 MODULE biolodynzwat

#if defined key_substance && defined key_biolo
   !!======================================================================
   !!                   ***  MODULE  biolodyn  ***
   !! Biologic dynamics: 
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters,   ONLY : rsh,rlg,nb_var,limin,limax,ljmin,ljmax,kmax,  &
                            epsilon,lchain,valmanq,imin,jmin,limaxp1,ljmaxp1, &
                            ljminp2,ljmaxm1,liminm1,limaxp2,ljminm1,ljmaxp2,  &
                            liminm2,ljminm2
   USE comvars2d,    ONLY : h0fond,id,ig,jb,jh,h0,jjulien,t,cdate,tdeb,   &
                            iscreenlog,imois,ijour,ian,iheure,iminu,isec
   USE comvars3d,    ONLY : sig,sigw
   USE comsubstance, ONLY : irk_mod,dtbio,t_cum_extinctionh,  &
                            extinction,extinction_ave4d,extinction_aveh,extinction_tab,  &
                            tdebsubs,nvp,nvpc,nv_adv,ws3,ws_free_min,ws_free_max,dcdt,cv_wat
   USE sflxatm,      ONLY : rad
   USE bioloinit
   USE toolgeom,     ONLY : f_dzu,f_dzw,f_zw

#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hc_sig,csw_sig,dcusds
   USE toolgeom,    ONLY : f_hzu
#endif

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_dyn_zwat        ! routine called by stepu.F90 and stepv.F90

   !! * Interface
   INTERFACE biolo_dyn_zwat
!     MODULE PROCEDURE verti_dinodiatzoo
     MODULE PROCEDURE verti_nut4phy3zoo2
   END INTERFACE

   !! * Shared module variables


   !! * Private variables

 CONTAINS

   !!======================================================================

  SUBROUTINE verti_nut4phy3zoo2(xe,cv_wat,sal,temp)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE  ***
   !&E
   !&E ** Purpose : calcul des effets inter-mailles non-hydrauliques
   !&E              (a estimer avant de calculer les variations a l interieur
   !&E               d une cellule donnee)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  (M. Sourisseau A. Menesguen) Original code
   !&E       !  2006-09  (V. Garnier)  module - mise en forme
   !&E       !  2006-09  (T. Odaka)  parallelisation openMP et MPI
   !&E       !  2008-01 (M. Sourisseau) evolutions
   !&E       !  2008-05 (M. Sourisseau) link between microzoo and diatoms
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2010-03  (A. Menesguen) addition of Pseudo-Nitzschia and N-tracking lines
   !&E       !  2012-10  (B. Thouvenin) fusion des versions biolo et mise a jour V906
   !&E       !  2013-09 (B.Thouvenin) integration of module tracer_N (A. Menesguen)
   !&E       !  2014-04 (B.Thouvenin) integration of module tracer_P (A. Menesguen) + update module Benthos
   !&E       !  2014-05 (B.Thouvenin) integration of key_zoo_prod (A. MENESGUEN)  
   !&E       !  2014-05 (B.Thouvenin) substance atmospheric inputs variable in space and time 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : obc_cv_ktherm_e,obc_cv_ktherm_w,    &
                            obc_cv_ktherm_n,obc_cv_ktherm_s,    &
                            alumplafond,attenuation,cmes_3dmgl
#ifdef key_karenia
   USE comvars3d,    ONLY : kz
#endif

!   USE comsubstance, ONLY : phytomoy,phytocarre
#ifdef key_sedim_anc
   USE sedimento,     ONLY : ksma,dzs
#endif
#ifdef key_parsub_newdt
   ! rajout pour calcul de ndt_part en fonction du temps, calcule en fonction de ws
   USE comsubstance, ONLY : imud1
#endif
#ifdef key_messat
   USE comsubstance, ONLY : messat,idateinf,idatesup,idimt_messat, &
                            t_clim_messat,date_s_annee
   USE comsubstance, ONLY : l_messat_clim,l_messat_obs,filemessat_obs,file_mes_obs_path &
                            ,messat_obs,idimt_messat_obs,t_obs_messat
#endif

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( in )   :: sal,temp
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( inout )   :: cv_wat

   !! * Local declarations
   INTEGER                  ::  iv,i,j,k,ivpc,is,js,ks         ! loop indexes
   CHARACTER(LEN=19)        :: tool_sectodat


   REAL(KIND=rsh),DIMENSION(nb_var,kmax,limin:limax,ljmin:ljmax)      :: cv_wat_pos
   REAL(KIND=rsh)   :: dzu,dzw

                         ! intermediate variable
                            ! --------------------------------------------
   REAL(KIND=rsh)           :: backgroundmes,fact_phyto_ChlNratio,cphyto

                            ! diagnoses
                            ! ---------
   INTEGER                  :: kmaxdiat,kmaxdino,kmaxnano,kminO2,kdcdz
   REAL(KIND=rsh)           :: xmaxdiat,datemaxdiat,xmaxdino,datemaxdino, &
                               xmaxnano,datemaxnano,xminO2,dateminO2,     &
                               dcdzmax,dcdz,cumuldiat,cumuldino,cumulnano,&
                               xmaxchloro,kmaxchloro,datemaxchloro,d,voleau

                            ! variables temperature, salinite
                            ! -------------------------------
   REAL(KIND=rsh)           :: temper,tempabs,effetchaleur,sali


#ifdef key_psnz
                            ! variables Pseudo-nitzschia
                            ! --------------------------
   REAL(KIND=rsh)           :: xmaxpsnz,datemaxpsnz,cumulpsnz
   INTEGER                  :: kmaxpsnz
#endif
#ifdef key_karenia
                            ! variables Karenia
                            ! -----------------
   REAL(KIND=rsh)           :: xmaxkarenia,datemaxkarenia,cumulkarenia
   INTEGER                  :: kmaxkarenia
#endif
#ifdef key_phaeocystis
                            ! variables Phaeocystis
                            ! ---------------------
   REAL(KIND=rsh)           :: xmaxphaeocystis,datemaxphaeocystis,cumulphaeocystis   
   INTEGER                  :: kmaxphaeocystis
#endif
#ifdef key_zoo_prod
                            ! variables production secondaire zooplanctonique
                            ! -----------------------------------------------
   REAL(KIND=rsh)           :: cumulpszoo_micr,cumulpszoo_meso   
#endif
                            ! calcul moyenne et ecart-type phyto surface
                            ! ------------------------------------------
!   REAL(KIND=rsh)           :: epcalc,cphyto
 !  REAL(KIND=rlg)           :: duree

#if defined key_messat
                            ! forcage MES par images satellites
                            ! ---------------------------------
   REAL(KIND=rsh), DIMENSION(limin:limax,ljmin:ljmax)   :: forcmes
   REAL(KIND=rsh)           :: interp
#endif
                            ! extinction moyennee sur 4 jours
                            ! ---------------------------------
   INTEGER                  :: iday,jhour,numday_extinction,numhour_extinction

   !!----------------------------------------------------------------------
   !! * Executable part

      cdate = tool_sectodat(t)
      CALL tool_decompdate(cdate,ijour,imois,ian,iheure,iminu,isec) 

           ! ==================================================================
           ! calcul du facteur d interpolation des MES lues dans fichiers de donnees satellitales
           ! ==================================================================

#if defined key_messat
      IF (l_messat_clim .EQV. .TRUE.) THEN

       IF (date_s_annee .NE. jjulien*24*60*60) THEN ! Definit des dates inf et sup pour l interp
        date_s_annee= jjulien*24*60*60              ! de la climato
        IF (idateinf .LT. idatesup) THEN
         IF (date_s_annee .GT. INT(t_clim_messat(idatesup))) THEN
          idateinf=idatesup
          IF (idatesup .NE. idimt_messat) THEN
           idatesup=idatesup+1
          ELSE
           idatesup=1
          END IF
         END IF
        ELSE
         IF ((date_s_annee .GT. INT(t_clim_messat(idatesup))) .AND.   &
               (date_s_annee .LT. INT(t_clim_messat(idateinf)))) THEN
          idateinf=1
          idatesup=idatesup+1
         END IF
        END IF

!        WRITE(iscreenlog,*) 'Simul a la date',date_s_annee,jjulien,imois,ijour
!        WRITE(iscreenlog,*) 'interpolation entre ',idateinf,'et',idatesup
!        WRITE(iscreenlog,*) 'soit, en s, entre ',INT(t_clim_messat(idateinf)), &
!                               ' et ',INT(t_clim_messat(idatesup))

       END IF !endif on date_s_annee

       IF (idatesup .GT. idateinf)  THEN
        interp= (date_s_annee -t_clim_messat(idateinf))/        &
                 (t_clim_messat(idatesup)-t_clim_messat(idateinf))
       ELSE
        IF (date_s_annee .GE. INT(t_clim_messat(idateinf))) THEN
        interp= (date_s_annee-t_clim_messat(idateinf))/&
                  ((366*24*60*60+t_clim_messat(idatesup))-t_clim_messat(idateinf))
        ELSE
        interp=(date_s_annee+366*24*60*60-t_clim_messat(idateinf))/ &
                  ((366*24*60*60+t_clim_messat(idatesup))-t_clim_messat(idateinf))
        END IF
       END IF

      END IF  !endif on l_messat_clim

      IF (l_messat_obs .EQV. .TRUE.) THEN
!       file_mes_obs_path=TRIM(filemessat_obs)//'/'//CHAR(ian)//'/'//    &
!                         CHAR(ian)//CHAR(imois)//CHAR(ijour)//'.nc'
         IF (t .GT. t_obs_messat(idatesup)) THEN
          idateinf=idatesup
          IF (idatesup .NE. idimt_messat_obs) THEN
           idatesup=idatesup+1
          ELSE
           write(*,*)'derniere date du fichier messat depassee'
           stop
          END IF
         END IF
!   WRITE(iscreenlog,*) 'date du run  = ',t
!   WRITE(iscreenlog,*) ' champs MES interp entre', idateinf,' et ',idatesup
!   WRITE(iscreenlog,*) ' champs MES interp entre', t_obs_messat(idateinf),' et ',t_obs_messat(idatesup)
       IF (idatesup .GT. idateinf)  THEN
        interp= (t -t_obs_messat(idateinf))/        &
                 (t_obs_messat(idatesup)-t_obs_messat(idateinf))
       END IF
       
      END IF

#endif

           ! ==================================================================
           ! mise a zero des vitesses de chute maximales de chaque groupe de particules
           ! ==================================================================

#ifdef key_parsub_newdt   
   ! rajoute pour calcul de ndt_part en fonction du temps
      ws3max_bio(:)=0.0_rsh
#endif
           ! ==================================================================
           ! implementation du cumul pour le calcul de l extinction moyenne horaire puis sur 4 jours
           !  t_cum_extinctionh a ete initialise dans subreadddat au debut de la simulation
           ! ==================================================================
      IF(iheure .ne. ihour_previous) THEN
         !write(*,*)'t_cum_extinctionh biolodynzwat',t_cum_extinctionh,dtbio,iminu,isec
         ihour_previous=iheure
         t_cum_extinctionh=0.0_rsh
      ELSE
         t_cum_extinctionh=t_cum_extinctionh+dtbio
      ENDIF
      numday_extinction=mod(jjulien,4)+1
      numhour_extinction=iheure+1
      !write(*,*)'t_cum_extinctionh biolodynzwat',t_cum_extinctionh,numday_extinction,numhour_extinction


!$OMP PARALLEL DEFAULT(none) SHARED(ig,id,jb,jh,nvp,nvpc,nv_adv,h0,xe,   &
!$OMP& h0fond,ljmin,ljmax,limin,limax,iv_spim,tdeb,                      &
!$OMP& iv_phyto_diat_N,id_spm_total,sigw,                                &
!$OMP& p_extincwat,iv_phyto_dino_N,iv_phyto_nano_N,p_phyto_ChlNratio,    &
!$OMP& p_extincChl1,p_extincChl2,id_extinctioncoeff,id_diat_max,         &
!$OMP& id_diat_datemax,id_diat_depthmax,id_dino_datemax,id_dino_depthmax,&
!$OMP& id_dino_max,id_nano_max,id_nano_datemax,id_nano_depthmax,         &
!$OMP& id_gradsali_max,id_gradsali_depthmax,id_gradtemp_max,             &
!$OMP& id_gradtemp_depthmax,iv_phyto_diat_pp,iv_phyto_dino_pp,           &
!$OMP& iv_phyto_nano_pp,id_diat_columnprod,id_dino_columnprod,           &
!$OMP& id_nano_columnprod,dtbio,p_extincspim,cmes_3dmgl,                 &
!$OMP& l_ChlNratio_var,extinction_aveh,t_cum_extinctionh,extinction_tab, &
!$OMP& numday_extinction,numhour_extinction,extinction_ave4d,            &
!$OMP& p_phyto_ChlN_ksmithextinct,p_phyto_ChlNratiomax,id_columnprodtotal,&
#if defined key_siggen || defined key_gencoord
!$OMP& hc_sig,csw_sig,                                                   &
#endif
#ifdef key_messat
!$OMP& messat,forcmes,idateinf,idatesup,interp,l_messat_clim,            &
!$OMP& l_messat_obs,                                                     &
#endif
!$OMP& cv_wat,irk_mod,ws_free_min, ws_free_max,                          &
!$OMP& diag_3d_wat,irk_diag,cv_wat_pos,                                  &
!$OMP& extinction,attenuation,alumplafond,rad,ws3,dcdt,t,                &
#ifdef key_parsub_newdt
!$OMP& ws3max_bio,imud1,                                                 &
#endif
#ifdef key_psnz
!$OMP& iv_phyto_psnz_N,id_psnz_max,id_psnz_datemax,                      &
!$OMP& id_psnz_depthmax,iv_phyto_psnz_pp,id_psnz_columnprod,             &
#endif
#ifdef key_karenia
!$OMP& iv_phyto_karenia_N,id_karenia_max,                                &
!$OMP& id_karenia_datemax,id_karenia_depthmax,iv_phyto_karenia_pp,       &
!$OMP& id_karenia_columnprod,                                            &
#endif
#ifdef key_phaeocystis
!$OMP& iv_phyto_phaeocystis_cell_N,iv_phyto_phaeocystis_colo_N,          &
!$OMP& id_phaeocystis_max,id_phaeocystis_datemax,id_phaeocystis_depthmax,&
!$OMP& iv_phyto_phaeocystis_pp,id_phaeocystis_columnprod,                &
#endif
#ifdef key_zoo_prod
!$OMP& iv_zoo_micr_ps,iv_zoo_meso_ps,id_zoo_columnprodtotal,		 &
!$OMP& id_zoo_micr_columnprod,id_zoo_meso_columnprod,			 &
#endif
#ifdef key_zostera
!$OMP& iv_zost_LB,p_zost_leafabscoef,p_zost_klai,                        &
#endif
#ifdef key_larve
!$OMP& i_ponte,iv_larve_1,iv_larve_2,iv_larve_3,iv_larve_4,iv_larve_5,   &
#endif
!$OMP& obc_cv_ktherm_s,obc_cv_ktherm_n,obc_cv_ktherm_w,obc_cv_ktherm_e,  &
!$OMP& diag_2d,sig,jjulien,sal,temp,imois,ijour,iheure)                  &
!$OMP& PRIVATE(i,j,k,iv,d,fact_phyto_ChlNratio,                          &
!$OMP& cphyto,dzu,dzw,backgroundmes,               &
#ifdef key_psnz
!$OMP& kmaxpsnz,xmaxpsnz,datemaxpsnz,cumulpsnz,                          &
#endif
#ifdef key_karenia
!$OMP& kmaxkarenia,xmaxkarenia,datemaxkarenia,cumulkarenia,              &
#endif
#ifdef key_phaeocystis
!$OMP& kmaxphaeocystis,xmaxphaeocystis,datemaxphaeocystis,               &
!$OMP& cumulphaeocystis,                                                 &
#endif
#ifdef key_zoo_prod
!$OMP& cumulpszoo_micr,cumulpszoo_meso,					 &
#endif
!$OMP& kmaxdiat,xmaxdiat,datemaxdiat,xmaxdino,kmaxdino,datemaxdino,      &
!$OMP& dcdzmax,kdcdz,dcdz,cumuldiat,cumuldino,effetchaleur,              &
!$OMP& xmaxnano,kmaxnano,datemaxnano,cumulnano)

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljmin,ljmax

     cv_wat_pos(:,:,limin:limax,j)=0.5_rsh*(cv_wat(:,:,limin:limax,j)+ABS(cv_wat(:,:,limin:limax,j)))

     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d=h0(i,j)+xe(i,j)

       ! no calcul de la profondeur en metres du plancher de chaque couche
       ! --------------------------------------------------------------
      ! DO k=1,kmax
      !         !! proposition benedicte : calcul comme dans sflxsurf
!#if defined key_siggen || defined key_gencoord
!               depthbottom(k,i,j) = - ( (xe(i,j)+hc_sig(i,j))*sigw(k-1) + (h0(i,j)-hc_sig(i,j))*Csw_sig(k-1) )
!#else
!               depthbottom(k,i,j) = -sigw(k-1)*d
!#endif
!               ! versin de Philippe/Clement : -sig(k)*d ???? pas bon en siggen et placé au milieu de la couche ??
      ! END DO
  
       IF (d.LE.h0fond) THEN
         ws3(:,1:nv_adv,i,j)=0.0_rsh      ! vitesses de chute ws3 nulles
#if defined key_messat
         forcmes(i,j)=valmanq
#endif
       ELSE

       ! calcul du forcage MES par donnees satellitales
       ! -----------------------------------------------

#if defined key_messat
        IF (l_messat_obs .EQV. .TRUE.) THEN
          forcmes(i,j)=messat_obs(i,j,idateinf)+                                &
               (messat_obs(i,j,idatesup)-messat_obs(i,j,idateinf))*interp

        END IF !endifon l_messat_obs

        IF (l_messat_clim .EQV. .TRUE.) THEN
          forcmes(i,j)=messat(i,j,idateinf)+                                &
               (messat(i,j,idatesup)-messat(i,j,idateinf))*interp

        END IF  !endif on l_messat_clim
#endif

        DO k=1,kmax
           ! epaisseur de la couche k
           ! ------------------------
           dzu=REAL(f_dzu(h0(i,j),xe(i,j),k,i,j),rsh)

   ! =================================================================
   ! Calcul par defaut des vitesses de chute  de toute variable particulaire (si pas key_sedim)
   !  sans modulation ni particularite (si different : calcul dans biolodyncellwat)
   ! =================================================================
           ! si key_sedim : les vitesses de chute sont calculees dans le module adequat
           ! ----------------------------------------------------------------
#if ! defined key_sedim
          ! ws3(k,iv_spim,i,j)=(xws(1,iv_spim)+xws(2,iv_spim))/2.0_rsh
           DO iv=1,nvp
             ws3(k,iv,i,j)=(ws_free_min(iv)+ws_free_max(iv))/2.0_rsh
           ENDDO
#endif
     
           ! bornage de la vitesse de chute des variables particulaires en chaque maille
           ! ---------------------------------------------------------------------------
           DO iv=1,nvp
             ws3(k,iv,i,j)=sign(MIN(0.95_rlg*dzu/dtbio,REAL(ABS(ws3(k,iv,i,j)),rlg)),ws3(k,iv,i,j))
           END DO

#ifdef key_parsub_newdt   
           ! calcul de la vitesse de chute max pour adaptation du nombre de sous pas de temps
           ! ---------------------------------------------------------------------------
           DO iv=imud1,nvp
                if (ws3(k,iv,i,j)>ws3max_bio(iv)) ws3max_bio(iv)=ws3(k,iv,i,j)
           ENDDO
#endif        


   ! ==================================
   ! Calcul du coefficient d extinction
   ! ==================================

   ! * Partie minerale:
   !   ---------------
           cmes_3dmgl(k,i,j)=0.0_rsh
           DO iv=1,nvpc
             cmes_3dmgl(k,i,j)=cmes_3dmgl(k,i,j)+cv_wat_pos(iv,k,i,j)*1000.0_rsh  !cmes_3dmgl passe en mg/l
           END DO

          ! backgroundmes=(0.03_rsh*d+10.0_rsh)/(0.1_rsh*d+0.5_rsh)
           backgroundmes=0.0_rsh

#if defined key_messat
#if defined key_biolo_opt2
           IF (forcmes(i,j)*1000.0_rsh .GT. cmes_3dmgl(k,i,j)) THEN  !forcage en g/l
                diag_3d_wat(irk_diag(id_spim_satused),k,i,j)=1.0_rsh
           ELSE
                diag_3d_wat(irk_diag(id_spim_satused),k,i,j)=0.0_rsh
           END IF
#endif
           backgroundmes=forcmes(i,j)*1000.0_rsh
#endif
          !       Cas ou on ne tient pas compte des MES simulees
          !           cmes_3dmgl(k,i,j)=backgroundmes               !M.E.S en mg/l
          !       Cas ou on tient compte des MES simulees
           cmes_3dmgl(k,i,j)=max(cmes_3dmgl(k,i,j),backgroundmes)     !M.E.S en mg/l
           cmes_3dmgl(k,i,j)=0.5_rsh*(cmes_3dmgl(k,i,j)+ABS(cmes_3dmgl(k,i,j)))     !Eq en mg/l

           diag_3d_wat(irk_diag(id_spm_total),k,i,j)=cmes_3dmgl(k,i,j)

   ! calcul de l extinction
   ! ----------------------
   !
         ! - Partie due au mineral (formule Gohin):
         ! --------------------------------------
           extinction(k,i,j)=p_extincspim*(cmes_3dmgl(k,i,j)+epsilon)+p_extincwat

   ! - Partie due au phytoplancton (formule Gohin):
   ! --------------------------------------------
        ! rapport Chloro/N dependant de l extinction lumineuse
          IF(l_ChlNratio_var) THEN
            IF((t-tdeb)>345600.0_rlg) THEN
               fact_phyto_ChlNratio=max(0.70_rsh,p_phyto_ChlNratiomax*(extinction_ave4d(k,i,j)/p_phyto_ChlN_ksmithextinct)/ &
                              (sqrt(1+(extinction_ave4d(k,i,j)/p_phyto_ChlN_ksmithextinct)**2))) 
           ELSE
               fact_phyto_ChlNratio=max(0.70_rsh,p_phyto_ChlNratiomax*(extinction(k,i,j)/p_phyto_ChlN_ksmithextinct)/ &
                              (sqrt(1+(extinction(k,i,j)/p_phyto_ChlN_ksmithextinct)**2)))
           ENDIF   
          ELSE
               fact_phyto_ChlNratio=p_phyto_ChlNratio
          ENDIF
          cphyto=(cv_wat_pos(iv_phyto_diat_N,k,i,j)+ cv_wat_pos(iv_phyto_dino_N,k,i,j)  &
                +cv_wat_pos(iv_phyto_nano_N,k,i,j) &
#ifdef key_psnz
                +cv_wat_pos(iv_phyto_psnz_N,k,i,j)   &
#endif
#ifdef key_karenia
                +cv_wat_pos(iv_phyto_karenia_N,k,i,j)   &
#endif
#ifdef key_phaeocystis
                +cv_wat_pos(iv_phyto_phaeocystis_cell_N,k,i,j)+ cv_wat_pos(iv_phyto_phaeocystis_colo_N,k,i,j)  & 
#endif
                +epsilon)*fact_phyto_ChlNratio

          cphyto=(ABS(cphyto)+cphyto)*0.5_rsh
          extinction(k,i,j)=extinction(k,i,j)+p_extincChl1*(cphyto)**p_extincChl2

#if defined key_zostera
   ! - Partie due aux zosteres :
   ! ---------------------------
           if(k==1 .and. cv_wat_pos(iv_zost_LB,k,i,j).gt.0.0_rsh) then
             extinction(k,i,j)=extinction(k,i,j)+p_zost_leafabscoef*cv_wat_pos(iv_zost_LB,k,i,j)*p_zost_klai
           endif
#endif

          diag_3d_wat(irk_diag(id_extinctioncoeff),k,i,j)=extinction(k,i,j)
          attenuation(k,i,j)=EXP(-extinction(k,i,j)*dzu)

   ! - calcul de l extinction moyenne sur 4 jours 
   ! --------------------------------------------
          IF(l_ChlNratio_var) THEN
             extinction_aveh(k,i,j)=extinction_aveh(k,i,j)+extinction(k,i,j)*dtbio
             !if(k==10 .and.i==20 .and.j==20)write(*,*)'extinction_aveh',extinction_aveh(k,i,j),extinction(k,i,j)
             IF(t_cum_extinctionh >= (3600._rsh-dtbio)) THEN
                extinction_aveh(k,i,j)=extinction_aveh(k,i,j)/t_cum_extinctionh
                extinction_tab(numday_extinction,numhour_extinction,k,i,j)=extinction_aveh(k,i,j)
                extinction_ave4d(k,i,j)=0.0_rsh
                DO iday=1,4
                DO jhour=1,24
                   extinction_ave4d(k,i,j)=extinction_ave4d(k,i,j)+extinction_tab(iday,jhour,k,i,j)
                END DO
                END DO
                extinction_ave4d(k,i,j)=extinction_ave4d(k,i,j)/96.0_rsh
              ENDIF
          ENDIF

        END DO  ! fin boucle sur k

   ! calcul de la lumiere au plafond de chaque maille
   ! ------------------------------------------------
         alumplafond(kmax,i,j)=rad(i,j)
         DO k=kmax-1,1,-1
           alumplafond(k,i,j)=alumplafond(k+1,i,j)*attenuation(k+1,i,j)
         END DO

       END IF
     END DO
   END DO
!$OMP END DO


   ! ---------------------------------------------
   ! Diagnostics
   ! ---------------------------------------------

!$OMP single
   obc_cv_ktherm_s(:)=0
   obc_cv_ktherm_n(:)=0
   obc_cv_ktherm_w(:)=0
   obc_cv_ktherm_e(:)=0
!$OMP END single

   ! calcul du max. de phyto avec date du max et profondeur du max
   ! -------------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)

   DO j=ljmin,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
#if ! defined key_biolo_opt2
       !variable diagnostique des max et des dat de max remises à zero tous les mois dans l option 1
       IF ((imois.eq.1).and.(ijour.eq.1).and.(iheure.eq.0)) THEN
            diag_2d(irk_diag(id_diat_max),i,j)=0.0_rsh
            diag_2d(irk_diag(id_dino_max),i,j)=0.0_rsh
            diag_2d(irk_diag(id_nano_max),i,j)=0.0_rsh
#ifdef key_psnz
            diag_2d(irk_diag(id_psnz_max),i,j)=0.0_rsh
#endif
#ifdef key_karenia
            diag_2d(irk_diag(id_karenia_max),i,j)=0.0_rsh
#endif
#ifdef key_phaeocystis
            diag_2d(irk_diag(id_phaeocystis_max),i,j)=0.0_rsh
#endif

            diag_2d(irk_diag(id_diat_datemax),i,j)=0.0_rsh
            diag_2d(irk_diag(id_dino_datemax),i,j)=0.0_rsh
            diag_2d(irk_diag(id_nano_datemax),i,j)=0.0_rsh

            diag_2d(irk_diag(id_diat_depthmax),i,j)=0.0_rsh
            diag_2d(irk_diag(id_dino_depthmax),i,j)=0.0_rsh
            diag_2d(irk_diag(id_nano_depthmax),i,j)=0.0_rsh
       ENDIF
#endif
       d=h0(i,j)+xe(i,j)
       IF (d > h0fond) THEN

   ! diatomees
   ! ---------
         kmaxdiat=1
         xmaxdiat=0.0_rsh
         datemaxdiat=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_diat_N,k,i,j).GE.xmaxdiat) THEN
             xmaxdiat=cv_wat_pos(iv_phyto_diat_N,k,i,j)
             kmaxdiat=k
             datemaxdiat=jjulien
           END IF
         END DO

         IF (xmaxdiat.GT.diag_2d(irk_diag(id_diat_max),i,j)) THEN
           diag_2d(irk_diag(id_diat_max),i,j)=xmaxdiat
           diag_2d(irk_diag(id_diat_datemax),i,j)=datemaxdiat
           diag_2d(irk_diag(id_diat_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxdiat,i,j)-xe(i,j))
         END IF

   ! dinoflagelles
   ! -------------
         xmaxdino=0.0_rsh
         kmaxdino=1
         datemaxdino=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_dino_N,k,i,j).GE.xmaxdino) THEN
             xmaxdino=cv_wat_pos(iv_phyto_dino_N,k,i,j)
             kmaxdino=k
             datemaxdino=jjulien
           END IF
         END DO

         IF (xmaxdino.GT.diag_2d(irk_diag(id_dino_max),i,j)) THEN
           diag_2d(irk_diag(id_dino_max),i,j)=xmaxdino
           diag_2d(irk_diag(id_dino_datemax),i,j)=datemaxdino
           diag_2d(irk_diag(id_dino_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxdino,i,j)-xe(i,j))
         END IF

   ! nanoflagelles
   ! -------------
         xmaxnano=0.0_rsh
         kmaxnano=1
         datemaxnano=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_nano_N,k,i,j).GE.xmaxnano) THEN
             xmaxnano=cv_wat_pos(iv_phyto_nano_N,k,i,j)
             kmaxnano=k
             datemaxnano=jjulien
           END IF
         END DO

         IF (xmaxnano.GT.diag_2d(irk_diag(id_nano_max),i,j)) THEN
           diag_2d(irk_diag(id_nano_max),i,j)=xmaxnano
           diag_2d(irk_diag(id_nano_datemax),i,j)=datemaxnano
           diag_2d(irk_diag(id_nano_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxnano,i,j)-xe(i,j))
         END IF

#ifdef key_psnz
   ! PseudoNitzschia
   ! -------------
         xmaxpsnz=0.0_rsh
         kmaxpsnz=1
         datemaxpsnz=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_psnz_N,k,i,j).GE.xmaxpsnz) THEN
             xmaxpsnz=cv_wat_pos(iv_phyto_psnz_N,k,i,j)
             kmaxpsnz=k
             datemaxpsnz=jjulien
           END IF
         END DO

         IF (xmaxpsnz.GT.diag_2d(irk_diag(id_psnz_max),i,j)) THEN
           diag_2d(irk_diag(id_psnz_max),i,j)=xmaxpsnz
           diag_2d(irk_diag(id_psnz_datemax),i,j)=datemaxpsnz
           diag_2d(irk_diag(id_psnz_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxpsnz,i,j)-xe(i,j))
         END IF
#endif
#ifdef key_karenia
   ! Karenia
   ! -------
         xmaxkarenia=0.0_rsh
         kmaxkarenia=1
         datemaxkarenia=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_karenia_N,k,i,j).GE.xmaxkarenia) THEN
             xmaxkarenia=cv_wat_pos(iv_phyto_karenia_N,k,i,j)
             kmaxkarenia=k
             datemaxkarenia=jjulien
           END IF
         END DO

         IF (xmaxkarenia.GT.diag_2d(irk_diag(id_karenia_max),i,j)) THEN
           diag_2d(irk_diag(id_karenia_max),i,j)=xmaxkarenia
           diag_2d(irk_diag(id_karenia_datemax),i,j)=datemaxkarenia
           diag_2d(irk_diag(id_karenia_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxkarenia,i,j)-xe(i,j))
         END IF
#endif
#ifdef key_phaeocystis
   ! Phaeocystis
   ! -----------
         xmaxphaeocystis=0.0_rsh
         kmaxphaeocystis=1
         datemaxphaeocystis=1
         DO k=1,kmax
           IF (cv_wat_pos(iv_phyto_phaeocystis_colo_N,k,i,j).GE.xmaxphaeocystis) THEN
             xmaxphaeocystis=cv_wat_pos(iv_phyto_phaeocystis_colo_N,k,i,j)
             kmaxphaeocystis=k
             datemaxphaeocystis=jjulien
           END IF
         END DO

         IF (xmaxphaeocystis.GT.diag_2d(irk_diag(id_phaeocystis_max),i,j)) THEN
           diag_2d(irk_diag(id_phaeocystis_max),i,j)=xmaxphaeocystis
           diag_2d(irk_diag(id_phaeocystis_datemax),i,j)=datemaxphaeocystis
           diag_2d(irk_diag(id_phaeocystis_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kmaxphaeocystis,i,j)-xe(i,j))
         END IF
#endif

   ! calcul de l intensite et profondeur de l halocline
   ! --------------------------------------------------
         dcdzmax=0.0_rsh
         kdcdz=kmax
         DO k=1,kmax-1
           ! version Philippe dzw : epaisseur de la couche entre sal(k) et sal(k+1)
            dzw=f_dzw(h0(i,j),xe(i,j),k,i,j)
            dcdz=(sal(k,i,j)-sal(k+1,i,j))/dzw
            dcdz=ABS(dcdz)
           IF (dcdz > dcdzmax) THEN
             kdcdz=k+1
             dcdzmax=dcdz
           END IF
         END DO
         diag_2d(irk_diag(id_gradsali_max),i,j)=dcdzmax
         diag_2d(irk_diag(id_gradsali_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kdcdz,i,j)-xe(i,j))

   ! calcul de l intensite et profondeur de la thermocline
   ! -----------------------------------------------------
         dcdzmax=0.0_rsh
         kdcdz=0
         DO k=1,kmax-1
          dzw=f_dzw(h0(i,j),xe(i,j),k,i,j)
          dcdz=ABS(temp(k,i,j)-temp(k+1,i,j))/dzw
           IF (dcdz > dcdzmax) THEN
             kdcdz=k+1
             dcdzmax=dcdz
           END IF
   !        diag_2d(irk_diag(40),i,j)=temp(kmax,i,j)-temp(1,i,j)
         END DO
         diag_2d(irk_diag(id_gradtemp_max),i,j)=dcdzmax
         diag_2d(irk_diag(id_gradtemp_depthmax),i,j)=- (f_zw(h0(i,j),xe(i,j),kdcdz,i,j)-xe(i,j))

   ! niveau du gradient vertical de temperature max pour injection
   ! des concentrations surface-fond en limite ouest et sud
   ! -------------------------------------------------------------
         IF (ljmin==jmin .AND. j.EQ.jb(i)) obc_cv_ktherm_s(i)=kdcdz
         IF (limin==imin .AND. i.EQ.ig(j)) obc_cv_ktherm_w(j)=kdcdz

   ! calcul de la production cumulee integree sur la verticale
   ! ATTENTION : production exprimee en carbone pour l option 1 et en azote pour l option 2 
   ! ---------------------------------------------------------------------------------------
#if ! defined key_biolo_opt2
        !production cumulee depuis le 1er janvier dans l option 1
         IF ((imois.eq.1).and.(ijour.eq.1).and.(iheure.eq.0)) THEN
          DO k=1,kmax
            cv_wat(iv_phyto_diat_pp,k,i,j)=0.0_rsh
            cv_wat(iv_phyto_dino_pp,k,i,j)=0.0_rsh
            cv_wat(iv_phyto_nano_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_diat_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_dino_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_nano_pp,k,i,j)=0.0_rsh
#ifdef key_psnz
            cv_wat(iv_phyto_psnz_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_psnz_pp,k,i,j)=0.0_rsh
#endif
#ifdef key_karenia
            cv_wat(iv_phyto_karenia_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_karenia_pp,k,i,j)=0.0_rsh
#endif
#ifdef key_phaeocystis
            cv_wat(iv_phyto_phaeocystis_pp,k,i,j)=0.0_rsh
            cv_wat_pos(iv_phyto_phaeocystis_pp,k,i,j)=0.0_rsh
#endif
#ifdef key_zoo_prod
            cv_wat(iv_zoo_micr_ps,k,i,j)=0.0_rsh
            cv_wat_pos(iv_zoo_micr_ps,k,i,j)=0.0_rsh
            cv_wat(iv_zoo_meso_ps,k,i,j)=0.0_rsh
            cv_wat_pos(iv_zoo_meso_ps,k,i,j)=0.0_rsh
#endif
          END DO
         ENDIF
#endif
         cumuldiat=0.0_rsh
         cumuldino=0.0_rsh
         cumulnano=0.0_rsh
#ifdef key_psnz
         cumulpsnz=0.0_rsh
#endif
#ifdef key_karenia
         cumulkarenia=0.0_rsh
#endif
#ifdef key_phaeocystis
         cumulphaeocystis=0.0_rsh
#endif
#ifdef key_zoo_prod
         cumulpszoo_micr=0.0_rsh
         cumulpszoo_meso=0.0_rsh
#endif
         DO k=1,kmax
           cumuldiat=cumuldiat+cv_wat_pos(iv_phyto_diat_pp,k,i,j)
           cumuldino=cumuldino+cv_wat_pos(iv_phyto_dino_pp,k,i,j)
           cumulnano=cumulnano+cv_wat_pos(iv_phyto_nano_pp,k,i,j)
#ifdef key_psnz
           cumulpsnz=cumulpsnz+cv_wat_pos(iv_phyto_psnz_pp,k,i,j)
#endif
#ifdef key_karenia
           cumulkarenia=cumulkarenia+cv_wat_pos(iv_phyto_karenia_pp,k,i,j)
#endif
#ifdef key_phaeocystis
           cumulphaeocystis=cumulphaeocystis+cv_wat_pos(iv_phyto_phaeocystis_pp,k,i,j)
#endif
#ifdef key_zoo_prod
           cumulpszoo_micr=cumulpszoo_micr+cv_wat_pos(iv_zoo_micr_ps,k,i,j)
           cumulpszoo_meso=cumulpszoo_meso+cv_wat_pos(iv_zoo_meso_ps,k,i,j)
#endif
         END DO
         diag_2d(irk_diag(id_diat_columnprod),i,j)=cumuldiat
         diag_2d(irk_diag(id_dino_columnprod),i,j)=cumuldino
         diag_2d(irk_diag(id_nano_columnprod),i,j)=cumulnano
#ifdef key_psnz
         diag_2d(irk_diag(id_psnz_columnprod),i,j)=cumulpsnz
#endif
#ifdef key_karenia
         diag_2d(irk_diag(id_karenia_columnprod),i,j)=cumulkarenia
#endif
#ifdef key_phaeocystis
         diag_2d(irk_diag(id_phaeocystis_columnprod),i,j)=cumulphaeocystis
#endif
#ifdef key_zoo_prod
         diag_2d(irk_diag(id_zoo_micr_columnprod),i,j)=cumulpszoo_micr
         diag_2d(irk_diag(id_zoo_meso_columnprod),i,j)=cumulpszoo_meso
#endif
#if ! defined key_biolo_opt2
   !   production cumulee dans le temps et sur la verticale de tous les types phytos   
         diag_2d(irk_diag(id_columnprodtotal),i,j)=cumuldiat+cumuldino+cumulnano
#ifdef key_psnz
         diag_2d(irk_diag(id_columnprodtotal),i,j)=diag_2d(irk_diag(id_columnprodtotal),i,j) + cumulpsnz
#endif 
#ifdef key_karenia
         diag_2d(irk_diag(id_columnprodtotal),i,j)=diag_2d(irk_diag(id_columnprodtotal),i,j) + cumulkarenia
#endif  
#ifdef key_phaeocystis
         diag_2d(irk_diag(id_columnprodtotal),i,j)=diag_2d(irk_diag(id_columnprodtotal),i,j) + cumulphaeocystis
#endif 
#ifdef key_zoo_prod
         diag_2d(irk_diag(id_zoo_columnprodtotal),i,j)=cumulpszoo_micr+cumulpszoo_meso
#endif                        
#endif                                             

       END IF

     END DO
   END DO
   
#ifdef key_larve
    !bdstbrieuc 
    
    !IF ((imois.eq.8).and.(ijour.eq.8).and.(iheure.eq.0).and.(i_ponte.eq.0)) then
   
   !bdeseine
   
   IF ((imois.eq.8).and.(ijour.eq.19).and.(iheure.eq.0).and.(i_ponte.eq.0)) then
    DO j=ljminm1,ljmaxp2
      DO i=liminm1,limaxp2
  
  !baie de st Brieuc
           
    !    if(j>=47.and.j<=51.and.i>=147.and.i<=150.or.j>=48.and.j<=51.and.i>=150.and.i<=155) then
    !       cv_wat(iv_larve_1,:,i,j)=1.0_rsh
    !  END IF
!	if(j>=44.and.j<=47.and.i>=148.and.i<=150.or.j>=44.and.j<=48.and.i>=150.and.i<=155) then
!          cv_wat(iv_larve_2,:,i,j)=1.0_rsh
!        END IF
!	if(j>=40.and.j<=44.and.i>=151.and.i<=153.or.j>=38.and.j<=44.and.i>=153.and.i<=155) then
!           cv_wat(iv_larve_3,:,i,j)=1.0_rsh
!        END IF
!	if(j>=38.and.j<=44.and.i>=155.and.i<=157.or.j>=38.and.j<=42.and.i>=157.and.i<=159) then
!           cv_wat(iv_larve_4,:,i,j)=1.0_rsh
!        END IF
!        if(j>=44.and.j<=48.and.i>=155.and.i<=157.or.j>=42.and.j<=48.and.i>=157.and.i<=159.or. &
!                         j>=40.and.j<=46.and.i>=159.and.i<=161) then
!           cv_wat(iv_larve_5,:,i,j)=1.0_rsh
!        END IF
!	if(j>=46.and.j<=48.and.i>=159.and.i<=162.or.j>=44.and.j<=48.and.i>=162.and.i<=167) then
!         cv_wat(iv_larve_6,:,i,j)=1.0_rsh
!       END IF
!	i_ponte=1
!    
    !baie de seine
     
      if(j>=85.and.j<=96.and.i>=206.and.i<=209.or.j>=85.and.j<=90.and.i>=209.and.i<=214) then
          cv_wat(iv_larve_1,:,i,j)=1.0
        END IF
        if(j>=90.and.j<=96.and.i>=209.and.i<=217.or.j>=85.and.j<=90.and.i>=214.and.i<=220) then
          cv_wat(iv_larve_2,:,i,j)=1.0
        END IF
         if(j>=90.and.j<=96.and.i>=217.and.i<=220.or.j>=85.and.j<=93.and.i>=220.and.i<=229) then
          cv_wat(iv_larve_3,:,i,j)=1.0
        END IF
        if(j>=85.and.j<=93.and.i>=229.and.i<=240) then
           cv_wat(iv_larve_4,:,i,j)=1.0
        END IF
        if(j>=93.and.j<=105.and.i>=220.and.i<=243) then
           cv_wat(iv_larve_5,:,i,j)=1.0
        END IF     
   
     i_ponte=1
   
     END DO
   END DO
  
  END IF
#endif

!$OMP END DO
!$OMP END PARALLEL

  END SUBROUTINE verti_nut4phy3zoo2

   !!======================================================================

  SUBROUTINE verti_bio_nut4phy3zoo2(xe,cv_wat)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE   ***
   !&E
   !&E ** Purpose : calcul du deplacement des var bio sur la verticale avec
   !&E              un simple schema d Euler (totalement conservatif)
   !&E ** Description :
   !&E
   !&E ** Called by : verti_dinodiatzoo
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E              25/07/2007  (M. Sourisseau) Original code
   !&E       !  2008-01 (M. Sourisseau) evolutions
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

   USE comvars2d,    ONLY : h0fond,id,ig,h0,hm
   USE comsubstance, ONLY : dtbio,nvp,ws3
   USE_MPI toolmpi,      ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT( inout )  :: cv_wat
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in )   :: xe


   !! * Local declarations
   INTEGER                  ::  i,j,k,l         ! loop indexes
   REAL(KIND=rsh)           ::  d
   REAL(KIND=rlg)           ::  sign_up,sign_do,test_in,test_out
   REAL(KIND=rlg) ,DIMENSION(kmax)    :: cv_wat_buff,cv_wat_buff_in
   REAL(KIND=rlg) ,DIMENSION(kmax)    :: epaiss

   !&E---------------------------------------------------------------------

!$OMP PARALLEL DEFAULT(none)                                            &
!$OMP& shared(h0fond,id,ig,h0,limin,limax,ljminp2,ljmaxm1,dtbio,hm,     &
!$OMP&        nvp,ws3,cv_wat,xe)                                        &
!$OMP& private(i,j,k,l,epaiss,cv_wat_buff,cv_wat_buff_in,sign_up,       &
!$OMP&         sign_do,d,test_in,test_out)

!$OMP DO SCHEDULE(RUNTIME)
   DO l=1,nvp
    DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d=h0(i,j)+xe(i,j)
       IF (d.GT.hm) THEN   ! no sink in 2D : vertical homogeneity - see tssub_eq_hm.F90
       test_in=0.0_rlg
       test_out=0.0_rlg
       DO k=1,kmax
        epaiss(k)=f_dzu(h0(i,j),xe(i,j),k,i,j)
        cv_wat_buff(k)=0.0_rlg
        cv_wat_buff_in(k)=0.5_rlg*REAL(cv_wat(l,k,i,j)+ABS(cv_wat(l,k,i,j)),rlg)
!        cv_wat_buff_in(l,k,i,j)=cv_wat(l,k,i,j)
        test_in=test_in+cv_wat_buff_in(k)*epaiss(k)
 !       IF (cv_wat_buff_in(l,k,i,j).lt.0.) THEN
 !         write(*,*) 'pb avant verti_bio_nut4phy3zoo2',cv_wat_buff_in(l,k,i,j),l,k,i,j,h0(i,j)
 !         write(*,*) 'mod plus conservatif'
 !       END IF
       END DO

      IF (test_in .GT. 10.0_rlg**-20) THEN
       DO k=1,kmax
        IF (k.LT.kmax) THEN
         IF (ws3(k+1,l,i,j).GT.0.0_rsh) THEN    !vitesse positive vers le bas
          sign_up=1.0_rlg
         ELSE
          sign_up=0.0_rlg
         END IF
        ELSE
         sign_up=0.0_rlg
        END IF
        IF (k.GT.1) THEN
         IF (ws3(k-1,l,i,j).LT.0.0_rsh) THEN
          sign_do=1.0_rlg
         ELSE
          sign_do=0.0_rlg
         END IF
        ELSE
         sign_do=0.0_rlg
        END IF

        IF (k.EQ.1) THEN
         ws3(k,l,i,j)=MIN(0.0_rsh,ws3(k,l,i,j))
         cv_wat_buff(k)=cv_wat_buff_in(k)                                           &  !quitte la cell
              -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)   &  !quitte la cell
              +sign_up*cv_wat_buff_in(k+1)*epaiss(k+1)/epaiss(k)               & !quitte la cell sup
                        *MIN(0.95_rlg,REAL(ws3(k+1,l,i,j),rlg)/epaiss(k+1)*dtbio)
        ELSE IF (k.EQ.kmax) THEN
         ws3(k,l,i,j)=MAX(0.0_rsh,ws3(k,l,i,j))
         cv_wat_buff(k)=cv_wat_buff_in(k)                                          &
                   -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)  &  !quitte la cell
                   +sign_do*cv_wat_buff_in(k-1)*epaiss(k-1)/epaiss(k)              & !quitte la cell inf
                        *MIN(0.95_rlg,REAL(ws3(k-1,l,i,j),rlg)/epaiss(k-1)*dtbio)
        ELSE
         cv_wat_buff(k)=cv_wat_buff_in(k)                                          &
                   -cv_wat_buff_in(k)*MIN(0.95_rlg,REAL(ABS(ws3(k,l,i,j)),rlg)/epaiss(k)*dtbio)  &  !quitte la cell
                   +sign_up*cv_wat_buff_in(k+1)*epaiss(k+1)/epaiss(k)              &  !quitte la cell sup
                        *MIN(0.95_rlg,REAL(ws3(k+1,l,i,j),rlg)/epaiss(k+1)*dtbio)                &
                   +sign_do*cv_wat_buff_in(k-1)*epaiss(k-1)/epaiss(k)              &  !quitte la cell inf
                        *MIN(0.95_rlg,REAL(ws3(k-1,l,i,j),rlg)/epaiss(k-1)*dtbio)
        END IF
       ENDDO !enddo on k

       DO k=1,kmax
         test_out=test_out+cv_wat_buff(k)*epaiss(k)
       ENDDO

       IF (test_in .GT. 10.0_rlg**-20) THEN
!       IF (abs(test_in-test_out)/test_in*REAL(100/dtbio*24*60*60,rsh) .gt. 0.1_rsh) THEN
       IF (ABS(test_in-test_out)/test_in*100.0_rlg .GT. 0.0000000001_rlg) THEN
        WRITE(*,*) 'pb-verti_bio_nut4phy3zoo2',ABS(test_in-test_out)/test_in*100/dtbio*24*60*60,       &
                                 test_in,test_out,i,j,l,d
        ! write(*,*) 'cv_wat_buff_in',cv_wat_buff_in(1:kmax)
        ! write(*,*) 'cv_wat_buff',cv_wat_buff(1:kmax)
        ! write(*,*) 'ws3',ws3(1:kmax,l,i,j)
        STOP
       END IF
       END IF

       DO k=1,kmax
         cv_wat(l,k,i,j)=REAL(cv_wat_buff(k),rsh)
       ENDDO

       END IF !end if on test_in

       END IF !end if on h0fond
     END DO !end do on i
    END DO !end do on j
   END DO !end do on nvp
!$OMP END DO
!$OMP END PARALLEL

   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))

  END SUBROUTINE verti_bio_nut4phy3zoo2

   !!======================================================================

#endif

 END MODULE
