 MODULE biolosinksource

#if defined key_substance && defined key_biolo

   !!======================================================================
   !!                   ***  MODULE  biolosinksource ***
   !! Biologic dynamics:  - estimate sink and source terms for each variables
   !!                     - update a few variable
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC biolo_sksc_sed,           &  ! routine called by stepu.F90 stepv.F90
          biolo_sksc_wat               ! routine called by stepu.F90 stepv.F90

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE biolo_sksc_sed

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_sksc_sed  ***
   !&E
   !&E ** Purpose : Estimate transformations inside sediments
   !&E              Estimate new concentrations in sediments (cv_sed)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          (A. Menesguen) Original code
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_sedim_anc  && defined key_biolo
   USE parameters,   ONLY : jmin,jmax,valmanq
   USE comvars2d,    ONLY : ig,id,h0
   USE comsubstance, ONLY : nv_state,dtbio
   USE biolodyncellsed,ONLY : biolo_dyn_cellsed
   USE sedimento,     ONLY : cv_sed,ksmi,ksma,dzs

   !! * Declaration Subroutine

   !! * Arguments

   !! * Local declarations
   INTEGER                :: i,j,k
   REAL(KIND=rsh),DIMENSION(nv_state)  :: c,dc

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP PARALLEL DEFAULT(none) shared(ig,id,h0,ksmi,ksma,dzs,cv_sed &
!$OMP& ,nv_state,dtbio,limin,limax,ljminp2,ljmaxm1)               &
!$OMP& private(i,j,k,c,dc )

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
       IF (h0(i,j).GT.-valmanq)  THEN
         DO k=ksmi(i,j),ksma(i,j)
           IF (dzs(k,i,j).GT.0.0_rsh) THEN
!cval             CALL advtobiosed(k,i,j)
             ! initialize to 0
             dc(1:nv_state)=0.0_rsh
             ! transfer 3D var cv_sed into 1D var c
             c(1:nv_state)=0.5_rsh*( cv_sed(1:nv_state,k,i,j)+   &
                                         ABS(cv_sed(1:nv_state,k,i,j)) )
             CALL biolo_dyn_cellsed(k,i,j,c,dc)
             ! update cv_sed and transfer from 1D to 3D 
             cv_sed(1:nv_state,k,i,j)=cv_sed(1:nv_state,k,i,j)+  &
                       dtbio/86400.0_rsh*dc(1:nv_state)
           ENDIF
         END DO
       END IF
     END DO
   END DO
!$OMP END DO

!$OMP END PARALLEL

#elif key_sedim_anc
        WRITE(*,*)  'biolo_sksc_sed : You should not enter this routine'
#endif

  END SUBROUTINE biolo_sksc_sed

   !!======================================================================

  SUBROUTINE biolo_sksc_wat(cv_wat,sal,temp,xe)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE biolo_sksc_wat  ***
   !&E
   !&E ** Purpose : Estimate variations of concentrations in the water.
   !&E              Estimate new concentrations of fixed variables.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          (A. Menesguen) Original code
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,   ONLY : kmax,limin,limax,ljmin,ljmax,rsh,nb_var,ljminp2,ljmaxm1, &
                            liminm1,limaxp2,ljminm1,ljmaxp2,limaxp1,ljmaxp1, &
                            liminm2,ljminm2
   USE comvars2d,    ONLY : h0,ig,id,h0fond
   USE comsubstance, ONLY : nv_adv,nv_state,dcdt,dtbio
   USE biolodyncellwat,ONLY : biolo_dyn_cellwat
   USE_MPI toolmpi,      ONLY : ex_i_rsh,ex_j_rsh

   !! * Declaration Subroutine

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: xe
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: sal,temp
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                :: i,j,k
   REAL(KIND=rsh),DIMENSION(nv_state)  :: c,dc

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP PARALLEL DEFAULT(none) shared(ig,id,h0,xe,h0fond,cv_wat   &
!$OMP& ,nv_state,nv_adv,dcdt,dtbio,limin,limax,ljminp2,ljmaxm1) &
!$OMP& private(i,j,k,c,dc )

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       IF (h0(i,j)+xe(i,j).GT.h0fond)  THEN
         DO k=1,kmax
           ! initialize to 0
           dc(1:nv_state)=0.0_rsh
           ! 3D to 1D for variable cv_wat(nb_var,kmax,limin:limax,ljmin:ljmax)
!cval                call advtobioeau(k,i,j,cv_wat,sal,temp)
           c(1:nv_state)=0.5_rsh*( cv_wat(1:nv_state,k,i,j)+   &
                                    ABS(cv_wat(1:nv_state,k,i,j)) )
           CALL biolo_dyn_cellwat(k,i,j,xe(i,j),c,dc)
!cval                call biotoadv(k,i,j)
           ! 1D to 3D or variable dcdt(nv_state,kmax,limin:limax,ljmin:ljmax)
           dcdt(1:nv_state,k,i,j)=dc(1:nv_state)/86400.0_rsh
           ! evolution des variables fixees :
           cv_wat(nv_adv+1:nv_state,k,i,j)=cv_wat(nv_adv+1:nv_state,k,i,j) &
                                      +dtbio*dcdt(nv_adv+1:nv_state,k,i,j)
         END DO
       END IF
     END DO
   END DO
!$OMP END DO

!$OMP END PARALLEL

   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))

  END SUBROUTINE biolo_sksc_wat
#endif

   !!======================================================================

 END MODULE
