 MODULE userconta
#if defined key_substance && defined key_contaminant
   !!======================================================================
   !!                   ***  MODULE  userconta  ***
   !! Chimestry - Contaminants dynamics:programs which have to be written 
   !!                        by users of key_contaminant 
   !!        in order to define some variable coefficients 
   !!            (cinetics, stochiometriques coefs etc..)
   !!         or initial conditions
   !!======================================================================
   !! * Modules used
   USE comsubstance
   USE parameters, ONLY : rsh,rlg,lchain,imin,imax,jmin,jmax,kmax,ljmin,ljmax,limin,limax  
   USE comvars2d,  ONLY : iscreenlog        
#ifdef key_sedim                          
   USE parameters, ONLY : ksdmin,ksdmax    
#endif
   USE parameters, ONLY : nb_var   

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC userconta_cin_re2,   &                 ! routine called by incellwat_GEN
          userconta_coef_re2                     ! routine called by incellwat_GEN
          

   !! * Shared module variables

   !! * Private variables



 CONTAINS

   !!======================================================================
  !!======================================================================
  SUBROUTINE userconta_cin_re2(ire,km,im,jm,cin_reac)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE userconta_cin_re2  ***
   !&E
   !&E ** Purpose : calcul de cinetique variable si besoin
   !&E                a programmer pour telle ou telle application
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          2008-06  B. thouvenin
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE cometemp, ONLY : coext
   USE sflxatm, ONLY : rad
   USE contareaddat, ONLY : cin_max_re,cin_re2_tmp,cf_flim3_re

   !! * Arguments
   INTEGER, INTENT( in )             :: ire,km,im,jm     
   REAL(KIND=rsh),INTENT(inout)      :: cin_reac

   !! * Local declarations
   INTEGER                           :: ivr,iv      
   REAL(KIND=rsh)                    :: radiation      
   
! calcul de la dgradation bacterienne en fonction de la luminosit
! d apres les mesures rejets urbains en Mediterranee
! cin(s-1)=cin_max_re*rad**cf_flim3_re
! calcule en surface seulement, sera attenue ensuite par fonction limitante
!  en fontion de l attenuation de la lumiere en profondeur
! exemple : on indique .false. pour cinetique non constante
!     on donne cin_max_re=3.e-6  3.e-8 s-1 
!     donc dans le fichier reaction.dat, on doit le donner en j-1=0.002 a 0.24
!    et on donne une fonction limitante de type 3 avec
!       cf_flim3=0.7  1.2
   if(km==1)then
     radiation=max(0.,rad(im,jm))
     cin_re2_tmp(im,jm)=cin_reac*radiation**cf_flim3_re(ire,1)
    else
     cin_reac=cin_re2_tmp(im,jm)
    endif
   
  END SUBROUTINE userconta_cin_re2
!
   !!======================================================================

  SUBROUTINE userconta_coef_re2(ire,coefstochio)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE userconta_coef_re2  ***
   !&E
   !&E ** Purpose : calcul de coef stochio variable si besoin
   !&E                 programmer pour telle ou telle application
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          2008-06  B. thouvenin
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE contareaddat, ONLY : cin_max_re

   !! * Arguments
   INTEGER, INTENT( in )                          ::   ire      
   REAL(KIND=rsh),DIMENSION(nv_tot),INTENT(inout) :: coefstochio

   !! * Local declarations
   INTEGER                     :: ivr,iv      
   
   

  END SUBROUTINE userconta_coef_re2
!
#endif
 END MODULE userconta
