! ***************************************************************************
!                 PROGRAMME sedimento  
! 
! ***************************************************************************  
!
!****************************************************************************
!
!     parametres et common en include:            fichier incsedim.h
!
! 
!     affectation des cond. init. dans sediment   Sub. i_inised       p.
!
!
!     calcul des echanges eau/sediment 
!           (erosion: definitif, depot: provis.)  Sub. s_erosion      p.
!             
!           > flux diffusion interface eau/sed.   Sub. s_difsed       p.
!
!           > calcul vitesses de chute            Sub. t_vitchut      p.
!
!     calculs dans le sediment
!
!           * depots effectifs (+erosion ?)       Sub. s_depeff       p.
!
!           * calcul du tassement                 Sub. s_tassdif       p.
!
!           * calcul transformations ds sediment  Sub. s_transfsed    p.
!
!     
!
!*************************************************************************      
!***********************************************************************
      subroutine i_inised
!
!     initialisation des variables au debut de la simulation
!***********************************************************************
!
#ifdef key_sedim_anc

use comsedim
use comsubstance, only : typart,nvpc,nvp,nv_state,flx_s2w,flx_w2s

implicit none
!
      integer i,j,k,iv,igsed,idsed,jbsed,jhsed,ksdma
      real(kind=rsh) :: somalp,dzsf,dzsunis,csedunis
!
!
!
   if(.not.l_repsed) then
     if(unised.eq.0) then

! fond de reference uniforme:

      z0sed(:,:)=z0seduni
      hsed(:,:)=0.0_rsh
      where (h0(:,:) /= -valmanq) hsed(:,:)=hseduni
      ksmi(:,:)=0
      ksma(:,:)=0
      WHERE (h0(:,:) /= -valmanq) ksmi(:,:)=ksmiuni
      WHERE (h0(:,:) /= -valmanq) ksma(:,:)=ksmauni
      c_sedtot(:,:,:)=0.0_rsh

      do j=jmin,jmax
        do i=ig(j),id(j)
!
          do k=ksmi(i,j),ksma(i,j)
            IF (h0(i,j) /= -valmanq) c_sedtot(k,i,j)=cseduni
            somalp=0.
            do iv=1,nvpc
!             - variables constitutives: cini_sed en pourcentage de la  masse de sediment
              cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)/100.0_rsh
              somalp=somalp+cv_sed(iv,k,i,j)/(ros(iv)+epsilon)
            enddo
            poro(k,i,j)=1.-somalp
            do iv=nvpc+1,nvp
!             - variables non constitutives: cini_sed en conc/kg de sediment
              cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
            enddo
!cval           do iv=nvp+1,nvar
            do iv=nvp+1,nv_state
!             - variables dissoutes: cini_sed en conc/m3 d eau interstitielle
              cv_sed(iv,k,i,j)=cini_sed(iv)
            enddo
            cv_sed(-1,k,i,j)=tini_sed
            cv_sed( 0,k,i,j)=sini_sed
         enddo
!
        enddo
      enddo
!
!
      if(imails.eq.0)then
!       maillage regulier
!       -----------------
        do j=jmin,jmax
          do i=imin,imax
            if(ksma(i,j).ne.0)then
              dzsf=hsed(i,j)/float(ksma(i,j)-ksmi(i,j)+1)
              do k=ksmi(i,j),ksma(i,j)-1
                dzs(k,i,j)=dzsf
              enddo
              k=ksma(i,j)
              dzs(k,i,j)=dzsf
            endif
          enddo
        enddo
      else 
!       
        write(*,*)'attention dans le sdiment:'
        write(*,*)'maillage irrgulier non programm'
        stop
      endif
!
      else 
!
! initialisation fond definie par utilisateur
!
         call inifond
!
      endif
      endif
#endif
      end      
!
!
!****************************************************************************
!
      subroutine s_depeff(xe,temp,sal,cv_wat)
!
!     calcul du depot effectif
!****************************************************************************
!
#ifdef key_sedim_anc
use comsedim
use comsubstance, only : typart,nv_adv,nvp,nvpc,flx_s2w,flx_w2s
use comvars2d, only : h0fond,t
implicit none
!
      real(kind=rsh),dimension(imin:imax,jmin:jmax) :: xe

      real(kind=rsh),dimension(kmax,imin:imax,jmin:jmax) :: sal,temp
      real(kind=rsh),dimension (nb_var,kmax,imin:imax,jmin:jmax) :: cv_wat

      integer i,j,k,iv,ksmin,ksmax,niter
      real(kind=rsh) :: fludep,qdep,vitdepo,ddzs,ddzs1,ddzs2,dzsa,dzsisu
      real(kind=rsh) :: poroa,somalp,somala,cdepiv,porosi,dzpoi,porodep


!
!     Reactualisation des cv_wat car le depot est implicite et ce sont les
!     nouveau cv_wat qu il faut considerer
!cval      do j=jmin,jmax
!cval         do i=ig(j),id(j)
!cval           do iv=1,nvar-2
!cval             cv_wat(iv,1,i,j)=tz(iv,1,i,j)
!cval           end do   
!cval           cv_wat(nvar-1,1,i,j)=temp(1,i,j)
!cval           cv_wat(nvar,1,i,j)=sal(1,i,j)
!cval         end do
!cval       end do 
!
!
!
    do  j=jmin,jmax
      do  i=ig(j),id(j)
!
      if(h0(i,j)+xe(i,j).gt.h0fond) then
!
        ksmax=ksma(i,j)
!
!       reactualisation du depot effectif (flx_w2s) car implicite

        do iv=1,nvp
!?      do iv=1,nvar nv_state
!         la ligne suivante a ne conserver 
!         que si le depot est implicite dans condif3
          flx_w2s(iv,i,j)=flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)
        enddo
!
        fludep=0.
        do iv=1,nvpc
          fludep=fludep+flx_w2s(iv,i,j)
        enddo
!
!
!
      if(fludep.gt.0.)then
!
!         cas ou il y a depot
!         *******************
          qdep=fludep*dtt
          vitdepo=fludep/csupi
          ddzs=dtt*vitdepo
          somalp=0.
          do iv=1,nvpc
            somalp=somalp+flx_w2s(iv,i,j)/(ros(iv)+epsilon)
          enddo
          porodep=1.-somalp/vitdepo
          epeaui(i,j)=ddzs*porodep
!.a.d.    ddzs=qdep/csupi
!
!
!
          if(ksmax.lt.ksmi(i,j))then
            ddzs2=ddzs
!
!
          else
!           il existe deja une couche
!           -------------------------
            ddzs1=min(dzsmax-dzs(ksmax,i,j),ddzs)
            ddzs1=.5*(ddzs1+abs(ddzs1))
            if(c_sedtot(ksmax,i,j).ge.csmaxmel) ddzs1=0.
            ddzs2=ddzs-ddzs1
            if(ddzs2.le.dzsmin)then
              ddzs1=ddzs
              ddzs2=0.
            endif
!
!           melange avec la couche superieure
!           ---------------------------------
            dzsa=dzs(ksmax,i,j)
            dzs(ksmax,i,j)=dzs(ksmax,i,j)+ddzs1
            dzsisu=1./dzs(ksmax,i,j)
!?          dzsiin(ksmax,i,j)=2.*dzsisu
            c_sedtot(ksmax,i,j)=(c_sedtot(ksmax,i,j)*dzsa+csupi*ddzs1)*dzsisu
            poroa=poro(ksmax,i,j)
            somalp=0.
            somala=0.
            do iv=1,nvp
              cdepiv=flx_w2s(iv,i,j)/vitdepo
              cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+cdepiv*ddzs1)     &
     &                         *dzsisu
              somala=somala+cdepiv*typart(iv)/(ros(iv)+epsilon)
              somalp=somalp+cv_sed(iv,ksmax,i,j)*typart(iv)/(ros(iv)+epsilon)
            enddo
            porosi=1.-somala
            poro(ksmax,i,j)=1.-somalp
            poroin(ksmax,i,j)=poro(ksmax,i,j)
            if(ksmax.gt.ksmi(i,j))then
             dzsiin(ksmax-1,i,j)=2./(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
             poroin(ksmax-1,i,j)=.5*(poro(ksmax-1,i,j)+poro(ksmax,i,j))
            endif
            dzpoi=dzsisu/poro(ksmax,i,j)
!           if (poro(ksmax-1,i,j).lt.0.)then
!           write(*,*)t,i,j,ksmax,somalp,poro(ksmax-1,i,j),poro(ksmax,i,j)
!           endif
            cv_sed(-1,ksmax,i,j)=(cv_sed(-1,ksmax,i,j)*poroa*dzsa             &
     &                       +temp(1,i,j)*porosi*ddzs1)*dzpoi
            cv_sed( 0,ksmax,i,j)=(cv_sed( 0,ksmax,i,j)*poroa*dzsa             &
     &                       +sal(1,i,j)*porosi*ddzs1)*dzpoi
            do iv=nvp+1,nv_adv
              cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*poroa*dzsa             &
     &                       +cv_wat(iv,1,i,j)*porosi*ddzs1)*dzpoi
!             attention, le puits dans l eau a pu etre different,   
!             car flx_w2s calcule dtt avant.
!     $                      +flx_w2s(iv,i,j)/vitdepo*ddzs1)*dzpoi
!             a condition que: flx_w2s(iv,i,j) en m/s =cv_wat*vitdepo*porosi
            enddo
!
          endif
!
!         poursuite du remplissage
!         ------------------------ 
!
          niter=0
777       niter=niter+1
          if(niter.ge.1000)then
            write(*,*)'probleme iteratif dans depeff'
            write(*,778)t,i,j,ksmax,ddzs2,(flx_w2s(iv,i,j),cv_wat(iv,1,i,j),iv=1,nvp)
778         format('a',f12.0,'s en i,j=',i3,1x,i3,' ksmax=',i3,' ddzs2=',g10.4,'flu+cv',8g9.3)
            stop
          endif
!
          if(ddzs2.gt.0.)then
            if(ksmax.eq.ksnmax)then
!
!             cas o on a atteint le nbre max de couches:
!             fusion des 2 premieres couches (fond) et decalage des autres
!             ------------------------------------------------------------
              ksmin=ksmi(i,j)
              dzsa=dzs(ksmin,i,j)
              poroa=poro(ksmin,i,j)
              dzs(ksmin,i,j)=dzs(ksmin,i,j)+dzs(ksmin+1,i,j)
              c_sedtot(ksmin,i,j)=0.
              somalp=0.
              do iv=1,nvp
                cv_sed(iv,ksmin,i,j)=(cv_sed(iv,ksmin,i,j)*dzsa                 &
     &          +cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j))/dzs(ksmin,i,j)
                c_sedtot(ksmin,i,j)=c_sedtot(ksmin,i,j)+cv_sed(iv,ksmin,i,j)        &
     &                          *typart(iv)
                somalp=somalp+cv_sed(iv,ksmin,i,j)*typart(iv)/(ros(iv)+epsilon)
              enddo
              poro(ksmin,i,j)=1.-somalp
              cv_sed(-1,ksmin,i,j)=(cv_sed(-1,ksmin,i,j)*dzsa*poroa           &
     &           +cv_sed(-1,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
     &                         /(dzs(ksmin,i,j)*poro(ksmin,i,j))
              cv_sed( 0,ksmin,i,j)=(cv_sed( 0,ksmin,i,j)*dzsa*poroa           &
     &           +cv_sed( 0,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
     &                         /(dzs(ksmin,i,j)*poro(ksmin,i,j))
!cval         do iv=nvp+1,nvar
              do iv=nvp+1,nv_adv
                cv_sed(iv,ksmin,i,j)=(cv_sed(iv,ksmin,i,j)*dzsa*poroa           &
     &         +cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
     &                         /(dzs(ksmin,i,j)*poro(ksmin,i,j))
              enddo
              poroin(ksmin,i,j)=.5*(poro(ksmin,i,j)+poro(ksmin+2,i,j))
              dzsiin(ksmin,i,j)=2./(dzs(ksmin,i,j)+dzs(ksmin+2,i,j))
              do k=ksmin+1,ksmax-1
                dzs(k,i,j)=dzs(k+1,i,j)
                dzsiin(k,i,j)=dzsiin(k+1,i,j)
                c_sedtot(k,i,j)=c_sedtot(k+1,i,j)
                poro(k,i,j)=poro(k+1,i,j)
                poroin(k,i,j)=poroin(k+1,i,j)
!                do iv=1,nvar
                do iv=-1,nv_adv
                  cv_sed(iv,k,i,j)=cv_sed(iv,k+1,i,j)
                enddo
              enddo
              ksmax=ksmax-1
!
!
!
            endif
!
!           creation d une nouvelle couche
!           ------------------------------
            ksmax=ksmax+1
!
!
            dzs(ksmax,i,j)=min(dzsmax,ddzs2)
            ddzs2=ddzs2-dzs(ksmax,i,j)
            if(ddzs2.le.dzsmin)then
              dzs(ksmax,i,j)=ddzs2+dzs(ksmax,i,j)
              ddzs2=0.
!
            endif
            dzsisu=1./dzs(ksmax,i,j)
!?          dzsiin(ksmax,i,j)=2.*dzsisu
            c_sedtot(ksmax,i,j)=csupi
            do iv=1,nvp
              cv_sed(iv,ksmax,i,j)=flx_w2s(iv,i,j)/vitdepo
            enddo
            poro(ksmax,i,j)=porodep
            poroin(ksmax,i,j)=poro(ksmax,i,j)
          if (ksmax.gt.ksmi(i,j))then
            poroin(ksmax-1,i,j)=.5*(poro(ksmax-1,i,j)+poro(ksmax,i,j))
            dzsiin(ksmax-1,i,j)=2./(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
!
          endif
            cv_sed(-1,ksmax,i,j)=temp(1,i,j)
            cv_sed( 0,ksmax,i,j)=sal(1,i,j)
            do iv=nvp+1,nv_adv
              cv_sed(iv,ksmax,i,j)=cv_wat(iv,1,i,j)
! eq?         cv_sed(iv,ksmax,i,j)=flx_w2s(iv,i,j)/
! eq?     $                    (poro(ksmax,i,j)*vitdepo)
            enddo
!
            go to 777
          endif
!
!         calcul du flx_w2s des var. diss. pour le pas de temps suivant:
!cval          do iv=nvp+1,nvar
          flx_w2s(-1,i,j)=porodep*vitdepo
          flx_w2s( 0,i,j)=porodep*vitdepo
          do iv=nvp+1,nv_adv
!           flx_w2s(iv,i,j)=fludep*porosi/csupi
!           flx_w2s est une vitesse de chute virtuelle
!           commune a toutes les variables dissoutes : 
            flx_w2s(iv,i,j)=porodep*vitdepo
          enddo
!
        else
!
!         cas ou pas de depot
!         *******************  
!         calcul du flx_w2s des var. diss. pour le pas de temps suivant:
          flx_w2s(-1,i,j)=0.0
          flx_w2s( 0,i,j)=0.0
!          do iv=nvp+1,nvar
          do iv=nvp+1,nv_adv
            flx_w2s(iv,i,j)=0.
          enddo
          epeaui(i,j)=0.
!
!         prise en compte des depots eventuels 
!         de variables particulaires non constitutives:
!c          if(ksmax.ge.ksmi(i,j).and.dzs(ksmax,i,j).ge.dzsmin)then
          if(ksmax.ge.1)then
            do iv=nvpc+1,nvp
              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)+flx_w2s(iv,i,j)*dtt      &
     &                         /dzs(ksmax,i,j)
            enddo
          else
!cval pas dangereux ca ? normal voir P. Le Hir
            do iv=nvpc+1,nvp
               cv_wat(iv,1,i,j)=cv_wat(iv,1,i,j)                              &
     &            +flx_w2s(iv,i,j)*dtt/(dzu(1)*(xe(i,j)+h0(i,j)))
!cval               tz(iv,1,i,j)=cv_wat(iv,1,i,j)
            enddo
          endif
!         
!
!
        endif   ! test on fludep
!
!       reactualisation de ksma ( ksmax a pu etre modifie dans la routine)
        ksma(i,j)=ksmax
!
      endif   ! test on h0+xe
!
      end do  ! loop on i
    end do    ! loop on j
!cval 10   continue
!
!
!
      return
#endif
      end
!
! **********************************************************************
!
#ifdef key_sedim_anc
      function s_difint(porosi)
!
! **********************************************************************
!
!     calcule le coefficient de diffusion a l interface eau-sediment
!
use comsedim
      real(kind=rsh) :: s_difint,porosi
!
      s_difint=xdifsi1*porosi**xdifsi2
!
      return
      end
!
!
! **********************************************************************
!
      function s_difsed(porosi)
!
! **********************************************************************
!
!     calcule le coefficient de diffusion dans le sediment
!
use comsedim
      real(kind=rsh) :: s_difsed,porosi
!
      s_difsed=xdifs1*porosi**xdifs2
!
      end

#endif
!
! **********************************************************************
!
      subroutine s_erosion(xe)
!
!     calcul du flux d erosion 
!     et du flux de depot (formule de Krone)
! **********************************************************************
!
#ifdef key_sedim_anc
use comsedim
use comsubstance, only : typart,nv_adv,nvp,flx_s2w,flx_w2s
use comfluxdo, only : phieau
use comvars2d, only : h0fond
implicit none
!

      real(kind=rsh),dimension(imin:imax,jmin:jmax) :: xe
      integer i,j,iv,ksmax,ksmaxa,k
      real(kind=rsh) :: dttinv,dt1,toce,erodab,ero,erosi,dzsa
      real(kind=rsh) :: ddzs,dzsam1,poroa,poroam1,dzsisu,somalp,dzpoi
      real(kind=rsh) :: s_tencrit
!
!
      dttinv=1./dtt
!
  do j=jmin+1,jmax
    do i=ig(j)+1,id(j)
!
!
      if(h0(i,j)+xe(i,j).gt.h0fond) then
!
      ksmax=ksma(i,j)
!
!
!cval      do iv=1,nvar
!cval quid pour flx_s2w de T et S ???
      flx_s2w(-1,i,j)=0.0
      flx_s2w( 0,i,j)=0.0
      do iv=1,nv_adv
        flx_s2w(iv,i,j)=0.
      enddo
!
!?      if(ksmax.ge.1.and.c_sedtot(ksmax,i,j).le.10.)then
!?c       erosion instantanee
!?        do iv=1,nvp
!?          flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+dzs(ksmax,i,j)
!?     $                     *cv_sed(iv,ksmax,i,j)
!?        enddo
!?        do iv=nvp+1,nvar
!?          flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+
!?     $              dzs(ksmax,i,j)*poro(ksmax,i,j)*cv_sed(iv,ksmax,i,j)
!?        enddo
!?        ksmax=ksmax-1
!?      endif
!
      if(ksmax.ge.ksmi(i,j))then
!
        dt1=dtt
!       epeaui represente l epaisseur d eau interstitielle recuperee 
!       par le sediment lors du depot (calcule par depeff precedent) 
!       ici, epeaui va representer la variation d epaisseur d eau a
!       prendre en compte dans la maille inferieure de la colonne d eau
        epeaui(i,j)=-epeaui(i,j)
 2      continue
!
        toce=s_tencrit(c_sedtot(ksmax,i,j))
        if(tenfon(i,j).gt.toce)then
!
          erodab=c_sedtot(ksmax,i,j)*dzs(ksmax,i,j)
!         relation de Partheniades:
          ero=xeros*(tenfon(i,j)/toce-1.)
          erosi=ero*dt1
!
          if(erosi.lt.erodab)then
            ddzs=erosi/c_sedtot(ksmax,i,j)
            dzs(ksmax,i,j)=dzs(ksmax,i,j)-ddzs
            epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
            do iv=1,nvp
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*cv_sed(iv,ksmax,i,j)
            enddo
!cval            do iv=nvp+1,nvar
            flx_s2w(-1,i,j)=flx_s2w(-1,i,j)+                              &
     &                     ddzs*poro(ksmax,i,j)*cv_sed(-1,ksmax,i,j)
            flx_s2w( 0,i,j)=flx_s2w( 0,i,j)+                              &
     &                     ddzs*poro(ksmax,i,j)*cv_sed( 0,ksmax,i,j)
            do iv=nvp+1,nv_adv
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+                            &
     &                     ddzs*poro(ksmax,i,j)*cv_sed(iv,ksmax,i,j)
            enddo
          else
            ddzs=erodab/c_sedtot(ksmax,i,j)
            epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
            do iv=1,nvp
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*cv_sed(iv,ksmax,i,j)
            enddo
!cval            do iv=nvp+1,nvar
            flx_s2w(-1,i,j)=flx_s2w(-1,i,j)+                              &
     &                     ddzs*poro(ksmax,i,j)*cv_sed(-1,ksmax,i,j)
            flx_s2w( 0,i,j)=flx_s2w( 0,i,j)+                              &
     &                     ddzs*poro(ksmax,i,j)*cv_sed( 0,ksmax,i,j)
            do iv=nvp+1,nv_adv
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+                            &
     &                     ddzs*poro(ksmax,i,j)*cv_sed(iv,ksmax,i,j)
            enddo
            dt1=dt1-erodab/ero
            ksmax=ksmax-1
            if(ksmax.ge.ksmi(i,j))then
              go to 2
            else
!c            write(*,*)'a t=',t,i,j,', il n"y a plus de sediment'
              go to 120
            endif
          endif
        endif
!
      endif
!
!
!
!
!     fusion des 2 couches superf., si dzs(ksmax) < dzsmin
!     ---------------------------------------------------
!      if(ksmax.ne.0)then
!bene d apres Benedicte, correction a revoir (7jv99) pour eviter k=0
!c      if(dzs(ksmax,i,j).le.dzsmin.and.ksmax.gt.ksmi(i,j))then
1000  if(ksmax.ge.2)then
      if(dzs(ksmax,i,j).le.dzsmin)then
        ksmaxa=ksmax
        dzsa=dzs(ksmax,i,j)
        dzsam1=dzs(ksmax-1,i,j)
        poroa=poro(ksmax,i,j)
        poroam1=poro(ksmax-1,i,j)
        ksmax=ksmax-1
        dzs(ksmax,i,j)=dzsa+dzs(ksmax,i,j)
        dzsisu=1./dzs(ksmax,i,j)
        c_sedtot(ksmax,i,j)=0.
        somalp=0.
!
        do iv=1,nvp
          cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*dzsa+                     &
     &                     cv_sed(iv,ksmax,i,j)*dzsam1)*dzsisu
          c_sedtot(ksmax,i,j)=c_sedtot(ksmax,i,j)                               &
     &                        +cv_sed(iv,ksmax,i,j)*typart(iv)
            somalp=somalp+cv_sed(iv,ksmax,i,j)*typart(iv)/(ros(iv)+epsilon)
        enddo
        poro(ksmax,i,j)=1.-somalp
        poroin(ksmax,i,j)=poro(ksmax,i,j)
        dzpoi=dzsisu/poro(ksmax,i,j)            
!cval        do iv=nvp+1,nvar
        cv_sed(-1,ksmax,i,j)=(cv_sed(-1,ksmaxa,i,j)*poroa*dzsa+               &
     &                     cv_sed(-1,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        cv_sed( 0,ksmax,i,j)=(cv_sed( 0,ksmaxa,i,j)*poroa*dzsa+               &
     &                     cv_sed( 0,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        do iv=nvp+1,nv_adv
!         les cs dissous sont en .../m3 d eau interst.
          cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*poroa*dzsa+               &
     &                     cv_sed(iv,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        enddo
!       et les variables benthiques ???
!?      dzsiin(ksmax,i,j)=2./dzs(ksmax,i,j)
        if(ksmax.gt.ksmi(i,j))then
          poroin(ksmax-1,i,j)=0.5*(poro(ksmax,i,j)+poro(ksmax-1,i,j))
          dzsiin(ksmax-1,i,j)=2./(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
        endif
!
        go to 1000
       endif
      endif
!
 120  continue
!
!
!
!cval      do iv=1,nvar
      do iv=-1,nv_adv
!       pour retrouver un flux d erosion en .../m2/s :
        flx_s2w(iv,i,j)=flx_s2w(iv,i,j)*dttinv
      enddo
!cval      do iv=nvp+1,nvar
      flx_s2w(-1,i,j)=flx_s2w(-1,i,j)-fludif(-1,i,j)+flutas(-1,i,j)
      flx_s2w( 0,i,j)=flx_s2w( 0,i,j)-fludif( 0,i,j)+flutas( 0,i,j)
      do iv=nvp+1,nv_adv
!       pour les dissoutes, on ajoute le flux diffusif a l interface
!       et celui qui resulte de l expulsion d eau interst. par tasst:
        flx_s2w(iv,i,j)=flx_s2w(iv,i,j)-fludif(iv,i,j)+flutas(iv,i,j)
      enddo
! 
!     
!     calcul du depot (en m/s, car implicite ds eq. de transport)
!     -----------------------------------------------------------
!     mis dans t_condif3 le 17 mai 2001, car Ws3 calcule aprs advection ...
!
!
!     reactualisation de ksma 
      ksma(i,j)=ksmax
!
!     rectification de l epaisseur de la maille d eau de fond
!     pour tenir compte de l apport (resp. fuite) d eau interstitielle par
!     erosion ou tassement (resp. depot)
!
!      phieau(1,i,j)=(epeaui(i,j)/dtt+winsurf(i,j))*surf(i,j) 
!
! attention dans la version med phieau est en m/s
!
!cval      phieau(1,i,j)=epeaui(i,j)/dtt+winsurf(i,j)
      phieau(1,i,j)=phieau(1,i,j)+(epeaui(i,j)/dtt+winsurf(i,j))*surf(i,j)
!
!
      endif
!
!
    end do
  end do
!
      return
#endif
      end
!
! **********************************************************************
!
      subroutine s_tassdif(xe)
!
!     calcule le tassement dans le sediment et
!     assure la gestion des couches correspondante
! **********************************************************************
!
#ifdef key_sedim_anc
use comsedim
use comsubstance, only : nvp,nvpc,nv_adv
use comvars2d, only : h0fond,t
use comvarp2d, only : temp,sal,cv_wat
implicit none
!
      real(kind=rsh),dimension(imin:imax,jmin:jmax) :: xe


      integer i,j,ksmin,ksmax,iv,k,it
      real(kind=rsh),dimension(ksdmin-1:ksdmax) :: winter
      real(kind=rsh),dimension(ksdmin  :ksdmax) :: ws,disvi,aa,bb,cc,dd, &
                                                   poroa
      real(kind=rsh) :: dzsa,dtrest,dzsan,dt1,volpwa,s_vitsed,s_difint,dt1s
      real(kind=rsh) :: dtsdzs,dzsam1,dvol,dzsisu,somalp
      real(kind=rsh) :: dzssdttas,hcrit
      real(kind=rlg) :: dttaseff,dttasinv,dtasmin

!
      dtasmin=10.
!
      dttaseff=dttas+(t-ttasse)
      ttasse=t
      dttasinv=1.0/dttaseff
!
   do j=jmin+1,jmax
    do  i=ig(j)+1,id(j)
!
     if(h0(i,j)+xe(i,j).gt.h0fond) then
!
      if (ksma(i,j).ge.ksmi(i,j))then
!
        ksmin=ksmi(i,j)
        ksmax=ksma(i,j)
!
!       memorisation des porosites, du volume d eau interstitielle
!       et de l epaisseur de la couche de surface
!       ----------------------------------------------------------
        do k=ksmin,ksmax
          poroa(k)=poro(k,i,j)
        end do
        dzsa=dzs(ksmax,i,j)
        volpwa=poro(ksmax,i,j)*dzsa
!cval        do 102 iv=nvp+1,nvar
        varpwa(-1)=volpwa*cv_sed(-1,ksmax,i,j)
        varpwa( 0)=volpwa*cv_sed( 0,ksmax,i,j)
        do iv=nvp+1,nv_adv
          varpwa(iv)=volpwa*cv_sed(iv,ksmax,i,j)
        end do
!
!
        if(ioptas.eq.0)then
!
!         1- calcul du tassement
!         **********************
!
!         actuellement, la vitesse de sedimentation est supposee 
!         identique pour toutes les particules
!
          dtrest=dttaseff
          it=0
  1       continue
          it=it+1
          dzsan=dzs(ksmax,i,j)
!         modif du 1-2-99:
          dt1=dtrest
          if(ksmax.ge.2)then
!
!           cas o il y a plus d une couche :
            do k=ksmin,ksmax-1
              ws(k)=s_vitsed(poroin(k,i,j))
              dt1=min(xcvgce*dzs(k,i,j)/(ws(k)+1.e-16),dt1)
            enddo
            k=ksmax
            ws(k)=s_vitsed(poroin(k,i,j))
            dt1s=min(xcvgce*dzs(k,i,j),dzsan-dzsmin)/(ws(k)+1.e-16)
            dt1=min(dt1,max(0.,dt1s))
!c         if(dt1.le.0.)write(*,*)'dt1',dt1,i,j,k,ws(ksmax),dzsan,it
            dzs(ksmax,i,j)=dzsan-dt1*ws(ksmax)
!
            k=ksmin
            dtsdzs=dt1/dzs(k,i,j)
            do iv=1,nvp
!c            cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(ws(k)*fexcs
!c     $                  *cv_sed(iv,k+1,i,j)+ws(k)*cexcs*cv_sed(iv,k,i,j))
!c            excentrement total impose pour gagner du temps:
              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*ws(k)*cv_sed(iv,k+1,i,j)
            enddo
            do k=ksmin+1,ksmax-1
              dtsdzs=dt1/dzs(k,i,j)
              do iv=1,nvp
!c              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(ws(k)*fexcs     
!c     $               *cv_sed(iv,k+1,i,j)+(ws(k)*cexcs-ws(k-1)*fexcs)
!c     $               *cv_sed(iv,k,i,j))-ws(k-1)*cexcs*cv_sed(iv,k-1,i,j))
                cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(-ws(k-1)*             &
     &                      cv_sed(iv,k,i,j)+ws(k)*cv_sed(iv,k+1,i,j))
              enddo
            enddo
            k=ksmax
            do iv=1,nvp
!c            cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsan-dt1*ws(k-1)*
!c     $                   (fexcs*cv_sed(iv,k,i,j)+cexcs*cv_sed(iv,k-1,i,j)))
!c     $                   /dzs(k,i,j)
              cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsan-dt1*ws(k-1)*             &
     &                     cv_sed(iv,k,i,j))/dzs(k,i,j)
            enddo
!
            if(dzs(ksmax,i,j).le.dzsmin)then
!             fusion des 2 couches de surface           
              dzsan=dzs(ksmax,i,j)
              dzsam1=dzs(ksmax-1,i,j)
!             reactualisation de dzsa et volpwa:
              dzsa=dzsa+dzsam1
              dvol=poroa(ksmax-1)*dzsam1
              volpwa=volpwa+dvol
!cval              do iv=nvp+1,nvar
              varpwa(-1)=varpwa(-1)+dvol*cv_sed(-1,ksmax-1,i,j)
              varpwa( 0)=varpwa( 0)+dvol*cv_sed(-1,ksmax-1,i,j)
              do iv=nvp+1,nv_adv
                varpwa(iv)=varpwa(iv)+dvol*cv_sed(iv,ksmax-1,i,j)
              enddo
              ksmax=ksmax-1
              dzs(ksmax,i,j)=dzs(ksmax,i,j)+dzsan
              dzsisu=1./dzs(ksmax,i,j)
              do iv=1,nvp
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax+1,i,j)*dzsan              &
     &                          +cv_sed(iv,ksmax,i,j)*dzsam1)*dzsisu
              enddo
            endif
!
          else
!           cas o il n y a qu une couche :
!           write(*,*)'attention: le sediment n"a qu"une couche !'
            k=ksmin
            if(dzs(k,i,j).gt.dzsmin)then
              ws(k)=s_vitsed(poro(k,i,j))
              dt1=min(xcvgce*dzs(k,i,j)/(ws(k)+1.e-16),dt1)
              dzs(k,i,j)=dzsan-dt1*ws(k)
              if(dzs(k,i,j).le.dzsmin)then
                dzs(k,i,j)=dzsmin
                dt1=dtrest
              endif            
              dzsisu=1./dzs(k,i,j)
              do iv=1,nvp
                cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*dzsan*dzsisu
              enddo
            endif
          endif
!
!         calcul de la densite (kg/m3) et de la porosite du sediment
!         ----------------------------------------------------------
          do k=ksmin,ksmax
            c_sedtot(k,i,j)=0.
            somalp=0.
            do iv=1,nvpc
              c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
              somalp=somalp+cv_sed(iv,k,i,j)/(ros(iv)+epsilon)
            enddo
!test          if(somalp.gt.1..or.c_sedtot(k,i,j).le.0.)then
!test            write(*,*)'somalp>1 ',t,i,j,k,ksmax,c_sedtot(k,i,j),
!test     &               somalp,dzsa,dzs(ksmax,i,j),dzs(k,i,j),it,wsurf
!test          endif
            poro(k,i,j)=1.-somalp
          enddo
          poroin(ksmax,i,j)=poro(ksmax,i,j)
          if(ksmax.ge.2)then
            do k=ksmin,ksmax-1
                poroin(k,i,j)=.5*(poro(k,i,j)+poro(k+1,i,j))
                dzsiin(k,i,j)=2./(dzs(k,i,j)+dzs(k+1,i,j))
            enddo
          endif
!
          dtrest=dtrest-dt1
          if(dtrest.gt.dtasmin)then
            if(it.le.200)then
              go to 1
            else
         write(*,555)it,i,j,dt1,ksmax,dzs(ksmax,i,j),poro(ksmax,i,j)
 555            format('tassdif iter',i3,' en i,j=',2i3                 &
     &         ,' dt1=',g8.2,' ksmax',i3,' dzs',g8.2,'po',f4.2)
            endif
          endif
!
!
!
!
!         2- calcul des vitesses du fluide interstitiel
!         *********************************************
!         considerees >0 vers le haut, centree a l interface superieure
!
          winter(0)=0.
          do k=ksmin,ksmax-1
            winter(k)=winter(k-1)+(poroa(k)-poro(k,i,j))                &
     &                *dzs(k,i,j)*dttasinv
          enddo
          k=ksmax
          winter(k)=winter(k-1)+(volpwa-poro(k,i,j)*dzs(k,i,j))         &
     &              *dttasinv
          winsurf(i,j)=winter(k)
!
        else                !if .not.ioptas
!         cas ou pas de tassement
!         ***********************        
          do k=ksmin,ksmax
            winter(k)=0.
          enddo
          winsurf(i,j)=0.
        endif               ! fin boucle sur ioptas
!
!       3- calcul des concentrations des var. dissoutes ds sediment
!       ***********************************************************
!
!cval              do iv=nvp+1,nvar
        do iv=nvp+1,nv_adv
!         la boucle sur les variables positionne ici permet de faire
!         varier les coef. de diffusion en fonction des variables
!
          k=ksmax
!c          disvi(k)=s_difint(poro(k,i,j))
!c          disvi(k)=xdifsi1*poro(k,i,j)**xdifsi2
            disvi(k)=xdifsi1                                            &
     &                /(dzs(k,i,j)*.5+epdifi)
!orrection du 18 avril 2002 :
            hcrit=max(0.,xe(i,j)+h0(i,j)-h0fond)
            disvi(k)=disvi(k)*hcrit/(hcrit+epsilon)
!gain            disve(k)=disvi(k)*cimpls
!gain            disvi(k)=disvi(k)*fimpls
!
!c        accroissement de la diffusion a l interface si charriage:
!c        if(iopcoh.ne.1)then
!c          efchar=xdifs3*teta**.5*max(0.,teta-tetac)
!c          disve(k)=disve(k)*efchar
!c          disvi(k)=disvi(k)*efchar
!c        endif
!
          if(ksmax.gt.ksmin)then
!
!           cas ou il y a plusieurs couches:
!           --------------------------------
!
!           calcul des vitesses de diffusion
!
            do k=ksmin,ksmax-1
!gain         disvi(k)=s_difsed(poroin(k,i,j))*dzsiin(k,i,j)
!gain         disvi(k)=xdifs1*poroin(k,i,j)**xdifs2*dzsiin(k,i,j)
              disvi(k)=xdifs1*dzsiin(k,i,j)
!gain         disve(k)=disvi(k)*cimpls
!gain         disvi(k)=disvi(k)*fimpls
            enddo
!
!           calcul d advection-diffusion 
!
!           coefficients de la matrice
!
!           calcul implicite pour l advection, partiellement implicite
!           (fimpls) pour la diffusion; 
!           flux advectifs partielt decentres amont (fexcs), totalt en surf
            k=ksmin
            dzssdttas=dzs(k,i,j)*dttasinv
            aa(k)=0.
            bb(k)=dzssdttas*poro(k,i,j)+disvi(k)+winter(k)*fexcs
            cc(k)=-disvi(k)+winter(k)*cexcs
!gain            dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)+disve(k)*
!gain     $            (cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
            dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)
            do k=ksmin+1,ksmax-1
              dzssdttas=dzs(k,i,j)*dttasinv
              aa(k)=-disvi(k-1)-winter(k-1)*fexcs
              bb(k)=dzssdttas*poro(k,i,j)-winter(k-1)*cexcs             &
     &              +winter(k)*fexcs+disvi(k-1)+disvi(k)
              cc(k)=-disvi(k)+winter(k)*cexcs
              dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)
!gain     $              +disve(k)*(cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
!gain     $              -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
            enddo
            k=ksmax
            aa(k)=-disvi(k-1)-winter(k-1)*fexcs
            bb(k)=dzs(k,i,j)*poro(k,i,j)*dttasinv                       &
     &           +disvi(k-1)+disvi(k)-winter(k-1)*cexcs+winter(k)
            cc(k)=0.
!gain            fludif(iv,i,j)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            fludif(iv,i,j)=disvi(k)*cv_wat(iv,1,i,j)
            dd(k)=varpwa(iv)*dttasinv+fludif(iv,i,j)
!gain     $            -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
!
!           resolution du systeme tridiagonal
!
            dd(1)=dd(1)/bb(1)
            do k=ksmin+1,ksmax
              bb(k)=bb(k)-aa(k)*cc(k-1)/bb(k-1)
              dd(k)=(dd(k)-aa(k)*dd(k-1))/bb(k)
            enddo
            cv_sed(iv,ksmax,i,j)=dd(ksmax)
            do k=ksmax-1,ksmin,-1
              cv_sed(iv,k,i,j)=dd(k)-cc(k)*cv_sed(iv,k+1,i,j)/bb(k)
            enddo
!
          else
!
!           cas ou il n y a qu une couche (ksmax=ksmin):
!           --------------------------------------------       
            k=ksmax        
!gain       fludif(iv,i,j)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            fludif(iv,i,j)=disvi(k)*cv_wat(iv,1,i,j)
            cv_sed(iv,k,i,j)=(varpwa(iv)+fludif(iv,i,j)*dttaseff)           &
     &      /(dzs(k,i,j)*poro(k,i,j)+(winter(k)+disvi(k))*dttaseff+epsilon)
          endif
!
!       flux d echange a l interface eau/sediment:
!       ------------------------------------------
!       fludif est le flux par diffusion a l interface, >0 vers le bas
!       flutas est le flux par tassement a l interface, >0 vers le haut
          fludif(iv,i,j)=fludif(iv,i,j)-disvi(ksmax)*cv_sed(iv,ksmax,i,j)
          flutas(iv,i,j)=winter(ksmax)*cv_sed(iv,ksmax,i,j)

        enddo                         ! fin boucke sur iv
!
!       3bis- calcul des concentrations de la temperature (-1) et salinite (0) 
!          ds sediment
!       ***********************************************************
!
        do iv=-1,0
!         la boucle sur les variables positionne ici permet de faire
!         varier les coef. de diffusion en fonction des variables
!
          k=ksmax
!c          disvi(k)=s_difint(poro(k,i,j))
!c          disvi(k)=xdifsi1*poro(k,i,j)**xdifsi2
            disvi(k)=xdifsi1                                            &
     &                /(dzs(k,i,j)*.5+epdifi)
!orrection du 18 avril 2002 :
            hcrit=max(0.,xe(i,j)+h0(i,j)-h0fond)
            disvi(k)=disvi(k)*hcrit/(hcrit+epsilon)
!gain            disve(k)=disvi(k)*cimpls
!gain            disvi(k)=disvi(k)*fimpls
!
!c        accroissement de la diffusion a l interface si charriage:
!c        if(iopcoh.ne.1)then
!c          efchar=xdifs3*teta**.5*max(0.,teta-tetac)
!c          disve(k)=disve(k)*efchar
!c          disvi(k)=disvi(k)*efchar
!c        endif
!
          if(ksmax.gt.ksmin)then
!
!           cas ou il y a plusieurs couches:
!           --------------------------------
!
!           calcul des vitesses de diffusion
!
            do k=ksmin,ksmax-1
!gain         disvi(k)=s_difsed(poroin(k,i,j))*dzsiin(k,i,j)
!gain         disvi(k)=xdifs1*poroin(k,i,j)**xdifs2*dzsiin(k,i,j)
              disvi(k)=xdifs1*dzsiin(k,i,j)
!gain         disve(k)=disvi(k)*cimpls
!gain         disvi(k)=disvi(k)*fimpls
            enddo
!
!           calcul d advection-diffusion 
!
!           coefficients de la matrice
!
!           calcul implicite pour l advection, partiellement implicite
!           (fimpls) pour la diffusion; 
!           flux advectifs partielt decentres amont (fexcs), totalt en surf
            k=ksmin
            dzssdttas=dzs(k,i,j)*dttasinv
            aa(k)=0.
            bb(k)=dzssdttas*poro(k,i,j)+disvi(k)+winter(k)*fexcs
            cc(k)=-disvi(k)+winter(k)*cexcs
!gain            dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)+disve(k)*
!gain     $            (cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
            dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)
            do k=ksmin+1,ksmax-1
              dzssdttas=dzs(k,i,j)*dttasinv
              aa(k)=-disvi(k-1)-winter(k-1)*fexcs
              bb(k)=dzssdttas*poro(k,i,j)-winter(k-1)*cexcs             &
     &              +winter(k)*fexcs+disvi(k-1)+disvi(k)
              cc(k)=-disvi(k)+winter(k)*cexcs
              dd(k)=dzssdttas*poroa(k)*cv_sed(iv,k,i,j)
!gain     $              +disve(k)*(cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
!gain     $              -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
            enddo
            k=ksmax
            aa(k)=-disvi(k-1)-winter(k-1)*fexcs
            bb(k)=dzs(k,i,j)*poro(k,i,j)*dttasinv                       &
     &           +disvi(k-1)+disvi(k)-winter(k-1)*cexcs+winter(k)
            cc(k)=0.
!gain            fludif(iv,i,j)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            if (iv==-1) fludif(iv,i,j)=disvi(k)*temp(1,i,j)
            if (iv== 0) fludif(iv,i,j)=disvi(k)*sal(1,i,j) 
!cval         fludif(iv,i,j)=disvi(k)*cv_wat(iv,1,i,j)
            dd(k)=varpwa(iv)*dttasinv+fludif(iv,i,j)
!gain     $            -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
!
!           resolution du systeme tridiagonal
!
            dd(1)=dd(1)/bb(1)
            do k=ksmin+1,ksmax
              bb(k)=bb(k)-aa(k)*cc(k-1)/bb(k-1)
              dd(k)=(dd(k)-aa(k)*dd(k-1))/bb(k)
            enddo
            cv_sed(iv,ksmax,i,j)=dd(ksmax)
            do k=ksmax-1,ksmin,-1
              cv_sed(iv,k,i,j)=dd(k)-cc(k)*cv_sed(iv,k+1,i,j)/bb(k)
            enddo
!
          else
!
!           cas ou il n y a qu une couche (ksmax=ksmin):
!           --------------------------------------------       
            k=ksmax        
!gain       fludif(iv,i,j)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            if (iv==-1) fludif(iv,i,j)=disvi(k)*temp(1,i,j)
            if (iv== 0) fludif(iv,i,j)=disvi(k)*sal(1,i,j)
!cval            fludif(iv,i,j)=disvi(k)*cv_wat(iv,1,i,j)
            cv_sed(iv,k,i,j)=(varpwa(iv)+fludif(iv,i,j)*dttaseff)           &
     &      /(dzs(k,i,j)*poro(k,i,j)+(winter(k)+disvi(k))*dttaseff+epsilon)
          endif
!
!       flux d echange a l interface eau/sediment:
!       ------------------------------------------
!       fludif est le flux par diffusion a l interface, >0 vers le bas
!       flutas est le flux par tassement a l interface, >0 vers le haut
          fludif(iv,i,j)=fludif(iv,i,j)-disvi(ksmax)*cv_sed(iv,ksmax,i,j)
          flutas(iv,i,j)=winter(ksmax)*cv_sed(iv,ksmax,i,j)

        enddo                         ! fin boucke sur temp et salinite
!
!       Reactualisation de ksma (ksmax a pu etre modifie)
!
        ksma(i,j)=ksmax
!
      endif             ! fin boucle sur ksma
!
     endif             ! fin boucle sur h0+xe
!
    end do
   end do
!
      ttasse=ttasse+dttas
#endif
      end
!
#ifdef key_sedim_anc
! **********************************************************************
!
      function s_tencrit(csedim)
!
!     calcul de la tension de cisaillement critique du sediment
! ******************************************************************
use comsedim
      real(kind=rsh) :: s_tencrit,csedim
!
! executable statements
!-------------------------
      s_tencrit=x1toce*max(csedim,csupi)**x2toce
      if(csedim.ge.1200.) s_tencrit=20.
!
      end
!
!
! ******************************************************************
!
      function s_vitsed(porosi)
!
!     calcule les vitesses de tassement dans le sediment
! **********************************************************************
!
!

use comsedim
       real(kind=rsh) :: s_vitsed,porosi
!
!
! executable statements
!-------------------------
!
      if(porosi.ge.poro1)then
        s_vitsed=xtas11*min(porosi,.98)**xtas12
      elseif(porosi.ge.poro2)then
        s_vitsed=xtas21*porosi**xtas22
      elseif(porosi.ge.poro3)then
        s_vitsed=xtas31*porosi**xtas32
      else
        s_vitsed=0.
      endif
!
      return

      end
#endif
!
!
!
!************************************************************************
!
      subroutine t_vitchut_init
!
!************************************************************************
#ifdef key_sedim_anc
use comsedim
use comsubstance, only : xws,nvp
implicit none
!
      integer iv
      real(kind=rsh) :: cfin,wsfin,voleau6,w6
!
!     formulation parametrique de la vitesse de chute, avec entravement,
!     issue des communications INTERCOH 98 et 2000
!
!     on suppose une variation de Ws en fonction de Cmes jusqu  la
!     concentration cfin = 500 g/l, pour laquelle la vitesse de chute, trs
!     entrave, est gale  wsfin = 10-7 m/s
      cfin=500.
      wsfin=0.0000001
!     dans SAM-1dv, la formule d entravement est applique jusqu  xws(6,iv)
!                   correspondant  une concentration volumique voleau6:
!                   voleau6=1.-xws(4,iv)*xws(6,iv)**xws(5,iv)
!                   au-del, une loi puissance de tassement est applique,
!                   passant par wsfin pour cmes=cfin
!      
!     xws(1,iv) est la valeur min de la vitesse de chute
!     xws(2,iv) est la valeur max de la vitesse de chute
!     xws(3,iv) est la valeur de concentration pour laquelle Ws est max
!     xws(4,iv) est le param. beta1 de passage de cmes  la concent.volum.
!     xws(5,iv) est le param. beta2 de passage de cmes  la concent.volum.
!     xws(6,iv) est la concentration max pour laquelle l entravement est appliq.
!     xws(7,iv) est un facteur de dpendance de la salinit (sal.max)
!     xws(8,iv) est un facteur de dpendance de la turbulence (vitmax**2)
!     xws(9,iv) est un facteur disponible (non programm) 
!
      do iv=1,nvp
        if(xws(1,iv).eq.0. .or. xws(2,iv).eq.0.) then
          alp1(iv)=0.
          alp2(iv)=0.
          alp3(iv)=0.
          alp4(iv)=0.
          alp5(iv)=1.
          xws(10,iv)=0.
          xws(11,iv)=0.
          xws(12,iv)=0.
          xws(13,iv)=0.
	else
          alp1(iv)=2.*(xws(2,iv)/(xws(1,iv))-1.)/xws(3,iv)
          alp2(iv)=(1.-xws(2,iv)/(xws(1,iv)))/xws(3,iv)**2.
          alp3(iv)=1./(1.-xws(4,iv)*xws(3,iv)**xws(5,iv))
!       dans SAM-1DV, w6 est la vitesse de chute pour cmes=xws(6,iv)
          voleau6=1.-xws(4,iv)*xws(6,iv)**xws(5,iv)
          w6=xws(2,iv)*(voleau6/                                        &
     &     (1.-xws(4,iv)*xws(3,iv)**xws(5,iv)))**4.65
          alp5(iv)=(log10(wsfin)-log10(w6))/                          &
     &             (log10(cfin)-log10(xws(6,iv)))
          alp4(iv)=10**(log10(w6)-alp5(iv)*log10(xws(6,iv)))
          xws(10,iv)=0.
          if(xws(7,iv).gt.0.)then
            xws(10,iv)=1.
            xws(12,iv)=1./xws(7,iv)
          endif
          xws(11,iv)=0.
          if(xws(8,iv).gt.0.)then
            xws(11,iv)=1.
            xws(13,iv)=1./xws(8,iv)
          endif
	endif
      enddo
      return

#endif
      end
!
!
!************************************************************************
!
      subroutine t_vitchut(xe,u,v,sal,cv_wat)
!
!
!************************************************************************
#ifdef key_sedim_anc
use comsedim
use comsubstance, only : xws,ws3,nvp,nvpc,nv_adv
use comvars2d, only : h0fond
implicit none 
!
      integer i,j,iv,k,ivpc
      real(kind=rsh),dimension(kmax) :: cmes
      real(kind=rsh) :: vitmodu,coeftur,ctursal,voleau
      real(kind=rsh),dimension(imin:imax,jmin:jmax) :: u,v,xe

      real(kind=rsh),dimension(kmax,imin:imax,jmin:jmax) :: sal
      real(kind=rsh),dimension (nb_var,kmax,imin:imax,jmin:jmax) :: cv_wat

!



      do j=jmin,jmax
       do i=ig(j),id(j)
       if(h0(i,j)+xe(i,j).gt.h0fond)  then
!
        vitmodu=u(i,j)**2+v(i,j)**2
!        
        do k=1,kmax
!        if (biologie) then 
!           xws(1,irang(5))=wbio(1,k,i,j)
!           xws(2,irang(5))=wbio(1,k,i,j)
!           xws(1,irang(7))=wbio(2,k,i,j)
!           xws(1,irang(8))=wbio(2,k,i,j)
!           xws(1,irang(9))=wbio(2,k,i,j)
!           xws(2,irang(7))=wbio(2,k,i,j)
!           xws(2,irang(8))=wbio(2,k,i,j)
!           xws(2,irang(9))=wbio(2,k,i,j)
!         endif
          cmes(k)=0.
          do ivpc=1,nvpc

!cval            cmes(k)=cmes(k)+tz(ivpc,k,i,j)
            cmes(k)=cmes(k)+cv_wat(ivpc,k,i,j)

          enddo

          cmes(k)=max(0.,cmes(k))

        do iv=1,nvp
!
        if((l_sable.or.l_sable3d).and.iv.ge.ndsable.and.iv.le.nfsable) then
        ws3(k,iv,i,j)=xws(1,iv)
!
        else
!

          coeftur=1.+xws(11,iv)*(-1.+                                   &
     &            xws(13,iv)*min(xws(8,iv),vitmodu+0.1))
            ctursal=coeftur*(1.+xws(10,iv)*(-1.+                        &
     &              xws(12,iv)*min(xws(7,iv),sal(k,i,j))))

            if(cmes(k).le.xws(3,iv))then

              ws3(k,iv,i,j)=max(xws(1,iv),xws(2,iv)*.5*                 &
     &                       amin1(2.,cmes(k)))

              ws3(k,iv,i,j)=max(xws(1,iv),ctursal*ws3(k,iv,i,j))

!
            elseif(cmes(k).le.xws(6,iv))then

              voleau=max(0.05,1.-xws(4,iv)*cmes(k)**xws(5,iv))
              ws3(k,iv,i,j)=max(xws(1,iv),ctursal*xws(2,iv))            &
     &                      *(voleau*alp3(iv))**4.65     
            else

              ws3(k,iv,i,j)=alp4(iv)*cmes(k)**alp5(iv)     
            endif


!
        endif
        enddo
!
!
!cval              do iv=nvp+1,nvar
        do iv=nvp+1,nv_adv
            ws3(k,iv,i,j)=0.0
        enddo
!
       enddo
         endif
        enddo
      enddo
!

#endif


      end
