MODULE ibm
#if defined key_ibm
   !!======================================================================
   !!                   ***  MODULE ibm   ***
   !! 
   !!         M. Huret 01/2011
   !!         IBM (Individual Based model)
   !!         To be adapted for each case for which a CPP key has to be defined
   !&E
   !&E ** History :
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2012-02 (M. Huret) Introduction of NAMELIST
   !&E
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters, ONLY :  imin,imax,jmin,jmax,kmax,ljmin,ljmax,ljmaxp1,  &
                           limaxp1,liminm1,limaxp2,ljminm1,ljmaxp2,       &
                           rsh,rlg,lchain,limin,liminm2,ljminm2,limax,    &
                           rg_valmanq_io,dg_valmanq_io,valmanq
   USE comtraj

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC ibm_3d            ! routine called by step3dxy.F90 step3dyx.F90
   PUBLIC ibm_init          ! routine called by main.F90

   !! * Shared module variables
   INTEGER, PUBLIC  :: nbouc_ibm

   !! * Private variables
   REAL(kind=rsh)                  :: w_max,alpha_w

 CONTAINS
   !!======================================================================

   SUBROUTINE ibm_init(xe,sal,temp,          &
#ifdef key_substance
                                   cv_wat,   &
#endif
                                   boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_init  ***
   !&E
   !&E ** Purpose : Initialize properties of particles with call to traj_init3d
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-11 (M. Huret) Original code
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE trajinitsave, ONLY: traj_init3d,nbouc_traj
   USE comvars2d,   ONLY : iscreenlog
#if defined key_siggen
   USE trajectools, ONLY:ztosiggen
#endif
#ifdef key_anchovy
   USE debmodel, only : deb_init,debuse,F_Fix,ffix,file_food,file_NBSS
   USE ibmmove, only : fish_move_init
   USE ibmtools, only :ibm_loc_xyz,gasdev_s,ibm_traint,w_dens
#endif

  USE ionc4,       ONLY : ionc4_openr,ionc4_read_trajt,ionc4_read_traj,ionc4_close,ionc4_read_time

! Arguments
     REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )       :: xe
     REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: sal,temp,boz
#ifdef key_substance
     REAL(KIND=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in ) :: cv_wat
#endif

   !! * Local declarations
     CHARACTER(LEN=lchain)    :: fileibm
     INTEGER                  :: IERR_MPI,eof
     REAL(KIND=rsh)           :: xe_init,h0_init
     INTEGER                  :: n,m,igg,idd,jhh,jbb,kp,km
     REAL(KIND=rsh)           :: px,py
     INTEGER                  :: hlb,hlt,hrb,hrt  
     INTEGER                  :: i_stage
     REAL(KIND=rsh)           :: i_w,i_age,i_size,i_density,i_Drate,i_number
     LOGICAL	              :: ibm_restart,ibm_l_time,ex
     REAL(KIND=rlg)           :: tread
     REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: sizenc,agenc,numbernc,densitync,dratenc,flagnc
     REAL(KIND=rlg), ALLOCATABLE, DIMENSION(:)   :: dateorignc
     INTEGER, ALLOCATABLE, DIMENSION(:)   :: stagenc,numnc
#ifdef key_anchovy
     INTEGER,        ALLOCATABLE, DIMENSION(:)   :: dayjuvnc
     REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: denspawnnc
     REAL(KIND=rsh)           :: sal_surf,temp_surf,dens_surf,prof,sprof
     REAL(KIND=rsh)           :: tir,ec_type,ec_type_size
#endif

   
!!----------------------------------------------------------------------
   !! * Executable part

   ! namelists in paraibm.txt
   !--------------------------

     if (nbouc_ibm==0) THEN  ! Fisrt check everything ok at start of simulation with minimum information!

        nbouc_ibm=1
        nbouc_traj=0

        NAMELIST/namibmin/ file_trajec,itypetraj
        NAMELIST/namibmphys/ ndtz
        fileibm='./paraibm.txt'
        OPEN(50,file=fileibm,status='old',form='formatted', access='sequential')
        READ(50,namibmin)
        READ(50,namibmphys)
        CLOSE(50)

        CALL traj_init3d(xe)

         ! save into simu.log
        !-------------------
        WRITE(iscreenlog,*) ' '
        WRITE(iscreenlog,*) ' '
        WRITE(iscreenlog,*) ' '
        WRITE(iscreenlog,*) '**************************************************'
        WRITE(iscreenlog,*) '***************** IBM_INIT.F90 ****************'
        WRITE(iscreenlog,*) '**************************************************'
        WRITE(iscreenlog,*) ' '
        WRITE(iscreenlog,*) 'fichier definissant les caracteristiques des trajectoires : ',trim(file_trajec)

     ELSE ! nbouc_ibm=1, then initialize everything

        NAMELIST/namibmin/ file_trajec,itypetraj
        NAMELIST/namibmphys/ ndtz
        NAMELIST/namibmrestart/ ibm_restart,ibm_l_time
        NAMELIST/namibminit/  i_stage, i_number,i_w,i_size,i_density,i_Drate
        NAMELIST/namibmbio/ w_max,alpha_w
#ifdef key_anchovy
        NAMELIST/namibmdeb/ debuse,F_Fix,ffix,file_NBSS,file_food
#endif
        fileibm='./paraibm.txt'
        OPEN(50,file=fileibm,status='old',form='formatted', access='sequential')
        READ(50,namibmin)
        READ(50,namibmphys)
        READ(50,namibmrestart)
        READ(50,namibminit)
        READ(50,namibmbio)
#ifdef key_anchovy
        READ(50,namibmdeb)
#endif
        CLOSE(50)
        
        CALL traj_init3d(xe)
        
        part(:,:)%stage=i_stage            
        part(:,:)%super=i_number
        part(:,:)%w=i_w
        part(:,:)%size=i_size !read in mm
        part(:,:)%density=i_density
        part(:,:)%temp=0.0_rsh   
        part(:,:)%Drate=i_Drate
        
        if (ibm_restart)  then
           
           WRITE(iscreenlog,*) 'debut read restart IBM = '   
           DO n = 1,nb_patch
              
              CALL ionc4_openr(trim(filetrajin(n)),.false.)
              
              ALLOCATE (stagenc(nb_part(n)),sizenc(nb_part(n)))
              ALLOCATE (numbernc(nb_part(n)),numnc(nb_part(n)))
              ALLOCATE (densitync(nb_part(n)),dratenc(nb_part(n)))
              ALLOCATE (flagnc(nb_part(n)))
              
              call ionc4_read_trajt(trim(filetrajin(n)),"STAGE",stagenc,1,nb_part(n),1)
              call ionc4_read_trajt(trim(filetrajin(n)),"SIZE",sizenc,1,nb_part(n),1)
              call ionc4_read_trajt(trim(filetrajin(n)),"NUM",numnc,1,nb_part(n),1) ! on ecrase ce qui est lu dans initraj
              call ionc4_read_trajt(trim(filetrajin(n)),"NUMBER",numbernc,1,nb_part(n),1)
              call ionc4_read_trajt(trim(filetrajin(n)),"DENSITY",densitync,1,nb_part(n),1)
              call ionc4_read_traj(trim(filetrajin(n)),"DRATE",dratenc,1,nb_part(n))
              call ionc4_read_traj(trim(filetrajin(n)),"flag",flagnc,1,nb_part(n))

              
              part(n,:)%stage=stagenc
              part(n,:)%size=sizenc
              part(n,:)%super=numbernc
              part(n,:)%density=densitync
              part(n,:)%Drate=dratenc
              part(n,:)%num=numnc
              part(n,:)%flag=flagnc

              
              ! update the date of restart, and savetraj is delayed not to have twice same time step in output
              if (ibm_l_time) then
                 CALL ionc4_read_time(trim(filetrajin(n)),1,tread)
                 tbegtraj(n)=tread
                 tsavtraj(n)=tread+dtsave_traj(n)*3600.0_rlg
              endif
              
              DEALLOCATE (stagenc,sizenc,numbernc,densitync,dratenc,flagnc,numnc)
              
#ifdef key_anchovy
              ALLOCATE (dateorignc(nb_part(n)),dayjuvnc(nb_part(n)),denspawnnc(nb_part(n)))
              
              call ionc4_read_traj(trim(filetrajin(n)),"DAYBIRTH",dateorignc,1,nb_part(n))
              call ionc4_read_traj(trim(filetrajin(n)),"DAYJUV",dayjuvnc,1,nb_part(n))
              call ionc4_read_traj(trim(filetrajin(n)),"DENSPAWN",denspawnnc,1,nb_part(n))
              
              part(n,:)%date_orig=dateorignc
              part(n,:)%dayjuv=dayjuvnc
              part(n,:)%denspawn=denspawnnc
              
              DEALLOCATE (dateorignc,dayjuvnc,denspawnnc)
              
              do m=1,nb_part(n)
                 if (part(n,m)%stage > 4) part(n,m)%itypevert=0
              end do
              
#endif
              
              call ionc4_close(filetrajin(n))
              
              
           END DO  ! loop on patch (n)
           
           
        else


#ifdef key_anchovy

           part(:,:)%size=part(:,:)%size*0.1_rsh ! egg diameter (mm ->cm)    
           part(:,:)%dayjuv=0
           
           !Goarant et al. 2007 (egg density =f(surf. salinity))
           DO n = 1,nb_patch
              DO m = 1,nb_part(n)
                 
                 IF_MPI ( part(n,m)%inoutflag ) THEN
                 
                 ! Egg density ----------------
                 
                 prof=min(4.0_rsh,part(n,m)%d3) ! surface
#ifdef key_siggen
                 prof=-prof+part(n,m)%xe ! switch from immersion to real z
                 CALL ztosiggen(prof,sprof,part(n,m)%xe,part(n,m)%h0,part(n,m)%hc)
#else
                 sprof=-(prof/part(n,m)%d3)
#endif
                 
                 CALL ibm_loc_xyz(xe,part(n,m)%xpos,part(n,m)%ypos,sprof, &
                      px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt,kp,km)
                 
                 
                 !egg density=f(surf. density)
                 sal_surf=ibm_traint(sal,xe,sprof,kp,km,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                 temp_surf=ibm_traint(temp,xe,sprof,kp,km,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                 dens_surf=w_dens(temp_surf,sal_surf)
                 part(n,m)%denspawn=-2.45_rsh + 1.064_rsh*(dens_surf-1000.0_rsh)+1000.0_rsh ! with thermal expansion correction
                 part(n,m)%denspawn=part(n,m)%denspawn+0.5_rsh ! cf Paul G.
                 
                 !           part(n,m)%denspawn=4.251_rsh + 0.787_rsh*(dens_surf-1000.0_rsh)+1000.0_rsh ! sardine
                 !           part(n,m)%denspawn=part(n,m)%denspawn+0.78_rh ! sardine
                 
                 
                 ! Egg density variability ----------------
                 ec_type=0.72_rsh !anchois
                 ! ec_type=0.83_rsh !sardine
                 CALL gasdev_s(tir)
                 part(n,m)%denspawn=part(n,m)%denspawn+tir*ec_type 
                 part(n,m)%density=part(n,m)%denspawn
                 
                 
                 ! Egg size variability----------------------
                 ec_type_size=0.0035_rsh
                 !call random_number(v1) ! 0 to +1 in each direction
                 !v1=2.0_rsh*v1-1.0_rsh  !-1 to +1 in each direction
                 !deltas=0.1_rsh*part(n,m)%size
                 part(n,m)%size=part(n,m)%size+tir*ec_type_size
                 
                 ENDIF_MPI
                 
                 
              ENDDO
           ENDDO
#endif

        endif ! restart or not

#ifdef key_anchovy
        if (debuse) call deb_init(ibm_restart)
        call fish_move_init
#endif
        
        ! save initialization
        CALL ibm_save
        
        nbouc_ibm=2
        
     ENDIF ! nbouc_ibm

  END SUBROUTINE ibm_init

   !!======================================================================

   SUBROUTINE ibm_3d(xe,uz,vz,sal,temp,    &
#ifdef key_substance
                                     cv_wat,   &
#endif
                                     boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_init  ***
   !&E
   !&E ** Purpose : Main routine with call to advection, behaviour, growth...
   !&E **  To be adapted by the user depending on case study
   !&E
   !&E ** Description :
   !&E ** Called by : traject3d
   !&E ** External calls :
   !&E ** Reference :
   !&E ** History : ! marc sourisseau 12/2008
   !&E              ! M. Huret 11/1010
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------

   !! * Modules usedk
   USE comvars2d, ONLY : t,dt,jjulien
#ifdef key_substance
   USE parameters, ONLY : nb_var
#endif
   USE traject3d, ONLY : traj_3d
   USE trajinitsave, ONLY : traj_init3d
   USE ibmtools, only : ibm_loc_xyz,ibm_buoy,ibm_traint,ibm_nycth_mig &
        ,ibm_proftraint, ibm_profuint, ibm_profvint
#if defined key_siggen
   USE trajectools, ONLY:ztosiggen
#endif
#ifdef key_anchovy
   USE debmodel, ONLY : deb_cycle,debuse,F_Fix,readfood3d,Hp,Hj,readtemp3d,climatemp
   USE ibmmove, ONLY : fish_move
   USE sflxatm,      ONLY : rad
   USE_MPI toolmpi, ONLY : ex_traj
#endif

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )       :: xe
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: sal,temp,boz
   REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )  :: uz,vz
#ifdef key_substance
   REAL(KIND=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in ) :: cv_wat
#endif

   !! * Local declarations
   INTEGER                    :: jj,mm,aaaa,hh,minu,sec,n,m
   REAL(KIND=rsh)             :: depth_day,depth_night,w1,w2
   REAL(KIND=rsh)             :: sal_part,zlag,slag
   REAL(KIND=rlg)             :: dtm
   LOGICAL                    :: update,night
   INTEGER                    :: kp,km
   INTEGER                    :: igg,idd,jhh,jbb  
   REAL(KIND=rsh)             :: px,py,mindepth,maxdepth
   INTEGER                    :: hlb,hlt,hrb,hrt  
   INTEGER                    :: IERR_MPI
   character(len=19)          :: tool_sectodat,date
   REAL(KIND=rsh)             :: tempo

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter IBM routine'

   IF (nbouc_ibm==1) THEN
      CALL ibm_init(xe,sal,temp, &
#ifdef key_substance
           cv_wat,       &
#endif
           boz)
   ENDIF

   ! save particle properties (before any change for getting exact initial properties) 
   CALL ibm_save

!if advection, depend de itypevert passive transport
   CALL traj_3d(xe,uz,vz)

   date=tool_sectodat(t) 
   call tool_decompdate(date,jj,mm,aaaa,hh,minu,sec)

!#ifdef key_dino
   !CALL ibm_detect_pycno(xe,boz,sal,temp)
!   CALL ibm_opt_depth(xe)   !model of convergence to an optimal depth
!#endif

#ifdef key_anchovy
   if (debuse .and. .not.F_Fix) then
      CALL readfood3d(jjulien)
      CALL readtemp3d(jjulien)
   endif
#endif


!--------------------------------------------------------------
!Common part to all species
!---------------------------------------------------------------
   dtm=dt*0.5_rlg

#ifdef key_MPI_2D
   down_give=0
   up_give=0
   right_give=0
   left_give=0
#endif

   DO n = 1,nb_patch
      IF ((t >= tbegtraj(n)).and.(t <= tendtraj(n))) THEN
      DO m = 1,nb_part(n)

         if (part(n,m)%super > 0.0_rsh .and. part(n,m)%flag /= -valmanq) THEN

         IF_MPI ( part(n,m)%inoutflag ) THEN

         CALL ibm_loc_xyz(xe,part(n,m)%xpos,part(n,m)%ypos,part(n,m)%spos, &
              px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt,kp,km)     

!--------------------------------------------------------------
! Specific models with calls to subroutines
!---------------------------------------------------------------

!Anchovy eggs and larvae
#ifdef key_anchovy
! size in cm

   update=.TRUE.
       
   IF (part(n,m)%stage == 1) THEN ! stade 1, eggs, buoyancy  
      part(n,m)%temp=ibm_traint(temp,xe,part(n,m)%spos,kp,km, &
           px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
       !egg density dependent on development (Adapted from Ospina 2012)
      part(n,m)%density=part(n,m)%denspawn+3.2029*part(n,m)%Drate-7.4937*(part(n,m)%Drate)**2 &
           -3.0858*(part(n,m)%Drate)**3+9.0376*(part(n,m)%Drate)**4 ! anchovy
      sal_part=ibm_traint(sal,xe,part(n,m)%spos,kp,km,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      part(n,m)%w = ibm_buoy(part(n,m)%density, part(n,m)%size,part(n,m)%temp,sal_part)
      !egg development rate (Regner, 1996)

      ! temp climato for Drate
      tempo=climatemp(kp,NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))
      if (tempo > 0.0_rsh .and. tempo < 30.0_rsh)  part(n,m)%temp=tempo
      part(n,m)%Drate=part(n,m)%Drate+dtm/86400.0_rsh*0.000559_rsh*part(n,m)%temp**2.290236_rsh
      !egg mortality
      part(n,m)%super=(1.0_rsh-0.266_rsh*dtm/86400.0_rsh)*part(n,m)%super
      if (part(n,m)%Drate.ge.1.0_rsh) then 
         part(n,m)%Drate=0.0_rsh
         part(n,m)%stage = 2
         part(n,m)%size = 0.35_rsh ! on force eclosion taille fixe
         part(n,m)%w=0.0_rsh
         update=.FALSE.
      endif
   ENDIF ! end of stage 1
      
   IF (part(n,m)%stage == 2) THEN ! stade 2, yolk sac larva, buoyancy  
     
      ! temp climato
      tempo=climatemp(kp,NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))
      if (tempo > 0.0_rsh .and. tempo < 30.0_rsh) then
         part(n,m)%temp=tempo
      else
         part(n,m)%temp=ibm_traint(temp,xe,part(n,m)%spos,kp,km, &
           px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      endif

  !    sal_part=ibm_traint(sal,xe,part(n,m)%spos,kp,km,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
  !    part(n,m)%w = ibm_buoy(part(n,m)%density,part(n,m)%size,part(n,m)%temp,sal_part)
  !yolk sac larva development rate (Regner, 1996)
      part(n,m)%Drate=part(n,m)%Drate+dtm/86400.0_rsh*0.000003702809_rsh*part(n,m)%temp**3.8079_rsh
  ! growth (simple as function of Drate as not handled in DEB model so far)
      part(n,m)%size=0.35_rsh+(0.4_rsh-0.35_rsh)*part(n,m)%Drate
      if (part(n,m)%Drate.ge.1.0_rsh) then 
         part(n,m)%stage = 3
         part(n,m)%size=0.4_rsh ! on force ouverture bouche  taille fixe (cm) tant que yolk sac pas dans DEB, Regner 1996
         update=.FALSE.
      endif
   ENDIF ! end of stage 2

   IF (part(n,m)%stage == 3.AND.update) THEN ! stage 3, larva without vertical migration
      ! temp climato
      tempo=climatemp(kp,NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))
      if (tempo > 0.0_rsh .and. tempo < 30.0_rsh) then
         part(n,m)%temp=tempo
      else
         part(n,m)%temp=ibm_traint(temp,xe,part(n,m)%spos,kp,km, &
           px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      endif
      depth_day=25.0_rsh
      depth_night=25.0_rsh
 
  !    sal_part=ibm_traint(sal,xe,part(n,m)%spos,kp,km,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
  !    part(n,m)%w = ibm_buoy(part(n,m)%density, part(n,m)%size,part(n,m)%temp,sal_part)

      w_max=part(n,m)%size/10.0_rsh/100.0_rsh ! m.s-1
      part(n,m)%w=ibm_nycth_mig(depth_day,depth_night,w_max,w_max,n,m,alpha_w)    ! to avoid deep diving at slope, to remove later on   
      !growth
      if (debuse) call deb_cycle(n,m,dtm,aaaa,mm)
      if (part(n,m)%size.ge.0.6_rsh) then 
         part(n,m)%stage = 4
         update=.FALSE.
      endif
   ENDIF ! end of stage 3
   
   IF (part(n,m)%stage == 4.AND.update) THEN ! stage 4, nycthemeral migration
      ! temp climato
      tempo=climatemp(kp,NINT(part(n,m)%xpos),NINT(part(n,m)%ypos))
      if (tempo > 0.0_rsh .and. tempo < 30.0_rsh) then
         part(n,m)%temp=tempo
      else
         part(n,m)%temp=ibm_traint(temp,xe,part(n,m)%spos,kp,km, &
           px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      endif
      depth_day=25.0_rsh
      depth_night=0.0_rsh
      ! calculate an optimal depth ?
      w_max=part(n,m)%size/10.0_rsh/100.0_rsh ! m.s-1
      part(n,m)%w=ibm_nycth_mig(depth_day,depth_night,w_max,w_max,n,m,alpha_w)       
      !growth
       if (debuse) call deb_cycle(n,m,dtm,aaaa,mm)
    !  if (part(n,m)%size.ge.3.5_rsh) then  ! to be made more continuous
       if (part(n,m)%H .ge. Hj) then
         part(n,m)%stage = 5
         update=.FALSE.
         part(n,m)%itypevert=0 ! Stop vertical advection/diffusion from stage 5
         part(n,m)%zpos=0.0_rsh
         part(n,m)%w=0.0_rsh
         part(n,m)%dayjuv=jjulien
      endif
   ENDIF ! end of stage 4

   IF (part(n,m)%stage == 5.AND.update) THEN ! stage 5, Horizontal swimming, distribution from file     
      ! Vertical position
      night=.FALSE.
      if(rad(NINT(part(n,m)%xpos),NINT(part(n,m)%ypos)).eq.0.0_rsh) THEN
         night=.TRUE.
      endif
      if (part(n,m)%H .lt. Hp .and. jjulien.ge.part(n,m)%dayjuv .and. jjulien.lt.330) then
         if (night) then ! Juvenile Nuit
            mindepth=0.0_rsh
            maxdepth=30.0_rsh
         else! Juvenile day
            mindepth=min(30.0_rsh, 2.27_rsh*part(n,m)%size) ! Boyra, 2013
            maxdepth=30.0_rsh+(120.0_rsh-30.0_rsh)*(1.0_rsh-(330.0_rsh-real(jjulien,rsh))/(330.0_rsh-real(part(n,m)%dayjuv,rsh)))
         !  maxdepth=min(part(n,m)%d3,120.0_rsh)
         endif
      else
         if (jjulien.le.90 .or. jjulien.ge.330) then !winter 
            mindepth=40.0_rsh
            maxdepth=120.0_rsh
         else 
            if (night) then ! spring/summer
               ! Adulte Nuit
               mindepth=0.0_rsh
               maxdepth=40.0_rsh
            else ! Adult day
               mindepth=30.0_rsh 
               maxdepth=120.0_rsh
            endif
         endif
      endif

      ! temp climato
      tempo=ibm_proftraint(climatemp,mindepth,maxdepth,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      if (tempo > 0.0_rsh .and. tempo < 30.0_rsh) then
         part(n,m)%temp=tempo
      else
         part(n,m)%temp=ibm_proftraint(temp,mindepth,maxdepth,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
      endif
      !growth
      if (debuse) call deb_cycle(n,m,dtm,aaaa,mm)
      ! Movement (time step 5H, cruising speed =1.5BL ~ 4km/h pour TL=15cm) 
      !! if dt or swimming speed too big may jump over 1 cell and MPI will crash (depend also on hor. resolution)
      if (hh >= part(n,m)%hmove+5) then
         call fish_move(n,m)
         part(n,m)%hmove=hh ! update of the saved hour
         if (part(n,m)%hmove.ge.19) part(n,m)%hmove=part(n,m)%hmove-24
      endif
   ENDIF  ! end of stage 5 

   ! mortality
 !  if (part(n,m)%stage > 1) part(n,m)%super=(1.0_rsh-0.28_rsh*dtm/86400.0_rsh)*part(n,m)%super


!anchovy
#endif
   

! Dynophysis
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#ifdef key_dino  !migration to an optimal depth
      !model de Neumann et Fennel : règles de migration
      ! a developper / Marc S.
      w_max=15.0_rsh/86400.0_rsh   !vitesse max de deplacement sur la verticale en m.j-1 conv en m.s-1
 !     part(n,m)%opt_depth=ibm_opt_depth(part(n,m)%xpos,part(n,m)%ypos)
 !     if (part(n,m)%opt_depth .lt. part(n,m)%d3) THEN
 !        part(n,m)%w=w_max*     &
 !             tanh(alpha_w*((part(n,m)%d3*(1.0_rsh-part(n,m)%zpos))-part(n,m)%opt_depth))
      else
         part(n,m)%w=0.0_rsh
      endif    
!dino
#endif

!--------------------------------------------------------------------------------
   ! Vertical movement and check for bottom and surface boundaries 
         
      zlag=-part(n,m)%zpos+part(n,m)%xe ! switch from immersion to real z
      zlag=zlag+part(n,m)%w*dtm         ! update position
      zlag=MIN(MAX(zlag,-part(n,m)%h0),part(n,m)%xe) ! check boundaries
      
      part(n,m)%zpos=-zlag+part(n,m)%xe  ! immersion
      
      if (part(n,m)%stage<=2) part(n,m)%zpos=min(part(n,m)%zpos,40.0_rsh) !patch...
      
#ifdef key_siggen 
      CALL ztosiggen(zlag,slag,part(n,m)%xe,part(n,m)%h0,part(n,m)%hc)
      part(n,m)%spos=slag
#else
      part(n,m)%spos=(zlag-part(n,m)%xe)/part(n,m)%d3                    
#endif

      ENDIF_MPI ! ends test on inoutflag
      ENDIF  ! super > 0, not dead
      ENDDO ! particle
   ENDIF ! date
   ENDDO ! patch

#ifdef key_anchovy
    CALL_MPI ex_traj(down_give, up_give, right_give, left_give) ! because fish_move
#endif

   END SUBROUTINE ibm_3d


   !!======================================================================

 SUBROUTINE ibm_save

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ibm_save  ***
   !&E
   !&E ** Purpose : Save trajectories
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : step.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  01-2011 (M. Huret) 
   !&E       !  2011-10 (M. Huret) Introduction of MPI
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,   ONLY : nbouc,date_ref,t,iscreenlog
   USE ionc4,       ONLY : ionc4_createfile_traj,ionc4_torigine,ionc4_tunite, &
                           ionc4_createvar_traj,ionc4_write_time,ionc4_sync,  &
                           ionc4_write_trajt, ionc4_write_traj
   USE_MPI toolmpi, ONLY : iproc,exchange_vectcpu_int

   !! * Arguments

   !! * Local declarations
   LOGICAL                                     :: l_out_nc4par
   INTEGER                                     :: n,m,k,kw,kwm,i,j,num1,num2,nbpart_out,p,nbproc,ind1
   REAL(kind=rsh)                              :: fillval
   REAL(KIND=rlg)                              :: tool_ind2lat,tool_ind2lon
   INTEGER,        ALLOCATABLE, DIMENSION(:)   :: num_out
   REAL(KIND=rlg), ALLOCATABLE, DIMENSION(:)   :: lat_out,lon_out,dateorig_out
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: temp_out,flag_out,spos_out,zpos_out,xpos_out,ypos_out 
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: h0pos_out,size_out,stage_out,nb_out,dens_out,food_out
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: E_out,H_out,R_out,Wdeb_out,Neggs_out,NRJ_out,Gam_out,Drate_out,Denspawn_out
   INTEGER,        ALLOCATABLE, DIMENSION(:)   :: dayjuv_out, dayspawn_out,yearspawn_out,season_out

   !!----------------------------------------------------------------------
   !! * Executable part

   IF(rsh==8) then 
      fillval = dg_valmanq_io
   ELSE
      fillval = rg_valmanq_io
   ENDIF

   ALLOCATE( xpos_out(0:nbmax_part),ypos_out(0:nbmax_part) )
   ALLOCATE( spos_out(0:nbmax_part),zpos_out(0:nbmax_part) )
   ALLOCATE( lat_out(0:nbmax_part),lon_out(0:nbmax_part) )
   ALLOCATE( temp_out(0:nbmax_part) )
   ALLOCATE( h0pos_out(0:nbmax_part),num_out(0:nbmax_part) )
   ALLOCATE( flag_out(0:nbmax_part),dens_out(0:nbmax_part) )
   ALLOCATE( size_out(0:nbmax_part),dateorig_out(0:nbmax_part) )
   ALLOCATE( stage_out(0:nbmax_part),nb_out(0:nbmax_part) )
   ALLOCATE( Drate_out(0:nbmax_part))

   xpos_out(:)=fillval ; ypos_out(:)=fillval ; spos_out(:)=fillval ; zpos_out(:)=-fillval ; h0pos_out(:)=-fillval
   flag_out(:)=fillval ; lat_out(:)=dg_valmanq_io ; lon_out(:)=dg_valmanq_io ; num_out(:)=0 ; temp_out(:)=fillval
   size_out(:)=fillval ; stage_out(:)=fillval ; nb_out(:)=fillval; dens_out(:)=fillval ; Drate_out(:)=fillval

#ifdef key_anchovy
   ALLOCATE( dayjuv_out(0:nbmax_part),dayspawn_out(0:nbmax_part) )
   ALLOCATE( yearspawn_out(0:nbmax_part),season_out(0:nbmax_part) )
   ALLOCATE( Denspawn_out(0:nbmax_part) )
   ALLOCATE( food_out(0:nbmax_part),Gam_out(0:nbmax_part) )
   ALLOCATE( H_out(0:nbmax_part),E_out(0:nbmax_part) )
   ALLOCATE( R_out(0:nbmax_part),Wdeb_out(0:nbmax_part) )
   ALLOCATE( Neggs_out(0:nbmax_part),NRJ_out(0:nbmax_part) )
   food_out(:)=fillval
   E_out(:)=fillval ; H_out(:)=fillval ; R_out(:)=fillval ;Gam_out(:)=fillval 
   Wdeb_out(:)=fillval ; Neggs_out(:)=fillval ; NRJ_out(:)=fillval; dateorig_out(:)=dg_valmanq_io
   yearspawn_out(:)=0 ; dayspawn_out(:)=0 ; dayjuv_out(:)=0
   season_out(:)=0 ; Denspawn_out(:)=fillval
#endif

   ! Create output file
   IF (nbouc_ibm == 1) THEN

#ifdef key_MPI_2D
     l_out_nc4par=.true.
#else
     l_out_nc4par=.false.
#endif

     DO n=1,nb_patch
#ifdef key_MPI_2D
        nbpart_out=nb_part(n)+1  ! part 0 fictive just for output of CPU without particles
#else
        nbpart_out=nb_part(n)
#endif
        CALL ionc4_createfile_traj(filetraj3dout(n),nbpart_out,0,0,l_out_nc4par=l_out_nc4par)
        CALL ionc4_torigine(filetraj3dout(n),date_ref)
        CALL ionc4_tunite(filetraj3dout(n),'seconds')
        CALL ionc4_createvar_traj(filetraj3dout(n),"DEPTH","m","depth as immersion",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"H0","m","h0",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NUM","","number of the particle",fill_value=0)
        CALL ionc4_createvar_traj(filetraj3dout(n),"flag","nbr","flag",fill_value=-fillval) 
        CALL ionc4_createvar_traj(filetraj3dout(n),"TEMP","degrees_Celsius","temperature",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"SIZE","Centimeters","Size",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"STAGE","","Stage",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NUMBER","Number","Number of individuals",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DENSITY","sigma","Density",fill_value=-fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DRATE","","Development rate of egg or larva",fill_value=fillval,ndims=1)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DAYBIRTH","","Date of birth",fill_value=dg_valmanq_io,ndims=1)


#ifdef key_anchovy
        CALL ionc4_createvar_traj(filetraj3dout(n),"FOOD","mg/m3","food",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"EDEB","Joules","Energy",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"HDEB","Joules","Maturity",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"RDEB","Joules","Repro",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"GAM","Joules","Energy gametes",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"WEIGHT","g","weight",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NEGGS","Number","Number of eggs",fill_value=fillval)
        CALL ionc4_createvar_traj(filetraj3dout(n),"NRJ","J/g","Energy Density",fill_value=fillval)
     
        CALL ionc4_createvar_traj(filetraj3dout(n),"YEARSPAWN","","Year authorised to spawn",fill_value=0,ndims=1)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DAYSPAWN","","Julian day start spawning",fill_value=0,ndims=1)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DAYJUV","","Julien day at metamorphosis",fill_value=0,ndims=1)
        CALL ionc4_createvar_traj(filetraj3dout(n),"SEASON","","Wether within spawning season",fill_value=0,ndims=1)
        CALL ionc4_createvar_traj(filetraj3dout(n),"DENSPAWN","sigma","Density of egg at spawning",fill_value=fillval,ndims=1)
#endif

     END DO
  END IF

  DO n=1,nb_patch
     IF ((t >= tsavtraj(n)).and.(t <= tendtraj(n))) THEN
        
        ! get the number of particles per cpus
#ifdef key_MPI_2D
        nb_part_percpu(n,iproc)=count(part(n,:)%inoutflag)
        CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
        
        IF_MPI ( nb_part_percpu(n,iproc) /= 0 ) THEN
        p=0
       DO m = 1,nb_part(n)
          IF_MPI ( part(n,m)%inoutflag ) THEN
          p=p+1
          xpos_out(p)=part(n,m)%xpos
          ypos_out(p)=part(n,m)%ypos
          lat_out(p)=tool_ind2lat(part(n,m)%xpos,part(n,m)%ypos)
          lon_out(p)=tool_ind2lon(part(n,m)%xpos,part(n,m)%ypos)
          dateorig_out(p)=part(n,m)%date_orig
          spos_out(p)=part(n,m)%spos
          zpos_out(p)=part(n,m)%zpos ! for output as immersion
          h0pos_out(p)=part(n,m)%d3 
          flag_out(p)=part(n,m)%flag   
          temp_out(p)=part(n,m)%temp
          size_out(p)=part(n,m)%size
          stage_out(p)=part(n,m)%stage
          dens_out(p)=part(n,m)%density
          nb_out(p)=part(n,m)%super
          num_out(p)=part(n,m)%num
          Drate_out(p)=part(n,m)%Drate

#ifdef key_anchovy
          food_out(p)=part(n,m)%X
          E_out(p)=part(n,m)%E
          H_out(p)=part(n,m)%H
          R_out(p)=part(n,m)%R
          Gam_out(p)=part(n,m)%Gam
          Wdeb_out(p)=part(n,m)%Wdeb
          Neggs_out(p)=part(n,m)%Neggs
          NRJ_out(p)=part(n,m)%NRJ
          
          yearspawn_out(p)=part(n,m)%yearspawn
          dayspawn_out(p)=part(n,m)%dayspawn
          dayjuv_out(p)=part(n,m)%dayjuv
          season_out(p)=part(n,m)%season
          denspawn_out(p)=part(n,m)%denspawn
#endif

          ENDIF_MPI

       END DO
       ENDIF_MPI

       CALL ionc4_write_time(filetraj3dout(n),0,t)

#ifdef key_MPI_2D
       nbproc=nb_part_percpu(n,iproc)
       IF_MPI ( nbproc /= 0 ) THEN
         ind1=1
         IF (iproc==0) THEN
            num1=2 ! 1 reserve a quand absence de part dans une CPU
         ELSE
            num1=SUM(nb_part_percpu(n,0:iproc-1))+2
         END IF
         num2=num1+nbproc-1
       ELSE_MPI  ! on met tout a 0 pour l indice 0. Indice d ecriture par defaut
          ind1=0
          num1=1
          num2=1
          lat_out(ind1)=REAL(ind1,rlg)
          lon_out(ind1)=lat_out(ind1)
          dateorig_out(ind1)=lat_out(ind1)
          spos_out(ind1)=REAL(ind1,rsh)
          zpos_out(ind1)=spos_out(ind1)
          xpos_out(ind1)=spos_out(ind1)
          ypos_out(ind1)=spos_out(ind1)
          h0pos_out(ind1)=spos_out(ind1)
          flag_out(ind1)=spos_out(ind1)
          temp_out(ind1)=spos_out(ind1)
          size_out(ind1)=spos_out(ind1)
          stage_out(ind1)=spos_out(ind1)
          dens_out(ind1)=spos_out(ind1)
          nb_out(ind1)=spos_out(ind1)
          Drate_out(ind1)=spos_out(ind1)
          num_out(ind1)=ind1

#ifdef key_anchovy
          food_out(ind1)=spos_out(ind1)
          E_out(ind1)=spos_out(ind1)
          H_out(ind1)=spos_out(ind1)
          R_out(ind1)=spos_out(ind1)
          Gam_out(ind1)=spos_out(ind1)
          Wdeb_out(ind1)=spos_out(ind1)
          Neggs_out(ind1)=spos_out(ind1)
          NRJ_out(ind1)=spos_out(ind1)

          yearspawn_out(ind1)=ind1
          dayspawn_out(ind1)=ind1
          dayjuv_out(ind1)=ind1
          season_out(ind1)=ind1
          denspawn_out(ind1)=spos_out(ind1)
#endif

       ENDIF_MPI
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(ind1:nbproc),num1,num2,0,0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'TEMP',temp_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'SIZE',size_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'STAGE',stage_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'DENSITY',dens_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUMBER',nb_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_traj(filetraj3dout(n),'DRATE',Drate_out(ind1:nbproc),num1,num2,fillval)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYBIRTH',dateorig_out(ind1:nbproc),num1,num2,dg_valmanq_io)

#ifdef key_anchovy
       CALL ionc4_write_trajt(filetraj3dout(n),'FOOD',food_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'EDEB',E_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'HDEB',H_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'RDEB',R_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'GAM',Gam_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'WEIGHT',Wdeb_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NEGGS',Neggs_out(ind1:nbproc),num1,num2,0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NRJ',NRJ_out(ind1:nbproc),num1,num2,0,fillval)

       CALL ionc4_write_traj(filetraj3dout(n),'YEARSPAWN',yearspawn_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYSPAWN',dayspawn_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYJUV',dayjuv_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_traj(filetraj3dout(n),'SEASON',season_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_traj(filetraj3dout(n),'DENSPAWN',denspawn_out(ind1:nbproc),num1,num2,fillval)
#endif

#else
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(1:nb_part(n)),1,nb_part(n),0,0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'TEMP',temp_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'SIZE',size_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'STAGE',stage_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'DENSITY',dens_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUMBER',nb_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_traj(filetraj3dout(n),'DRATE',Drate_out(1:nb_part(n)),1,nb_part(n),fillval)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYBIRTH',dateorig_out(1:nb_part(n)),1,nb_part(n),dg_valmanq_io)

#ifdef key_anchovy
       CALL ionc4_write_trajt(filetraj3dout(n),'FOOD',food_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'EDEB',E_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'HDEB',H_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'RDEB',R_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'GAM',Gam_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'WEIGHT',Wdeb_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NEGGS',Neggs_out(1:nb_part(n)),1,nb_part(n),0,fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NRJ',NRJ_out(1:nb_part(n)),1,nb_part(n),0,fillval)

       CALL ionc4_write_traj(filetraj3dout(n),'YEARSPAWN',yearspawn_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYSPAWN',dayspawn_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_traj(filetraj3dout(n),'DAYJUV',dayjuv_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_traj(filetraj3dout(n),'SEASON',season_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_traj(filetraj3dout(n),'DENSPAWN',denspawn_out(1:nb_part(n)),1,nb_part(n),fillval)
#endif

#endif


       ! To write the data on the disk and not loose data in case of run crash
       CALL ionc4_sync(filetraj3dout(n))

       tsavtraj(n)=tsavtraj(n)+dtsave_traj(n)*3600.0_rlg
     END IF
   END DO

   END SUBROUTINE ibm_save

! key_ibm
#endif

 END MODULE
