 MODULE comcolig3d

   !!======================================================================
   !!                   ***  MODULE comcolig3d  ***
   !! 
   !&E ** History :
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-07  (V. Garnier) MPI2d : initialize cdvit
   !&E
   !!======================================================================
   !! * Modules used
   USE parameters

   !! * Accessibility

   !! * Shared or public module variables

   !! * Private variables
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:)   :: cdvit
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:)   :: intz_exp
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:)   :: intz_exp_ustar, intz_exp_vstar
#if defined key_lans_turb || defined key_leray_turb
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:)   :: intz_exp_us_star, intz_exp_vs_star
#endif
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:) :: yz_exp
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: uz_v, vz_u
#if defined key_lans_turb || defined key_leray_turb
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: uzs_v, vzs_u
#endif

#if key_dyn_pg_djcs
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: zr,pr
#endif

#ifdef key_lans_turb
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: dzdx, dzdy 
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: pdyn
#endif

 CONTAINS

   !!======================================================================

  SUBROUTINE alloc_comcolig3d

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE alloc_comcolig3d  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#if key_dyn_pg_djcs
   ALLOCATE(zr(kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   ALLOCATE(pr(kmax,limin:limaxp1,ljmin:ljmaxp1))
   zr(:,:,:)=0.0_rsh  ! to manage with sleeping cpus
   pr(:,:,:)=0.0_rsh
#endif
   ALLOCATE(cdvit(limin:limax,ljmin:ljmax))
   ALLOCATE(intz_exp(limin:limax,ljmin:ljmax))
   ALLOCATE(intz_exp_ustar(limin:limax,ljmin:ljmax))
   ALLOCATE(intz_exp_vstar(limin:limax,ljmin:ljmax))
#if defined key_lans_turb || defined key_leray_turb
   ALLOCATE(intz_exp_us_star(limin:limax,ljmin:ljmax))
   ALLOCATE(intz_exp_vs_star(limin:limax,ljmin:ljmax))
#endif
   ALLOCATE(yz_exp(kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(uz_v(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2))
   ALLOCATE(vz_u(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2))
#if defined key_lans_turb || defined key_leray_turb
   ALLOCATE(uzs_v(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2))
   ALLOCATE(vzs_u(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2))
#endif

#ifdef key_lans_turb
   ALLOCATE(dzdx(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(dzdy(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(pdyn(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2))
#endif

   cdvit(:,:)=0.0_rsh
   intz_exp(:,:)=0.0_rlg
   yz_exp(:,:,:)=0.0_rlg
   intz_exp_ustar(:,:)=0.0_rsh
   intz_exp_vstar(:,:)=0.0_rsh
#if defined key_lans_turb || defined key_leray_turb
   intz_exp_us_star(:,:)=0.0_rsh
   intz_exp_vs_star(:,:)=0.0_rsh
#endif
   uz_v(:,:,:)=0.0_rsh
   vz_u(:,:,:)=0.0_rsh
#if defined key_lans_turb || defined key_leray_turb
   uzs_v(:,:,:)=0.0_rsh
   vzs_u(:,:,:)=0.0_rsh
#endif

#ifdef key_lans_turb
   dzdx(:,:,:)=0.0_rsh
   dzdy(:,:,:)=0.0_rsh
   pdyn(:,:,:)=0.0_rsh
#endif

  END SUBROUTINE alloc_comcolig3d

   !!======================================================================

  SUBROUTINE dealloc_comcolig3d

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dealloc_comcolig3d  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part
   DEALLOCATE(cdvit,intz_exp,yz_exp,intz_exp_ustar,intz_exp_vstar, uz_v, vz_u)
#if defined key_lans_turb || defined key_leray_turb
   DEALLOCATE(uzs_v, vzs_u)
!   DEALLOCATE(uzs_v, vzs_u, intz_exp_us_star, intz_exp_vs_star)
#endif
#ifdef key_lans_turb
   DEALLOCATE(dzdx, dzdy)
   DEALLOCATE(pdyn)
#endif

#ifdef key_dyn_pg_djcs
   DEALLOCATE(zr,pr)
#endif

  END SUBROUTINE dealloc_comcolig3d

   !!======================================================================

 END MODULE comcolig3d
