 MODULE comoasis

   !!======================================================================
   !!                   ***  MODULE comoasis   ***
   !! 
   !!======================================================================
   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC init_comoasis                ! routine called by init.F90 and cas*.F90

   !! * Shared or public module variables
                            ! Ocean variables
   LOGICAL, PUBLIC       :: l_cpl_ocn_maskt, l_cpl_ocn_masku, l_cpl_ocn_maskv, & ! land mask at t/u/v location
                            l_cpl_ocn_ssh,l_cpl_ocn_sst, &                       ! sea surface height, sea surface temperature
                            l_cpl_ocn_ucur, l_cpl_ocn_vcur                       ! zonal / meridional current

                            ! Atmospherical variables
   LOGICAL, PUBLIC       :: l_cpl_atm_fwsu, l_cpl_atm_fwsv, &   ! zonal / meridional wind stress
                            l_cpl_atm_heat, l_cpl_atm_rad,  &   ! total heat fluxes, solar radiation
                            l_cpl_atm_ir, l_cpl_atm_lat,l_cpl_atm_sens, &   ! long-wave, latent, sensible heat fluxes
                            l_cpl_atm_evap, l_cpl_atm_rain, &   ! evaporation, rain
                            l_cpl_atm_pa                        ! atmospherical pressure

                            ! Wave variables
   LOGICAL,PUBLIC        :: l_cpl_wave_mask,    & ! land mask
                            l_cpl_wave_tmn,     & ! Mean wave period (tmn in s) (m0,-1)
                            l_cpl_wave_hs,      & ! Significant wave height (hs in m)
                            l_cpl_wave_dir,     & ! Mean wave direction (degrees)
                            l_cpl_wave_bhd,     & ! Wave-induced Bernoulli head pressure (bhd in N.m-1) (J term, Smith JPO 2006)
                            l_cpl_wave_two,     & ! Wave-ocean momentum flux (tauox/y in m2.s-2)
                            l_cpl_wave_uba,     & ! rms amplitude of orbital bottom velocity of the waves (uba in m/s)
                            l_cpl_wave_foc,     & ! Wave-to-ocean TKE flux (phioc in W.m-2)
                            l_cpl_wave_taw,     & ! Net wave-supported stress (tauwix/y in m2.s-2)
                            l_cpl_wave_lm         ! Mean wave length (wlm in m)

 CONTAINS

   !!======================================================================

  SUBROUTINE init_comoasis

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_comoasis  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ! Ocean variables
   l_cpl_ocn_maskt = .False.
   l_cpl_ocn_masku = .False.
   l_cpl_ocn_maskv = .False.
   l_cpl_ocn_ssh   = .False.
   l_cpl_ocn_sst   = .False.
   l_cpl_ocn_ucur  = .False.
   l_cpl_ocn_vcur  = .False.

   ! Atmospherical variables
   l_cpl_atm_fwsu = .False.
   l_cpl_atm_fwsv = .False.
   l_cpl_atm_heat = .False.
   l_cpl_atm_rad  = .False.
   l_cpl_atm_ir   = .False.
   l_cpl_atm_lat  = .False.
   l_cpl_atm_sens = .False.
   l_cpl_atm_evap = .False.
   l_cpl_atm_rain = .False.
   l_cpl_atm_pa   = .False.

   ! Wave variables
   l_cpl_wave_mask = .False.
   l_cpl_wave_tmn = .False.
   l_cpl_wave_hs = .False.
   l_cpl_wave_dir = .False.
   l_cpl_wave_bhd = .False.
   l_cpl_wave_two = .False.
   l_cpl_wave_uba = .False.
   l_cpl_wave_foc = .False.
   l_cpl_wave_taw = .False.
   l_cpl_wave_lm = .False.
!   l_cpl_wave_ = .False.

  END SUBROUTINE init_comoasis

   !!======================================================================

  END MODULE comoasis
