
  MODULE comwave

#ifdef key_wave
   !!======================================================================
   !!                       ***  MODULE comwave   ***
   !!======================================================================

   !&E
   !&E ** Called by:
   !&E      module wave_read (in wave.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of cometdyn.F90
   !&E       !  2010-07 (R. Verney) Introduction of orbital velocity and wave period variables
   !&E       !  2013-08 (AC. Bennis) Introduction of arrays for 3D radiation stress tensor
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !!======================================================================
   !! * Modules used
   USE parameters

   !! * Public variables
   INTEGER               :: idimt ! Number of sample in the wave file
   INTEGER               :: iwavemin,iwavemax,jwavemin,jwavemax ! Extrem spatial size of wave array
   INTEGER               :: ilec_wave ! For manage time sample
   INTEGER               :: ilecmem_wave ! For manage time sample
   INTEGER               :: iwmin0  ! shift to read the correct index in the netcdf file


   CHARACTER(LEN=lchain) :: file_wave ! Location of the NetCDF file
   REAL(KIND=rsh)        :: valfill_wave ! Fill value (ground, the same for each variable of waves file)
   INTEGER               :: cdf_id_wave
   REAL(KIND=rlg)        :: tnc1_wave,tnc2_wave

   !! For locate MARS grid from WAVE grid
   !!   Related to center of the MARS cell
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1xe_wave,j1xe_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpxe_wave,ydpxe_wave
   !!   Related to zonal velocity (East face)
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1u_wave,j1u_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpu_wave,ydpu_wave
   !!   Related to meridional velocity (North face)
   INTEGER,       DIMENSION(:,:),ALLOCATABLE :: i1v_wave,j1v_wave
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: xdpv_wave,ydpv_wave

   !! * Waves quantities -neww-
   !!   - Significant height
   LOGICAL               :: l_wave_rhs
   CHARACTER(LEN=lchain) :: name_wave_hs
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_hs1,wave_hs2,wave_hs
   !!   - Bottom orbital velocity
   LOGICAL               :: l_wave_rubot
   CHARACTER(LEN=lchain) :: name_wave_ubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_ubot1,wave_ubot2,wave_ubot
   LOGICAL               :: l_wave_ruubot
   CHARACTER(LEN=lchain) :: name_wave_uubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_uubot1,wave_uubot2,wave_uubot
   LOGICAL               :: l_wave_rvubot
   CHARACTER(LEN=lchain) :: name_wave_vubot
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_vubot1,wave_vubot2,wave_vubot
   !!  - Energie flux from waves to ocean
   LOGICAL               :: l_wave_rfoc
   CHARACTER(LEN=lchain) :: name_wave_foc
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_foc1,wave_foc2
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: wave_foc
   !!  - Directionality
   LOGICAL               :: l_wave_rdir
   CHARACTER(LEN=lchain) :: name_wave_dir
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_dir1,wave_dir2
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: wave_dir
   !!  - Wave peak period
   LOGICAL               :: l_wave_rtp
   CHARACTER(LEN=lchain) :: name_wave_tp
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE :: wave_tp1,wave_tp2,wave_tp
   !!  - If data have been interpolated on the MARS grid in advance
   LOGICAL               :: l_wave_marsgrid
   ! 3D dynamics
#if key_wave_qeuler_dynamics
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE :: hrms
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE :: THETA
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: USX,USY,USZ
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: USX_u
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: USY_v
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: dsiguz_u
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: dsigvz_v
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: dsigx_u,dsigy_v
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: pk_u,pk_v
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: taustr
   REAL(KIND=rsh), DIMENSION(:,:,:),ALLOCATABLE  :: pk,peps
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE  :: wave_freq
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE  :: tausurf
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE :: IUSX_u,IUSY_v
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE :: wave_number
   REAL(KIND=rsh) :: c1eps
   REAL(KIND=rsh), parameter :: cflin=0.05_rsh
#endif
#ifdef key_wave_soulsby_bottom_stress
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE  :: nuz,nvz
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE  :: uorb
   REAL(KIND=rsh), DIMENSION(:,:), ALLOCATABLE  :: FWST
#endif

  CONTAINS

   !!======================================================================

  SUBROUTINE alloc_comwave

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE alloc_comwave  ***
   !&E
   !&E ** Purpose : Allocate array use wave
   !&E
   !&E ** Called by :
   !&E       ! module alloc (file alloc.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of alloc_cometdyn
   !&E       !  2013-08 (AC. Bennis) Introduction of arrays for 3D radiation stress tensor
   !&E
   !!----------------------------------------------------------------------
   !! * Executable part

   !! * Locate MARS cell from Wave cell
   ALLOCATE(i1v_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1v_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(i1u_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1u_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(i1xe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(j1xe_wave(limin:limax,ljmin:ljmax))

   ALLOCATE(xdpxe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(xdpu_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(xdpv_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpxe_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpu_wave(limin:limax,ljmin:ljmax))
   ALLOCATE(ydpv_wave(limin:limax,ljmin:ljmax))
   !! * Waves quantities -neww-
   !!   - directionality
   ALLOCATE(wave_dir1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_dir2(limin:limax,ljmin:ljmax))
   wave_dir1(:,:)=0.0_rsh ; wave_dir2(:,:)=0.0_rsh
   !!   - significant height
   ALLOCATE(wave_hs(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_hs1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_hs2(limin:limax,ljmin:ljmax))
   wave_hs1(:,:) =0.0_rsh ; wave_hs2(:,:) =0.0_rsh
   !!   - energy flux from waves to ocean
   ALLOCATE(wave_foc1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_foc2(limin:limax,ljmin:ljmax))
   wave_foc1(:,:)=0.0_rsh ; wave_foc2(:,:)=0.0_rsh
   !!   - orbital velocity
   ALLOCATE(wave_ubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_ubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_ubot(limin:limax,ljmin:ljmax))
   wave_ubot1(:,:)=0.0_rsh ; wave_ubot2(:,:)=0.0_rsh ; wave_ubot(:,:)=0.0_rsh
   ALLOCATE(wave_uubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_uubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_uubot(limin:limax,ljmin:ljmax))
   wave_uubot1(:,:)=0.0_rsh ; wave_uubot2(:,:)=0.0_rsh ; wave_uubot(:,:)=0.0_rsh
   ALLOCATE(wave_vubot1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_vubot2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_vubot(limin:limax,ljmin:ljmax))
   wave_vubot1(:,:)=0.0_rsh ; wave_vubot2(:,:)=0.0_rsh ; wave_vubot(:,:)=0.0_rsh
   !!   - wave period
   ALLOCATE(wave_tp1(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_tp2(limin:limax,ljmin:ljmax))
   ALLOCATE(wave_tp(limin:limax,ljmin:ljmax))
   wave_tp1(:,:)=0.0_rsh ; wave_tp2(:,:)=0.0_rsh ; wave_tp(:,:)=0.0_rsh


#ifdef key_wave_qeuler_dynamics
   ! 3D arrays
   ALLOCATE(USX(1:kmax,limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(USY(1:kmax,limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(USZ(1:kmax,limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(USX_u(1:kmax,liminm1:limax,ljmin:ljmax))
   ALLOCATE(USY_v(1:kmax,limin:limax,ljminm1:ljmax))
   ALLOCATE(pk_u(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(pk_v(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(pk(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(peps(1:kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(taustr(1:kmax,limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(dsiguz_u(kmax,liminm1:limax,ljmin:ljmaxp1))
   ALLOCATE(dsigvz_v(kmax,limin:limaxp1,ljminm1:ljmax))
   ALLOCATE(dsigx_u(kmax,liminm1:limax,ljmin:ljmaxp1))
   ALLOCATE(dsigy_v(kmax,limin:limaxp1,ljminm1:ljmax))
   USX(:,:,:)=0.0_rsh
   USY(:,:,:)=0.0_rsh
   USZ(:,:,:)=0.0_rsh
   USX_u(:,:,:)=0.0_rsh
   USY_v(:,:,:)=0.0_rsh
   pk_u(:,:,:)=0.0_rsh
   pk_v(:,:,:)=0.0_rsh
   pk(:,:,:)=0.0_rsh
   peps(:,:,:)=0.0_rsh
   taustr(:,:,:)=0.0_rsh
   dsiguz_u(:,:,:)=0.0_rsh
   dsigvz_v(:,:,:)=0.0_rsh
   dsigx_u(:,:,:)=0.0_rsh
   dsigy_v(:,:,:)=0.0_rsh

   ! 2D arrays
   ALLOCATE(hrms(limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(THETA(limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(wave_freq(limin:limax,ljmin:ljmax))
   ALLOCATE(tausurf(limin:limax,ljmin:ljmax))
   ALLOCATE(IUSX_u(liminm1:limax,ljmin:ljmax))
   ALLOCATE(IUSY_v(limin:limax,ljminm1:ljmax))
   ALLOCATE(wave_number(limin:limax,ljmin:ljmax))
   hrms(:,:)=0.0_rsh
   THETA(:,:)=0.0_rsh
   wave_freq(:,:)=0.0_rsh
   tausurf(:,:)=0.0_rsh
   IUSX_u(:,:)=0.0_rsh
   IUSY_v(:,:)=0.0_rsh
   wave_number(:,:)=0.0_rsh

#ifdef key_wave_soulsby_bottom_stress
   ALLOCATE(uorb(limin:limaxp1,ljmin:ljmaxp1))    ! a mettre hors clef
   ALLOCATE(FWST(limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(nuz(liminm1:limax,ljmin:ljmax))
   ALLOCATE(nvz(limin:limax,ljminm1:ljmax))
   uorb(:,:)=0.0_rsh
   FWST(:,:)=0.0_rsh
   nuz(:,:)=0.0_rsh
   nvz(:,:)=0.0_rsh
#endif

#endif

  END SUBROUTINE alloc_comwave

   !!======================================================================

  SUBROUTINE dealloc_comwave

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dealloc_comwave  ***
   !&E
   !&E ** Purpose : Deallocate array use wave
   !&E
   !&E ** Called by :
   !&E       ! module dealloc (file dealloc.F90)
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Creation  on the basis of dealloc_cometdyn
   !&E       !  2013-08 (AC. Bennis) Introduction of arrays for 3D radiation stress tensor
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E---------------------------------------------------------------------
   !! * Executable part

   !! * Locate MARS cell from Wave cell
   DEALLOCATE(xdpxe_wave,ydpxe_wave,xdpu_wave,ydpu_wave,xdpv_wave,ydpv_wave)
   DEALLOCATE(i1v_wave,j1v_wave,i1u_wave,j1u_wave,i1xe_wave,j1xe_wave)

   !! * Waves quantities -neww-
   !!   - directionality
   DEALLOCATE(wave_dir1,wave_dir2)
   !!   - significant height
   DEALLOCATE(wave_hs1,wave_hs2,wave_hs)
   !!   - energy flux from waves to ocean
   DEALLOCATE(wave_foc1,wave_foc2)
   !!   - orbital velocity
   DEALLOCATE(wave_ubot1,wave_ubot2,wave_ubot)
   DEALLOCATE(wave_uubot1,wave_uubot2,wave_uubot)
   DEALLOCATE(wave_vubot1,wave_vubot2,wave_vubot)
   !!   - wave period
   DEALLOCATE(wave_tp1,wave_tp2,wave_tp)

   !!   - radiative stress
#ifdef key_wave_soulsby_bottom_stress
   DEALLOCATE(uorb)
   DEALLOCATE(FWST)
   DEALLOCATE(nuz,nvz)
#endif
#ifdef key_wave_qeuler_dynamics
   DEALLOCATE(USY_v)
   DEALLOCATE(dsiguz_u,dsigvz_v)
   DEALLOCATE(dsigx_u,dsigy_v)
   DEALLOCATE(pk_u,pk_v)
   DEALLOCATE(pk,peps,taustr)
   DEALLOCATE(wave_freq)
   DEALLOCATE(hrms,THETA)
   DEALLOCATE(USX,USY,USZ)
   DEALLOCATE(wave_number)
   DEALLOCATE(IUSX_u,IUSY_v)
#endif

  END SUBROUTINE dealloc_comwave

   !!======================================================================

#endif

  END MODULE comwave
