 MODULE modcasinit

   !!======================================================================
   !!                   ***  MODULE  modcasinit  ***
   !! Casinit :  Initialization of the test cases
   !!            cas_init : call the correct test case
   !!            cas_init_* : global initialisation of each component
   !!
   !!   History :
   !!        !  2012-07 (V. Garnier) Reorganisation of the implementation of the test cases
   !!
   !!======================================================================

   !! * Modules used
   USE parameters, ONLY       : rsh

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC cas_init_special ! routine called by casacc test case

#ifdef key_casacc
   !! * Shared module variables
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:),  PUBLIC  :: ssh0,u0,v0
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:),PUBLIC  :: temp0,uz0,vz0
#endif

 CONTAINS

   !!======================================================================

  SUBROUTINE cas_init_special
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_special  ***
   !&E
   !&E ** Purpose : initialize special variables related to some test cases
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Used modules
   USE parameters, ONLY : kmax,liminm1,limaxp2,ljminm1,ljmaxp2, &
                          liminm2,ljminm2

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ! allocation of variables without any perturbation
   ! (used as a reference by the nudging)
#ifdef key_casacc
   ALLOCATE( temp0(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2) )
   ALLOCATE(   uz0(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(   vz0(1:kmax,liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(    u0(       liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(    v0(       liminm2:limaxp2,ljminm2:ljmaxp2) )
   ALLOCATE(  ssh0(       liminm2:limaxp2,ljminm2:ljmaxp2) )
   temp0(:,:,:)=0.0_rsh
   uz0(:,:,:)=0.0_rsh
   vz0(:,:,:)=0.0_rsh
   ssh0(:,:)=0.0_rsh
   u0(:,:)=0.0_rsh
   v0(:,:)=0.0_rsh
#endif

  END SUBROUTINE cas_init_special

   !!======================================================================

  END MODULE modcasinit
