 MODULE parameters

   !!======================================================================
   !!                   ***  MODULE parameters_rank0  ***
   !! 
   !! ** Description : parameters for a rank 0 configuration
   !! 
   !!
   !! ** Reference :
   !!                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !!                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !!
   !! ** History :
   !!       !  2008-07 (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !!       !  2008-09 (G. Galibert / ALTRAN) attributes of output files
   !!                   to respect NetCDF OCO v1.1 file format
   !!                   modification of rg_valmanq_io and addition of dg_valmanq_io
   !!
   !!======================================================================
   !! * Modules used

   !! * Shared or public module variables

   INTEGER,PARAMETER :: riosh=4,riolg=8,rlg=8,rsh=4,lchain=200
   CHARACTER(len=4),PARAMETER :: nrac='RAZB'
   LOGICAL,PARAMETER :: l_testcase=.FALSE.
   INTEGER,PARAMETER :: num_testcase=0

   INTEGER,PARAMETER :: no=0 ! rank
   INTEGER           &
#if ! defined key_agrif
          ,PARAMETER &
#endif
                     :: imin=0,imax=649,jmin=0,jmax=475 ! configuration rang_0
! obc_width=number of lines (or columns) in your obc files - 1
   INTEGER,PARAMETER :: kmax=20,obc_width=2
   ! shift of obc for non-linear advection terms
   INTEGER          &
#if ! defined key_agrif
         ,PARAMETER &
#endif
           :: ishift_obc_west=0,ishift_obc_east=0,jshift_obc_south=0,jshift_obc_north=0

#ifdef key_tide_saverotatedobc
   INTEGER,PARAMETER :: imin_child=0,jmin_child=0,imax_child=131,jmax_child=100
#endif

#ifdef key_substance
#if ! defined key_biolo
! choix du nombre de variables a simuler si pas key_biolo ni key_castestsub ou castestconta : 
   INTEGER,PARAMETER :: nb_var_base=1
#else
#ifdef key_biolo_opt2
! option 2 : Marc : 4 variables supplementaires (N2, et N,P,Si detritique dissous)
   INTEGER,PARAMETER :: nb_var_base=21
#else
! option 1 par defaut : Alain, Philippe 
   INTEGER,PARAMETER :: nb_var_base=17
#endif
#ifdef key_psnz
   INTEGER,PARAMETER :: nb_var_psnz=4
#else
   INTEGER,PARAMETER :: nb_var_psnz=0
#endif
#ifdef key_karenia
   INTEGER,PARAMETER :: nb_var_karenia=3
#else
   INTEGER,PARAMETER :: nb_var_karenia=0
#endif 
#ifdef key_phaeocystis
   INTEGER,PARAMETER :: nb_var_phaeocystis=4
#else
   INTEGER,PARAMETER :: nb_var_phaeocystis=0
#endif 
#ifdef key_zoo_prod
   INTEGER,PARAMETER :: nb_var_zoo_ps=2
#else
   INTEGER,PARAMETER :: nb_var_zoo_ps=0
#endif
#ifdef key_ulvas
   INTEGER,PARAMETER :: nb_var_ulv=8
#else
   INTEGER,PARAMETER :: nb_var_ulv=0
#endif
#ifdef key_oyster
   INTEGER,PARAMETER :: nb_var_oys=3
#else
   INTEGER,PARAMETER :: nb_var_oys=0
#endif
#ifdef key_oxygen
   INTEGER,PARAMETER :: nb_var_oxy=1
#else
   INTEGER,PARAMETER :: nb_var_oxy=0
#endif
#ifdef key_microtracers
   INTEGER,PARAMETER :: nb_var_microtracers=6
#else
   INTEGER,PARAMETER :: nb_var_microtracers=0
#endif 
#ifdef key_benthos
   INTEGER,PARAMETER :: nb_var_benth_det=3
#ifdef key_diatbenth
   INTEGER,PARAMETER :: nb_var_benth_plus=1
#elif key_zostera
   INTEGER,PARAMETER :: nb_var_benth_plus=3
#else
   INTEGER,PARAMETER :: nb_var_benth_plus=0
#endif
#else
   INTEGER,PARAMETER :: nb_var_benth_det=0
   INTEGER,PARAMETER :: nb_var_benth_plus=0
#endif
#ifdef key_suspensivores
   INTEGER,PARAMETER :: nb_var_benth_susp=1
#else
   INTEGER,PARAMETER :: nb_var_benth_susp=0
#endif
#ifdef key_benthos_gener
   INTEGER,PARAMETER :: nb_var_benth_gener=4
#else
   INTEGER,PARAMETER :: nb_var_benth_gener=0
#endif
#ifdef key_larve
  INTEGER,PARAMETER :: nb_var_larve=6
#else
   INTEGER,PARAMETER :: nb_var_larve=0
#endif
#ifdef key_N_tracer
   INTEGER,PARAMETER :: nb_sourceN=1
   INTEGER,PARAMETER :: nb_var_tracerN_defaut=8
!  modele de base=8, ajouter 1 pour Karenia ou pour Pseudo-Nitzschia   
#ifdef key_psnz
   INTEGER,PARAMETER :: nb_var_tracerN_psnz=1
#else
   INTEGER,PARAMETER :: nb_var_tracerN_psnz=0
#endif
#ifdef key_karenia
   INTEGER,PARAMETER :: nb_var_tracerN_kar=1
#else
   INTEGER,PARAMETER :: nb_var_tracerN_kar=0
#endif 
#ifdef key_phaeocystis
   INTEGER,PARAMETER :: nb_var_tracerN_phae=2
#else
   INTEGER,PARAMETER :: nb_var_tracerN_phae=0
#endif 
#ifdef key_ulvas
   INTEGER,PARAMETER :: nb_var_tracerN_ulv=2
#else
   INTEGER,PARAMETER :: nb_var_tracerN_ulv=0
#endif
#ifdef key_benthos
   INTEGER,PARAMETER :: nb_var_tracerN_bent=1
#else
   INTEGER,PARAMETER :: nb_var_tracerN_bent=0
#endif
   INTEGER,PARAMETER :: nb_var_tracerN= nb_var_tracerN_defaut+nb_var_tracerN_bent+nb_var_tracerN_ulv+nb_var_tracerN_phae+nb_var_tracerN_kar+nb_var_tracerN_psnz
#ifdef key_age_tracer
   INTEGER,PARAMETER :: nb_var_N_age_tracer=nb_var_tracerN
#else    
   INTEGER,PARAMETER :: nb_var_N_age_tracer=0
#endif 
#else    
   INTEGER,PARAMETER :: nb_sourceN=0
   INTEGER,PARAMETER :: nb_var_tracerN=0
   INTEGER,PARAMETER :: nb_var_N_age_tracer=0
#endif
#ifdef key_P_tracer
   INTEGER,PARAMETER :: nb_sourceP=2
   INTEGER,PARAMETER :: nb_var_tracerP_defaut=8
!  modele de base=8, ajouter 1 pour Karenia ou pour Pseudo-Nitzschia   
#ifdef key_psnz
   INTEGER,PARAMETER :: nb_var_tracerP_psnz=1
#else
   INTEGER,PARAMETER :: nb_var_tracerP_psnz=0
#endif
#ifdef key_karenia
   INTEGER,PARAMETER :: nb_var_tracerP_kar=1
#else
   INTEGER,PARAMETER :: nb_var_tracerP_kar=0
#endif 
#ifdef key_phaeocystis
   INTEGER,PARAMETER :: nb_var_tracerP_phae=2
#else
   INTEGER,PARAMETER :: nb_var_tracerP_phae=0
#endif 
#ifdef key_benthos
   INTEGER,PARAMETER :: nb_var_tracerP_bent=1
#else
   INTEGER,PARAMETER :: nb_var_tracerP_bent=0
#endif
   INTEGER,PARAMETER :: nb_var_tracerP= nb_var_tracerP_defaut+nb_var_tracerP_bent+nb_var_tracerP_phae+nb_var_tracerP_kar+nb_var_tracerP_psnz
#ifdef key_age_tracer
   INTEGER,PARAMETER :: nb_var_P_age_tracer=nb_var_tracerP
#else    
   INTEGER,PARAMETER :: nb_var_P_age_tracer=0
#endif
#else    
   INTEGER,PARAMETER :: nb_sourceP=0
   INTEGER,PARAMETER :: nb_var_tracerP=0
   INTEGER,PARAMETER :: nb_var_P_age_tracer=0
#endif
#ifdef key_zostera
   INTEGER,PARAMETER :: nb_var_zost=13
#else
   INTEGER,PARAMETER :: nb_var_zost=0
#endif

!!  fin key_biolo
#endif

#if defined key_castestsub
   INTEGER,PARAMETER :: nb_var_castest=4
#elif key_castestconta
   INTEGER,PARAMETER :: nb_var_castest=8
#else
   INTEGER,PARAMETER :: nb_var_castest=nb_var_base
#endif

#ifdef key_contaminant
   INTEGER,PARAMETER :: ntypreac=7         !! nb types de reactions deja programmeess
!!  4 parameters for preliminary and temporary arrays of contaminants
   INTEGER,PARAMETER  :: nfracvmax=10       !! nb max fractions possibles regroupees dans une variable
   INTEGER,PARAMETER  :: nflimcomax=10      !! nb max limiting functions for generic reaction of contaminant
   INTEGER,PARAMETER  :: nrecomax=20        !! nb max reaction affected contaminant
   INTEGER,PARAMETER  :: nvactcomax=10      !! nb max active variables for contaminant
#endif
#ifdef key_sedim
   INTEGER,PARAMETER :: ksdmin=1,ksdmax=10
#endif
#endif
   INTEGER,PARAMETER :: nbpartmax=1,nbtachmax=1
   INTEGER,PARAMETER :: ngridmax=10,nmarmax=143,nbchprox=1

   ! global variables for characteristics method (obc)
   ! coefficient used when correcting matrixes
   REAL(KIND=rsh),PARAMETER :: al1=1.0_rsh,al2=1.0_rsh,al3=-1.0_rsh,ac0=1.0_rsh-al1/4.0_rsh, &
                               ac1=al2+al3/2.0_rsh,ac2=al2/4.0_rsh+al3/2.0_rsh

   ! implicit and explicit coefficients
   REAL(kind=rsh)  ,PARAMETER :: cfroe=0.5_rsh,cfroi=1.0_rsh-cfroe
   ! alphap (p for previous) must be lower than 0.5
   ! alphan (n for next)
   REAL(kind=rsh)  ,PARAMETER :: alphap=0.45_rsh,alphan=1.0_rsh-alphap
   REAL(kind=rsh)  ,PARAMETER :: valmanq=999.0_rsh
   REAL(kind=riosh),PARAMETER :: rg_valmanq_io=999.0_riosh
   REAL(kind=riolg),PARAMETER :: dg_valmanq_io=-1.7e+38
   REAL(kind=rlg)  ,PARAMETER :: pi=3.141592654_rlg,tref=59958230400_rlg,epsilon=0.00000001_rlg
   REAL(KIND=rsh)  ,PARAMETER :: grav=9.81_rsh
   REAL(KIND=rlg)  ,PARAMETER :: exp1=2.718281828_rlg

#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rlg)  ,PARAMETER :: smooth_coeff=1.75_rsh
   REAL(KIND=rlg)  ,PARAMETER :: aws=1.0_rsh, bws=0.45_rsh, cws=0.4_rsh, dws=0.35_rsh, ews=0.3_rsh
   INTEGER,PARAMETER :: smooth_method=1!neighborAveraging=1; inverseHelmholtzOperator=2
   INTEGER,PARAMETER :: filter_width=9
   INTEGER,PARAMETER :: iumin = imin+2, iumax=imax-2, jumin=jmin+3, jumax=jmax-2, &
                        idu  = iumax-iumin-1, jdu  = jumax-jumin-1, ijdu = idu*jdu
   INTEGER,PARAMETER :: ivmin = imin+3, ivmax=imax-2, jvmin=jmin+2, jvmax=jmax-2, &
                        idv  = ivmax-ivmin-1, jdv  = jvmax-jvmin-1, ijdv = idv*jdv
#endif

#ifdef key_agrif
   INTEGER :: imaxm2=imax-2
   INTEGER :: jmaxm2=jmax-2
   INTEGER :: lax=imax+jmax
   INTEGER,PARAMETER :: nb_connect=3 ! nb of mesh similar between mother and child grids
#endif

   INTEGER :: ljmin=jmin,ljminp1=jmin+1+jshift_obc_south,ljminp2=jmin+2+jshift_obc_south, &
              ljminm1=jmin,ljminm2=jmin,ljminp1m2=jmin+1+jshift_obc_south,ljminp1m1=jmin+1+jshift_obc_south
   INTEGER :: ljmax=jmax,ljmaxm1=jmax-1-jshift_obc_north,ljmaxm1p1=jmax-1-jshift_obc_north, &
              ljmaxp1=jmax,ljmaxp2=jmax,ljmaxp3=jmax
   INTEGER :: limin=imin,liminp1=imin+1+ishift_obc_west,liminp2=imin+2+ishift_obc_west, &
              liminm1=imin,liminm2=imin,liminp1m2=imin+1+ishift_obc_west,liminp1m1=imin+1+ishift_obc_west
   INTEGER :: limax=imax,limaxm1=imax-1-ishift_obc_east,limaxm1p1=imax-1-ishift_obc_east, &
              limaxp1=imax,limaxp2=imax,limaxp3=imax

#ifdef key_substance
#ifdef key_biolo
INTEGER, PARAMETER :: nb_var=nb_var_base+nb_var_psnz+nb_var_karenia+nb_var_phaeocystis+nb_var_ulv+nb_var_oys+nb_var_oxy+nb_var_zoo_ps + &
                      nb_var_microtracers+nb_var_tracerN*nb_sourceN+nb_var_N_age_tracer*nb_sourceN+nb_var_tracerP*nb_sourceP+ &
                      nb_var_P_age_tracer*nb_sourceP+nb_var_benth_det+nb_var_benth_plus+nb_var_benth_susp+nb_var_benth_gener + &
                      nb_var_larve+nb_var_zost
#else
   INTEGER,PARAMETER :: nb_var=nb_var_castest
#endif
#endif

#if defined key_lans_turb || defined key_leray_turb
   INTEGER :: ljminm3=jmin, liminm3=imin
   INTEGER :: ljminm4=jmin, liminm4=imin, ljmaxp4=jmax, limaxp4=imax
#endif

 END MODULE parameters
