   !!======================================================================

  SUBROUTINE agrif_obc_ssh(xeuv)
!
!********************************************************************
!
!     objet    : routine de calculs des conditions aux limites
!                en niveau dans le cas d un modele emboite, pour lequel
!                on dispose d un fichier de conditions aux limites etabli
!                par un grand modele le contenant. Cas des frontieres
!                rectiligne.
!
!
!     appelant         : init_xe, stepu, stepv, step2dxy, step2dyx
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : nbouc, nbouc2d, filecl, ieac, iwac
!     variables in     : jnac, jsac, dgac, dfiac, dgp, dfip
!     variables in     : t, it
!                      :
!     variables out    : xe, ul, vl
!                      :
!     variables inout  : xlim, ylim
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas (test sur profondeur 2m au lieu de 2*h0fond)
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrifcommonid
   USE obccombine, ONLY : l_obc_west,l_obc_east,l_obc_north,l_obc_south
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   IMPLICIT NONE
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( inout ) :: xeuv
!
   INTEGER              :: nb
   REAL(kind=rlg)       :: TimeIndex
   INTEGER :: i,j
   INTEGER,pointer :: Nbouc_Parent
   External :: interpssh

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter AGRIF_OBC_SSH routine',Agrif_Root()

!     Initialisation
!************************************************************
!
   IF (.not.Agrif_Root()) THEN
     Agrif_SpecialValue=valmanq
     Agrif_USESpecialValue = .TRUE.
     c_agrif_interp='ssh_bc'
     nesting_tabtemp_2d(:,:,1) = xeuv
     CALL Agrif_Bc_Variable(ssh_interp_id,procname=interpssh)
     xeuv = nesting_tabtemp_2d(:,:,1)
     Agrif_SpecialValue=valmanq
     Agrif_USESpecialValue = .FALSE.
   END IF

   IF ((Agrif_Grandmother())) THEN

   ! western boundary
     IF_MPI (limin==imin) THEN
       IF ( l_obc_west ) THEN
         WHERE (h0(imin+1,ljminm2:ljmaxp2)/=-valmanq)
         xeuv(imin+1,ljminm2:ljmaxp2) = xeuv(imin+2,ljminm2:ljmaxp2)
       ELSE WHERE
         xeuv(imin+1,ljminm2:ljmaxp2) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

! eastern boundary
     IF_MPI (limax==imax) THEN
       IF ( l_obc_east ) THEN
         WHERE (h0(imax,ljminm2:ljmaxp2)/=-valmanq)
         xeuv(imax,ljminm2:ljmaxp2) = xeuv(imax-1,ljminm2:ljmaxp2)
       ELSE WHERE
         xeuv(imax,ljminm2:ljmaxp2) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

! southern boundary
     IF_MPI (ljmin==jmin) THEN
       IF ( l_obc_south ) THEN
         WHERE (h0(liminm2:limaxp2,jmin+1)/=-valmanq)
         xeuv(liminm2:limaxp2,jmin+1) = xeuv(liminm2:limaxp2,jmin+2)
       ELSE WHERE
         xeuv(liminm2:limaxp2,jmin+1) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

! northern boundary
     IF_MPI (ljmax==jmax) THEN
       IF ( l_obc_north ) THEN
         WHERE (h0(liminm2:limaxp2,jmax)/=-valmanq)
         xeuv(liminm2:limaxp2,jmax) = xeuv(liminm2:limaxp2,jmax-1)
       ELSE WHERE
         xeuv(liminm2:limaxp2,jmax) = valmanq
         END WHERE
       END IF
     ENDIF_MPI
   END IF

#ifdef key_quefairela
! d apres config BRES mieux sans mettre ces echanges (ce que vous aviez fait)
! cval echanges ajoutes a la revision 1268 (extension spatiale de ssh)
! cval la routine Agrif_User.F90 a ete adaptee au meme moment
! cval sauf que Franck me dit qu il y avait peut-etre une tres bonne raison pour
! ne pas faire les echanges. On les fait pour les vitesses.
! cval Aie que faut-il faire ici ?
! cval il est vrai que sur la config BRES, le pas de temps augmentait moins vite
! cval apres cette "correction", donc code moins stable ? Revenir en arriere ?
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(ssh)
#endif

#else

   USE parameters, ONLY : rsh,liminm2,limaxp2,ljminm2,ljmaxp2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT( in ) :: xeuv

#endif
   PRINT_DBG*, 'exit AGRIF_OBC_SSH routine'

  END SUBROUTINE agrif_obc_ssh

   !!======================================================================

#if defined key_agrif

  SUBROUTINE interpssh(tabres,i1,i2,j1,j2,k1,k2,before)
   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrif_types
   USE agrifcommonid
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   LOGICAL :: before
   REAL(kind=rsh) :: tabres(i1:i2,j1:j2,k1:k2)
   INTEGER :: i,j

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERPSSH routine',Agrif_Root()

   IF (before) THEN
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)+ssh(i,j) <= h0fond) THEN
! IF (h0(i,j)==-valmanq) THEN
           tabres(i,j,1) = Agrif_SpecialValue
           tabres(i,j,2) = 0.
         ELSE
           tabres(i,j,1) = ssh(i,j)
           tabres(i,j,2) = 1.
         END IF
       END DO
     END DO
   ELSE
     SELECT CASE(c_agrif_interp)
       CASE ('ssh_init')
         DO j=j1,j2
           DO i=i1,i2
!  IF (h0(i,j)+tabres(i,j) >= h0fond) THEN
             IF (h0(i,j) == -valmanq) THEN
               nesting_tabtemp_2d(i,j,1) = valmanq
             ELSE IF (h0(i,j)+tabres(i,j,1) < h0fond) THEN
               nesting_tabtemp_2d(i,j,1) = -h0(i,j)
             ELSE
               nesting_tabtemp_2d(i,j,1) = tabres(i,j,1)
             END IF
           END DO
         END DO
       CASE('ssh_bc')
         DO j=j1,j2
           DO i=i1,i2
             IF (i==imin+1.or.i==imax.or.j==jmax.or.j==jmin+1) THEN
               IF (h0(i,j) == -valmanq) THEN
                 nesting_tabtemp_2d(i,j,1) = valmanq
               ELSE IF (h0(i,j)+tabres(i,j,1) < h0fond) THEN
                 nesting_tabtemp_2d(i,j,1) = -h0(i,j)
               ELSE
                 nesting_tabtemp_2d(i,j,1) = tabres(i,j,1)
               END IF
             END IF
           END DO
         END DO
     END SELECT
   END IF

   PRINT_DBG*, 'exit INTERPSSH routine',Agrif_Root()

   RETURN

   END SUBROUTINE interpssh

   !!======================================================================

  SUBROUTINE updateh0(tabres,i1,i2,j1,j2,before)

   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrif_types
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2
   LOGICAL :: before
   REAL(kind=rsh) :: tabres(i1:i2,j1:j2)
   INTEGER :: i,j

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter UPDATEH0 routine',Agrif_Root()

   IF (before) THEN

     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)==-valmanq) THEN
           tabres(i,j) = Agrif_SpecialValueFineGrid
         ELSE
           tabres(i,j) = h0(i,j)
         END IF
       END DO
     END DO

   ELSE

     DO j=j1,j2
       DO i=i1,i2

!    IF (tabres(i,j) == Agrif_SpecialValueFineGrid) THEN
!      h0(i,j) = -valmanq
!    ELSE
!      h0(i,j) = tabres(i,j)
!    ENDIF


         IF ((h0(i,j) /= -valmanq).AND.(tabres(i,j) /= Agrif_SpecialValueFineGrid)) THEN
           h0(i,j) = tabres(i,j)
         END IF

       END DO
     END DO

   END IF

   PRINT_DBG*, 'exit UPDATEH0 routine',Agrif_Root()

   RETURN

  END SUBROUTINE updateh0

   !!======================================================================

  SUBROUTINE interph0(h0_out,tabres,i1,i2,j1,j2,k1,k2,before,nb,ndir)

   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrif_types
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   LOGICAL :: before
   REAL(kind=rsh) :: tabres(i1:i2,j1:j2,k1:k2)
   INTEGER :: i,j
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2,1:2) :: h0_out
   INTEGER :: nb,ndir
   LOGICAL :: western_side, eastern_side,southern_side,northern_side
   REAL(kind=rsh),DIMENSION(i1:i2,j1:j2) :: coeff
   INTEGER :: i3,j3,dist

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERPH0 routine',Agrif_Root()

   IF (before) THEN

     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)==-valmanq) THEN
           tabres(i,j,1) = Agrif_SpecialValue
           tabres(i,j,2) = 0.
         ELSE
           tabres(i,j,1) = h0(i,j)
           tabres(i,j,2) = 1.
         END IF
       END DO
     END DO

   ELSE

     western_side   = (nb == 1).AND.(ndir == 1)
     eastern_side   = (nb == 1).AND.(ndir == 2)
     southern_side  = (nb == 2).AND.(ndir == 1)
     northern_side  = (nb == 2).AND.(ndir == 2)

     coeff = 0.

     DO j=j1,j2
       DO i=i1,i2
         dist = 2*nb_connect*Agrif_Irhox()
         dist = min(dist,i-(imin+1))
         dist = min(dist,imax-i)
         dist = min(dist,j-(jmin+1))
         dist = min(dist,jmax-j)

         IF (dist <= nb_connect*Agrif_Irhox()) THEN
           coeff(i,j) = 1.
         ELSE
           coeff(i,j) = 1.-REAL(dist-nb_connect*Agrif_Irhox())/real(nb_connect*Agrif_Irhox())
         END IF

       END DO
     END DO

     DO j=j1,j2
       DO i=i1,i2
         IF (i/=imin.and.j/=jmin) THEN
           IF (tabres(i,j,2) <= 0.5) THEN
             h0_out(i,j,1) = -valmanq
           ELSE
             IF (h0(i,j) == -valmanq) THEN
               h0_out(i,j,1) = tabres(i,j,1)
             ELSE
               h0_out(i,j,1) = coeff(i,j)*tabres(i,j,1)+(1.-coeff(i,j))*h0(i,j)
             END IF
           END IF
         ELSE
           h0_out(i,j,1) = -valmanq
         END IF
       END DO
     END DO
     DO j=j1,j2
       DO i=i1,i2
         h0_g(i,j)=h0_out(i,j,1)
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit INTERPH0 routine',Agrif_Root()

   RETURN

   END SUBROUTINE interph0
#endif
