   !!======================================================================

  SUBROUTINE agrif_obc_u(uv)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de u.
!
!
!
!     appelant         : stepu, step2dxy
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : jh, jb, ig, id, hx, valmanq, h0fond
!                      :
!     variables out    :
!                      :
!     variables inout  : u
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
   USE agrif_types
   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrifcommonid
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

   IMPLICIT NONE
!
   INTEGER :: i,j
   REAL(KIND=rsh) :: htxtemp, bid0
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: uv


   external interpu_stepu

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter AGRIF_OBC_U routine',Agrif_Root()

! Initialization

   agrif_SpecialValue=0.
   c_agrif_interp='u_bc'

   nesting_tabtemp_2d(:,:,1) = uv
   CALL Agrif_Bc_variable(u_id,procname=interpu_stepu)
   uv = nesting_tabtemp_2d(:,:,1)

!  IF_MPI(limin==imin) &
!  uv(imin,ljminm2:ljmaxp2)  =utemp(imin,ljminm2:ljmaxp2)
!  IF_MPI(limax==imax) &
!  uv(imax,ljminm2:ljmaxp2)  =utemp(imax,ljminm2:ljmaxp2)
!  IF_MPI(ljmax==jmax) &
!  uv(liminm2:limaxp2,jmax)  =utemp(liminm2:limaxp2,jmax)
!  IF_MPI(ljmin==jmin) &
!  uv(liminm2:limaxp2,jmin+1)=utemp(liminm2:limaxp2,jmin+1)
   agrif_SpecialValue=valmanq

   IF (Agrif_Grandmother()) THEN

     DO j=ljminp2,ljmaxm1

   ! Western boundary

       i=ig(j)-1
       IF_MPI (limin==imin .AND. limin==i) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uv(i,j)=uv(i+1,j)
       ENDIF_MPI

   ! Eastern boundary

       i=id(j)
       IF_MPI (limax==imax .AND. limax==i) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uv(i,j)=uv(i-1,j)
       ENDIF_MPI

     END DO


     DO i=liminp1,limaxm1

   ! Southern open boundary

       j=jb(i)
       IF_MPI (ljmin==jmin .AND. ljmin==j-1) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uv(i,j)=uv(i,j+1)
       ENDIF_MPI

   ! Northern open boundary

       j=jh(i)
       IF_MPI (ljmax==jmax .AND. ljmax==j) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uv(i,j)=uv(i,j-1)
       ENDIF_MPI

     END DO

   END IF    ! end Grand Mother

OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull call ex_j_rsh(-2,2,1,liminm1,limax,ljminm2,ljmaxp2,uv(liminm1:limax,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,uv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(uv)

   PRINT_DBG*, 'exit AGRIF_OBC_U routine',Agrif_Root()

   RETURN

#else
   USE parameters, ONLY : rsh,liminm2,limaxp2,ljminm2,ljmaxp2
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: uv
#endif

  END SUBROUTINE agrif_obc_u

   !!======================================================================

#ifdef key_agrif

  SUBROUTINE interpu_stepu(tabres,i1,i2,j1,j2,k1,k2,before)
   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE comcolig2d, ONLY : igdu
   USE agrifcommonid
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(i1:i2,j1:j2,k1:k2)
   INTEGER i,j
   REAL htxtemp, hextemp
   REAL bid0
   LOGICAL :: before

   REAL hex_loc(i1:i2,j1:j2)
   INTEGER igdu_loc(i1:i2,j1:j2)
   REAL hzex_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERPU_STEPU routine',Agrif_Root()

   hex_loc(i1:i2,j1:j2) = 0.0

   IF (before) THEN     ! mother

     test = 1
!     CALL agrif_dyn3d_height_mask(ssh,igdu_loc,hex_loc, &
!#if defined key_siggen || defined key_gencoord
!                                  hzex_loc, &
!#endif
!                                  i1,i2,j1,j2,test)

     DO j=j1,j2
       DO i=i1,i2
         IF (igdu_loc(i,j) /= 4) THEN
!           tabres(i,j,1) = hex_loc(i,j)*u(i,j)
           tabres(i,j,2) = u(i,j)
         ELSE
           tabres(i,j,:) = 0.
         END IF
       END DO
     END DO

   ELSE                 ! child

     SELECT CASE(c_agrif_interp)

       CASE('u_init')

         DO j=MAX(j1,ljmin),MIN(j2,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             IF (hx(i,j) == -valmanq-h0fond) THEN
               nesting_tabtemp_2d(i,j,1) = 0.0_rsh
             ELSE
               nesting_tabtemp_2d(i,j,1) = tabres(i,j,2)
             END IF
           END DO
         END DO

       CASE('u_bc')

         DO j=MAX(j1,ljmin),MIN(j2,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             IF (hx(i,j) == -valmanq-h0fond) THEN
               IF (i==imin.or.i==imax.or.j==jmax.or.j==jmin+1) nesting_tabtemp_2d(i,j,1) = 0.0_rsh
             ELSE
               IF ((i==imin.or.i==imax.or.j==jmax.or.j==jmin+1)) THEN
                 nesting_tabtemp_2d(i,j,1) = tabres(i,j,2)
               END IF
             END IF
           END DO
         END DO

     END SELECT

   END IF

   PRINT_DBG*, 'exit INTERPU_STEPU routine',Agrif_Root()
   RETURN

  END SUBROUTINE interpu_stepu

   !!======================================================================

  SUBROUTINE updatehx(tabres,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE agrif_util
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2
   REAL tabres(i1:i2,j1:j2)
   INTEGER i,j
   REAL htxtemp
   REAL bid0
   LOGICAL before

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter UPDATEHX routine',Agrif_Root()

   IF (before) THEN

     DO j=j1,j2
       DO i=i1,i2
         IF (hx(i,j)==-valmanq-h0fond) THEN
           tabres(i,j) = Agrif_SpecialValueFineGrid
         ELSE
           tabres(i,j) = hx(i,j)
         END IF
       END DO
     END DO

   ELSE

     DO j=j1,j2
       DO i=i1,i2
!    IF (tabres(i,j) == Agrif_SpecialValueFineGrid) THEN
!      hx(i,j) = -valmanq -h0fond
!    ELSE
!      hx(i,j) = tabres(i,j)
!    ENDIF

         IF ((hx(i,j) /= -valmanq -h0fond).AND.(tabres(i,j) /= Agrif_SpecialValueFineGrid)) THEN
           hx(i,j) = tabres(i,j)
         END IF
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit UPDATEHX routine',Agrif_Root()

   RETURN

  END SUBROUTINE updatephx

   !!======================================================================

  SUBROUTINE interphx(hx_out,tabres,i1,i2,j1,j2,k1,k2,before,nb,ndir)

   USE parameters
   USE comvars2d
   USE comvarp2d
   USE agrif_types
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   LOGICAL :: before
   REAL(KIND=rsh) :: tabres(i1:i2,j1:j2,k1:k2)
   INTEGER :: i,j
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2,1:2) :: hx_out
   INTEGER :: nb,ndir
   LOGICAL :: western_side, eastern_side,southern_side,northern_side
   REAL(KIND=rsh),DIMENSION(i1:i2,j1:j2) :: coeff
   INTEGER :: i3,j3,dist

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERPHX routine',Agrif_Root()

   IF (before) THEN

     DO j=j1,j2
       DO i=i1,i2
         IF (hx(i,j)==-valmanq-h0fond) THEN
           tabres(i,j,1) = Agrif_SpecialValue
           tabres(i,j,2) = 0.
         ELSE
           tabres(i,j,1) = hx(i,j)
           tabres(i,j,2) = 1.
         END IF
       END DO
     END DO

   ELSE

     western_side   = (nb == 1).AND.(ndir == 1)
     eastern_side   = (nb == 1).AND.(ndir == 2)
     southern_side  = (nb == 2).AND.(ndir == 1)
     northern_side  = (nb == 2).AND.(ndir == 2)

     coeff = 0.

     DO j=j1,j2
       DO i=i1,i2
         dist = 2*nb_connect*Agrif_Irhox()
         dist = min(dist,i-(imin+1))
         dist = min(dist,imax-i)
         dist = min(dist,j-(jmin+1))
         dist = min(dist,jmax-j)

         IF (dist <= nb_connect*Agrif_Irhox()) THEN
           coeff(i,j) = 1.
         ELSE
           coeff(i,j) = 1.-REAL(dist-nb_connect*Agrif_Irhox())/real(nb_connect*Agrif_Irhox())
         END IF

       END DO
     END DO

     DO j=j1,j2
       DO i=i1,i2
         IF (i/=imin.and.j/=jmin) THEN
           IF (tabres(i,j,2) <= 0.5) THEN
             hx_out(i,j,1) = -valmanq-h0fond
           ELSE
             IF (hx(i,j) == -valmanq-h0fond) THEN
               hx_out(i,j,1) = tabres(i,j,1)
             ELSE
               hx_out(i,j,1) = coeff(i,j)*tabres(i,j,1)+(1.-coeff(i,j))*hx(i,j)
             END IF
!     if (hx_out(i,j,1) >= hrdef) THEN
!       hx_out(i,j,1) = 0.5*(h0(i,j)+h0(i+1,j))
!     endif
           END IF
         END IF
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit INTERPHX routine',Agrif_Root()

   RETURN

   END SUBROUTINE interphx
#endif
