   !!======================================================================

  SUBROUTINE agrif_obc_uz(uvz,uv,xeuv)
!
!********************************************************************
!
!     objet    :  conditions aux limites ouvertes en vitesse.
!                 On remplit le cadre de u.
!
!
!
!     appelant         : dyn3dx, stepu
!                      :
!     appeles          :
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          :
!                      :
!     variables in     : jh, jb, ig, id, valmanq, h0fond, hx
!                      :
!     variables out    :
!                      :
!     variables inout  : uz
!                      :
!     dernieres modifs : 13/10/2005
!                      : 12/12/2007 F. Dumas
   !&E       !  2009-05  (F. Dumas)  AGRIF pour nouveau schema temporel
!                      :
!***********************************************************

#include "toolcpp.h"

#ifdef key_agrif
   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE agrifcommonid
!   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

   IMPLICIT NONE
!
   INTEGER        :: i,j,k
   REAL(KIND=rsh) :: htxtemp, bid0
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: uvz
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uv
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: xeuv
   external interpuz_stepu
!
!************************************************************
!     Executable statements
!************************************************************
!
   PRINT_DBG*, 'enter AGRIF_OBC_UZ routine',Agrif_Root()

! Initialization

   agrif_SpecialValue=0.
   c_agrif_interp='uz_bc'

   nesting_tabtemp_3d = uvz
   nesting_tabtemp_2d(:,:,1) = uv
   nesting_tabtemp_2d(:,:,2) = xeuv
   CALL Agrif_Bc_variable(uz_id,procname=interpuz_stepu)
   uvz = nesting_tabtemp_3d

!   IF_MPI(limin==imin) &
!   uvz(:,imin,ljminm2:ljmaxp2)  =uztemp(:,imin,ljminm2:ljmaxp2)
!   IF_MPI(limax==imax) &
!   uvz(:,imax,ljminm2:ljmaxp2)  =uztemp(:,imax,ljminm2:ljmaxp2)
!   IF_MPI(ljmax==jmax) &
!   uvz(:,liminm2:limaxp2,jmax)  =uztemp(:,liminm2:limaxp2,jmax)
!   IF_MPI(ljmin==jmin) &
!   uvz(:,liminm2:limaxp2,jmin+1)=uztemp(:,liminm2:limaxp2,jmin+1)
   agrif_SpecialValue=valmanq

   IF (Agrif_Grandmother()) THEN

     DO j=ljminp2,ljmaxm1

   ! Western boundary

       i=ig(j)-1
       IF_MPI (limin==imin .AND. limin==i) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uvz(:,i,j)=uvz(:,i+1,j)
       ENDIF_MPI

   ! Eastern boundary

       i=id(j)
       IF_MPI (limax==imax .AND. limax==i) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uvz(:,i,j)=uvz(:,i-1,j)
       ENDIF_MPI

     END DO


     DO i=liminp1,limaxm1

   ! Southern open boundary

       j=jb(i)
       IF_MPI (ljmin==jmin .AND. ljmin==j-1) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uvz(:,i,j)=uvz(:,i,j+1)
       ENDIF_MPI

   ! Northern open boundary

       j=jh(i)
       IF_MPI (ljmax==jmax .AND. ljmax==j) THEN
         IF (hx(i,j) /= -valmanq-h0fond) uvz(:,i,j)=uvz(:,i,j-1)
       ENDIF_MPI

     END DO

   END IF    ! end Grand Mother

!!!!!!!!!OMPMPINOAGRIF barrier
!!!!!!!!!OMPMPINOAGRIF master
!!!!!!!!!   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull CALL ex_j_rsh(-2,2,kmax,liminm1,limax,ljminm2,ljmaxp2,uvz(:,liminm1:limax,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
!!!!!!!!   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uvz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!!!!!!!!OMPMPINOAGRIF end master
!!!!!!!!OMPMPINOAGRIF barrier
!!!!!!!!OMPMPINOAGRIF flush(uvz)

   PRINT_DBG*, 'exit AGRIF_OBC_UZ routine',Agrif_Root()

   RETURN

#else
   USE parameters, ONLY : rsh,liminm2,limaxp2,ljminm2,ljmaxp2,kmax
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: uvz
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uv
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: xeuv
#endif
  END SUBROUTINE agrif_obc_uz

   !!======================================================================

#ifdef key_agrif

  SUBROUTINE interpuz_stepu(tabres,k1,k2,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE comcolig2d
   USE agrifcommonid
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,dcusds
   USE toolgeom,  ONLY : f_hzu_x
#endif
   USE comvars3d, ONLY : dsigu
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   REAL uvdiff(k1:k2,i1:i2,j1:j2)
   INTEGER i,j,k
   REAL htxtemp, hextemp
   REAL hzextemp(1:kmax)
   REAL bid0
   LOGICAL :: before
   REAL(KIND=rsh),DIMENSION(k1:k2,i1:i2,j1:j2) :: Fx, Fy
   REAL(KIND=rsh),DIMENSION(i1:i2,j1:j2) :: coeff
   REAL(KIND=rsh) :: visc_sponge
   REAL(KIND=rsh) :: umoy, umoy_a
   INTEGER :: dist
   REAL hex_loc(i1:i2,j1:j2)
   INTEGER igdu_loc(i1:i2,j1:j2)
   REAL hzex_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERPUZ_STEPU routine',Agrif_Root()

   IF (before) THEN     ! mother

     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           tabres(k,i,j) = uz(k,i,j)
         END DO
       END DO
     END DO

   ELSE                 ! child

     SELECT CASE(c_agrif_interp)

       CASE('uz_init')

         test = 1
         CALL agrif_dyn3d_height_mask(ssh,igdu_loc,hex_loc, &
#if defined key_siggen || defined key_gencoord
                                      hzex_loc, &
#endif
                                      i1,i2,j1,j2,test)

         DO j=MAX(j1,ljmin),MIN(j2,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             IF (igdu_loc(i,j) == 1) THEN
               nesting_tabtemp_3d(:,i,j) = tabres(1:kmax,i,j)
               umoy = SUM(nesting_tabtemp_3d(:,i,j)*hzex_loc(:,i,j)*dsigu(:))/hex_loc(i,j)
               nesting_tabtemp_3d(:,i,j) = nesting_tabtemp_3d(:,i,j)-umoy+u(i,j)
             ELSE IF (igdu_loc(i,j) == 5) THEN
               nesting_tabtemp_3d(:,i,j) = u(i,j)
             ELSE
               nesting_tabtemp_3d(:,i,j) = 0.
             END IF
           END DO
         END DO

       CASE('uz_bc')

         test = 1
         CALL agrif_dyn3d_height_mask(nesting_tabtemp_2d(:,:,2),igdu_loc,hex_loc, &
#if defined key_siggen || defined key_gencoord
                                      hzex_loc, &
#endif
                                      i1,i2,j1,j2,test)

         uvdiff = 0.
         DO j=MAX(j1,ljmin),MIN(j2,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             IF (i==imin.or.i==imax.or.j==jmax.or.j==jmin+1) THEN
               IF (igdu_loc(i,j) == 1) THEN
                 nesting_tabtemp_3d(:,i,j) = tabres(1:kmax,i,j)
               ELSE IF (igdu_loc(i,j) == 5) THEN
                 nesting_tabtemp_3d(:,i,j) = nesting_tabtemp_2d(i,j,1) ! u(i,j)
               ELSE
                 nesting_tabtemp_3d(:,i,j) = 0.
               END IF
             END IF

             IF (igdu_loc(i,j) /=4) THEN
               uvdiff(:,i,j) = nesting_tabtemp_3d(:,i,j)-tabres(:,i,j)
             ELSE
               uvdiff(:,i,j) = 0.
             END IF

           END DO
         END DO

!      return

         coeff = 0.
!cval WARNING AGRIF en dur !!!!
         visc_sponge=1./8.

         DO j=MAX(j1,ljmin),MIN(j2,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             dist = nb_connect*Agrif_Irhox()
             dist = min(dist,i-(imin+1))
             dist = min(dist,imax-i)
             dist = min(dist,j-(jmin+1))
             dist = min(dist,jmax-j)
             coeff(i,j) = (1.-REAL(dist)/real(nb_connect*Agrif_Irhox()))*visc_sponge* &
             (dx_u(i,j)*dy_v(i,j))**2/(halfdt*(dx_u(i,j)**2+dy_v(i,j)**2))
           END DO
         END DO

         Fx = 0.

         DO j=max(j1,ljminm1),min(j2,ljmaxp1)
!WARNING pourquoi cette modif ????
           DO i=i1,MIN(i2-1,limaxp1-1)
             DO k=k1,k2
               Fx(k,i,j) = coeff(i,j)*(uvdiff(k,i+1,j)-uvdiff(k,i,j))/dx(i+1,j)*halfdt
             END DO
             IF ((i<imin+1).OR.(i>imax-1)) Fx(:,i,j) = 0.
           END DO
         END DO

         Fy = 0.
         DO j=max(j1,ljmin),min(j2-1,ljmax)
           DO i=MAX(i1,limin),MIN(i2,limax)
             DO k=k1,k2
               Fy(k,i,j) = mn(i,j)*coeff(i,j)*(uvdiff(k,i,j+1)-uvdiff(k,i,j))/dy_f(i,j)*halfdt
               IF ((j<jmin+1).OR.(j>jmax-1)) Fy(:,i,j) = 0.
             END DO
           END DO
         END DO
!
         DO j=MAX(j1+1,ljmin),MIN(j2-1,ljmax)
!WARNING pourquoi cette modif ????
           DO i=MAX(i1+1,liminm1),MIN(i2-1,limax)
!cval le test induit appel a agrif_dyn3d_height_mask, vraiment utile ????????????
             IF ((igdu_loc(i,j)/=4).AND.(hex_loc(i,j)/=0.)) THEN
               DO k=k1,k2
                 nesting_tabtemp_3d(k,i,j) = nesting_tabtemp_3d(k,i,j)+((1./dx_u(i,j))*(Fx(k,i,j)-Fx(k,i-1,j))  &
                 +(1./dy_u(i,j))*(Fy(k,i,j)-Fy(k,i,j-1)))
               END DO
             END IF
           END DO
         END DO

     END SELECT

   END IF

   PRINT_DBG*, 'exit INTERPUZ_STEPU routine',Agrif_Root()

   RETURN

  END SUBROUTINE interpuz_stepu
#endif
