#if defined key_agrif

   !!======================================================================

  SUBROUTINE update_agrif
!
!*************************************************
!
!     objet    : actualisation des valeurs des variables de la grille
!                parent par les valeur des variables sur la grille
!                fine sur tout ou partie de la zone de recouvrement.
!
!
!     appelant         : stepu (contexte agrif, voir stepuagrif.h)
!                      :
!     appeles          : Agrif_Root, Agrif_NbStepint
!     appeles          : Agrif_Rhox, Agrif_Rhoy, Agrif_ixl, Agrif_iyl
!     appeles          : COUNT, SUM
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture :
!                      :
!     cle cpp          : key_agrif
!                      :
!     variables in     : sal, temp, ssh, uz, u
!                      :
!     variables out    : valmanq
!                      :
!     variables inout  :
!                      :
!     dernieres modifs : 19/09/02
!                      :
!***********************************************************
!
#include "toolcpp.h"
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comvarp3d
   USE agrifcommonid
#ifdef key_substance
   USE comsubstance
#endif
   IMPLICIT NONE
!
   external :: updatetemp, updatesal, updateuz,updatevz,updatessh
   external :: updateturb, updatekz, updatenz, updatehzex, updatehzey
   external :: updatesurf
#ifdef key_substance
   external updatecv_wat
#endif

!
!************************************************************
!     Executable statements
!************************************************************
!
   PRINT_DBG*, 'enter UPDATE_AGRIF routine'
!
! s il n y a pas de grille grand mere et que l on est sur la grille mere : on sort
   IF ( Agrif_Root() .AND. (.not.agrif_coarse)) RETURN
! s il y a une grille grand mere et que l'on est dessus on sort aussi
   IF (Agrif_GrandMother()) RETURN
! si on est une sur une grille fille quelconque et que l'on n'est pas rendu au meme instant d'integration
! que sa grille parent on sort
   IF (Agrif_NbStepint().NE.(Agrif_IRhot()-1)) RETURN
!
! a partir de la on va faire de l update soit parce que l on est sur un grille fille rendue
! au bon moment, soit on a une grille grand mere et que l on est sur la grille mere
!
   Agrif_USESpecialValueInUpdate = .TRUE.
   Agrif_SpecialValueFineGrid=valmanq
!
!cval quand est-ce que l'on elimine cela ?
!  on update les traceurs actifs
   IF (.NOT.(Agrif_Root().AND.(agrif_nb_step()>=4))) THEN
! si l on  n est pas sur la grille mere ou  que  l on est sur la grille mere et que l on est sur les tout premiers pas
! de temps pour tester le coarsening on update les champs de T et S puis ensuite ils evolueront avec la dynamique
! c est plus  une etape de travail a virer ulterieurement.
     CALL Agrif_Update_Variable(sal_id ,procname = updatesal)
     CALL Agrif_Update_Variable(temp_id,procname = updatetemp)
#ifdef key_substance
   CALL Agrif_Update_Variable(cv_wat_id,procname = updatecv_wat)
#endif
   END IF
!
! si on arrive la on a forcement coasening cf IF ( Agrif_Root() .AND.  (.not.agrif_coarse)) RETURN

   ! Update of kz (coarsening only)
   ! -----------------------------

   IF (agrif_root()) THEN
! !     Call Agrif_Update_Variable(turb_id,procname = updateturb)
! si l on  est sur la grille mere et qu il y a une grille grand mere on update le kz pour le melange sur
! la grille grand mere : on le fait a chaque pas de temps d integration de la grille mere ce qui
! signifie qu il n y a pas de raffinement en temps entre les deux alors qu'il pourrait
!
     CALL Agrif_Update_Variable(kz_id,procname = updatekz)
!     Call Agrif_Update_Variable(nz_id,procname = updatenz)   ! inutile sur la ! grille grand mere puisque pas de dynamique
   END IF

! CVAL mais pourquoi ne le fait-on pas (kz + nz) quand on est sur la grille enfant ?
! on pourrait mais coûteux


#ifdef key_virerquandreturneclairci
!
! Recompute water heights with updated ssh
! on recalcule les hauteurs d eau avec le bon ssh
! oui mais la pour le moment on n a pas encore update la ssh
! normal c est compatible avec ce qui est fait dans dyn3dxy.F90 et  dyn3dyx.F90
! ou les hzex ne sont pas mis a jour une fois que l on dispose des nouveaux uz et de la nouvelle ssh
!
   agrif_indic = 1
!
! agrif_indic : dit si on est en u(1) ou v(1) pour indiquer à compute_heights (qui appelle dyn3d_height_mask) si il faut recalculer les hex ou hey
!
   CALL Agrif_ChildGrid_To_ParentGrid()
   CALL Compute_heights()
   CALL Agrif_ParentGrid_To_ChildGrid()
#endif


   ! Update of fxeau (through updateuz)
   ! ----------------------------------

   Agrif_USESpecialValueInUpdate = .FALSE.

! locupdate2 :  dimension 2 du tableau 3D (donc x ou i dans mars)
! locupdate3 :  dimension 3 du tableau 3D (donc y ou j dans mars)

   IF (.not.agrif_root()) THEN    ! cas agrif general, on update depuis une grille fille
!pour cas realiste ?      Call Agrif_Update_Variable(uz_update_id  ,procname = updateuz) 
! on update les fxeau (les uz sont deduits dans updatewz() )  partout (/-1,-2/) sur la grille parent d une grille fille mais pas sur le point u qui correspond a la frontiere
! les locupdate ici ajoute semblent ameliorer bien les choses
! locupdate2=(/-1,-2/) : 1ere borne < 2eme borne donc on travaille sur zone de recouvrement de toute la grille fille
     CALL Agrif_Update_Variable(uz_update_id  , locupdate2=(/-1,-2/) , procname = updateuz)
   ELSE  ! update of the grandmother grid
! update of hzex and fxeau including at boundary points
! on update les uz partout sur la grille grand mere en incluant aussi le premier point u qui correspond a la frontiere de la grille fine (ie premier point u calcule)
!  locupdate2=(/-1,-2/) signifie que l on update vers l exterieur d un point (-1) et sur toute la grille du fait du -2 qui est inferieur au -1
!  locupdate2=(/-1,0/) on updaterait le point sur la frontiere et le premier point a l interieur
     CALL Agrif_Update_Variable(hzex_id, locupdate2=(/-1,-2/) , procname = updatehzex)
     CALL Agrif_Update_Variable(uz_update_id , locupdate2=(/-1,-2/), procname = updateuz)
!     Call Agrif_Update_Variable(uz_update_id  ,locupdate1=(/0,-1/),locupdate2=(/1,0/) ,procname = updateuz)
   END IF

#ifdef key_virerquandreturneclairci
! pourquoi on recalcule encore les hauteurs  : on  n a toujours pas update la surface libre !!
   agrif_indic = 2
   CALL Agrif_ChildGrid_To_ParentGrid()
   CALL Compute_heights()
   CALL Agrif_ParentGrid_To_ChildGrid()
#endif


   ! Update of fyeau (through updatevz)
   ! ----------------------------------

   Agrif_USESpecialValueInUpdate = .FALSE.

! locupdate2 :  dimension 2 du tableau 3D (donc x ou i dans mars)
! locupdate3 :  dimension 3 du tableau 3D (donc y ou j dans mars)

   IF (.not.agrif_root()) THEN
!pour cas realiste ?      Call Agrif_Update_Variable(vz_update_id  ,procname = updatevz)
! update de v partout sauf  sur la frontiere
     CALL Agrif_Update_Variable(vz_update_id  , locupdate3=(/-1,-2/), procname = updatevz)
   ELSE! update of the grandmother grid
! meme remarque pour la grille grand mere que pour le u  haut dessus
     CALL Agrif_Update_Variable(hzey_id, locupdate3=(/-1,-2/), procname = updatehzey)
     CALL Agrif_Update_Variable(vz_update_id, locupdate3=(/-1,-2/), procname = updatevz)
   END IF
!      Call Agrif_Update_Variable(vz_update_id  ,locupdate1=(/1,0/) ,locupdate2=(/0,-1/),procname = updatevz)


   ! update fwet
   ! -----------

   IF (fwetmin /= 1.0_rsh) THEN
     Agrif_USESpecialValueInUpdate = .FALSE.
     agrif_indic = 2    ! switch pour traiter fwet et non surf
     CALL Agrif_Update_Variable(h0_id, procname = updatesurf)
   END IF

   ! update ssh de la grille fille vers la grille mere (from child grid)
   ! -------------------------------------------------

   Agrif_USESpecialValueInUpdate = .TRUE.
   Agrif_SpecialValueFineGrid=valmanq
   IF (.not.agrif_root()) THEN
     CALL Agrif_Update_Variable(ssh_id ,procname = updatessh)
   END IF

!CVAL pour test ou garder tout le temps ? test coasening encore A garder ?
!cval quand est-ce que l on vire ?
   IF (agrif_root().AND.agrif_nb_step()<=9) THEN
! on update la ssh sur la grille grand mere et au debut (sur les 9 premiers pas de temps)
! ensuite on ne le fait plus et la ssh est recalculee sur la grille grand mere (cf  update_wz() )
     CALL Agrif_Update_Variable(ssh_id ,procname = updatessh)
   END IF

! on calcule les wz sur la grille parent (grand mere ou autre)
! on calcule la ssh sur la grille grand mere (car pour les traceurs il faut etre super precis. On evite donc pertes liees aux interpolations)
! on calcule les u et v sur la grille parent compatible avec la continuite
   Agrif_USESpecialValueInUpdate = .FALSE.
   CALL Agrif_ChildGrid_To_ParentGrid()
   CALL update_wz()
   CALL Agrif_ParentGrid_To_ChildGrid()

   PRINT_DBG*, 'exit UPDATE_AGRIF routine'

  END SUBROUTINE update_agrif

   !!======================================================================

  SUBROUTINE update_wz()
#include "toolcpp.h"
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE agrifcommonid
   USE obccombine, ONLY : l_obc_west,l_obc_east,l_obc_north,l_obc_south
#if defined key_MPI_2D && defined key_diag_physwz
   USE toolmpi, ONLY : ex_i_rsh,ex_j_rsh
#endif

   INTEGER :: i,j
   INTEGER :: test
   REAL(KIND=rsh),DIMENSION(limin:limax)                     :: tfh,tfb
   REAL(KIND=rsh),DIMENSION(ljmin:ljmax)                     :: tfg,tfd
   REAL(KIND=rlg) :: somme_xe,cff1
   REAL(KIND=rsh) :: cff2

   ! Coarsening
   ! Estimate of the ssh from fluxes
   ! ---------------------------------------------------------------

   IF (Agrif_Grandmother()) THEN

     DO j=ljminp2,ljmaxm1
       DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
!     cff2=(ssh_old(i,j)*fwetp(i,j)-(SUM(dsigu(1:kmax)*fxeau(1:kmax,i  ,j))*dy_u(i  ,j) &
!                                      -SUM(dsigu(1:kmax)*fxeau(1:kmax,i-1,j))*dy_u(i-1,j) &
!                                      +SUM(dsigu(1:kmax)*fyeau(1:kmax,i  ,j))*dx_v(i  ,j) &
!                                      -SUM(dsigu(1:kmax)*fyeau(1:kmax,i,j-1))*dx_v(i,j-1) &
!                                      -SUM(phieau(1:kmax,i,j))*halfdt )/(surf(i,j)))/fwet(i,j)
!if (ssh(i,j)/=0.) THEN
!if (abs(ssh(i,j)-cff2)/abs(ssh(i,j)).GT.1.E-3) THEN
! print *,i,j,cff2,ssh(i,j),abs(ssh(i,j)-cff2)/abs(ssh(i,j)),fwet(i,j)
! endif
! endif

         ssh(i,j)=(ssh_old(i,j)*fwetp(i,j) &
                  -(SUM(dsigu(1:kmax)*fxeau(1:kmax,i  ,j))*dy_u(i  ,j) &
                  -SUM(dsigu(1:kmax)*fxeau(1:kmax,i-1,j))*dy_u(i-1,j) &
                  +SUM(dsigu(1:kmax)*fyeau(1:kmax,i  ,j))*dx_v(i  ,j) &
                  -SUM(dsigu(1:kmax)*fyeau(1:kmax,i,j-1))*dx_v(i,j-1) &
                  -SUM(phieau(1:kmax,i,j))*halfdt )/(surf(i,j)))/fwet(i,j)
       END DO
     END DO

   ! western boundary
     IF_MPI (limin==imin) THEN
       IF ( l_obc_west ) THEN
         WHERE (h0(imin+1,ljminm2:ljmaxp2)/=-valmanq)
         ssh(imin+1,ljminm2:ljmaxp2) = ssh(imin+2,ljminm2:ljmaxp2)
       ELSE WHERE
         ssh(imin+1,ljminm2:ljmaxp2) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

   ! eastern boundary
     IF_MPI (limax==imax) THEN
       IF ( l_obc_east ) THEN
         WHERE (h0(imax,ljminm2:ljmaxp2)/=-valmanq)
         ssh(imax,ljminm2:ljmaxp2) = ssh(imax-1,ljminm2:ljmaxp2)
       ELSE WHERE
         ssh(imax,ljminm2:ljmaxp2) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

   ! southern boundary
     IF_MPI (ljmin==jmin) THEN
       IF ( l_obc_south ) THEN
         WHERE (h0(liminm2:limaxp2,jmin+1)/=-valmanq)
         ssh(liminm2:limaxp2,jmin+1) = ssh(liminm2:limaxp2,jmin+2)
       ELSE WHERE
         ssh(liminm2:limaxp2,jmin+1) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

   ! northern boundary
     IF_MPI (ljmax==jmax) THEN
       IF ( l_obc_north ) THEN
         WHERE (h0(liminm2:limaxp2,jmax)/=-valmanq)
         ssh(liminm2:limaxp2,jmax) = ssh(liminm2:limaxp2,jmax-1)
       ELSE WHERE
         ssh(liminm2:limaxp2,jmax) = valmanq
         END WHERE
       END IF
     ENDIF_MPI

#if defined key_MPI_2D && defined key_diag_physwz
   CALL ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif

   END IF !if (Agrif_Grandmother())

!$OMP PARALLEL

   ! coarsening ET cas general
   ! on calcule les vitesses au lieu de les updater via agrif. On est dans la grille parent
   ! --------------------------------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
   DO j=ljminp2,ljmaxm1
     DO i=liminp1,limaxm1
       IF (igdu(i,j)/=4) THEN
#if defined key_siggen || defined key_gencoord
         uz(:,i,j) = fxeau(:,i,j)/(hzex(:,i,j)*halfdt)
         u(i,j)=SUM(uz(:,i,j)*hzex(:,i,j)*dsigu(:))/hex(i,j)
#else
         uz(:,i,j) = fxeau(:,i,j)/(hex(i,j)*halfdt)
         u(i,j)=SUM(uz(:,i,j)*dsigu(:))
#endif
       ELSE
         uz(:,i,j)=0.
         u(i,j) = 0.
       END IF
     END DO
   END DO
!$OMP END DO

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
   DO j=ljminp1,ljmaxm1
     DO i=liminp2,limaxm1
       IF (igdv(i,j)/=4) THEN
#if defined key_siggen || defined key_gencoord
         vz(:,i,j) = fyeau(:,i,j)/(hzey(:,i,j)*halfdt)
         v(i,j)=SUM(vz(:,i,j)*hzey(:,i,j)*dsigu(:))/hey(i,j)
#else
         vz(:,i,j) = fyeau(:,i,j)/(hey(i,j)*halfdt)
         v(i,j)=SUM(vz(:,i,j)*dsigu(:))
#endif
       ELSE
         vz(:,i,j) = 0.
         v(i,j) = 0.
       END IF
     END DO
   END DO
!$OMP END DO

!$OMP END PARALLEL

#if defined key_MPI_2D
   ! MPI exchanges of uz,vz,u,v
   CALL exchange_uv()
#endif

#if defined key_siggen || defined key_gencoord
   CALL dyn3d_wz(ssh_old,ssh,uz,vz,u,v)
#else
   CALL dyn3d_wz(ssh_old,ssh,uz,vz)
#endif

   RETURN

  END SUBROUTINE update_wz

   !!======================================================================

  SUBROUTINE Updatesal(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k

!$OMP PARALLEL

   IF (before) THEN

     IF (first_agrif_update) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
       DO j=j1,j2
         DO i=i1,i2
           DO k=k1,k2
             IF (h0(i,j) == -valmanq) THEN
               tabres(k,i,j) = Agrif_SpecialValueFineGrid
             ELSE
               tabres(k,i,j) = sal(k,i,j)
             END IF
           END DO
         END DO
       END DO
!$OMP END DO
     ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
       DO j=j1,j2
         DO i=i1,i2
           DO k=k1,k2
             IF (h0(i,j)+ssh(i,j) < h0fond) THEN
!if(h0(i,j) == -valmanq) THEN
               tabres(k,i,j) = Agrif_SpecialValueFineGrid
             ELSE
               tabres(k,i,j) = sal(k,i,j)
             END IF
           END DO
         END DO
       END DO
!$OMP END DO
     END IF
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF ((tabres(k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
             sal(k,i,j) = tabres(k,i,j)
           END IF
           IF (h0(i,j) == -valmanq) THEN
             sal(k,i,j) = valmanq
           END IF
         END DO
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN

  END SUBROUTINE Updatesal

   !!======================================================================

  SUBROUTINE Updatetemp(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL :: test_temp

!$OMP PARALLEL

   IF (before) THEN

     IF (first_agrif_update) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
       DO j=j1,j2
         DO i=i1,i2
           DO k=k1,k2
             IF (h0(i,j) == -valmanq) THEN
               tabres(k,i,j) = Agrif_SpecialValueFineGrid
             ELSE
               tabres(k,i,j) = temp(k,i,j)
             END IF
           END DO
         END DO
       END DO
!$OMP END DO
     ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
       DO j=j1,j2
         DO i=i1,i2
           DO k=k1,k2
             IF (h0(i,j)+ssh(i,j) < h0fond) THEN
!if(h0(i,j) == -valmanq) THEN
               tabres(k,i,j) = Agrif_SpecialValueFineGrid
             ELSE
               tabres(k,i,j) = temp(k,i,j)
             END IF
           END DO
         END DO
       END DO
!$OMP END DO
     END IF
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
#ifdef key_debug_agrif
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) '          '
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateTemp',minval(temp(:,i,j)),maxval(temp(:,i,j))
#endif
         DO k=k1,k2
           IF ((tabres(k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
             temp(k,i,j) = tabres(k,i,j)
           END IF
           IF (h0(i,j) == -valmanq) THEN
             temp(k,i,j) = valmanq
           END IF
         END DO
#ifdef key_debug_agrif
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateTemp',minval(temp(:,i,j)),maxval(temp(:,i,j))
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) '          '
#endif
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatetemp

   !!======================================================================

  SUBROUTINE Updateturb(tabres,l1,l2,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comturb
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2,l1,l2
   REAL tabres(l1:l2,k1:k2,i1:i2,j1:j2)
   LOGICAL before

   INTEGER i,j,k,l

!$OMP PARALLEL

   IF (before) THEN

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k,l)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2

           IF (h0(i,j) == -valmanq) THEN
             DO l=l1,l2
               tabres(l,k,i,j) = Agrif_SpecialValueFineGrid
             END DO
           ELSE
             tabres(1,k,i,j) = ect(k,i,j)
             tabres(2,k,i,j) = psi(k,i,j)
             IF (turb_nbeq == 2) THEN
               tabres(3,k,i,j) = eps(k,i,j)
             END IF
           END IF
         END DO

       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF ((tabres(1,k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
             ect(k,i,j) = tabres(1,k,i,j)
             psi(k,i,j) = tabres(2,k,i,j)
             IF (turb_nbeq == 2) THEN
               eps(k,i,j) = tabres(3,k,i,j)
             END IF

           ELSE
             ect(k,i,j) = valmanq
             psi(k,i,j) = valmanq
             IF (turb_nbeq == 2) THEN
               eps(k,i,j) = valmanq
             END IF

           END IF
         END DO
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN

  END SUBROUTINE Updateturb

   !!======================================================================

  SUBROUTINE Updatekz(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comturb
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before

   INTEGER i,j,k

!$OMP PARALLEL

   IF (before) THEN

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF (h0(i,j) == -valmanq) THEN
             tabres(k,i,j) = Agrif_SpecialValueFineGrid
           ELSE
             tabres(k,i,j) = kz(k,i,j)
           END IF
         END DO

       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF ((tabres(k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
             kz(k,i,j)  = tabres(k,i,j)
           ELSE
             kz(k,i,j) = valmanq
           END IF
         END DO
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatekz

   !!======================================================================

  SUBROUTINE Updatenz(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comturb
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before

   INTEGER i,j,k

!$OMP PARALLEL

   IF (before) THEN

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF (h0(i,j) == -valmanq) THEN
             tabres(k,i,j) = Agrif_SpecialValueFineGrid
           ELSE
             tabres(k,i,j) = nz(k,i,j)
           END IF
         END DO

       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           IF ((tabres(k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
             nz(k,i,j)  = tabres(k,i,j)
           ELSE
             nz(k,i,j) = valmanq
           END IF
         END DO
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatenz

   !!======================================================================

#ifdef key_substance
  SUBROUTINE Updatecv_wat(tabres,l1,l2,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comsubstance

   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2,l1,l2
   REAL tabres(l1:l2,k1:k2,i1:i2,j1:j2)
   LOGICAL before

   INTEGER i,j,k,l

!$OMP PARALLEL

   IF (before) THEN

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k,l)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           DO l=l1,l2
             tabres(l,k,i,j) = cv_wat(l,k,i,j)
           END DO
         END DO
       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,k,l)
     DO j=j1,j2
       DO i=i1,i2
         DO k=k1,k2
           DO l=l1,l2
!            IF ( cv_wat(l,k,i,j)/=valmanq .AND. tabres(l,k,i,j)/=valmanq ) THEN
             IF ((tabres(l,k,i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
               cv_wat(l,k,i,j) = tabres(l,k,i,j)
             END IF
           END DO
         END DO
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN

  END SUBROUTINE Updatecv_wat
#endif

   !!======================================================================

  SUBROUTINE Updatessh(tabres,i1,i2,j1,j2,before)

   !&E       !  2014-09  (S. Petton, F. Dumas) Review of update of ssh and xeadv
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comfluxdo

   IMPLICIT NONE
   INTEGER i1,i2,j1,j2
   REAL tabres(i1:i2,j1:j2)
   LOGICAL before

   INTEGER i,j

!$OMP PARALLEL

   IF (before) THEN
     IF (first_agrif_update) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
           IF (h0(i,j)== -valmanq ) THEN
             tabres(i,j) = Agrif_SpecialValueFineGrid
!            if(h0(i,j)+ssh(i,j) < h0fond) THEN
!               tabres(i,j) = 0.
           ELSE
             tabres(i,j) = ssh(i,j) ! *surf(i,j)
           END IF
         END DO
       END DO
!$OMP END DO 
     ELSE

!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
           IF (h0(i,j)+ssh(i,j) < h0fond) THEN
             tabres(i,j) = 0.
             tabres(i,j) = Agrif_SpecialValueFineGrid
           ELSE
             tabres(i,j) = ssh(i,j) ! *surf(i,j)
           END IF
         END DO
       END DO
!$OMP END DO
     END IF

   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=j1,j2
       DO i=i1,i2
#ifdef key_debug_agrif
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) '        '
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateSsh',ssh(i,j),h0(i,j),tabres(i,j)
#endif
         IF ((tabres(i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
           ssh(i,j) = tabres(i,j)
!spetton fdumas
         ELSE IF ((tabres(i,j).NE.Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq).AND.(h0(i,j)+tabres(i,j)<h0fond)) THEN
           ssh(i,j) = - h0(i,j)
         ELSE IF ((tabres(i,j)==Agrif_SpecialValueFineGrid).AND.(h0(i,j)/=-valmanq)) THEN
!cval ce test  la est vraiment utile ?
           ssh(i,j) = - h0(i,j)
!spetton fdumas
         END IF
!          IF ((tabres(i,j).NE.0.).AND.(h0(i,j)/=-valmanq)) THEN
!            ssh(i,j) = tabres(i,j)/(fwet(i,j)*surf(i,j))
!          ENDIF
#ifdef key_debug_agrif
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updatessh',ssh(i,j)
         IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) '        '
#endif
       END DO
     END DO
!$OMP END DO
!spetton     if (first_agrif_update) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         IF ((tabres(i,j).NE.0.).AND.(h0(i,j)/=-valmanq)) THEN
           xeadv(i,j) = ssh(i,j)
         END IF
       END DO
     END DO
!$OMP END DO
!spetton   endif
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatessh

   !!======================================================================

  SUBROUTINE Updatesurf(tabres,i1,i2,j1,j2,before)

   !&E       !  2014-09  (S. Petton, F. Dumas, limited update of fwet)

   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comfluxdo
   USE agrifcommonid

   IMPLICIT NONE
   INTEGER i1,i2,j1,j2
   REAL tabres(i1:i2,j1:j2)
   LOGICAL before
   REAL :: rhox, rhoy

   INTEGER i,j

!$OMP PARALLEL PRIVATE(rhox,rhoy)

   IF (before) THEN

     rhox = REAL(agrif_irhox())
     rhoy = REAL(agrif_irhoy())

     IF (agrif_indic == 1) then ! surf
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
           tabres(i,j) = surf(i,j)*rhox*rhoy
         END DO
       END DO
!$OMP END DO
     ELSE ! agrif_indic = 2 fwet
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
           IF (h0(i,j)+ssh(i,j) < h0fond) THEN
             tabres(i,j) = 0.
           ELSE
             tabres(i,j) = surf(i,j)*rhox*rhoy
           END IF
         END DO
       END DO
!$OMP END DO
     END IF
   ELSE ! before is false
     IF (agrif_indic == 1) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
#ifdef key_debug_agrif
           IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateSurf',surf(i,j),tabres(i,j)
#endif
           IF (tabres(i,j) /= 0.) THEN
             surf(i,j) = tabres(i,j)
           END IF
#ifdef key_debug_agrif
           IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateSurf',surf(i,j)
#endif
         END DO
       END DO
!$OMP END DO
     ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
#ifdef key_debug_agrif
           IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateFwet',tabres(i,j),surf(i,j)
#endif
           IF (tabres(i,j) /= 0.) THEN
!spetton
             fwet(i,j) = MAX(tabres(i,j)/surf(i,j),fwetmin)
!spetton
           END IF
#ifdef key_debug_agrif
           IF (i==ich .and. j==jch) write(100*(agrif_fixed()+1),*) 'updateFwet',fwet(i,j)
#endif
         END DO
       END DO
!$OMP END DO
       IF (first_agrif_update) THEN
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
       DO j=j1,j2
         DO i=i1,i2
           fwetp(i,j) = fwet(i,j)
         END DO
       END DO
!$OMP END DO
       END IF
     END IF
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatesurf

   !!======================================================================

  SUBROUTINE Updateuz(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdu
   USE comfluxdo,    ONLY : fxeau,fyeau
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,dcusds
   USE toolgeom,  ONLY : f_hzu_x
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL umoy
   REAL hex_loc(i1:i2,j1:j2)
   INTEGER igdu_loc(i1:i2,j1:j2)
   REAL hzex_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test
   REAL :: rhoy

!$OMP PARALLEL PRIVATE(rhoy)

!child (or mother if coarsening)
   IF (before) THEN
     rhoy = REAL(agrif_irhoy())
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,liminm1),MIN(i2,limax)
         tabres(1:kmax,i,j) = rhoy*fxeau(:,i,j)*dy_u(i,j)/halfdt
! rhoy car dans agrifdeclarevariables on fait bien une moyenne et non une somme
! (uz_update_id,update2=type_update_u_x,update3=type_update_u_y , 
!    type_update_u_x     = Agrif_Update_Copy  (on copie tous u de la grille)
!  type_update_u_y     = Agrif_Update_Average 

       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,liminm1),MIN(i2,limax)
         fxeau(:,i,j)=tabres(1:kmax,i,j)*halfdt/dy_u(i,j)
       END DO
     END DO
!$OMP END DO

   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updateuz

   !!======================================================================

  SUBROUTINE Updatehzex(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdu
   USE comfluxdo,    ONLY : fxeau,fyeau
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,dcusds
   USE toolgeom,  ONLY : f_hzu_x
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL umoy
   REAL hex_loc(i1:i2,j1:j2)
   INTEGER igdu_loc(i1:i2,j1:j2)
   REAL hzex_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test

!$OMP PARALLEL

   IF (before) THEN

     test = 1
!cval car agrif_dyn3d_height_mask pas parallelise pour instant
!$OMP SINGLE
     CALL agrif_dyn3d_height_mask(ssh,igdu_loc,hex_loc, &
#if defined key_siggen || defined key_gencoord
                                  hzex_loc, &
#endif
                                  i1,i2,j1,j2,test)
!$OMP END SINGLE

#if defined key_siggen || defined key_gencoord
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         tabres(1:kmax,i,j) = hzex_loc(:,i,j)
       END DO
     END DO
!$OMP END DO
#endif
   ELSE

#if defined key_siggen || defined key_gencoord
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         hzex(:,i,j) = tabres(:,i,j)
       END DO
     END DO
!$OMP END DO
#endif
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatehzex

   !!======================================================================

  SUBROUTINE Updatevz(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdv
   USE comfluxdo,    ONLY : fxeau,fyeau
   USE comcolig3d,   ONLY : intz_exp_vstar
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzey,dcusds
   USE toolgeom,  ONLY : f_hzu_y
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL vmoy
   REAL hey_loc(i1:i2,j1:j2)
   INTEGER igdv_loc(i1:i2,j1:j2)
   REAL hzey_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test
   REAL :: rhox

!$OMP PARALLEL PRIVATE(rhox)

   IF (before) THEN
     rhox = REAL(agrif_irhox())
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljminm1),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         tabres(1:kmax,i,j) = rhox*fyeau(:,i,j)*dx_v(i,j)/halfdt
       END DO
     END DO
!$OMP END DO
   ELSE
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljminm1),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=tabres(1:kmax,i,j)*halfdt/dx_v(i,j)
       END DO
     END DO
!$OMP END DO
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatevz

   !!======================================================================


  SUBROUTINE Updateuz_old(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdu
   USE comfluxdo,    ONLY : fxeau,fyeau
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex,dcusds
   USE toolgeom,  ONLY : f_hzu_x
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL umoy
   REAL hex_loc(i1:i2,j1:j2)
   INTEGER igdu_loc(i1:i2,j1:j2)
   REAL hzex_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test

   IF (before) THEN


     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         tabres(1:kmax,i,j) = uz(:,i,j)
         tabres(kmax+2,i,j) = u(i,j)
       END DO
     END DO
   ELSE

     test = 1
     CALL agrif_dyn3d_height_mask(ssh,igdu_loc,hex_loc, &
#if defined key_siggen || defined key_gencoord
                                  hzex_loc, &
#endif
                                  i1,i2,j1,j2,test)

     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         IF (igdu_loc(i,j) == 1) THEN
           uz(:,i,j) = tabres(1:kmax,i,j)
#if defined key_siggen || defined key_gencoord
           umoy = SUM(uz(:,i,j)*hzex_loc(:,i,j)*dsigu(:))/hex_loc(i,j)
#else
           umoy = SUM(uz(:,i,j)*dsigu(:))
#endif
           u(i,j) = tabres(kmax+2,i,j)
           uz(:,i,j) = uz(:,i,j)-umoy+u(i,j)
         ELSE IF (igdu_loc(i,j) == 5) THEN
           u(i,j) = tabres(kmax+2,i,j)
           uz(:,i,j) = u(i,j)
         ELSE
           u(i,j) = 0.
           uz(:,i,j) = 0.
         END IF

         IF ((abs(uz(kmax,i,j))*dt/dx_u(i,j)) > 0.83*cflcrt) THEN
           IF (uz(kmax,i,j)>0.) THEN
             uz(:,i,j) = 0.83_rsh*cflcrt*dx_u(i,j)/dt
           ELSE
             uz(:,i,j) = -0.83_rsh*cflcrt*dx_u(i,j)/dt
           END IF
           u(i,j) = uz(kmax,i,j)
         END IF
#if defined key_siggen || defined key_gencoord
         fxeau(:,i,j)=tabres(1:kmax,i,j)*hzex_loc(:,i,j)*halfdt
#else
         fxeau(:,i,j)=tabres(1:kmax,i,j)*hex_loc(i,j)*halfdt
#endif

       END DO
     END DO
   END IF

   RETURN
  END SUBROUTINE Updateuz_old

   !!======================================================================

  SUBROUTINE Updatevz_old(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdv
   USE comfluxdo,    ONLY : fxeau,fyeau
   USE comcolig3d,   ONLY : intz_exp_vstar
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzey,dcusds
   USE toolgeom,  ONLY : f_hzu_y
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL vmoy
   REAL hey_loc(i1:i2,j1:j2)
   INTEGER igdv_loc(i1:i2,j1:j2)
   REAL hzey_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test


   IF (before) THEN

     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         tabres(1:kmax,i,j) = vz(:,i,j)
         tabres(kmax+2,i,j) = v(i,j)
       END DO
     END DO
   ELSE

     test = 2
     CALL agrif_dyn3d_height_mask(ssh,igdv_loc,hey_loc, &
#if defined key_siggen || defined key_gencoord
                                  hzey_loc, &
#endif
                                  i1,i2,j1,j2,test)

     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         IF (igdv_loc(i,j) == 1) THEN
           vz(:,i,j) = tabres(1:kmax,i,j)
           v(i,j) = tabres(kmax+2,i,j)
#if defined key_siggen || defined key_gencoord
           vmoy = SUM(vz(:,i,j)*hzey_loc(:,i,j)*dsigu(:))/hey_loc(i,j)
#else
           vmoy = SUM(vz(:,i,j)*dsigu(:))
#endif
           vz(:,i,j) = vz(:,i,j)-vmoy+v(i,j)
         ELSE IF (igdv_loc(i,j) == 5) THEN
           v(i,j) = tabres(kmax+2,i,j)
           vz(:,i,j) = v(i,j)
         ELSE
           v(i,j) = 0.
           vz(:,i,j) = 0.
         END IF

         IF ((abs(vz(kmax,i,j))*dt/dy_v(i,j)) > 0.83*cflcrt) THEN
           IF (vz(kmax,i,j)>0.) THEN
             vz(:,i,j) = 0.83_rsh*cflcrt*dy_v(i,j)/dt
           ELSE
             vz(:,i,j) = -0.83_rsh*cflcrt*dy_v(i,j)/dt
           END IF
           v(i,j) = vz(kmax,i,j)
         END IF
#if defined key_siggen || defined key_gencoord
         IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=tabres(1:kmax,i,j)*hzey_loc(:,i,j)*halfdt
#else
         IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=tabres(1:kmax,i,j)*hey_loc(i,j)*halfdt
#endif
       END DO
     END DO
   END IF

   RETURN
  END SUBROUTINE Updatevz_old

   !!======================================================================

  SUBROUTINE Updatehzey(tabres,k1,k2,i1,i2,j1,j2,before)
   USE parameters
   USE comvars3d
   USE comvars2d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d, ONLY : igdv
   USE comfluxdo,    ONLY : fxeau,fyeau
   USE comcolig3d,   ONLY : intz_exp_vstar
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzey,dcusds
   USE toolgeom,  ONLY : f_hzu_y
#endif
   IMPLICIT NONE
   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   LOGICAL before
   INTEGER i,j,k
   REAL vmoy
   REAL hey_loc(i1:i2,j1:j2)
   INTEGER igdv_loc(i1:i2,j1:j2)
   REAL hzey_loc(1:kmax,i1:i2,j1:j2)
   INTEGER test


!$OMP PARALLEL

   IF (before) THEN
     test = 2
!cval car agrif_dyn3d_height_mask pas parallelise pour instant
!$OMP SINGLE
     CALL agrif_dyn3d_height_mask(ssh,igdv_loc,hey_loc, &
#if defined key_siggen || defined key_gencoord
                                  hzey_loc, &
#endif
                                  i1,i2,j1,j2,test)
!$OMP END SINGLE

#if defined key_siggen || defined key_gencoord
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         tabres(1:kmax,i,j) = hzey_loc(:,i,j)
       END DO
     END DO
!$OMP END DO
#endif
   ELSE
#if defined key_siggen || defined key_gencoord
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j)
     DO j=MAX(j1,ljmin),MIN(j2,ljmax)
       DO i=MAX(i1,limin),MIN(i2,limax)
         hzey(:,i,j) = tabres(:,i,j)
       END DO
     END DO
!$OMP END DO
#endif
   END IF

!$OMP END PARALLEL

   RETURN
  END SUBROUTINE Updatehzey

   !!======================================================================

#else
  SUBROUTINE update_agrif_empty
   RETURN
  END SUBROUTINE update_agrif_empty
#endif
