   !!======================================================================

  SUBROUTINE dyn2dx_tls_obc_smooth(i1,i2,j,tfg,tfd,u,xe,xeuv)
#if defined key_lans_turb || defined key_leray_turb
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn2dx_tls_obc  ***
   !&E
   !&E ** Purpose : Modify coefficients of matrix
   !&E              to introduce implicit part of the sea surface pressure gradient
   !&E
   !&E ** Description : - Characteristics method (ssh and u)
   !&E                  - or Pure prescription of sea surface height
   !&E
   !&E ** Called by : dyn2dxy, dyn2dyx, dyn3dxy, dyn3dyx
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xe(i1,j), xe(i1+1,j), xe(i2,j), xe(i2-1,j)
   !&E                     u(i1,j), u(i1+1,j), u(i2-1,j), u(i2-2,j)
   !&E
   !&E ** Modified variables : y2dvs
   !&E
   !&E ** Reference : Blayo & Debreu, 2005. Ocean modelling, 9, 231-252
   !&E                Vandermeirsch, convention 03/2.210 458F, sept 2004
   !&E
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E
   !&E ** History :
   !&E       !  2007-05  (F. Vandermeirsch, J. Marin) Original code
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-01  (A. Mano T. Odaka) MPI2D
   !&E       !  2009-01  (F. Vandermeirsch) introduction of obc_ssh_* and characteristics method
   !&E       !  2009-07  (F. Vandermeirsch)  MC 2nd order scheme
   !&E       !  2009-07  (V. Garnier) MPI2d for characteristics method
   !&E       !  2010-04  (F. Vandermeirsch) OBC : introduction of Flather operator
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2011-11  (F. Vandermeirsch) correction : spatial average
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comcolig2d, ONLY : y2dv,y2dvs,y2dv_moy,av,cv

   USE comfluxdo
   USE obccombine, ONLY : obc_ue, obc_uw, obc_sshe, obc_sshw, &
                          l_obc_west,l_obc_east,              &
                          l_obc_char
   USE_MPI toolmpi,     only : ex_i_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                            INTENT( in )     :: i1,i2,j
   REAL(KIND=rsh),                                     INTENT( in )     :: tfg,tfd
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: xe
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: xeuv
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: u

   !! * local declarations
   REAL(KIND=rsh)               :: h,w1,w3

   !!----------------------------------------------------------------------
   !! * Executable part

   ! introduce sea suface height at open boundaries

     IF (l_obc_west) THEN
     IF_MPI ( i1==imin+1+ishift_obc_west .AND. limin<=i1 .AND. i1<=limax) THEN
      IF_AGRIF (Agrif_Root()) THEN
       y2dv_moy(i1,j) = y2dv(i1,j) 
      ELSE_AGRIF
       INST_AGRIF y2dv_moy(i1,j)=y2dv(i1,j)
      ENDIF_AGRIF
     ENDIF_MPI
     END IF

     IF (l_obc_east) THEN
     IF_MPI ( i2==imax-ishift_obc_east .AND. limin<=i2 .AND. i2<=limax) THEN
      IF_AGRIF (Agrif_Root()) THEN
       y2dv_moy(i2-1,j) = y2dv(i2-1,j)
      ELSE_AGRIF
       INST_AGRIF y2dv_moy(i2-1,j) = y2dv(i2-1,j)
      ENDIF_AGRIF
     ENDIF_MPI
     END IF
   PRINT_DBG*, 'exit DYN2DX_TLS_OBC routine'
#endif
  END SUBROUTINE dyn2dx_tls_obc_smooth
