   !&E   ***************   include dyn2dy_cty   **********************************
   !&E
   !&E ** Purpose : remplissage des termes du systeme lineaire
   !&E              provenant de la discretisation de l equation de
   !&E              continuite dans le systeme d equation de Saint-
   !&E              Venant sur l ensemble de la grille de calcul.
   !&E
   !&E ** Description : Equation couplee a celle de la conservation de la
   !&E                  quantite de mouvement dans la direction Nord-Sud.
   !&E
   !&E
   !&E ** Called by : dyn2dy_tls_coef
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xeu,u,v,ustar,dx_v,dy_u,hey,hex,surf,phieau
   !&E
   !&E ** Modified variables : y2dxe,axe,bxe,cxe
   !&E
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E        !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E        !  2008-07  (L. Debreu, T. Duhaut, M. Honnorat) time scheme
   !&E        !  2009-07  (N. Bruneau F. Dumas) coupling with swan
   !&E
   !&E
   !&E-------------------------------------------------------------------------

       j1=jb(i)
       j2=jh(i)
       IF (j >= j1+1.and.j <= j2-1) THEN

   ! 1- terme de diagonale de la matrice tridiagonale
   !    du modele barotrope
   !--------------------------------------------------

   ! 2- terme de sous-diagonale de la matrice tridiagonale
   !    issu de l implicitation de la divergence de flux meridienne
   !    (flux au sud  du point xe(i,j) considere). Remplissage
   !    du tableau hey qui ressert dans les flux de traceurs
   !------------------------------------------------------------

         axe(i,j)=-alphan*halfdt*dx_v(i,j-1)/surf(i,j)*hey(i,j-1)
         bxe(i,j)=fwet(i,j)

   ! 3- terme de sous-diagonale de la matrice tridiagonale
   !    issu de l implicitation de la divergence de flux meridienne
   !    (flux au nord du point xe(i,j) considere). Remplissage
   !    du tableau hey qui ressert dans les flux de traceurs
   !------------------------------------------------------------
!
         cxe(i,j)=alphan*halfdt*dx_v(i,j)/surf(i,j)*hey(i,j)
!
   ! 4- terme du vecteur second membre du systeme lineaire
   !    issu de la divergence de flux zonale
   !    (flux a l est et a l ouest du point xe(i,j) considere)
   !    Prise en compte de tous les apports d eau dans
   !    la colonne d eau. Remplissage
   !    du tableau hex qui ressert dans les flux de traceurs
   !------------------------------------------------------------

         phieautot=0.0_rsh
         DO k=1,kmax
           phieautot=phieautot+phieau(k,i,j)
         END DO

         y2dxe(i,j)=xeu(i,j)*fwetp(i,j)-halfdt/surf(i,j)*(                   &
                              alphap*( hey(i,j  )*v(i,j  )*dx_v(i,j  )       &
                                     - hey(i,j-1)*v(i,j-1)*dx_v(i,j-1) )     &
                             +alphap*( hex(i  ,j)*u(i  ,j)*dy_u(i  ,j)       &
                                     - hex(i-1,j)*u(i-1,j)*dy_u(i-1,j) )     &
                             +alphan*( hex(i  ,j)*ustar(i  ,j)*dy_u(i  ,j)   &
                                     - hex(i-1,j)*ustar(i-1,j)*dy_u(i-1,j) ) &
                                     - phieautot )

#ifdef key_waveforcing
         IF (l_wave_brgm_smith .OR. l_wave_brgm_phillips) THEN
           IF (h0(i  ,j  )+xeu(i  ,j  ) >= hlim_calc .AND. &
               h0(i+1,j  )+xeu(i+1,j  ) >= hlim_calc .AND. &
             h0(i-1,j  )+xeu(i-1,j  ) >= hlim_calc) y2dxe(i,j) = y2dxe(i,j) - halfdt * (Qx(i+1,j)-Qx(i-1,j))/(dx_u(i,j)+dx_u(i-1,j))
           IF (h0(i  ,j  )+xeu(i  ,j  ) >= hlim_calc .AND. &
               h0(i  ,j+1)+xeu(i  ,j+1) >= hlim_calc .AND. &
             h0(i  ,j-1)+xeu(i  ,j-1) >= hlim_calc) y2dxe(i,j) = y2dxe(i,j) - halfdt * (Qy(i,j+1)-Qy(i,j-1))/(dy_v(i,j)+dy_v(i-1,j))
         END IF
! Attention forme non conservative a priori
#endif

       END IF

   !&E   ***************   end include dyn2dy_cty   ******************************
