   !!======================================================================

  SUBROUTINE dyn2dy_tls_obc_smooth(j1,j2,i,tfb,tfh,v,xe,xeuv)
#if defined key_lans_turb || defined key_leray_turb
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn2dy_tls_obc  ***
   !&E
   !&E ** Purpose : Modify coefficients of matrix
   !&E              to introduce implicit part of the sea surface pressure gradient
   !&E
   !&E ** Description : - Characteristics method (ssh and u)
   !&E                  - or Pure prescription of sea surface height
   !&E
   !&E ** Called by : dyn2dxy, dyn2dyx, dyn3dxy, dyn3dyx
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xe(i,j1), xe(i,j1+1), xe(i,j2), xe(i,j2-1)
   !&E                     v(i,j1), v(i,j1+1), v(i,j2-1), v(i,j2-2)
   !&E
   !&E ** Reference : Blayo & Debreu, 2005. Ocean modelling, 9, 231-252
   !&E                Vandermeirsch, convention 03/2.210 458F, sept 2004
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !  2007-05  (F. Vandermeirsch, J. Marin) Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, T. Duhaut, M. Honnorat) time scheme
   !&E       !  2008-08  (T. Odaka) MPI
   !&E       !  2009-01  (A. Mano T. Odaka) MPI2D
   !&E       !  2009-01  (F. Vandermeirsch) introduction of obc_ssh_* and characteristics method
   !&E       !  2009-07  (F. Vandermeirsch)  MC 2nd order scheme
   !&E       !  2009-07  (V. Garnier) MPI2d for characteristics method
   !&E       !  2010-04  (F. Vandermeirsch) OBC : introduction of Flather operator
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2011-11  (F. Vandermeirsch) correction : spatial average
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comcolig2d, ONLY : y2dv,y2dvs,y2dv_moy,av,cv
   USE comfluxdo
   USE obccombine, ONLY : obc_vn, obc_vs, obc_sshn, obc_sshs, &
                          l_obc_south,l_obc_north,            &
                          l_obc_char
   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                            INTENT( in )     :: j1,j2,i
   REAL(KIND=rsh),                                     INTENT( in )     :: tfh,tfb
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: xe
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: xeuv
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT ( in ) :: v

   !! * local declarations
   REAL(KIND=rsh)               :: h,w1,w3

   !!----------------------------------------------------------------------
   !! * Executable part 

   ! introduce sea suface height at open boundaries

     IF (l_obc_south) THEN
     IF_MPI ( j1==jmin+1+jshift_obc_south .AND. ljmin<=j1 .AND. j1<=ljmax) THEN
      IF_AGRIF (Agrif_Root()) THEN
       y2dv_moy(i,j1) = y2dv(i,j1) 
      ELSE_AGRIF
       INST_AGRIF y2dv_moy(i,j1) = y2dv(i,j1) 
      ENDIF_AGRIF
     ENDIF_MPI
     END IF

     IF (l_obc_north) THEN
     IF_MPI ( j2==jmax-jshift_obc_north .AND. ljmin<=j2 .AND. j2<=ljmax) THEN
      IF_AGRIF (Agrif_Root()) THEN
       y2dv_moy(i,j2-1) = y2dv(i,j2-1)
      ELSE_AGRIF
       INST_AGRIF y2dv_moy(i,j2-1) = y2dv(i,j2-1)
      ENDIF_AGRIF
     ENDIF_MPI
     END IF
#endif
  END SUBROUTINE dyn2dy_tls_obc_smooth
