  SUBROUTINE dyn3d_wzs(wz)
#ifdef key_leray_turb
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE flx_cd  ***
   !&E
   !&E ** Purpose : estimate of the vertical velocity from the vertical
   !&E              integration (from the bottom to the surface)
   !&E              of the local continuity equation 
   !&E
   !&E ** Description : prognostic estimate
   !&E
   !&E ** Called by : step3dxy.F90,step3dyx.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,xe3,xe2,fsxeau,fsyeau,phieau,dy_u,dx_v,surf
   !&E
   !&E ** Modified variables : wz,fzeau
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E
   !&E---------------------------------------------------------------------

# include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE smoothing

   USE comvars3d, ONLY : wzs
   USE comvars2d, ONLY : ig,id,h0,dy_u,dx_v,l_obc_cycl_y,l_obc_cycl_x,jb,jh,surf,halfdt,hm
   USE comcolig2d, ONLY   : wz2d, wzs2d
  
#ifdef key_MPI_2D
   USE toolmpi,   ONLY : ex_j_rsh, ex_i_rsh,ex_peri_x_rsh,ex_peri_y_rsh
#endif

   IMPLICIT NONE

   !! * Arguments

  !! * Arguments
   REAL(KIND=rsh),DIMENSION(0:kmax,liminm1:limaxp1,ljminm1:ljmaxp1),INTENT(IN) :: wz

   INTEGER :: k, j, i

   !!----------------------------------------------------------------------
   !! * Executable part

   DO  k=0,kmax

         wz2d(limin:limax,ljmin:ljmax) = wz(k,limin:limax,ljmin:ljmax)

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,wz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,wz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(wz2d)

       CALL smoothw(wz2d, wzs2d)

       wzs(k,limin:limax,ljmin:ljmax)  = wzs2d(limin:limax,ljmin:ljmax)

   END DO

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     CALL ex_peri_x_rsh(1,kmax+1,liminm1,limaxp1,ljminm1,ljmaxp1,wzs(:,liminm1:limaxp1,ljminm1:ljmaxp1))
#else
     DO j=ljmin,ljmax
       wzs(:,ig(j),j)=wzs(:,id(j)-1,j)
       wzs(:,id(j),j)=wzs(:,ig(j)+1,j)
     END DO
#endif
   ELSE
     IF (limax==imax) wzs(:,imax-ishift_obc_east,:)=wzs(:,imax-1-ishift_obc_east,:)
     IF (limin==imin) wzs(:,imin+1+ishift_obc_west,:)=wzs(:,imin+2+ishift_obc_west,:)
   END IF
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
     CALL ex_peri_y_rsh(1,kmax+1,liminm1,limaxp1,ljminm1,ljmaxp1,wzs(:,liminm1:limaxp1,ljminm1:ljmaxp1))
#else
     DO i=limin,limax
       wzs(:,i,jb(i))=wzs(:,i,jh(i)-1)
       wzs(:,i,jh(i))=wzs(:,i,jb(i)+1)
     END DO
#endif
   ELSE
     IF (ljmin==jmin) wzs(:,limin:limax,jmin+1+jshift_obc_south)=wzs(:,limin:limax,jmin+2+jshift_obc_south)
     IF (ljmax==jmax) wzs(:,limin:limax,jmax-jshift_obc_north)=wzs(:,limin:limax,jmax-1-jshift_obc_north)
   END IF
!$OMP END SINGLE


   CALL_MPI ex_i_rsh(-1,1,kmax+1,liminm1,limaxp1,ljminm1,ljmaxp1,wzs(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax+1,liminm1,limaxp1,ljminm1,ljmaxp1,wzs(:,liminm1:limaxp1,ljminm1:ljmaxp1))

   PRINT_DBG*, 'exit DYN3D_WZ routine'

#endif
  END SUBROUTINE dyn3d_wzs
