!*********    including dyn3dx_adv_centered.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_adv_centered.h  ***
   !&E
   !&E ** Purpose : estimate the advection terms of the zonal momentum
   !&E              equation
   !&E
   !&E ** Description : explicit part
   !&E                  centered scheme
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,wz,mn,ms,dx,dy_f,tfiusr
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------


    k=1

    wdui=0.5_rsh*(wz(k,i,j)+wz(k,i+1,j))*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

   !   u * du/dx
     yz_exp(k,i,j)=yz_exp(k,i,j)-uz(k,i,j)*(centered_duzdx(uz,  k, i, j) &

   !   w * du/dz
                  +wdui) &
   !   v * du/dy : fully explicit 
                   -vz_u(k,i,j)*centered_duzdy(uz, k, i, j)  &                  
   !   u * v * tg(phi) / R
                  +uz(k,i,j)*vz_u(k,i,j)*tfiusr(i,j)

#ifdef key_diagdyn_details
   !   u * du/dx
    diagx_nlt_ududx(k,i,j)=uz(k,i,j)*(uz(k,i+1,j)-uz(k,i-1,j))/(dx(i,j)+dx(i+1,j))
   !   w * du/dz
    diagx_nlt_wdudz(k,i,j)=wdui
   !   v * du/dy : fully explicit 
    diagx_nlt_vdudy(k,i,j)=((uz(k,i,j  )-uz(k,i,j-1))*vz_u(k,i,j)*ms(i,j)/dy_f(i,j-1)           &
                   +(uz(k,i,j+1)-uz(k,i,j  ))*vz_u(k,i,j)*mn(i,j)/dy_f(i,j  ) )*0.5_rsh
#endif

    DO k=2,kmax-1

      wdus=0.5_rsh*(wz(k,i,j)+wz(k,i+1,j))*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

   !   u * du/dx
     yz_exp(k,i,j)=yz_exp(k,i,j)-(uz(k,i,j)*centered_duzdx(uz, k, i, j) &

   !   w * du/dz
                  +(wdui*dsigw(k)+wdus*dsigw(k-1))/(dsigw(k)+dsigw(k-1))) &

   !   v * du/dy : fully explicit
                   -vz_u(k,i,j)*centered_duzdy(uz, k, i, j)  &

   !   u * v * tg(phi) / R
                   +uz(k,i,j)*vz_u(k,i,j)*tfiusr(i,j)

        wdui=wdus

#ifdef key_diagdyn_details
   !   u * du/dx
       diagx_nlt_ududx(k,i,j)=uz(k,i,j)*(uz(k,i+1,j)-uz(k,i-1,j))/(dx(i,j)+dx(i+1,j))
   !   w * du/dz
       diagx_nlt_wdudz(k,i,j)=(wdui*dsigw(k)+wdus*dsigw(k-1))/(dsigw(k)+dsigw(k-1))
   !   v * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=((uz(k,i,j  )-uz(k,i,j-1))*vz_u(k,i,j)*ms(i,j)/dy_f(i,j-1)           &
                              +(uz(k,i,j+1)-uz(k,i,j  ))*vz_u(k,i,j)*mn(i,j)/dy_f(i,j  ) )*0.5_rsh
#endif

     END DO

     k=kmax

   !   u * du/dx
     yz_exp(k,i,j)=yz_exp(k,i,j)-(uz(k,i,j)*centered_duzdx(uz, k, i, j) &

   !w* du/dz
                  + wdui*dsigw(k)/(dsigw(k)+dsigw(k-1))) &
   !   v * du/dy : fully explicit
                   -vz_u(k,i,j)*centered_duzdy(uz, k, i, j)  &

   !   u * v * tg(phi) / R
                  +uz(k,i,j)*vz_u(k,i,j)*tfiusr(i,j)

#ifdef key_diagdyn_details
   !   u * du/dx
     diagx_nlt_ududx(k,i,j)=uz(k,i,j)*(uz(k,i+1,j)-uz(k,i-1,j))/(dx(i,j)+dx(i+1,j))
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdui*dsigw(k)/(dsigw(k)+dsigw(k-1))
   !   v * du/dy : fully explicit 
     diagx_nlt_vdudy(k,i,j)=((uz(k,i,j  )-uz(k,i,j-1))*vz_u(k,i,j)*ms(i,j)/dy_f(i,j-1)           &
                            +(uz(k,i,j+1)-uz(k,i,j  ))*vz_u(k,i,j)*mn(i,j)/dy_f(i,j  ) )*0.5_rsh
#endif
!********* end including dyn3dx_adv_centered.h      ****************************
