!*********    including dyn3dx_adv_centered_lans.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_adv_centered.h  ***
   !&E
   !&E ** Purpose : estimate the advection terms of the zonal momentum
   !&E              equation
   !&E
   !&E ** Description : explicit part
   !&E                  centered scheme
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,wz,mn,ms,dx,dy_f,tfiusr
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------


    k=1

!1-Non linear Leray terms

    wdui=0.5_rsh*(wzs(k,i,j)+wzs(k,i+1,j))*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

   !   us * du/dx

     yz_exp(k,i,j)=yz_exp(k,i,j)-uzs(k,i,j)*(centered_duzdx(uz, k, i, j) &

   !   w * du/dz
                  +wdui) &

   !   v * du/dy : fully explicit 

       -vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)  &
                  
   !   u * v * tg(phi) / R
                  +uzs(k,i,j)*vzs_u(k,i,j)*tfiusr(i,j)

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl  = 0.

#ifdef key_lans_turb
        lans_nl     = lans_nl_centered_lon(uz, uzs, vz_u, vzs_u, k, i, j)
#endif
        yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   us * du/dx
    diagx_nlt_ududx(k,i,j)=uzs(k,i,j)*centered_duzdx(uz, k, i, j)
   !   w * du/dz
    diagx_nlt_wdudz(k,i,j)=wdui
   !   v * du/dy : fully explicit 
    diagx_nlt_vdudy(k,i,j)=vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)
#endif

    DO k=2,kmax-1

!1-Non linear Leray terms

      wdus=0.5_rsh*(wzs(k,i,j)+wzs(k,i+1,j))*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

   !   u * du/dx

     yz_exp(k,i,j)=yz_exp(k,i,j)-(uzs(k,i,j)*centered_duzdx(uz, k, i, j) &

   !   w * du/dz
                  +(wdui*dsigw(k)+wdus*dsigw(k-1))/(dsigw(k)+dsigw(k-1))) &

   !   v * du/dy : fully explicit
                   -vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)  &

   !   u * v * tg(phi) / R
                   +uzs(k,i,j)*vzs_u(k,i,j)*tfiusr(i,j)

        wdui=wdus

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl  = 0.

#ifdef key_lans_turb
        lans_nl = lans_nl_centered_lon(uz, uzs, vz_u, vzs_u, k, i, j) 
#endif
        yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   u * du/dx
       diagx_nlt_ududx(k,i,j)=uzs(k,i,j)*centered_duzdx(uz, k, i, j)
   !   w * du/dz
       diagx_nlt_wdudz(k,i,j)=(wdui*dsigw(k)+wdus*dsigw(k-1))/(dsigw(k)+dsigw(k-1))
   !   v * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)
#endif

     END DO

     k=kmax

!1-Non linear Leray terms

   !   u* du/dx

     yz_exp(k,i,j)=yz_exp(k,i,j)-(uzs(k,i,j)*centered_duzdx(uz, k, i, j) &

   !w* du/dz
                  + wdui*dsigw(k)/(dsigw(k)+dsigw(k-1))) &

   !   vs * du/dy : fully explicit
                   -vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)  &

   !   u * v * tg(phi) / R
                  +uzs(k,i,j)*vzs_u(k,i,j)*tfiusr(i,j)

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl  = 0.

#ifdef key_lans_turb
        lans_nl = lans_nl_centered_lon(uz, uzs, vz_u, vzs_u, k, i, j) 
#endif
        yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   u * du/dx
     diagx_nlt_ududx(k,i,j)=uzs(k,i,j)*centered_duzdx(uz, k, i, j)
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdui*dsigw(k)/(dsigw(k)+dsigw(k-1))
   !   v * du/dy : fully explicit 
     diagx_nlt_vdudy(k,i,j)=vzs_u(k,i,j)*centered_duzdy(uz, k, i, j)
#endif

!********* end including dyn3dx_adv_centered_lans.h      ****************************
