!**************** include dyn3dx_atmforc *********************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_atmforc.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the met. forcing
   !&E              (atmospherical pressure gradient and wind stress)
   !&E              for the zonal momentum forcing (2D and 3d equation)
   !&E
   !&E ** Description : explicit
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : pa,tosx,hzutx,htx,dx_u
   !&E
   !&E ** Modified variables : yz_exp,intz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E
   !&E---------------------------------------------------------------------

   	!Sea surface pressure

        grad_pa = forward_derivative_2d_xdir(pa, dx_u(i,j), i, j)/rhoref

        pressure_term = grad_pa

#ifdef key_lans_turb
        pressure_lans = forward_derivative_2d_xdir(dyn_press_lans, dx_u(i,j), i, j)
	pressure_term = grad_pa + pressure_lans
#endif
    	yz_exp(:,i,j) = yz_exp(:,i,j) - pressure_term

    	intz_exp(i,j) = intz_exp(i,j) - pressure_term

    	!tau_sx

    	yz_exp(kmax,i,j)= yz_exp(kmax,i,j)+tosx(i,j)/MAX(htx(i,j),hminfrot)/rhoref/dsigu(kmax)

    	intz_exp(i,j)   = intz_exp(i,j)   +tosx(i,j)/MAX(htx(i,j),hminfrot)/rhoref

!**************** en d include dyn3dx_atmforc *********************************
