   !&E   ***************   include dyn3dx_cont.h   **********************************
   !&E
   !&E ** Purpose : remplissage des termes du systeme lineaire
   !&E              provenant de la discretisation de l equation de
   !&E              continuite dans le systeme d equation de Saint-
   !&E              Venant sur l ensemble de la grille de calcul.
   !&E
   !&E ** Description : Equation couplee a celle de la conservation de la
   !&E                  quantite de mouvement dans la direction Ouest-Est.
   !&E
   !&E
   !&E ** Called by : dyn2dx
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History : 2006-05
   !&E              2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-08  (T. Duhaut) time scheme Rq : schema des traceurs 
   !&E                               semble instable si u est decomposee en
   !&E                               alpha u et (alpha - 1) u
   !&E       !  2013-08  (AC. Bennis) Add Stokes drift divergence
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E-------------------------------------------------------------------------

   ! 1- terme de diagonale de la matrice tridiagonale
   !    du modele barotrope
   !--------------------------------------------------

   ! 2- terme de sous-diagonale de la matrice tridiagonale
   !    issu de l implicitation de la divergence de flux zonale
   !    (flux a l ouest du point xe(i,j) considere). Remplissage
   !    du tableau hex qui ressert dans les flux de traceurs
   !------------------------------------------------------------

      axe(i,j)=-halfdt*dy_u(i-1,j)/surf(i,j)*hex(i-1,j)
      bxe(i,j)=fwet(i,j)

   ! 3- terme de sous-diagonale de la matrice tridiagonale
   !    issu de l implicitation de la divergence de flux zonale
   !    (flux a l est du point xe(i,j) considere). Remplissage
   !    du tableau hex qui ressert dans les flux de traceurs
   !-----------------------------------------------------------

       cxe(i,j)=halfdt*dy_u(i,j)/surf(i,j)*hex(i,j)

   ! 4- terme du vecteur second membre du systeme lineaire
   !    issu de la divergence de flux meridienne
   !    (flux au nord et sud  du point xe(i,j) considere)
   !    Prise en compte de tous les apports d eau dans
   !    la colonne d eau. Remplissage
   !    du tableau hey qui ressert dans les flux de traceurs
   !---------------------------------------------------------- 

#ifdef key_lans_turb
       y2dxe(i,j)=xec(i,j)*fwetp(i,j)-halfdt/surf(i,j)*(              &
                           ( hey(i,j  )*(gam*vstar(i,j)+gams*vs_star(i,j))*dx_v(i,j  )      &
                           - hey(i,j-1)*(gam*vstar(i,j-1)+gams*vs_star(i,j-1))*dx_v(i,j-1) )    &
                           - SUM(phieau(:,i,j))                       &
#else
       y2dxe(i,j)=xec(i,j)*fwetp(i,j)-halfdt/surf(i,j)*(              &
                           ( hey(i,j  )*vstar(i,j  )*dx_v(i,j  )      &
                           - hey(i,j-1)*vstar(i,j-1)*dx_v(i,j-1) )    &
                           - SUM(phieau(:,i,j))                       &
#endif
#ifdef key_wave_qeuler_dynamics
                           +( hey(i,j)*IUSY_v(i,j)*dx_v(i,j)          &
                           -  hey(i,j-1)*IUSY_v(i,j-1)*dx_v(i,j-1) )  &
                           +( hex(i,j)*IUSX_u(i,j)*dy_u(i,j)          &
                           -  hex(i-1,j)*IUSX_u(i-1,j)*dy_u(i-1,j) )  &		           
#endif 
			   )               

   !&E   ***************   end include dyn3dx_cont.h   ******************************
