!*********    including dyn3dx_ipg_dj.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_dj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the density jacobian
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dx_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Second order density jacobian of ROMS/SCRUM
   !&E                Shchepetkin and Mc Williams (2003)
   !&E
   !&E ** History :
   !&E       !  2010-09  (M. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

   dxb=dx_u(i,j)

!    dxex=(xec(i+1,j)-xec(i,j))/dxb

   ! surface
   k=kmax
   z1 = f_zu(h0(i,j),xec(i,j),k,i,j)
   z3 = f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)

   ! extrapolation to estimate the buoyancy at the sea surface
#ifdef key_profil_back
   dzb=f_zu(h0(i  ,j),xec(i  ,j),k,i  ,j)-f_zu(h0(i  ,j),xec(i  ,j),k-1,i  ,j)
   b2surf=boz2(k,i  ,j)+(boz2(k,i  ,j)-boz2(k-1,i  ,j))/dzb*(xec(i  ,j)-z1)
   dzb=f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)-f_zu(h0(i+1,j),xec(i+1,j),k-1,i+1,j)
   b4surf=boz2(k,i+1,j)+(boz2(k,i+1,j)-boz2(k-1,i+1,j))/dzb*(xec(i+1,j)-z3)
#else
   dzb=f_zu(h0(i  ,j),xec(i  ,j),k,i  ,j)-f_zu(h0(i  ,j),xec(i  ,j),k-1,i  ,j)
   b2surf=boz(k,i  ,j)+(boz(k,i  ,j)-boz(k-1,i  ,j))/dzb*(xec(i  ,j)-z1)
   dzb=f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)-f_zu(h0(i+1,j),xec(i+1,j),k-1,i+1,j)
   b4surf=boz(k,i+1,j)+(boz(k,i+1,j)-boz(k-1,i+1,j))/dzb*(xec(i+1,j)-z3)
#endif

#ifdef key_song
!cval je ne comprends pas pourquoi je dois utiliser la pseudo formule en surface
!cval alors que dans les couches internes elle est moins precise ?????
#ifdef key_profil_back
   ! horizontal gradient of sigma-averaged buoyancy dxbs = -(b1+b2-b3-b4)
   dxbs = 0.5_rlg*(b4surf+boz2(k,i+1,j)-b2surf-boz2(k,i,j))
   ! vertical gradient of x-averaged depth (z4-z3+z2-z1)
   dszx = 0.5_rlg*(-z3-z1+xec(i+1,j)+xec(i,j))
   ! vertical gradient of x-averaged buoyancy (b4+b3-b2-b1)/dx
   dsbx = 0.5_rlg*(b4surf+b2surf-boz2(k,i+1,j)-boz2(k,i,j))
   ! horizontal gradient of sigma-averaged depth dxzs = -(z1+z2-z3-z4)
   dxzs = 0.5_rlg*(z3+z4-z1-z2)
#else
   ! horizontal gradient of sigma-averaged buoyancy dxbs = -(b1+b2-b3-b4)
   dxbs = 0.5_rlg*(b4surf+boz(k,i+1,j)-b2surf-boz(k,i,j))
   ! vertical gradient of x-averaged depth (z4-z3+z2-z1)
   dszx = 0.5_rlg*(-z3-z1+xec(i+1,j)+xec(i,j))
   ! vertical gradient of x-averaged buoyancy (b4+b3-b2-b1)/dx
   dsbx = 0.5_rlg*(b4surf+b2surf-boz(k,i+1,j)-boz(k,i,j))
   ! horizontal gradient of sigma-averaged depth dxzs = -(z1+z2-z3-z4)
   dxzs = 0.5_rlg*(z3+z4-z1-z2)
#endif

   ! vertical integration of the jacobian
   jacobian = (dxbs*dszx-dsbx*dxzs)/dxb

#else
#ifdef key_profil_back
   ! pseudo-flux form
   ! f21=0.5_rlg*(b1+b2)*(z2-z1)
   f21=0.5_rlg*(boz2(k,i,j)+b2surf)*(xec(i,j)-z1)
   ! f42=0.5_rlg*(b2+b4)*(z4-z2)
   f42=0.5_rlg*(b2surf+b4surf)*(xec(i+1,j)-xec(i,j))
   ! f43=0.5_rlg*(b3+b4)*(z4-z3)
   f43=0.5_rlg*(boz2(k,i+1,j)+b4surf)*(xec(i+1,j)-z3)
   ! f31=0.5_rlg*(b1+b3)*(z3-z1)
   f31=0.5_rlg*(boz2(k,i,j)+boz2(k,i+1,j))*(z3-z1)
#else
   ! pseudo-flux form
   ! f21=0.5_rlg*(b1+b2)*(z2-z1)
   f21=0.5_rlg*(boz(k,i,j)+b2surf)*(xec(i,j)-z1)
   ! f42=0.5_rlg*(b2+b4)*(z4-z2)
   f42=0.5_rlg*(b2surf+b4surf)*(xec(i+1,j)-xec(i,j))
   ! f43=0.5_rlg*(b3+b4)*(z4-z3)
   f43=0.5_rlg*(boz(k,i+1,j)+b4surf)*(xec(i+1,j)-z3)
   ! f31=0.5_rlg*(b1+b3)*(z3-z1)
   f31=0.5_rlg*(boz(k,i,j)+boz(k,i+1,j))*(z3-z1)
#endif

   ! vertical integration of the jacobian
   jacobian=(f31+f43-f42-f21)/dxb
#endif

#ifdef key_ktest_iniv3d
   ipx(k,i,j) = jacobian
#endif
   yz_exp(k,i,j)= yz_exp(k,i,j)                   &
                  ! vertical integration
                  +jacobian
                  ! buoyancy * gradient of ssh
!                  +dxex*(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg

   DO k=kmax-1,1,-1
     z2 = z1
     z4 = z3
     z1 = f_zu(h0(i,j),xec(i,j),k,i,j)
     z3 = f_zu(h0(i+1,j),xec(i+1,j),k,i+1,j)

#ifdef key_profil_back
     ! horizontal gradient of sigma-averaged buoyancy dxbs = -(b1+b2-b3-b4)
     dxbs = 0.5_rlg*(boz2(k+1,i+1,j)+boz2(k,i+1,j)-boz2(k+1,i,j)-boz2(k,i,j))
     ! vertical gradient of x-averaged depth (z4-z3+z2-z1)
     dszx = 0.5_rlg*(z4-z3+z2-z1)
     ! vertical gradient of x-averaged buoyancy (b4+b3-b2-b1)/dx
     dsbx = 0.5_rlg*(boz2(k+1,i+1,j)+boz2(k+1,i,j)-boz2(k,i+1,j)-boz2(k,i,j))
     ! horizontal gradient of sigma-averaged depth dxzs = -(z1+z2-z3-z4)
     dxzs = 0.5_rlg*(z3+z4-z1-z2)
#else
     ! horizontal gradient of sigma-averaged buoyancy dxbs = -(b1+b2-b3-b4)
     dxbs = 0.5_rlg*(boz(k+1,i+1,j)+boz(k,i+1,j)-boz(k+1,i,j)-boz(k,i,j))
     ! vertical gradient of x-averaged depth (z4-z3+z2-z1)
     dszx = 0.5_rlg*(z4-z3+z2-z1)
     ! vertical gradient of x-averaged buoyancy (b4+b3-b2-b1)/dx
     dsbx = 0.5_rlg*(boz(k+1,i+1,j)+boz(k+1,i,j)-boz(k,i+1,j)-boz(k,i,j))
     ! horizontal gradient of sigma-averaged depth dxzs = -(z1+z2-z3-z4)
     dxzs = 0.5_rlg*(z3+z4-z1-z2)
#endif
     ! vertical integration of the jacobian
     jacobian = jacobian + (dxbs*dszx-dsbx*dxzs)/dxb

     ! vertical integration
#ifdef key_ktest_iniv3d
     ipx(k,i,j) = jacobian
#endif
     yz_exp(k,i,j)=yz_exp(k,i,j)+jacobian

   END DO


!*********  end including dyn3dx_ipg_dj.h      ****************************
