!*********    including dyn3dx_ipg_pj.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_pj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the pressure jacobian
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dx_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E
   !&E---------------------------------------------------------------------

    dxb=dx_u(i,j)

!    dhxx=(h0(i+1,j)-h0(i,j))/dxb
    dhxx=(xec(i+1,j)-xec(i,j))/dxb
    htxee=h0(i+1,j)+xec(i+1,j)
    htxeo=h0(i,j)+xec(i,j)
    dhtx=(htxee-htxeo)/dxb
    pri=0.0_rlg

    k=kmax
   ! internal pressure gradient : dP/dx part
#ifdef key_profil_back
    bms=(htxee*boz2(kmax,i+1,j)-htxeo*boz2(kmax,i,j))/dxb
    pri=pri-bms*sig(kmax)
    tbc=pri+(boz2(k,i,j)+boz2(k,i+1,j))*0.5_rlg*(sig(k)*dhtx+dhxx)
#else
    bms=(htxee*boz(kmax,i+1,j)-htxeo*boz(kmax,i,j))/dxb
    pri=pri-bms*sig(kmax)
    tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*(sig(k)*dhtx+dhxx)
#endif
#ifdef key_ktest_iniv3d
    ipx(k,i,j)=tbc
#endif
    yz_exp(k,i,j)  = yz_exp(k,i,j)+tbc

    DO k=kmax-1,1,-1
#ifdef key_profil_back
      bm=(htxee*boz2(k,i+1,j)-htxeo*boz2(k,i,j))/dxb
      pri=pri+(bm+bms)*0.5_rlg*dsigw(k)
      tbc=pri+(boz2(k,i,j)+boz2(k,i+1,j))*0.5_rlg*(sig(k)*dhtx+dhxx)
#else
      bm=(htxee*boz(k,i+1,j)-htxeo*boz(k,i,j))/dxb
      pri=pri+(bm+bms)*0.5_rlg*dsigw(k)
      tbc=pri+(boz(k,i,j)+boz(k,i+1,j))*0.5_rlg*(sig(k)*dhtx+dhxx)
#endif
#ifdef key_ktest_iniv3d
      ipx(k,i,j)=tbc
#endif

      yz_exp(k,i,j)  = yz_exp(k,i,j)+tbc
      bms=bm
    END DO

!********* end  including dyn3dx_ipg_pj.h      ****************************
