   !&E   ***************   include dyn3dx_mtm   **********************************
   !&E
   !&E ** Purpose : remplissage des termes du systeme lineaire
   !&E              provenant de la discretisation de l equation de conservation
   !&E              de la quantite de mouvement du systeme d equations
   !&E              relatif au mode barotrope du modele 3D dans la direction Est-Ouest
   !&E              sur l ensemble de la grille.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History : 2006-05
   !&E              2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2010-07  (L. Debreu, F. Dumas) rewritting of ADI
   !&E       !  2011-03  (F. Dumas) bottom stress parametrisation + remove case 2 and 3 for tidal flat processing
   !&E       !  2011-10  (L. Pineau, F. Dumas, V. garnier) correction bottom stress parametrisation
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E       !  2014-01  (F. Dumas, V. Garnier) Correction bottom stress parametrisation to take into account
   !&E                                          the bottom velocity in the implicit part of the 2D momentum eq.
   !&E
   !&E
   !&E  RQ V Garnier vis2d est homogene spatialement donc OK
   !&E     neanmoins, il faudrait plutot utiliser vish_xe(1,:,:) et vis
   !&E     car ainsi le coef pourrait varier spatialement
   !&E     Aujourd hui, faux si cela varie spatialement, il faut definir
   !&E     coef de viscosite en u et v
   !&E
   !&E-------------------------------------------------------------------------

     gdt = grav*dt

   ! cas d un point u sans eau de part ni d autre
   ! drying u-point : no water on the right and left

     IF (igdu(i,j) == 4) THEN

       av(i,j)=0.0_rlg
       bv(i,j)=1.0_rlg
       cv(i,j)=0.0_rlg
       y2dv(i,j)=0.0_rlg
#if defined key_lans_turb || defined key_leray_turb
       y2dv_aux(i,j)=0.0_rlg
       y2dv_moy(i,j)=0.0_rlg
#endif

   ! cas d un point u avec de l eau de part et d autre
   ! u-point surrounded by water

     ELSE IF (igdu(i,j) == 1) THEN

       ! sea surface pressure gradient : implicit part
       av(i,j)=-alphan*0.5_rlg*gdt/dx_u(i,j)                ! i-1
       cv(i,j)= alphan*0.5_rlg*gdt/dx_u(i,j)                ! i

      ! time evolution (u at n+1/2)
#ifdef key_dyn_adv_linear
       bv(i,j)=1.0_rlg
#else
       ! bottom stress (implicit part)
       ft=halfdt*cdvit(i,j)/htx(i,j)
#ifdef key_no_bottom_stress
       ft=0.0_rsh
#elif key_linear_bottom_stress
       !ft=frot_lin/htx(i,j)
       ft=halfdt*frot_lin/MAX(htx(i,j),hminfrot)
#endif
       bv(i,j)=1.0_rlg/(                                &
               ! time evolution (u at n+1/2)
               1.0_rlg                                  &
               ! bottom stress : implicit part
#if defined key_lans_turb || defined key_leray_turb
               +ft)
#else
               +ft*cfroi)
#endif
#endif

#if defined key_lans_turb || defined key_leray_turb
       ! rhs : explicit terms
       y2dv_aux(i,j)=                           	&
                 ! time evolution (u at n-1/2)
                   u(i,j)      		        &
                 ! other explicit terms integrated from the bottom to the sea surface
                  +halfdt*intz_exp(i,j)      &     
                 ! explicit part of the sea surface pressure gradient
                  -0.5_rlg*alphap*gdt*forward_dxedx(xec, i, j)  
                 ! bottom stress : taking into account the bottom velocity to correct the implicit part
                 !-ft*(uz(1,i,j)-u(i,j))*cfroi                          
                 ! -ft*uz(1,i,j)*cfroi
		 ! +ft*u(i,j)*cfroi
#else
       ! rhs : explicit terms
       y2dv(i,j)=                                       &
                 ! time evolution (u at n-1/2)
                 u(i,j)                                 &
                 ! other explicit terms integrated from the bottom to the sea surface
                 +halfdt*intz_exp(i,j)                  &
                 ! explicit part of the sea surface pressure gradient
                 -0.5_rlg*gdt*alphap*forward_dxedx(xec, i, j) &
                 ! bottom stress : taking into account the bottom velocity to correct the implicit part
                 -ft*(uz(1,i,j)-u(i,j))*cfroi
#endif

     ELSE IF (igdu(i,j) == 5) THEN

   ! low depth

       v_utmp  = interp_v_to_east(v, i, j)
#if defined key_lans_turb || defined key_leray_turb
       vs_utmp = interp_v_to_east(vs, i, j)
#endif
       ! sea surface pressure gradient : implicit part
       av(i,j)=-alphan*0.5_rlg*gdt/dx_u(i,j)                ! i-1
       cv(i,j)= alphan*0.5_rlg*gdt/dx_u(i,j)                ! i
#ifdef key_dyn_adv_linear
       ! time evolution (u at n+1/2)
       bv(i,j)=1.0_rlg
#else
       ! bottom stress (implicit part)

       vit=SQRT(u(i,j)*u(i,j)+v_utmp*v_utmp)
       ft=halfdt*vit/htx(i,j)*(0.4_rsh/LOG(htx(i,j)/(exp1*z0b(i,j))+1.0_rsh))**2
#ifdef key_no_bottom_stress
       ft=0.0_rsh
#elif key_linear_bottom_stress
       !ft=frot_lin/htx(i,j)
       ft=halfdt*frot_lin/MAX(htx(i,j),hminfrot)
#endif
       bv(i,j)=1.0_rlg/(                                &
               ! time evolution (u at n+1/2)
               1.0_rlg                                  &
               ! bottom stress : implicit part
#if defined key_lans_turb || defined key_leray_turb
               +ft)
#else
               +ft*cfroi)
#endif
#endif


#if defined key_lans_turb || defined key_leray_turb
#      include "dyn3dx_mtm_lowdepth_lans.h"
#else 
#      include "dyn3dx_mtm_lowdepth.h"
#endif
	
   !&E   ***************   end include dyn3dx_mtm   ******************************
