!*********    including dyn3dx_wave.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_corio.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the GLM formulation of the radiation
   !&E              stress for the zonal momentum forcing (3D equation)
   !&E
   !&E ** Description : explicit
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2013-08  (AC Bennis)  Original code
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E---------------------------------------------------------------------

   ! add Coriolis term
   yz_exp(:,i,j)=yz_exp(:,i,j)+fu(i,j)*USY_v(:,i,j)


   hrmsm=0.5_rsh*(hrms(i,j)+hrms(i+1,j))

   DO k=1,kmax

!cval change les resultats via htx sans doute
     ! smb1xm is assigned to mean wave induced pressure (J/bhd : Bernouilli head) and the wind stress from WW3
     smb1xm=-(wave_bhd(i+1,j)-wave_bhd(i,j))/dx_u(i,j) + 0.5_rsh*(wave_tauwx(i,j)+wave_tauwx(i+1,j))
     ! We add here the momentum source due to wave breaking from the surface down to hrms/2 depth
     IF (hrmsm /= 0.0_rsh .and. ABS(htx(i,j)*(sig(kmax)-sig(k))) <= 0.5_rsh*hrmsm ) &
       smb1xm=smb1xm+4.0_rsh*0.5_rsh*(wave_tauox(i,j)+wave_tauox(i+1,j))/hrmsm* &
                                     (1.0_rsh-2.0_rsh*htx(i,j)*(sig(kmax)-sig(k))/hrmsm)

!cval L inttroduction de la ligne ci-dessous m a change les resultats. Je ne comprends pas pourquoi
!le test IF (igdu(i,j)/=4) comme dans wavequeler n a aucune importance
! a cause des OBC sur vz ?
     ! compute the dv/dx at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dxvz_u=0.5_rsh*(vz(k,i+1,j)+vz(k,i+1,j-1)-(vz(k,i,j)+vz(k,i,j-1)))/dx_f(i,j)

     ! compute the du/dy at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dyuz_u=(uz(k,i,j+1)-uz(k,i,j-1))/(dy_f(i,j-1)+dy_f(i,j))

     ! compute the y-component of the STokes drift  at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     USY_u=0.5_rsh*(USY(k,i,j)+USY(k,i+1,j))

     ! compute the z-component of the STokes drift  at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     USZ_u=0.5_rsh*(USZ(k,i,j)+USZ(k,i+1,j))

     ! compute the dv/dsigma at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dsigvz_u=0.25_rsh*(dsigvz_v(k,i,j-1)+dsigvz_v(k,i,j)+dsigvz_v(k,i+1,j-1)+dsigvz_v(k,i+1,j))

     ! compute the dsigma/dy at u point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dsigy_u=0.25_rsh*(dsigy_v(k,i,j)+dsigy_v(k,i+1,j)+dsigy_v(k,i,j-1)+dsigy_v(k,i+1,j-1))

     yz_exp(k,i,j)= yz_exp(k,i,j)+dxvz_u*USY_u                   &
                                 -dyuz_u*USY_u                   &
                                 -USZ_u/htx(i,j)*dsiguz_u(k,i,j) &
                                 +smb1xm                         &
                                 +dsigvz_u*dsigx_u(k,i,j)*USY_u  &
                                 -dsiguz_u(k,i,j)*dsigy_u*USY_u  &
                                 +pk_u(k,i,j)

   END DO

!********* end  including dyn3dx_wave.h      ****************************
