!*******************    including dyn3dy_adv_centered.h      *******************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_adv_centered.h  ***
   !&E
   !&E ** Purpose : estimate the advection terms of the meridional momentum
   !&E              equation
   !&E
   !&E ** Description : explicit part
   !&E                  centered scheme
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,wz,mo,me,dy_f,dy,tfiusr
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------


    k=1
!1-Non linear leray term

    wdvi=0.5_rsh*(wzs(k,i,j)+wzs(k,i,j+1))*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)

   !	v * dv/dy

    	yz_exp(k,i,j)=yz_exp(k,i,j)-(vzs(k,i,j)*centered_dvzdy(vz, k, i, j) &

   !   	w * dv/dz

        +wdvi) &

   !   	u * dv/dx

	-uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)	&

   !   u**2 * tg(phi) / R

        -uzs_v(k,i,j)*uzs_v(k,i,j)*tfivsr(i,j)

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl =  0.

#ifdef key_lans_turb
        lans_nl = lans_nl_centered_lat(uz_v, uzs_v, vz, vzs, k, i, j) 
#endif

	yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   	v * dv/dy
     diagy_nlt_vdvdy(k,i,j)=vzs(k,i,j)*centered_dvzdy(vz, k, i, j) 
   !   	w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvi
   !   	u * dv/dx
     diagy_nlt_udvdx(k,i,j)=uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)
#endif

    DO k=2,kmax-1

!1-Non linear Leray terms

      wdvs=0.5_rsh*(wzs(k,i,j)+wzs(k,i,j+1))*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)

   !   	v * dv/dy

	yz_exp(k,i,j)=yz_exp(k,i,j)-(vz(k,i,j)*centered_dvzdy(vz, k, i, j) &

   !   	w * dv/dz

        +(wdvi*dsigw(k)+wdvs*dsigw(k-1))/(dsigw(k)+dsigw(k-1))) &

   !   	u * dv/dx

       -uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)	&

   !   u**2 * tg(phi) / R

      -uzs_v(k,i,j)*uzs_v(k,i,j)*tfivsr(i,j)		

       wdvi=wdvs

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl =  0.

#ifdef key_lans_turb
        lans_nl  = lans_nl_centered_lat(uz_v, uzs_v, vz, vzs, k, i, j) 
#endif
        yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   v * dv/dy
       diagy_nlt_vdvdy(k,i,j)=vz(k,i,j)*centered_dvzdy(vz, k, i, j)
   !   w * dv/dz
       diagy_nlt_wdvdz(k,i,j)=(wdvi*dsigw(k)+wdvs*dsigw(k-1))/(dsigw(k)+dsigw(k-1))
   !   u * dv/dx
       diagy_nlt_udvdx(k,i,j)=uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)
#endif

     END DO

     k = kmax

!1-Non linear Leray terms

   !   v * dv/dy

     yz_exp(k,i,j)=yz_exp(k,i,j)-(vzs(k,i,j)*centered_dvzdy(vz, k, i, j) &

   !   w * dv/dz
                   +wdvi*dsigw(k)/(dsigw(k)+dsigw(k-1))) &
   !   u * dv/dx

                   -uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)	&

   !   u**2 * tg(phi) / R

                  -uzs_v(k,i,j)*uzs_v(k,i,j)*tfivsr(i,j)

!2-Terms to ensure the Kelvin circulation theorem

        lans_nl =  0.

#ifdef key_lans_turb
        lans_nl = lans_nl_centered_lat(uz_v, uzs_v, vz, vzs, k, i, j)
#endif

        yz_exp(k,i,j) = yz_exp(k,i,j) - lans_nl

#ifdef key_diagdyn_details
   !   v * dv/dy
     diagy_nlt_vdvdy(k,i,j) = vz(k,i,j)*centered_dvzdy(vz, k, i, j)
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvi*dsigw(k)/(dsigw(k)+dsigw(k-1))
   !   u * dv/dx
     diagy_nlt_udvdx(k,i,j)=uzs_v(k,i,j)*centered_dvzdx(vz, k, i, j)
#endif

!********* end including dyn3dy_adv_centered.h      ******************************
