!*************include  dyn3dy_botstress   **********************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_botstress.h  ***
   !&E
   !&E ** Purpose : estimate the drag coefficient and the trend term
   !&E              relative to the bottom stress
   !&E              for the meridional momentum forcing (2D and 3D equation)
   !&E
   !&E ** Description : explicit
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,hzuty,hty,z0b,cdvit
   !&E
   !&E ** Modified variables : yz_exp,intz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original cod
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-07  (F. Dumas) Generalized sigmas
   !&E       !  2013-08  (A-C. Bennis and F. Dumas) Soulsby (1995) and
   !&E                      Yu and Slinn (2003) parameterizations
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E---------------------------------------------------------------------

#ifdef key_wave_soulsby_bottom_stress
   uorb_v=0.5_rsh*uorb(i,j+1)+uorb(i,j)
   FWST_v=0.5_rsh*(FWST(i,j)+FWST(i,j+1))
   shcl=( 0.4_rsh/log(hty(i,j)/(2*z0b(i,j))) )**2*sqrt(nuz(i,j)**2+nvz(i,j)**2)
   shc=shcl*sqrt(nuz(i,j)**2+nvz(i,j)**2)
   shw=0.5_rsh*FWST_v*uorb_v**2
   
   cdvit(i,j)=1.0e-10_rsh
   IF ((shw+shc) /= 0._rsh) &
      cdvit(i,j)=shcl*(1.0_rsh+1.2_rsh*((shw/(shw+shc))**3.2))

#elif key_wave_yu_slinn_bottom_stress
   uorb_v=0.5_rsh*uorb(i,j+1)+uorb(i,j)
   cdvit(i,j)=max(2.0_rsh/pi*cflin*uorb_v,1.0e-10_rsh)
!cval BUG non ? il y avait uorb_u(i,j)
#else

#ifdef key_linear_bottom_stress
    cdvit(i,j)=frot_lin
#else
    vitf=SQRT(uz_v(1,i,j)*uz_v(1,i,j)+vz(1,i,j)*vz(1,i,j))
#if defined key_siggen || defined key_gencoord
    cdf=(0.4_rsh/LOG(0.5_rsh*dsigu(1)*hzuty(1,i,j)/z0b(i,j)+1.0_rsh))**2
    !cdvit(i,j)=min(cdf*vitf,0.01_rsh)
    cdvit(i,j)=cdf*vitf
#else
    cdf = drag_coef(hty(i,j), sig(1), z0b(i,j))
    cdvit(i,j)=cdf*vitf
#endif
#endif
#endif
#if defined key_lans_turb || defined key_leray_turb
   tofyexp = cdvit(i,j)*vz(1,i,j)/hty(i,j)
#else
   tofyexp = cdvit(i,j)*vz(1,i,j)/hty(i,j)*cfroe
#endif
!Update intz_exp expression
    intz_exp(i,j)=intz_exp(i,j)-tofyexp
!*************   end include dyn3dy_botstress   *******************
