!**************** including dyn3dy_hdiff  *****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_hdiff.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to horizontal viscosity
   !&E              terms for the meridional momentum equation
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit part
   !&E                  d(nu*d(vz)/dx)/dx + d(nu*d(vz)/dy)/dy
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : yz_exp,me,mo,dy,dy_v,dx_f,vz,vish_xe,vish_phi
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  1990-10  (F. Dumas ) new formulation
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E
   !&E---------------------------------------------------------------------

     yz_exp(:,i,j)=yz_exp(:,i,j)+ &
!-----------------------------------
!    d2v/dy2
!-----------------------------------
        (vish_xe(:,i,j+1)*(vz(:,i,j+1)-vz(:,i,j  ))/dy(i,j+1) &
        -vish_xe(:,i,j  )*(vz(:,i,j  )-vz(:,i,j-1))/dy(i,j  ))&
         /dy_v(i,j) &

        ! (vish_xe(:,i,j+1)*forward_derivative_3d_ydir(vz, dy(i,j+1), i, j)   &
        ! -vish_xe(:,i,j)*backward_derivative_3d_ydir(vz, dy(i,j), i, j))/dy_v(i,j) &

!-----------------------------------
!    d2v/dx2
!-----------------------------------
       +(vish_phi(:,i  ,j)*me(i,j)*(vz(:,i+1,j)-vz(:,i  ,j))/dx_f(i  ,j)  &
        -vish_phi(:,i-1,j)*mo(i,j)*(vz(:,i  ,j)-vz(:,i-1,j))/dx_f(i-1,j)) &
        /dx_v(i,j)

!	+(vish_phi(:,i,j)*me(i,j)*forward_derivative_3d_xdir(vz, dx_f(i,j), i, j) &
!        -vish_phi(:,i-1,j)*mo(i,j)*backward_derivative_3d_xdir(vz, dx_f(i-1,j), i, j))/dx_v(i,j)

#ifdef key_diagdyn_details
!    d2v/dy2
    diagy_hdif_dnudvdy2(:,i,j)=(vish_xe(:,i,j+1)*(vz(:,i,j+1)-vz(:,i,j  ))/dy(i,j+1) &
                               -vish_xe(:,i,j  )*(vz(:,i,j  )-vz(:,i,j-1))/dy(i,j  ))&
                               /dy_v(i,j)
!    d2v/dx2
    diagy_hdif_dnudvdx2(:,i,j)= (vish_phi(:,i  ,j)*me(i,j)*(vz(:,i+1,j)-vz(:,i  ,j))/dx_f(i  ,j)  &
                                -vish_phi(:,i-1,j)*mo(i,j)*(vz(:,i  ,j)-vz(:,i-1,j))/dx_f(i-1,j)) &
                                /dx_v(i,j)
#endif
!**************** end including dyn3dy_hdiff  *********************
