!*********    including dyn3dy_wave.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_corio.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the GLM formulation of the radiation
   !&E              stress for the meridian momentum forcing (3D equation)
   !&E
   !&E ** Description : explicit
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2013-08  (AC Bennis)  Original code
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E---------------------------------------------------------------------

   ! add Coriolis term
   yz_exp(:,i,j)=yz_exp(:,i,j)-fv(i,j)*USX_u(:,i,j)


   hrmsm=0.5_rsh*(hrms(i,j)+hrms(i,j+1))

   DO k=1,kmax

!cval change les resultats via hty sans doute
     ! smb1xm is assigned to mean wave induced pressure (J/bhd : Bernouilli head) and the wind stress from WW3
     smb1ym=-(wave_bhd(i,j+1)-wave_bhd(i,j))/dy_v(i,j) + 0.5_rsh*(wave_tauwy(i,j)+wave_tauwy(i,j+1))
     ! We add here the momentum source due to wave breaking from the surface down to hrms/2 depth
     IF (hrmsm /= 0.0_rsh .and. ABS(htx(i,j)*(sig(kmax)-sig(k))) <= 0.5_rsh*hrmsm ) &
       smb1ym=smb1ym+4.0_rsh*0.5_rsh*(wave_tauoy(i,j)+wave_tauoy(i,j+1))/hrmsm* &
                                     (1.0_rsh-2.0_rsh*htx(i,j)*(sig(kmax)-sig(k))/hrmsm)

!cval L inttroduction de la ligne ci-dessous m a change les resultats. Je ne comprends pas pourquoi
! SI OBC differente sur vitesses ?
     ! compute the du/dy at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dyuz_v=0.5_rsh*(uz(k,i,j+1)+uz(k,i-1,j+1)-(uz(k,i,j)+uz(k,i-1,j)))/dy_f(i,j)

     ! compute the dv/dx at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dxvz_v=(vz(k,i+1,j)-vz(k,i-1,j))/(dx_f(i-1,j)+dx_f(i,j))

     ! compute the x-component of the STokes drift  at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     USX_v=0.5_rsh*(USX(k,i,j)+USX(k,i,j+1))

     ! compute the z-component of the STokes drift  at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     USZ_v=0.5_rsh*(USZ(k,i,j)+USZ(k,i,j+1))

     ! compute the dv/dsigma at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dsiguz_v=0.25_rsh*(dsiguz_u(k,i,j  )+dsiguz_u(k,i,j+1)+dsiguz_u(k,i-1,j)+dsiguz_u(k,i-1,j+1))

     ! compute the dsigma/dx at v point of the C-grid to compute the vortex force (see Bennis et al 2011
     ! appendix A for the formulation)
     dsigx_v=0.25_rsh*(dsigx_u(k,i,j)+dsigx_u(k,i,j+1)+dsigx_u(k,i-1,j)+dsigx_u(k,i-1,j+1))

     yz_exp(k,i,j)= yz_exp(k,i,j)-dxvz_v*USX_v                   &
                                 +dyuz_v*USX_v                   &
                                 -USZ_v/hty(i,j)*dsigvz_v(k,i,j) &
                                 +smb1ym                         &
                                 -dsigvz_v(k,i,j)*dsigx_v*USX_v  &
                                 +dsiguz_v*dsigy_v(k,i,j)*USX_v  &
                                 +pk_v(k,i,j)

   END DO

!*************    end including dyn3dy_wave.h     ************
