  SUBROUTINE dyn3dy_tls_coef(vz,        &
#if defined key_lans_turb || defined key_leray_turb
                             vzs,       &
#endif
                             boz,       &
                             uz,        &
#if defined key_lans_turb || defined key_leray_turb
                             uzs,       &
#endif
                             u,         &
                             v,         &
#if defined key_lans_turb || defined key_leray_turb
                             us,        &
                             vs,        &
#endif
                             xec,       &

#if defined key_lans_turb || defined key_leray_turb
                             us_star,   &
#endif
                             ustar      &
                            )

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_tls_coef.F90  ***
   !&E
   !&E ** Purpose : estimate all the coefficient of the tridiagonal
   !&E              linear system representative of the meridional momentum 
   !&E              and continuity equations (2D and 3D parts)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dxy.F90, dyn3dyx.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,hc_sig,xec(xeu,xev),zr,boz,igdv,fv,ustar
   !&E                     fv,uz    (dyn3dy_corio.h)
   !&E                     xec,h0,boz   (dyn3dy_ipg_*.h)
   !&E                     vz,wz,why,mo,me,tfivsr,dx_v,dy_v,dx_f,dy (dyn3dy_adv_*.h)
   !&E                     vz,vish_xe,vish_phi,me,mo,dy,dx_f  (dyn3dy_hdiff.h)
   !&E                     vz,hty,z0b (dyn3dy_botstress.h)
   !&E                     pa,tosy,dy_v,hty (dyn3dy_atmforc.h)
   !&E                     potgene,dy_v (dyn3dy_genpot.h)
   !&E                     u,v,xec(xeu,xev),igdv,hy,tfivsr,htx,hty,vis2d,fv,
   !&E                     pa,potgene,intz_exp,dx_f,dx_v,dy,dy_v (dyn3dy_mtm.h)
   !&E                     hex,hey,surf,dx_v,dy_u,ustar,phieau (dyn3dy_cont.h)
   !&E
   !&E
   !&E ** Modified variables : zr,pr,yz_exp,intz_exp
   !&E                         av,bv,cv,y2dv     (dyn3dx_mtm.h)
   !&E                         axe,bxe,cxe,y2dxe (dyn3dx_cont.h)
   !&E
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-08  (T. Odaka) MPI
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2010-03  (A.-C. Bennis)  Introduction of waves radiative stress
   !&E       !  2011-03  (F. Dumas)  bottom stress parametrisation + remove case 2 and 3 for tidal flat
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2014-03  (A.-C. Bennis, F. Dumas, V. Garnier) Review of the implementation of wave-current quasi-eulerian formalism
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"
# define ipx !ipx
# define ipy !ipy

   !! * Modules used
   USE parameters                              ! imin imax rsh etc
   USE interpolation_methods
   USE derivative_methods
   USE bottom_stress
#ifdef key_lans_turb
   USE lans_terms
#endif

   USE comvars2d,  ONLY : l_obc_cycl_y,l_obc_cycl_x
   USE comaree,    ONLY : l_genpot,potgene        ! l_genpot, potgene(dyn3dy_genpot)
   USE comvars2d,  ONLY : ig,id,icon               ! ig id
   USE comvars2d,  ONLY : h0,hx,hy                 ! dyn3dy_ipg_pj.h(h0)
   USE comvars2d,  ONLY : fwet,fwetp
   USE comvars2d,  ONLY : dx_v,dy_u                ! dyn3dy_cont.h
   USE comvars2d,  ONLY : dy_v,dx_f                ! dyn3dy_mtm.h
#if defined key_lans_turb || defined key_leray_turb
   USE comvars2d,  ONLY : dx, dx_u, dy_f, dy     !dyn3dx_adv_quick_lans.h
#endif
   USE comvars2d,  ONLY : dy,tfivsr,fv             ! dyn3dy_ipg_pj.h(h0,hx)
   USE comvars2d,  ONLY : vish_xe,vish_phi         ! dyn3dy_adv_centered.h ( dy,tfiusr)
   USE comvars2d,  ONLY : htx,surf,govlet
   USE comvars2d,  ONLY : jb,jh,vis2d,goulet,hey,hex,z0b,hminfrot,hty,botstressmax,dt,halfdt
#ifdef key_linear_bottom_stress
   USE comvars2d,  ONLY : frot_lin
#endif
   USE comcolig2d, ONLY : igdv                     ! fv(dyn3dy_corio)
   USE comcolig2d, ONLY : y2dxe,y2dv,axe,bxe,cxe,cv,bv,av,y2d_expv ! igdv
!MBo#ifdef key_lans_turb
#if defined key_lans_turb || defined key_leray_turb
!MBo   USE comcolig2d, ONLY : y2dv_aux      ! dyn3dx_cont.h
   USE comcolig2d, ONLY : y2dv_aux,y2dv_moy
#endif
   USE comcolig2d, ONLY : me,mo,why                ! dyn3dy_adv_scheme.h
   USE comcolig3d, ONLY : yz_exp,intz_exp          ! yz_exp(dyn3dy_corio,dyn3dy_ipg_pj.h,dyn3dx_adv_centered.h)
   USE comcolig3d, ONLY : cdvit
#if key_dyn_pg_djcs
   USE comcolig3d, ONLY : zr,pr
#endif
   USE comcolig3d, ONLY : uz_v
#if defined key_lans_turb || defined key_leray_turb
   USE comcolig3d, ONLY : uzs_v
#endif
#ifdef key_lans_turb
  USE comcolig3d, ONLY :  dzdy
  USE comcolig3d, ONLY :  pdyn
  USE comvarp3d,  ONLY :  lans_u, lans_v
#endif
   USE comvars3d,  ONLY : sig, dsigu, sigw, wz, dsigw, rhoref       ! dyn3dy_ipg_pj.h(sig,dsigw)
#if defined key_lans_turb || defined key_leray_turb
   USE comvars3d,  ONLY :  wzs !dyn3dx_adv_quick_lans.h
#endif
#ifdef key_profil_back
   USE comvars3d,  ONLY : boz2
#endif
#ifdef key_wave_soulsby_bottom_stress
   use comwave,    ONLY : nuz,nvz,FWST,uorb
#endif
#ifdef key_wave_qeuler_dynamics
   USE comvars2d,  ONLY : dy_f                     ! dyn3dy_wave.h
   use comwave,    ONLY : IUSX_u,USX_u,USX,USZ,IUSY_v,dsigy_v,   &
                          dsigx_u,dsiguz_u,dsigvz_v,pk_v,hrms
   USE wave,       ONLY : wave_bhd,wave_tauwy,wave_tauoy
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen,  ONLY : hzuty,hzwty,hzex,hzey,hc_sig,csu_sig
#endif
   USE toolgeom,   ONLY : f_hzw,f_zu,f_hzu
   USE cometdyn,   ONLY : pa,tosy                   ! forcmet
   USE comfluxdo,  ONLY : phieau                   ! phieau(dyn3dy_cont)
#ifdef key_MPI_2D
   USE toolmpi,    ONLY : ex_i_rsh,  ex_j_rsh
   USE_MPI toolmpi, ONLY: ADD_TAG,min_all_mpi_rsh,max_all_mpi_rsh
#endif
#ifdef key_diagdyn
   USE diagdyn,   ONLY : mask_diagy,diagy_cor,diagy_corcor,diagy_ipg,diagy_nlt,diagy_hdif,diagy_vdif,diagy_epg
#ifdef key_diagdyn_details
   USE diagdyn,   ONLY : diagy_nlt_udvdx,diagy_nlt_vdvdy,diagy_nlt_wdvdz,diagy_hdif_dnudvdx2,diagy_hdif_dnudvdy2
#endif
#endif

   USE obcapplyuz

   IMPLICIT NONE

!! * Arguments

   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: u,v
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: us, vs
#endif
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: xec


   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: ustar
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: us_star
#endif
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uz,vz
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uzs,vzs
#endif
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: boz

   !! * Local declarations

   INTEGER                            :: i,j,k,j1,j2

#ifdef key_dyn_pg_djcs
   REAL(KIND=rsh), PARAMETER          :: demi   = 0.5_rsh         !dyn3dy_ipg_cs.h
   REAL(KIND=rsh), PARAMETER          :: uncinq = 0.2_rsh
   REAL(KIND=rsh), PARAMETER          :: undouz = 1.0_rsh/12.0_rsh
   REAL(KIND=rsh)                     :: cff,tbc,dyzr,dyboz, &
                                      dyzrp1,dybozp1,     &
                                      dyzrp2,dybozp2
   REAL(KIND=rsh),DIMENSION(0:kmax)         :: dzr,dboz
!real(KIND=rsh),dimension(kmax,imin:imax,jmin:jmax) :: zr,pr
#elif key_dyn_pg_pint
   REAL(KIND=rsh),DIMENSION(kmax)   :: p_primu_j,p_primu_jp1,sec_gp
   REAL(KIND=rsh),DIMENSION(0:kmax) :: p_prim_interface_j,  &
                                       p_prim_interface_jp1
   REAL(KIND=rsh)                   :: hz_j_kp1,hz_j,hz_jp1_kp1,hz_jp1, &
                                       zk_j,zk_jp1,htxes,htxen,dyb
#elif key_dyn_pg_dj
   REAL(KIND=rsh)    :: dyb,jacobian,dybs,dszy,dsby,dyzs,z1,z2,z3,z4,dzb,b2surf,b4surf
   REAL(KIND=rsh)    :: f31,f43,f42,f21
#else
#if defined key_siggen || defined key_gencoord
   REAL(KIND=rlg)    :: bms,bm,pri,tbc,dyb,dhyy,dhcy,dxey,bhzn,bhzs       !dyn3dy_ipg_pj_sig.h
#else
   REAL(KIND=rsh)    :: bms,bm,pri,tbc,dyb,dhyy,dhty,htxen,htxes !dyn3dy_ipg_pj.h
#endif
#endif
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#if defined key_dyn_adv_quick || defined key_dyn_adv_quickest
   REAL(KIND=rsh)    :: wmo,wm,wp,wdvdz,vp,um,vc,vdvdy,up,vm,udvdx,uv,vel_adv  !dyn3dy_adv_quickest.h
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh)    :: wsdvdz, usdvdx, vsdvdy, udusdy, vdvsdy        !dyn3dy_adv_quick_lans.h
#endif

#else
   REAL(KIND=rsh)    :: wdvi,wdvs                                     !dyn3dy_adv_centered.h
#endif
#endif
#endif
!#ifdef key_lans_turb
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2) :: us_v
   REAL(KIND=rsh), DIMENSION(1:kmax)  :: lans_nl_sig
   REAL(KIND=rsh)    :: lans_nl, dusdy
#endif
   REAL(KIND=rsh)    :: vitf,cdf,tofyexp                              !dyn3dy_botstress.h

   REAL(KIND=rsh)    :: grad_pa                            !

#ifdef key_wave_soulsby_bottom_stress
   REAL(KIND=rsh)    :: uorb_v,shw,shcl,shc,FWST_v                    !dyn3dx_botstress.h
#endif
#ifdef key_wave_qeuler_dynamics

   REAL(KIND=rsh)    :: USX_v,dxvz_v,dyuz_v,smb1ym,hrmsm,USZ_v, &     !dyn3dy_wave.h
                        dsiguz_v,dsigx_v
#endif
   REAL(KIND=rsh)    :: ft,gdt,vit,factft,dvdx, u_vtmp       !dyn3dy_mtm.h
#ifdef key_diagdyn  
   REAL(KIND=rsh)    :: zp,effet_pression ! zp :save previous value of yz_exp(kmax) before adding wind stress
   REAL(kind=rsh),DIMENSION(1:kmax)  :: z_prev
#endif

   REAL(KIND=rsh)    :: value

   LOGICAL   ::   Agrif_Root_OMP

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter DYN3DY_TLS_COEF routine'

   Agrif_Root_OMP = agrif_root()

#if key_dyn_pg_djcs
   !************************
   !     Preliminary step
   !************************
#ifdef key_MPI_2D
!$OMP DO schedule(runtime) private(i,j,k,cff,dzr,dboz)
#else
!$OMP DO schedule(dynamic,5) private(i,j,k,cff,dzr,dboz)
#endif
   DO j = ljmin, ljmax
     DO i = liminp2,limaxm1
       IF (j >= jb(i) .AND. j<=jh(i) ) THEN
         IF (h0(i,j) > -valmanq) THEN
! if dzr too small over wet-drying cells         IF (h0(i,j)+xec(i,j) > h0fond) THEN

           zr(kmax,i,j) = f_zu(h0(i,j),xec(i,j),kmax,i,j)
           DO k = kmax-1, 1, -1
             zr(k,i,j) = f_zu(h0(i,j),xec(i,j),k,i,j)
             dzr(k) = zr(k+1,i,j)-zr(k,i,j)
#ifdef key_profil_back
             dboz(k) = boz2(k+1,i,j)-boz2(k,i,j)
#else
             dboz(k) = boz(k+1,i,j)-boz(k,i,j)
#endif
           END DO
           dzr(kmax) = dzr(kmax-1)
           dboz(kmax) = dboz(kmax-1)
           dzr(0) = dzr(1)
           dboz(0) = dboz(1)

           DO k = kmax,1,-1
             cff = 2.0_rsh*dboz(k)*dboz(k-1)
             IF (cff > epsilon) THEN
               dboz(k) = cff / (dboz(k)+dboz(k-1))
             ELSE
               dboz(k) = 0.0_rsh
             ENDIF
             dzr(k) = 2.0_rsh*dzr(k)*dzr(k-1) / (dzr(k)+dzr(k-1))
           END DO

! pressure vertically interpolated
#ifdef key_profil_back
           pr(kmax,i,j) = (  boz2(kmax,i,j) + &
                             0.5_rsh * (xec(i,j)-zr(kmax,i,j)) * (boz2(kmax,i,j)-boz2(kmax-1,i,j)) / (zr(kmax,i,j)-zr(kmax-1,i,j)) ) &
                                     * (xec(i,j)-zr(kmax,i,j))
#else
           pr(kmax,i,j) = (  boz(kmax,i,j) + &
                             0.5_rsh * (xec(i,j)-zr(kmax,i,j)) * (boz(kmax,i,j)-boz(kmax-1,i,j)) / (zr(kmax,i,j)-zr(kmax-1,i,j)) ) &
                                     * (xec(i,j)-zr(kmax,i,j))
#endif
           DO k = kmax-1, 1, -1
#ifdef key_profil_back
             pr(k,i,j) = pr(k+1,i,j) +                                               &
                         demi*(  (boz2(k+1,i,j)+boz2(k,i,j))*(zr(k+1,i,j)-zr(k,i,j)) &
                         - uncinq*( (dboz(k+1)-dboz(k))*(zr(k+1,i,j)-zr(k,i,j)   &
                                             -undouz*(dzr(k+1)+dzr(k)))          &
                                   -(dzr(k+1)-dzr(k))*(boz2(k+1,i,j)-boz2(k,i,j) &
                                             -undouz*(dboz(k+1)+dboz(k))) )  )
#else
             pr(k,i,j) = pr(k+1,i,j) +                                               &
                         demi*(  (boz(k+1,i,j)+boz(k,i,j))*(zr(k+1,i,j)-zr(k,i,j))   &
                         - uncinq*( (dboz(k+1)-dboz(k))*(zr(k+1,i,j)-zr(k,i,j)   &
                                             -undouz*(dzr(k+1)+dzr(k)))          &
                                   -(dzr(k+1)-dzr(k))*(boz(k+1,i,j)-boz(k,i,j)   &
                                             -undouz*(dboz(k+1)+dboz(k))) )  )
#endif
           END DO

         END IF
       END IF
     END DO
   END DO
!$OMP END do

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,zr(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh( 0,1,kmax,limin  ,limaxp1,ljmin  ,ljmaxp1,pr(:,limin  :limaxp1,ljmin  :ljmaxp1))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(zr,pr)
#endif

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljmin,ljmax
        DO i=limin,limax

!                IF (i >= MAX0(limin,2+ishift_obc_west) .AND. i <= MIN0(limax,imax-1-ishift_obc_east) .AND. j >= ljminp1 .AND. j <= ljmax ) THEN
                IF (i >= MAX0(limin,2+ishift_obc_west) .AND. i <=MIN0(limax,imax-1-ishift_obc_east) .AND. j >= ljmin .AND. j <= ljmaxm1 ) THEN
        
                                uz_v(:,i,j)  = interp_uz_to_north(uz, i, j)
#if defined key_lans_turb || defined key_leray_turb
                                uzs_v(:,i,j) = interp_uz_to_north(uzs, i, j)
                                us_v(i,j)  = interp_u_to_north(us, i, j)
#endif
                ELSE
                                uz_v(:,i,j)  = uz(:,i, j)
#if defined key_lans_turb || defined key_leray_turb
                                uzs_v(:,i,j) = uzs(:,i, j)
                                us_v(i,j) = us( i, j)
#endif              
                END IF
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2, ljmaxp2, uz_v(:,liminm2:limaxp2, ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2, ljmaxp2, uz_v(:,liminm2:limaxp2, ljminm2:ljmaxp2))
!MBo#ifdef key_lans_turb
#if defined key_lans_turb || defined key_leray_turb
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2, ljmaxp2, uzs_v(:,liminm2:limaxp2, ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2, ljmaxp2, uzs_v(:,liminm2:limaxp2, ljminm2:ljmaxp2))

   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2, ljmaxp2, us_v(liminm2:limaxp2, ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2, ljmaxp2, us_v(liminm2:limaxp2, ljminm2:ljmaxp2))
#endif
OMPMPI end master
OMPMPI barrier
#if defined key_lans_turb || defined key_leray_turb
OMPMPI flush(uz_v, uzs_v, us_v)
#else
OMPMPI flush(uz_v)
#endif

#ifdef key_lans_turb
!$OMP DO SCHEDULE(RUNTIME)
    DO k = 1, kmax
        DO j=ljmin,ljmax
                DO i=limin,limax
                        IF(igdv(i,j) == 1) THEN
                                dzdy(k,i,j) = calc_dzdy(xec, k, i, j)
                                pdyn(k,i,j) = pdyn_lans_lat(uzs_v, vzs, k, i, j)
                        END IF
                END DO
        END DO
    END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,pdyn(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,pdyn(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(pdyn)
#endif

!$OMP DO schedule(dynamic,5)                                    &
!$OMP& private(i,j,k)                                           &
#ifdef key_dyn_pg_djcs
!$OMP& ,cff,tbc,dyzr,dyboz,dyzrp1,dybozp1,dyzrp2,dybozp2        &
#elif key_dyn_pg_pint
!$OMP& ,p_primu_j,p_primu_jp1,p_prim_interface_j                &
!$OMP& ,p_prim_interface_jp1,sec_gp,zk_j,zk_jp1,htxes,htxen,dyb &
!$OMP& ,hz_j_kp1,hz_j,hz_jp1_kp1,hz_jp1                         &
#elif key_dyn_pg_dj
!$OMP& ,dyb,jacobian,dybs,dszy,dsby,dyzs,z1,z2,z3,z4            &
!$OMP& ,dzb,b2surf,b4surf,f31,f43,f42,f21                       &
#else
#if defined key_siggen || defined key_gencoord
!$OMP& ,bms,bm,pri,tbc,dyb,dhyy,dhcy,dxey,bhzn,bhzs             &
#else
!$OMP& ,bms,bm,pri,tbc,dyb,dhyy,dhty,htxen,htxes                &
#endif
#endif
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#if defined key_dyn_adv_quick || defined key_dyn_adv_quickest
!$OMP& ,vm,vp,vdvdy,up,udvdx,wm,wp,wdvdz,wmo,um,vc,uv           &
#if defined key_lans_turb || defined key_leray_turb
!$OMP& ,vsdvdy,usdvdx,wsdvdz,us_vtmp                            &
#endif
#if defined key_lans_turb || defined key_leray_turb
!$OMP& ,udusdy, vdvsdy                                          &
#endif
# else
!$OMP& ,wdvi,wdvs                                               &
#endif
#endif
#endif
#if defined key_diagdyn
!$OMP& ,z_prev                                                  &
#endif
!$OMP& ,vitf,cdf,tofyexp                                        &
!$OMP& ,ft,gdt,vit,factft,dvsdx,u_vtmp                          &
!$OMP& ,j1,j2                                                   &
!$OMP& )


!do j = jmin+1, jmax-1
   DO j=ljminp1,ljmaxm1
!     DO i=liminp2,limaxm1
     DO i=MAX0(limin,2+ishift_obc_west), MIN0(limax,imax-1-ishift_obc_east)
       IF(igdv(i,j) == 1) THEN

!  debut des calculs des termes de tendances
!-------------------------------------------
                                     ! 1- coriolis a l instant n-1/2
#        include "dyn3dy_corio.h"

#ifdef key_diagdyn
         IF (mask_diagy(i,j)) diagy_cor(:,i,j)=fv(i,j)*uz_v(:,i,j)
# endif

#        include "dyn3dy_intz.h"

         !removes barotropic part of coriolis term at time n (or n+1/2 depending on the order of calculation of u and v)
         yz_exp(:,i,j) = yz_exp(:,i,j)-intz_exp(i,j)

#ifdef key_diagdyn
         ! premiere correction pour Coriolis
         IF (mask_diagy(i,j)) diagy_corcor(:,i,j)=yz_exp(:,i,j)
         IF (icon<=1 .AND. mask_diagy(i,j)) z_prev(:)=yz_exp(:,i,j)
#endif	

                                         ! 1.1- coupling with WW3
#ifdef key_wave_qeuler_dynamics
#     include "dyn3dy_wave.h"
#endif

         IF(icon==2) THEN                ! 2- gradient de pression interne a n-1/2
 
#ifdef key_diagdyn
           IF (mask_diagy(i,j)) z_prev(:)=yz_exp(:,i,j)
#endif	

#ifdef key_dyn_pg_djcs
#          include "dyn3dy_ipg_cs.h"
#elif key_dyn_pg_pint
#          include "dyn3dy_ipg_pint.h"
#elif key_dyn_pg_dj
#          include "dyn3dy_ipg_dj.h"
#else
#if defined key_siggen || defined key_gencoord
#          include "dyn3dy_ipg_pj_sig.h"
#else
#          include "dyn3dy_ipg_pj.h"
#endif
#endif

#ifdef key_diagdyn
           IF (mask_diagy(i,j)) THEN
             diagy_ipg(:,i,j)=yz_exp(:,i,j)-z_prev(:)
             z_prev(:)=yz_exp(:,i,j)
           END IF
#endif
         END IF
                                     ! 3- termes d advection a l instant n-1
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#ifdef key_dyn_adv_quick
#if defined key_lans_turb || defined key_leray_turb
#        include "dyn3dy_adv_quick_lans.h"
#else
#        include "dyn3dy_adv_quick.h"
#endif
#elif key_dyn_adv_quickest
#if defined key_lans_turb || defined key_leray_turb
#        include "dyn3dy_adv_quickest_lans.h"
#else
#        include "dyn3dy_adv_quickest.h"
#endif
#else
#if defined key_lans_turb || defined key_leray_turb
#        include "dyn3dy_adv_centered_lans.h"
#else
#        include "dyn3dy_adv_centered.h"
#endif
#endif
#endif
#endif

#if defined key_diagdyn && ! defined key_diagdyn_details
         IF (mask_diagy(i,j)) THEN
           diagy_nlt(:,i,j)=-(yz_exp(:,i,j)-z_prev(:))
           z_prev(:)=yz_exp(:,i,j)
         END IF
#endif
                                     ! 4- terme de diffusion horizontale a n-1
#        include "dyn3dy_hdiff.h" 


#if defined key_diagdyn && ! defined key_diagdyn_details
         IF (mask_diagy(i,j)) diagy_hdif(:,i,j)=yz_exp(:,i,j)-z_prev(:)
#endif	
                                     ! 5- integration des termes precedents sur
                                     !    la verticale a destination du modele
                                     !    barotrope

#        include "dyn3dy_intz.h"

                                     ! 6- tension sur le fond  a l instant n-1
#if ! defined key_no_bottom_stress
#        include "dyn3dy_botstress.h"
#endif
                                     ! 8- forcage meteo tension de surface et
                                     !    pression a n-1/2

#ifdef key_diagdyn
         IF (mask_diagy(i,j)) THEN
           zp=yz_exp(kmax,i,j)  ! on sauve en surface pour recuperer ensuite  le vent
           z_prev(:)=yz_exp(:,i,j) ! sur la verticale on va ensuite ajouter la pression
         END IF
#endif
#        include "dyn3dy_atmforc.h"

#ifdef key_diagdyn
         IF (mask_diagy(i,j)) THEN
           effet_pression=yz_exp(1,i,j)-z_prev(1)
           diagy_vdif(kmax,i,j)=yz_exp(kmax,i,j)-zp-effet_pression
           diagy_epg(:,i,j)=effet_pression ! on aura pu creer un tableau en plus...
         END IF
#endif

         IF (l_genpot) THEN             ! 9- potentiel generateur de maree a n-1/2
#          include "dyn3dy_genpot.h" 
         END  IF

   ! adds barotropic part of coriolis term estimated with velocity extrapolated in time (ustar)

!Interpolation method
#if defined key_lans_turb || defined key_leray_turb
         yz_exp(:,i,j) = yz_exp(:,i,j)-fv(i,j)*interp_u_to_north(us_star, i, j)
         intz_exp(i,j) = intz_exp(i,j)-fv(i,j)*interp_u_to_north(us_star, i, j)
#else
         yz_exp(:,i,j) = yz_exp(:,i,j)-fv(i,j)*interp_u_to_north(ustar, i,j)
         intz_exp(i,j) = intz_exp(i,j)-fv(i,j)*interp_u_to_north(ustar, i,j)
#endif

#ifdef key_diagdyn
         ! seconde correction pour Coriolis
         IF (mask_diagy(i,j)) diagy_corcor(:,i,j)= &
            -(diagy_corcor(:,i,j)-fv(i,j)*0.25_rlg*(us_star(i,j)+us_star(i,j+1)+us_star(i-1,j)+us_star(i-1,j+1)))
#endif	  

       END  IF                             ! fin test igdv

! 15- calcul des coefficients des matrices tridiagonales pour
!     chaque colonne de la grille C, issus de l implicitation
!     des termes de gradient de pression externe (equation de
!     quantite de mouvement dyn3dy_mtm ) et des termes de divergence
!     du flux Nord-Sud (equation de conservation de la masse dyn3dy_cont)
!     et des vecteurs second-membre

#      include "dyn3dy_mtm.h"
#      include "dyn3dy_cont.h"

!#ifdef key_lans_turb
!       y2dv_aux(i,j) = y2dv_aux(i,j)*bv(i,j)
!#endif
     END DO
   END DO
!$OMP end do

   PRINT_DBG*, 'exit DYN3DY_TLS_COEF routine'

  END SUBROUTINE dyn3dy_tls_coef
