   !!======================================================================

  SUBROUTINE init_flux

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_flux  ***
   !&E
   !&E ** Purpose : initialize all the variables that are cumulated over several half time-steps
   !&E              for the advection computation
   !&E
   !&E ** Description : step3dxy, step3dxy
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2004-10-04  last modifications
   !&E       !  2011-11     (V. Garnier, B. Thouvenin) New organisation for the initializations of the flux
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used

   USE parameters
   USE comfluxdo,    ONLY : phieau
#ifdef key_castest
   USE comfluxdo,    ONLY : evap,rain
   USE comvars2d,    ONLY : surf,ijour,imois
   USE comvars2d,    ONLY : dx, dy      
   USE sflxatm,      ONLY : rad, senslos
   USE cometemp,     ONLY : sflx_solarcst
   USE comvars2d,    ONLY : t,tdeb,dt
   USE comvarp2d,    ONLY : temp
   USE cometdyn    , ONLY : tosx,tosy,wind_veloc,wind_dir
#endif
#ifdef key_substance
   USE comsubstance, ONLY : phicon
#if defined key_phicondrycell 
   USE comsubstance, ONLY : phicon_drycell
   USE comvars2d,    ONLY : h0fond,h0
   USE comvarp2d,    ONLY : ssh
#endif
#endif
   !USE cometemp,ONLY:phitemp,phisal
   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER :: i,j
   REAL(kind=rsh)  :: d3
   
#ifdef key_castest
  INTEGER,DIMENSION(31)    :: inter_vent
  REAL(kind=rsh)  :: windvit,windvit2,winddir,radpert
  REAL(kind=rlg)  :: tdeban
  REAL(kind=rlg)  :: x_ij, y_ij, Ly
 ! DATA inter_vent /1,1,0,0,0,1,1,0,0,0,1,1,0,0,0,1,1,0,0,0,1,1,0,0,0,0,1,1,0,0,0/
  DATA inter_vent /1,0,0,1,0,1,0,0,1,0,1,0,1,0,0,1,0,0,1,0,1,0,0,1,0,0,1,0,1,0,0/
!  DATA inter_vent /1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0/
#endif
   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP PARALLEL DO PRIVATE(i,j,d3)
   DO j= ljmin,ljmax
     phieau(:,:,j)=0.0_rlg
#ifdef key_substance
     phicon(:,:,:,j)=0.0_rsh
#ifdef key_phicondrycell
     DO i=liminp1,limax
       d3=h0(i,j)+ssh(i,j)
       IF (d3 > h0fond) THEN
         phicon(:,:,i,j)=phicon_drycell(:,:,i,j)
         phicon_drycell(:,:,i,j)=0.0_rsh
       ENDIF
     END DO
#endif
#endif
   ENDDO
!$OMP END PARALLEL DO

#ifdef key_castest
   IF (num_testcase==5) THEN
!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
     DO j=ljminp1,ljmax
       DO i = liminp1,limax
         phieau(kmax,i,j) = 0.001_rsh*(rain(i,j)-evap(i,j))*surf(i,j)
       END DO
     END DO
!$OMP END DO
!$OMP END PARALLEL

   ELSE IF (num_testcase == 36 .OR. num_testcase == 37 ) THEN
! cas test 1DV bio
      tdeban=INT((t-tdeb)/31536000.0_rlg)*31536000.0_rlg+tdeb
    ! sflx_solarcst=(200.0_rsh+150.0_rsh*sin(2*pi/365.0_rsh/86400.0_rsh*(t-tdeb+21600000.0_rsh)))*max(0.0_rsh,sin(2*pi/86400.0_rsh*(t-tdeb-21600.0_rsh)))
     sflx_solarcst=(200.0_rsh+150.0_rsh*sin(2*pi/365.0_rsh/86400.0_rsh*(t-tdeb+21600000.0_rsh)))
     rad(limin:limax,ljmin:ljmax)=sflx_solarcst*max(0.0_rsh,sin(2*pi/86400.0_rsh*(t-tdeb-21600.0_rsh)))
     radpert=-10.0_rsh+80.0_rsh*sin(2*pi/365.0_rsh/86400.0_rsh*(t-tdeban-24480000.0_rsh))- &
                       260.0_rsh*sin(2.0_rsh*pi*tanh(0.00013_rsh*(t-tdeban-21000000.0_rsh)/3600.0_rsh))
     senslos(limin:limax,ljmin:ljmax)=radpert-sflx_solarcst*0.4_rsh

     windvit2=(wind_veloc*(1.0_rsh+0.4_rsh*sin(2*pi/365.0_rsh/86400.0_rsh*(t-tdeb)))*inter_vent(ijour))**2
     winddir=wind_dir+80.0_rsh*sin(2*pi/365.0_rsh/86400.0_rsh*(t-tdeb+5356800))
     if(mod(t,43200.0_rlg)<=dt .and.limin==imin .and. ljmin==jmin ) THEN
    !         write(*,*) 'vent : windvit2,winddir',windvit2,winddir
             write(*,*) 't,tdeb,jour,mois',t,tdeb,ijour,imois
             write(*,*) 'solar,rad,senslos,radpert',sflx_solarcst,rad(limin,ljmax),senslos(limin,ljmax),radpert
     endif
     tosx(:,:)=-windvit2*SIN((winddir)*pi/180.0_rsh)
     tosy(:,:)=-windvit2*COS((winddir)*pi/180.0_rsh)
   END IF

!  IF (num_testcase==11) THEN
!
!        Ly = dy(1,1)*jmax
!        DO i = limin,limax
!           DO j = ljmin,ljmax
!              y_ij = j*dy(i,j)
!              tosx(i,j)=SIN(pi*y_ij/Ly)
!         WRITE(*,*) "y_ij=", y_ij, "tosx=", tosx(i,j)
!           END DO
!        END DO
!
!   END IF
#endif

   PRINT_DBG*, 'exit INIT_FLUX routine'

  END SUBROUTINE init_flux
