SUBROUTINE cas_lans_bc_current

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_advcst  ***
   !&E
   !&E ** Purpose : definition of a test case with log inlet velocity
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : cas_init
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables : hx, hy, h0, sal, temp, dy, dhx, dhy, dsigu, dsigw
   !&E                         surf, fisac, gwac, dgac, dfiac, fu, fv, sig, sigw 
   !&E                         tfiusr, tfivsr, goulet, govlet, ig, id, jb, jh 
   !&E                         ssh,u,v,uz,vz
   !&E
   !&E ** Reference : 
   !&E
   !&E ** History :
   !&E       !  2006     (F. Vandermeirsch) module -Original code
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2011-11  (V. Garnier) Spatial extension of
   !ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

    !! * Modules used
    USE parameters, ONLY : rsh,rlg,riolg,kmax,jmin,ljmin,ljminm2,jmax,ljmax,ljmaxp2, &
                           imin,limin,liminm2,imax,limax,limaxp2,obc_width,grav,pi, &
                           ishift_obc_west,ishift_obc_east,jshift_obc_south,jshift_obc_north, &
                           valmanq
 
    USE comvars2d, ONLY : fisac,gwac,fu,fv,dtmin,dtmax,dt,suffix,qmax,l_modele2d, &
                          h0fond,hx_g,hy_g,h0_g,hx,hy,h0,icon,jh,jb,ig,id,l_obc_cycl_x,l_obc_cycl_y, &
                          t,tdeb,fvisc,l_smagor,iscreen,vish_phi, vish_xe,vis2d

    USE comvarp2d, ONLY : sal,temp,ssh,u,v

#if defined key_lans_turb || defined key_leray_turb
    USE comvarp2d, ONLY : us,vs
#endif

    USE comvarp3d, ONLY : uz,vz

#if defined key_lans_turb || defined key_leray_turb
    USE comvarp3d, ONLY : uzs,vzs
#endif

    USE comvars3d, ONLY : l_stateeq_lin,tetaref_lin,saliref_lin,kzinit,nzinit, &
                          saliref,tetaref,rhoref,qmaxz,sig,turb_nbeq,turb_2eq_option

    USE output,    ONLY : pasor, l_out_nc4par


    USE comcfo,    ONLY : uext,xext,h0_b,uzext,vzext,uzInlet

    USE obccombine,ONLY : l_obc_south,l_obc_north,l_obc_west,l_obc_east,l_obc_cycl, &
                          obc_ue,obc_uw,obc_us,obc_un,obc_uze,obc_uzw,obc_uzs,obc_uzn, &
                          obc_ve,obc_vw,obc_vs,obc_vn,obc_vze,obc_vzw,obc_vzs,obc_vzn, &
                          obc_sshe,obc_sshw,obc_sshs,obc_sshn,l_obc_char,l_obc_diri,l_obc_mars,l_obc_tide,obc_tide_sshw

    USE obcogcm,   ONLY : obc_tw,obc_se,obc_te,obc_ss,obc_ts,obc_sn,obc_tn,obc_sw,obc_tsref

    USE casinit,   ONLY : cas_init_param,cas_init_grid,cas_init_bathy, &
                          cas_init_io,cas_init_obc,cas_init_meteo,     &
                          cas_init_visc,cas_init_turb,cas_init_tspar,  &
                          cas_init_progvar,cas_init_misc,cas_init_river

    IMPLICIT NONE

    !! * Arguments

    !! * Local declarations
    CHARACTER(LEN=19) :: date_start,date_end,date_start3d,date_startagrif
    INTEGER           :: k, i, j, imid, jmid
    REAL(KIND=rsh)    :: depth, visc_num
    REAL(KIND=rsh)    :: xc, yc, xi, yj, xic, yjc, diam_bump, height_bump
    REAL(KIND=rsh)    :: dxval,dyval,z0bottom,z0surface
    REAL(KIND=rsh)    :: ubar,u0,zpos,sshe
    REAL(KIND=rlg)    :: tool_datosec
 
    !!----------------------------------------------------------------------
    !! * Executable part

    !------------------------------------------------
    !------------------------------------------------
    !!     courant constant zonal et barotrope     !!
    !!     surface libre lineaire                  !!
    !!     densite constant                        !!
    !------------------------------------------------
    !------------------------------------------------


    ! general parametrization (each parameter can be reassigned)
    ! -----------------------

    ! dynamics
    CALL cas_init_param

    ! over-assignement
    !l_corio_cst=.FALSE.  ! beta-plane
    rhoref=1024.0_rsh

     !-----------------------------------------------------
     ! Paramètres bathymétques
     !-----------------------------------------------------

    WRITE(*,*)'####################################'
    WRITE(*,*)'      LANS  TEST CASE 01:           '
    WRITE(*,*)'      BC CONDITIONS:                '
    WRITE(*,*)'                                    '
    WRITE(*,*)'                                   '
    WRITE(*,*)'####################################'


    !Bathymetry
    !----------

    !Bathymetry
    !----------

    depth = 60_rsh ! 60 meters

    ! assignement : input variables for cas_init_grid routine
    dxval = 50._rsh !200 m
    dyval = 50._rsh !200 m

    height_bump  = 10_rsh
    diam_bump    = 600_rsh

    xc = imax*dxval/2_rsh
    yc = jmax*dyval/2_rsh

    h0_g(:,:) = depth

    DO i = imin,imax
         DO j = jmin,jmax

            xi = i*dxval
            yj = j*dyval

            IF((xi > xc - diam_bump/2.).AND.(xi < xc + diam_bump/2.)) THEN

                xic = xi - xc

                h0_g(i,j) = depth - height_bump*EXP(-xic**2.0_rsh/diam_bump**2.0_rsh)

            END IF

         END DO
    END DO

    h0_g(:,:) = depth

    !compute hx

    hx_g(0,:) = h0_g(0,:)
    DO i=imin+1,imax

         hx_g(i,:) = 0.5_rsh*(h0_g(i-1,:) + h0_g(i,:))

    ENDDO

    !compute hy

    hy_g(:,0) = h0_g(:,0)
    DO j=jmin+1,jmax

          hy_g(:,j) = 0.5_rsh*(h0_g(:,j-1) + h0_g(:,j))

    ENDDO
     
    h0(:,:) = h0_g(:,:)
    hx(:,:) = hx_g(:,:)
    hy(:,:) = hy_g(:,:)

    ! grid parametrization
    ! --------------------

    CALL cas_init_grid(dxval,dyval)

    !IO parametrization
    !------------------

    !assignement : input variables for cas_init_io routine
     !assignement : input variables for cas_init_io routine
    date_start= '01/01/2018 00:00:00'
    date_end=   '01/01/2018 06:00:00'
    date_start3d='01/01/2018 00:00:00'
    date_startagrif='01/01/2018 00:00:00'


    tdeb = tool_datosec(date_start)
    !t = tdeb

    dtmin = 1._rsh ! second

    dtmax = dtmin
    dt = dtmax

    suffix='courant'

    CALL cas_init_io(date_start,date_end,date_start3d,date_startagrif,suffix)

    !over-assignement
    pasor(1)= 10*60_rsh! 10 minutes 
    l_out_nc4par = .FALSE.
    iscreen=12

    ! viscosity
    !-----------

    ! none
    CALL cas_init_visc

    !over-assignement
    l_smagor=.FALSE.
    fvisc=10.0_rsh

    visc_num  = fvisc*0.01_rsh*(dxval)**1.15_rsh

    vis2d(:,:)      = visc_num
    vish_xe(:,:,:)  = visc_num
    vish_phi(:,:,:) = visc_num

    !Turbulence
    !----------

    z0bottom   = 1.0e-12
    z0surface  = 1.0e-12
    kzinit     = 1.0e-6
    nzinit     = 1.0e-6  ! Gaspard parametrization

    CALL cas_init_turb(kzinit,nzinit,z0surface,z0bottom)

    !over-assignement
    !turb_nbeq=2
    !turb_2eq_option=2

    !Meteorology
    !-----------

    !No forcing
    CALL cas_init_meteo

    !River and outflows
    !-------------------

    !No river
    CALL cas_init_river

   ! Initialization of velocity field
   !---------------------------------

    !Inflow BC condition

    ubar = 1.0 

    DO k=1,kmax
 
       zpos = sig(k)*depth 
       u0   = ubar/LOG(depth/(2.7183*z0bottom))
       uzInlet(k) = u0*LOG((depth+zpos)/z0bottom)
        
    END DO

    obc_uw(:,:) = ubar

    DO k=1,kmax

      obc_uzw(k,:,:) = uzInlet(k)

    END DO

   ! Initialization of velocity field
   !---------------------------------
    !Initialization
    !--------------

    u(:,:)  = 0.
#if defined key_lans_turb || defined key_leray_turb
    us(:,:) = 0.
#endif

    DO k=1,kmax

       uz(k,:,:)  = u(:,:)

#if defined key_lans_turb || defined key_leray_turb
       uzs(k,:,:) = us(:,:)
#endif

    END DO

    !Open boundaries
    !--------------

    CALL cas_init_obc

    !Over-assignement
    !geometry (where are the open boundaries)
   
    l_obc_south = .TRUE.
    l_obc_north = .TRUE.
    l_obc_west  = .TRUE.
    l_obc_east  = .TRUE.

    ! used scheme to manage obc
    l_obc_mars = .TRUE.

    !l_obc_cycl   = .TRUE.
    !l_obc_cycl_x = .FALSE.
    !l_obc_cycl_y = .TRUE.

    !Percent of wet surface on each cell
    CALL cas_init_misc

    ! over-assignement
    id(:)=id(:)-ishift_obc_east
    ig(:)=ig(:)+ishift_obc_west
    jh(:)=jh(:)-jshift_obc_north
    jb(:)=jb(:)+jshift_obc_south

END SUBROUTINE cas_lans_bc_current
