SUBROUTINE cas_lans_wind_stress 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_advcst  ***
   !&E
   !&E ** Purpose : definition of a test case with constant zonal and
   !baroctropic current
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : cas_init
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables : hx, hy, h0, sal, temp, dy, dhx, dhy, dsigu, dsigw
   !&E                         surf, fisac, gwac, dgac, dfiac, fu, fv, sig, sigw 
   !&E                         tfiusr, tfivsr, goulet, govlet, ig, id, jb, jh 
   !&E                         ssh,u,v,uz,vz
   !&E
   !&E ** Reference : 
   !&E
   !&E ** History :
   !&E       !  2006     (F. Vandermeirsch) module -Original code
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2011-11  (V. Garnier) Spatial extension of
   !ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

    !! * Modules used
    USE parameters, ONLY : rsh,rlg,riolg,kmax,jmin,ljmin,ljminm2,jmax,ljmax,ljmaxp2, &
                           imin,limin,liminm2,imax,limax,limaxp2,obc_width,grav,pi, &
                           ishift_obc_west,ishift_obc_east,jshift_obc_south,jshift_obc_north, &
                           valmanq
 
    USE comvars2d, ONLY : fisac,gwac,fu,fv,dtmin,dtmax,dt,suffix,qmax,l_modele2d, l_corio_cst,&
                          h0fond,hx_g,hy_g,h0_g,hx,hy,h0,icon,jh,jb,ig,id,l_obc_cycl_x,l_obc_cycl_y, &
                          t,tdeb,fvisc,l_smagor,iscreen,vish_xe,vish_phi,vis2d

    USE comvarp2d, ONLY : sal,temp,ssh,u,v

#if defined key_lans_turb || defined key_leray_turb
    USE comvarp2d, ONLY : us,vs
#endif

    USE comvarp3d, ONLY : uz,vz

#if defined key_lans_turb || defined key_leray_turb
    USE comvarp3d, ONLY : uzs,vzs
#endif


    USE comvars3d, ONLY : l_stateeq_lin,tetaref_lin,saliref_lin,kzinit,nzinit, &
                          saliref,tetaref,rhoref,qmaxz,sig,turb_nbeq,turb_2eq_option

    USE output,    ONLY : pasor, l_out_nc4par

    USE obccombine,ONLY : l_obc_south,l_obc_north,l_obc_west,l_obc_east,l_obc_cycl

    USE casinit,   ONLY : cas_init_param,cas_init_grid,cas_init_bathy, &
                          cas_init_io,cas_init_obc,cas_init_meteo,     &
                          cas_init_visc,cas_init_turb,cas_init_tspar,  &
                          cas_init_progvar,cas_init_misc,cas_init_river

    USE cometdyn,    ONLY : tosx,tosy

    IMPLICIT NONE

    !! * Arguments

    !! * Local declarations
    CHARACTER(LEN=19) :: date_start,date_end,date_start3d,date_startagrif
    INTEGER           :: k,i,j
    REAL(KIND=rsh)    :: depth, visc_num
    REAL(KIND=rsh)    :: xc, yc, xi, yj, xic, diam_bump, height_bump
    REAL(KIND=rsh)    :: dxval,dyval,z0bottom,z0surface
    REAL(KIND=rlg)    :: tool_datosec
    REAL(KIND=rlg)    :: beta
 
    !!----------------------------------------------------------------------
    !! * Executable part

    !------------------------------------------------
    !------------------------------------------------
    !!          !!
    !!                       !!
    !!                             !!
    !------------------------------------------------
    !------------------------------------------------


    ! general parametrization (each parameter can be reassigned)
    ! -----------------------

    ! dynamics
    CALL cas_init_param

    ! over-assignement
    l_corio_cst=.FALSE.  ! beta-plane
    rhoref=1024.0_rsh

     !-----------------------------------------------------
     ! Paramètres bathymétques
     !-----------------------------------------------------

    WRITE(*,*)'####################################'
    WRITE(*,*)'     LANS  TEST CASE 02:            '
    WRITE(*,*)'      BC CONDITIONS:   PERIODICi    '
    WRITE(*,*)'     FORCING: WIND STRESS           '
    WRITE(*,*)'                                    '
    WRITE(*,*)'                                    '
    WRITE(*,*)'####################################'

    !Bathymetry
    !----------

    depth = 1632_rsh !1.632 Km

    ! assignement : input variables for cas_init_grid routine
    dxval = 40000._rsh !40 Km
    dyval = 40000._rsh !40 Km
    beta  = 2.0e-11_rsh

    height_bump  = 200._rsh
    diam_bump    = 730000._rsh ! 730 Km

    xc = imax*dxval/2.0_rsh
    yc = jmax*dyval/2.0_rsh

    h0_g(:,:) = depth

    DO i = imin,imax
         DO j = jmin,jmax

            xi = i*dxval
            yj = j*dyval

            IF((xi > xc - diam_bump/2.).AND.(xi < xc + diam_bump/2.)) THEN

                xic = xi - xc

                h0_g(i,j) = depth - height_bump*EXP(-xic**2.0_rsh/diam_bump**2.0_rsh)

            END IF

         END DO
    END DO

   !compute hx

    hx_g(0,:) = h0_g(0,:)
    DO i=imin+1,imax

         hx_g(i,:) = 0.5_rsh*(h0_g(i-1,:) + h0_g(i,:))

    ENDDO

    !compute hy

    hy_g(:,0) = h0_g(:,0)
    DO j=jmin+1,jmax

          hy_g(:,j) = 0.5_rsh*(h0_g(:,j-1) + h0_g(:,j))

    ENDDO
     
    h0(:,:) = h0_g(:,:)
    hx(:,:) = hx_g(:,:)
    hy(:,:) = hy_g(:,:)

    ! grid parametrization
    ! --------------------

    CALL cas_init_grid(dxval,dyval,beta)

    !IO parametrization
    !------------------

    !assignement : input variables for cas_init_io routine
    date_start= '01/01/2012 00:00:00'
    date_end=   '01/01/2013 00:00:00'
    date_start3d='01/01/2012 00:00:00'
    date_startagrif='01/01/2012 00:00:00'

    tdeb = tool_datosec(date_start)
    t = tdeb

    dtmin = 30*60_rsh ! 30 minutes 
    dtmax = dtmin
    dt = dtmax

    suffix='courant'

    CALL cas_init_io(date_start,date_end,date_start3d,date_startagrif,suffix)

    !over-assignement
    pasor(1)= 10*86400_rsh! 10 Days
    l_out_nc4par = .FALSE.
    iscreen=12

    ! viscosity
    !-----------

    ! none
    CALL cas_init_visc

    !over-assignement
    l_smagor=.FALSE.
    !l_smagor=.TRUE.
    fvisc=10_rsh

    visc_num  = fvisc*0.01_rsh*(dxval)**1.15_rsh

    vis2d(:,:)      = visc_num
    vish_xe(:,:,:)  = visc_num
    vish_phi(:,:,:) = visc_num

    !Turbulence
    !----------

    z0bottom  = 1.0e-12
    z0surface = 1.0e-12
    kzinit    = 1.0e-06 
    nzinit    = 1.0e-06

    CALL cas_init_turb(kzinit,nzinit,z0surface,z0bottom)

    !over-assignement
    !turb_nbeq=2
    !turb_2eq_option=2

    !Meteorology
    !-----------

    !No forcing
    CALL cas_init_meteo

    !River and outflows
    !-------------------

    !No river
    CALL cas_init_river

   ! Initialization of velocity field
   !---------------------------------
    u(:,:)  = 0.0_rsh
    v(:,:)  = 0.0_rsh
#if defined key_lans_turb || defined key_leray_turb
    us(:,:) = 0.0_rsh
    vs(:,:) = 0.0_rsh
#endif

    uz(:,:,:)  = 0.0_rsh
    vz(:,:,:)  = 0.0_rsh 
#if defined key_lans_turb || defined key_leray_turb
    uzs(:,:,:) = 0.0_rsh
    vzs(:,:,:) = 0.0_rsh
#endif

    !Open boundaries
    !--------------

    CALL cas_init_obc

    !Over-assignement
    !geometry (where are the open boundaries)

    l_obc_south = .FALSE.
    l_obc_north = .FALSE.
    l_obc_west  = .TRUE.
    l_obc_east  = .TRUE.

    l_obc_cycl   = .TRUE.
    l_obc_cycl_x = .TRUE.
    l_obc_cycl_y = .FALSE.

   !Percent of wet surface on each cell
    CALL cas_init_misc

    ! over-assignement
    id(:)=id(:)-ishift_obc_east
    ig(:)=ig(:)+ishift_obc_west
    jh(:)=jh(:)-jshift_obc_north
    jb(:)=jb(:)+jshift_obc_south

END SUBROUTINE cas_lans_wind_stress 
