  SUBROUTINE cas_rip

   !&E-----------------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_rip  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : cas_init
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        !  2010-03  (A.-C. Bennis) Creation
   !&E        !  2014-03  (V. Garnier) New formalism for the test case initialization
   !&E        !  2014-05  (B.Thouvenin) compatibility with substance (first corrections)
   !&E
   !&E-----------------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters,ONLY : rsh,rlg,riosh,riolg,imin,imax,jmin,jmax,kmax, &
                         jshift_obc_north,jshift_obc_south
   USE comvars2d, ONLY : fisac,gwac,fu,fv,dtmin,dtmax,dt,suffix,     &
                         vish_xe,vish_phi,l_maxu,icon,h0fond,hm,     &
                         hx_g,hy_g,h0_g,hx,hy,dhx,dhy,dhx_g,dhy_g,   &
                         iscreen,l_obc_cycl_y,frot_lin,tobs,jb,jh
   USE comvars2d, ONLY : l_saverestart_bydate,saverestart_step
   USE comvars3d, ONLY : kzinit,nzinit,l_stateeq_lin,bgdiff,         &
                         turb_nbeq,turb_2eq_option
   USE comvarp3d, ONLY : ect
   USE obccombine,ONLY : l_obc_cycl,l_obc_east,l_obc_mars,l_obc_tide

   USE output, ONLY : pasor,l_out_ssh,l_out_ubt,l_out_u3d,l_out_sal,l_out_temp
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : theta_sig,b_sig,hc
#endif
   USE_MPI toolmpi, ONLY : ADD_TAG,ex_i_rsh,ex_j_rsh,MASTER
   USE casinit,   ONLY : cas_init_param,cas_init_grid,cas_init_bathy, &
                         cas_init_io,cas_init_obc,cas_init_meteo,     &
                         cas_init_visc,cas_init_turb,cas_init_tspar,  &
                         cas_init_progvar,cas_init_misc,cas_init_river
#ifdef key_wave_qeuler_dynamics
   USE comwave,   ONLY : c1eps
#endif
#ifdef key_substance
   USE comsubstance, ONLY : nvpc,nvp,nv_cp,nv_dis,nv_fix,nv_dri,nv_int,nv_sorb,     &
                            name_var,long_name_var,standard_name_var,nv_ncp,        &
                            name_filesubs,cv_wat,                        &
                            valid_min_var,valid_max_var,cv_wat,nb_var,date_startsub
   USE casinit,   ONLY : cas_init_subs
#endif
   USE comoasis,  ONLY :    init_comoasis
#ifdef key_oasis_mars_ww3
   USE comoasis,  ONLY :    l_cpl_wave_mask, l_cpl_wave_tmn, l_cpl_wave_hs, &
                            l_cpl_wave_dir, l_cpl_wave_bhd, l_cpl_wave_two, &
                            l_cpl_wave_uba, l_cpl_wave_foc, l_cpl_wave_taw, &
                            l_cpl_wave_lm
#endif

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   LOGICAL           :: l_nc4par
   CHARACTER(LEN=19) :: date_start,date_end,date_start3d,date_startagrif
   INTEGER           :: i,j,k
   INTEGER           :: ni_wall_w,nb_outfile
   REAL(KIND=rsh)    :: dyval,dxval
   REAL(kind=rsh)    :: L1,L2,NP1,NP2
   REAL(KIND=rsh)    :: z0bottom,z0surface,depthcst,limit,nzinit0
   REAL(KIND=rsh),DIMENSION(jmin:jmax-jshift_obc_north-jshift_obc_south) :: hx_read
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_tmp
   LOGICAL           :: l_filesubs

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CAS_PLAGE3D routine'

   ! general parametrization (each parameter can be reassigned)
   ! -----------------------

   ! dynamics
   CALL cas_init_param

   ! over-assignement
   l_maxu=.FALSE.
   h0fond=0.001_rsh
   hm=0.01_rsh

   ! tracers : among others, saliref=saliref_lin=35.5_rsh,
   ! tetaref=tetaref_lin=10.0_rsh
   CALL cas_init_tspar

   ! over-assignement
   icon=0
   l_stateeq_lin=.FALSE.


   ! Bathymetry
   ! ----------

   ! assignement : input variables for cas_init_bathy routine
   depthcst = 0.0_rsh   ! analytical distribution
   ni_wall_w = 2  ! land over ni_wall_w cells (from imin column) at the western part of the domain
#if defined key_siggen || defined key_gencoord
   hc= 5.0_rsh
#endif

#ifdef key_oasis_mars_ww3
   open(20,file="../../data/bathy_to_mars_idduck.txt",status='old',form='formatted')
#else
   open(20,file="../inputs/bathy_to_mars_idduck.txt",status='old',form='formatted')
#endif
   hx_tmp(:,:)=0.0_rsh
   do i=imin,imax
     read(20,*) (hx_read(j),j=jmin,jmax-jshift_obc_north-jshift_obc_south)
     do j=jmin,jmax-jshift_obc_north-jshift_obc_south
#ifdef key_shiftobc
       IF (j==jmin) THEN
         hx_tmp(i,j  )=hx_read(j)
         hx_tmp(i,j+1)=hx_read(j)
         hx_tmp(i,j+2)=hx_read(j)
         hx_tmp(i,j+3)=hx_read(j)
       END IF
#endif
       hx_tmp(i,j+jshift_obc_south)=hx_read(j)
#ifdef key_shiftobc
       IF (j==jmax-jshift_obc_north-jshift_obc_south) THEN
         hx_tmp(i,jmax-jshift_obc_north+1)=hx_read(j)
         hx_tmp(i,jmax-jshift_obc_north+2)=hx_read(j)
         hx_tmp(i,jmax-jshift_obc_north+3)=hx_read(j)
         hx_tmp(i,jmax-jshift_obc_north+4)=hx_read(j)
       END IF
#endif
     enddo
   enddo
   close(20)
   do i=imin,imax
   do j=jmin,jmax
   h0_g(i,j)=hx_tmp(i,j)
   enddo
   enddo
   do i=imin,imax-1
   do j=jmin,jmax
   hx_g(i,j)=0.5*(h0_g(i,j)+h0_g(i+1,j))
   enddo
   enddo
   do i=imin,imax
   do j=jmin,jmax-1
   hy_g(i,j)=0.5*(h0_g(i,j)+h0_g(i,j+1)) 
   enddo
   enddo
   hy_g(:,jmax)=hy_g(:,jmax-1)
   hx_g(imax,:)=hx_g(imax-1,:)

! version initiale
   do i=imin,imax
   do j=jmin,jmax
   hx_g(i,j)=hx_tmp(i,j)
   hy_g(i,j)=hx_tmp(i,j)
   enddo
   enddo
   do j=jmin+1,jmax
   do i=imin+1,imax
   h0_g(i,j)=max(hx_g(i,j),hy_g(i,j),hx_g(i-1,j),hy_g(i,j-1))
   end do
   end do
   h0_g(:,jmin)=h0_g(:,jmin+1)
   h0_g(imin,:)=h0_g(imin+1,:)


   ! flat bottom, plus walls
   CALL cas_init_bathy(depthcst,ni_wall_w=ni_wall_w)

   ! over-assignement
   dhx(:,:)=0.5_rsh
   dhy(:,:)=0.5_rsh
   dhx_g(:,:)=0.5_rsh
   dhy_g(:,:)=0.5_rsh


   ! grid parametrization
   ! --------------------

   ! assignement : input variables for cas_init_grid routine

   ! parametres de forme
   !---------------------
   L1=2328.0_rsh
   L2=1512.0_rsh
   NP1=imax-imin-1
   NP2=jmax-jshift_obc_north-jshift_obc_south-jmin-1
   dxval=L1/(REAL(imax,rsh)-1.5_rsh)
   dxval=L1/NP1
   dyval=L2/(REAL(jmax-jshift_obc_north-jshift_obc_south,rsh)-1.5_rsh)
   dyval=L2/NP2
   print*,dxval,dyval

   fisac=0.0_riolg
   gwac=0.0_riolg

#if defined key_siggen || defined key_gencoord
   theta_sig= 4.0_rsh ! entre 0 et 20
   theta_sig= 0.0000001_rsh ! entre 0 et 20
   b_sig= 1.0_rsh     ! entre 0 et 1
#endif

   CALL cas_init_grid(dxval,dyval)

   ! over-assignement
   fu(:,:)=0.0_rsh
   fv(:,:)=0.0_rsh


   ! IO parametrization
   ! ------------------

   ! assignement : input variables for cas_init_io routine
   date_start=   '31/12/1900 12:00:00'
   date_end=   '03/01/1901 14:00:00'
#ifdef key_oasis_mars_ww3
   date_end=   '31/12/1900 12:09:30'
   date_end=   '31/12/1900 12:09:29'   ! Coupling_duration - dt/2
!non   date_end=   '31/12/1900 12:09:28'
#else
   date_end=   '31/12/1900 14:00:00'
#endif
   !date_end=   '01/02/1901 14:00:00'
   !date_end=   '31/12/1900 12:10:00'
   date_start3d=  '31/12/1900 12:00:00'
   date_startagrif='31/12/1900 00:00:00'
   dtmin=2.0_rlg
   dtmax=2.0_rlg
   dt=2.0_rlg

   suffix='qeuler'
   l_nc4par = .TRUE.
!   l_nc4par = .FALSE.

   nb_outfile=1

   CALL cas_init_io(date_start,date_end,date_start3d,date_startagrif,suffix,nb_file=nb_outfile,l_nc4par=l_nc4par)

   ! over-assignement
#ifdef key_oasis_mars_ww3
   pasor(:)=30.0_rlg
#else
   pasor(:)=24.0_rlg*3600.0_rlg
   pasor(:)=4.0_rlg*60.0_rlg
#endif
   l_out_sal(:)=.FALSE.
   l_out_temp(:)=.FALSE.
   iscreen=6
   tobs=500.0
!   l_saverestart_bydate=.TRUE.
!   saverestart_step=0.00001


   ! open boundaries
   ! ---------------

   ! no open boundary
   CALL cas_init_obc

   ! over-assignement
   ! geometry (where are the open boundaries ?)
   l_obc_east=.TRUE.
   ! used scheme to manage obc
   l_obc_mars = .TRUE.
   l_obc_cycl = .TRUE.
   l_obc_cycl_y=.TRUE.
   ! physics introduced into the simulated area (tide or ogcm solution)
   l_obc_tide=.TRUE.


   ! viscosity
   !-----------

   ! none
   CALL cas_init_visc

   ! over-assignement
   vish_xe(:,:,:) = 0.2_rsh
   vish_phi(:,:,:)= 0.2_rsh


   ! turbulence
   ! ----------

   z0bottom   =0.005_rsh
   z0surface  =0.0_rsh
   nzinit0=0.0_rsh
   kzinit=0.0_rsh

   ! Gaspard parametrization
   CALL cas_init_turb(kzinit,nzinit0,z0surface,z0bottom)

   ! over-assignement
   turb_nbeq=2
   turb_2eq_option=2
   frot_lin=0.004_rsh
   ect(:,:,:)=1.0e-6
   bgdiff=1.0e-6
#ifdef key_wave_qeuler_dynamics
   c1eps=1.44_rsh
!cval donne souci a gestion de la limite sans doute, car pas propre dans coin bas gauche
#endif
! attention ces coefs ont change de base (initturb)       c1turb=1.38_rsh
! attention ces coefs ont change de base (initturb)       c2turb=1.98_rsh
! attention ces coefs ont change de base (initturb)       cmu=0.5544_rsh


   ! meteorology
   ! -----------

   ! no forcing
   CALL cas_init_meteo


   ! River and outflows
   ! -------------------

   ! no river
   CALL cas_init_river


   ! OASIS coupling
   ! --------------

   CALL init_comoasis

   ! over-assignement
#ifdef key_oasis_mars_ww3
   l_cpl_wave_mask = .True.
   l_cpl_wave_tmn = .True.
   l_cpl_wave_hs = .True.
   l_cpl_wave_dir = .True.
   l_cpl_wave_bhd = .True.
   l_cpl_wave_two = .True.
   l_cpl_wave_uba = .True.
   l_cpl_wave_foc = .True.
   l_cpl_wave_taw = .True.
   l_cpl_wave_lm = .True.
#endif

   ! initialization of the pronostic variables
   ! -----------------------------------------

   ! set to 0,tetaref,saliref
   CALL cas_init_progvar

   ! over-assignement


   ! Miscellaneous
   ! -------------

   ! Domain of calculation
   ! Percent of wet surface on each cell
   CALL cas_init_misc

   ! over-assignement
   jh(:)=jh(:)-jshift_obc_north
   jb(:)=jb(:)+jshift_obc_south


!fdumas ajout tracer cas test 060112
#ifdef key_substance
   nvpc=0; nvp=0; nv_cp=0; nv_ncp=0; nv_sorb=0; nv_fix=0; nv_dri=0; nv_int=0;
   nv_dis=1 
   l_filesubs=.FALSE.  ! TRUE si variable lu dans le fichier variable.dat
   IF (l_filesubs) THEN  ! we use "sub_read_var" instead of "cas_init_subs"
     name_filesubs = '../inputs/variable.dat'
   ELSE
   ! allocation et initialisation des tableaux cv_wat,f90,t90...
     CALL cas_init_subs(date_start)    ! nvp must be defined before
   ENDIF

#ifdef ajouterlasuiteavantcasinitsubs?
   
   name_filesubs='variable.dat'
   file_outflow = 'ouflow.dat'
!
   cv_wat(:,:,:,:)= 0.0_rsh
! traceur pres de la plage
   cv_wat(1,kmax-5:kmax,1:16,:)= 50.0_rsh
   cv_wat(1,kmax-5:kmax,17:32,:)= 40.0_rsh
   cv_wat(1,kmax-5:kmax,33:48,:)= 30.0_rsh
   cv_wat(1,kmax-5:kmax,49:64,:)= 20.0_rsh
   cv_wat(1,kmax-5:kmax,65:80,:)= 10.0_rsh
   cv_wat(1,kmax-5:kmax,81:imax,:)= 5.0_rsh
! traceur au large
!    cv_wat(1,kmax-5:kmax,imax-20:imax,:)= 50.0_rsh
#endif
! fin fdumas
#endif
! faire en fonction du choix de BThouvenin


   PRINT_DBG*, 'exit CAS_PLAGE3D routine'

  END SUBROUTINE cas_rip
