  SUBROUTINE cas_sedim

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_sedim  ***
   !&E
   !&E ** Purpose : visualisation of evolution of homogeneously spread 
   !&E              sediments under the effect of tide in an estuary
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : cas_init
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2007-11  (A. Jouon) Original code
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comvarp2d, ONLY : ssh,u,v,sal,temp
   USE comvarp3d, ONLY : uz,vz,ect
   USE comvars3d
   USE output, ONLY : fileresu,tdebsor,tfinsor,nb_out,tinter,tsort,pasor,   &
                      l_out_precdate,l_out_demer,l_out_ssh,l_out_ubt,l_out_u3d, &
                      l_out_sal,l_out_temp,l_out_wind,l_out_buoy,           &
                      l_out_visc,l_out_turb,l_out_off,l_out_atmsflx,nbatm,  &
                      iextmin,iextmax,l_out_nc4par,l_out_pack,              &
                      jextmin,jextmax,kextmin,kextmax,stepi,stepj,stepk,    &
                      igextmin,igextmax,jgextmin,jgextmax,l_filebydate,     &
                      l_out_zaxis_w
   USE obcogcm, ONLY : obc_coefrel
   USE cometdyn
   USE comturb !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   USE comdiag, ONLY : l_diag,file_diag,tdebdiag,tfindiag
   USE cometemp
   USE comfluxdo
#ifdef key_substance
   USE comsubstance, ONLY : nvpc,nvp,nv_cp,nv_dis,nv_fix,nv_dri,nv_int,        &
                            nv_adv,nv_state,nv_tot,f90,t90,irk_fil,l_out_subs, &
                            name_var,unit_var,long_name_var,standard_name_var, &
                            valid_min_var,valid_max_var,cv_wat,c2,alloc_fxfyfz
#endif
   USE comaree
   USE obccombine
   USE_MPI toolmpi, ONLY: ADD_TAG,ex_i_rsh,ex_j_rsh,MASTER,min_all_mpi_rsh,max_all_mpi_rsh
#if defined key_siggen || defined key_gencoord
   USE comsiggen
#endif
#ifdef key_sedim
   USE sedimento
#endif

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   CHARACTER(LEN=19) :: date_start,date_end,date_start3d,date_startagrif
   CHARACTER(LEN=19) :: date_startout,date_endout
   CHARACTER(LEN=19) :: tool_sectodat
   CHARACTER(LEN=lchain) :: logfilename
   INTEGER           :: i,j,k
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_t,hy_t
   REAL(KIND=rsh)    :: dxval,dyval
   REAL(KIND=rlg)    :: x,y,Lx,Ly
   REAL(KIND=rlg)    :: tool_datosec
#ifdef key_substance
   REAL(KIND=rlg)    :: tdb,tfi
   INTEGER           :: n
#endif
   REAL(KIND=rsh)    :: value

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT*, '!!!!!!!!!!!!!!!!!!!'
   PRINT*, 'TEST CASE : sedim  '
   PRINT*, '!!!!!!!!!!!!!!!!!!!'

   ! grid parametrization
   ! --------------------
   dxval = 500.0_rsh
   dyval = 500.0_rsh
   fisac=0.0_riolg
   gwac=0.0_riolg
   dfiac=0.01_riolg
   dgac= 0.01_riolg
   goulet(:,:)=1.0_rsh
   govlet(:,:)=1.0_rsh
   DO j = ljmin,ljmax
     DO i = limin,limax
       dx(i,j)=dxval
       dx_u(i,j)=dxval
       dx_v(i,j)=dxval
       dx_f(i,j)=dxval
       dy(i,j)=dyval
       dy_u(i,j)=dyval
       dy_v(i,j)=dyval
       dy_f(i,j)=dyval
       surf(i,j)=dy(i,j)**2
       fu(i,j)=0.0_rsh
       fv(i,j)=0.0_rsh
       tfiusr(i,j)=0.0_rsh
       tfivsr(i,j)=0.0_rsh
     ENDDO
   ENDDO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,govlet(limin:limax,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dx(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dx(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dy(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dy(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,dx_u(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,dx_f(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,dy_v(limin:limax,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,dy_f(limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(govlet,dx,dx_v,dy,dy_u,dx_u,dx_f,dy_v,dy_f)

   jb(:)=jmin+1
   jh(:)=jmax
   ig(:)=imin+1
   id(:)=imax
   dhx(:,:)=0.2_rsh
   dhy(:,:)=0.2_rsh
   dhx_g(:,:)=0.2_rsh
   dhy_g(:,:)=0.2_rsh

   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljmin,ljmax,dx(liminm1:limaxp1,ljmin:ljmax))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmaxp1,dx_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh( 0,1,1,liminm1,limax,ljmin,ljmaxp1,dx_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh( 0,1,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,dx_f(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,1,1,limin,limax,ljminm1,ljmaxp1,dy(limin:limax,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh( 0,1,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmax,dy_v(liminm1:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,liminm1,limaxp1,ljminm1,ljmax,dy_v(liminm1:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,dy_f(limin:limax,ljminm1:ljmax))

   l_obc_cycl_x=.FALSE.
   l_obc_cycl_y=.FALSE.

   DO k=1,kmax
     sig(k)=(REAL(k-kmax,rsh)-0.5_rsh)/REAL(kmax,rsh)
   END DO
   sig(kmax+1)=-sig(kmax)
   sig(0)=ABS(sig(1))-2.0_rsh

   DO k=1,kmax
     dsigw(k)=sig(k+1)-sig(k)
     sigw(k)=0.5_rsh*(sig(k)+sig(k+1))
   END DO
   sigw(kmax+1)=-sigw(kmax)
   sigw(0)=-1.0_rsh

   DO k=1,kmax
     dsigu(k)=sigw(k)-sigw(k-1)
   END DO

#if defined key_siggen || defined key_gencoord

   theta_sig= 4.0_rsh ! entre 0 et 20
   theta_sig= 0.0000001_rsh ! entre 0 et 20
   b_sig= 1.0_rsh     ! entre 0 et 1
   hc_sig(:,:)= 5.0_rsh
   hc_sig_g(:,:)= 5.0_rsh
   hcx_sig(limin:limaxp1,ljmin:ljmaxp1)= hc_sig_g(limin:limaxp1,ljmin:ljmaxp1)
   hcy_sig(limin:limaxp1,ljmin:ljmaxp1)= hc_sig_g(limin:limaxp1,ljmin:ljmaxp1)
   ! hc_sig(:,:)= 0.0_rsh
   DO k=0,kmax+1
     Csu_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sig(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sig(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
   DO k=0,kmax
     Csw_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sigw(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sigw(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO

   DO k=1,kmax
     dcusds(k)=(Csw_sig(k)-Csw_sig(k-1))/(sigw(k)-sigw(k-1))
   END DO

   DO k=1,kmax-1
     dcwsds(k)=(Csu_sig(k+1)-Csu_sig(k))/(sig(k+1)-sig(k))
   END DO

#endif

   ! longitude latitude
   ! ------------------

   lon2d_g(:,0)=(/ (dxval *(i-1), i=0,imax) /)
   DO j=1,jmax
     lon2d_g(:,j)=lon2d_g(:,0)
   END DO
   lon2du_g(:,:)=lon2d_g(:,:)+dxval/2.0_rsh
   lon2dv_g(:,:)=lon2d_g(:,:)
   lat2d_g(0,:)=(/ (dyval *(j-1), j=0,jmax) /)
   DO i=1,imax
     lat2d_g(i,:)=lat2d_g(0,:)
   END DO
   lat2du_g(:,:)=lat2d_g(:,:)
   lat2dv_g(:,:)=lat2d_g(:,:)+dyval/2.0_rsh

   lon2du(liminm1:limax,ljmin:ljmaxp1)=lon2du_g(liminm1:limax,ljmin:ljmaxp1)
   lon2d(limin:limax,ljmin:ljmax)=lon2d_g(limin:limax,ljmin:ljmax)
   lon2dv(limin:limax,ljmin:ljmax)=lon2dv_g(limin:limax,ljmin:ljmax)

   lat2dv(limin:limax,ljminm1:ljmax)=lat2dv_g(limin:limax,ljminm1:ljmax)
   lat2d(limin:limax,ljmin:ljmax)=lat2d_g(limin:limax,ljmin:ljmax)
   lat2du(limin:limax,ljmin:ljmax)=lat2du_g(limin:limax,ljmin:ljmax)

   ! dates
   ! -----

   date_start=   '01/01/1900 00:00:00'
   date_end=   '20/01/1900 00:00:00'
   date_start3d=  '01/01/1900 00:00:00'
   date_startagrif='01/01/1900 00:00:00'
   date_ref = '01-Jan-1900 00:00:00'
   tdeb     = tool_datosec(date_start) 
   t=tdeb
   tfin     = tool_datosec(date_end)
   tdeb3d   = tool_datosec(date_start3d)
   tdebagrif = tool_datosec(date_startagrif)
   date_startshift= 0.0_rlg
   dtmin=5.0_rlg
   dtmax=400.0_rlg
   dt=400.0_rlg
   halfdt=0.5_rlg*dt
   l_modele2d=.TRUE.
   IF ((kmax==1) .AND. (l_modele2d.EQV..FALSE.) ) THEN
     l_modele2d=.TRUE.
     PRINT*,'variable l_modele2d should be set to .true. as kmax=1'
     PRINT*,'corrected directly in routine cassedim.F90'
   END IF
   cflcrt=0.6_rsh
   tobs=12.0_rlg*3600.0_rlg
   tdebobs=t
   cflumaxmax=0.0_rsh
   cflvmaxmax=0.0_rsh

   l_initfromfile=.FALSE.
   l_saverestart_1file=.FALSE.
   l_saverestart_bydate=.FALSE.
   file_saverestart='save.nc'
   saverestart_step=5.0_rlg*3600.0_rlg   ! in hours
   name_in_dt='dt'
   name_bathy_h0='h0'
   name_bathy_hx='hx'
   name_bathy_hy='hy'
   name_in_xev='xev'
   name_in_u='u'
   name_in_v='v'
   name_in_uz='uz'
   name_in_vz='vz'
   name_in_wz='wz'
   name_in_temp='temp'
   name_in_sal='sal'
   name_in_ect='ect'
   name_in_psi='psi'
   name_in_kz='kz'
   name_in_nz='nz'
   name_in_vis2d='name_in_vis2d'
   name_in_vis3d_xe='name_in_vis3d_xe'
   name_in_vis3d_phi='name_in_vis3d_phi'

   ! outputs
   ! -------

   nb_out=1
   ALLOCATE( fileresu(nb_out),tdebsor(nb_out),tfinsor(nb_out),tinter(nb_out) )
   ALLOCATE( tsort(nb_out),pasor(nb_out),l_filebydate(nb_out),l_out_precdate(nb_out) )
   ALLOCATE( l_out_off(nb_out),l_out_atmsflx(nb_out),l_out_visc(nb_out),l_out_turb(nb_out) )
   ALLOCATE( l_out_buoy(nb_out), l_out_zaxis_w(nb_out) )
   ALLOCATE( l_out_demer(nb_out),l_out_ubt(nb_out),l_out_u3d(nb_out) )
   ALLOCATE( l_out_sal(nb_out),l_out_temp(nb_out),l_out_ssh(nb_out),l_out_wind(nb_out) )
   ALLOCATE( iextmin(nb_out),iextmax(nb_out),jextmin(nb_out),jextmax(nb_out) )
   ALLOCATE( igextmin(nb_out),igextmax(nb_out),jgextmin(nb_out),jgextmax(nb_out) )
   ALLOCATE( kextmin(nb_out),kextmax(nb_out),stepi(nb_out),stepj(nb_out),stepk(nb_out) )
   fileresu(1)='results.nc'
   suffix='sedim'
   date_startout='01/01/1900 00:00:00'
   date_endout='20/01/1900 00:00:00'
   l_maxu=.FALSE.
   tdebsor(1)=tool_datosec(date_startout)
   tfinsor(1)=tool_datosec(date_endout)
   pasor(1)=3600.0_rlg
   l_filebydate(1)=.FALSE.
   IF_MPI ( .NOT. l_filebydate(1) ) THEN
     CALL_MPI ADD_TAG(fileresu(1),LEN(fileresu(1)))
   ENDIF_MPI
   l_out_off(1)=.FALSE.
   l_out_precdate(1)=.TRUE.
   l_out_demer(1)=.FALSE.
   igextmin(1)=imin
   igextmax(1)=imax
   jgextmin(1)=jmin
   jgextmax(1)=jmax
   iextmin(1)=limin
   iextmax(1)=limax
   jextmin(1)=ljmin
   jextmax(1)=ljmax
   kextmin(1)=1
   kextmax(1)=kmax
   stepi(1)=1
   stepj(1)=1
   stepk(1)=1
   l_out_ssh(1)=.TRUE.
   l_out_ubt(1)=.TRUE.
   l_out_u3d(1)=.TRUE.
   l_out_sal(1)=.TRUE.
   l_out_temp(1)=.FALSE.
   l_out_wind(1)=.FALSE.
   l_out_atmsflx(1)=.FALSE.
   l_out_visc(1)=.FALSE.
   l_out_turb(1)=.FALSE.
   l_out_buoy(1)=.FALSE.
   l_out_zaxis_w(1)=.FALSE.
   l_obc2drank_save=.FALSE.
   l_stability=.FALSE.
   iscreen=12
   iscreenlog=18
   ierrorlog=17
   iwarnlog=16
   l_out_nc4par = .false.
   l_out_pack = .false.

   IF_MPI (MASTER) THEN
   logfilename= 'simu_'//TRIM(suffix)//'.log'
   IF (iscreenlog/=6) OPEN(unit=iscreenlog,file=logfilename,status="replace")
   logfilename='error_'//TRIM(suffix)//'.log'
   IF (ierrorlog/=6) OPEN(unit=ierrorlog,file=logfilename,status="replace")
   logfilename='warning_'//TRIM(suffix)//'.log'
   IF (iwarnlog/=6) OPEN(unit=iwarnlog,file=logfilename,status="replace")
   ENDIF_MPI

   ! diagnoses
   ! ---------

   l_diag=.FALSE.
   file_diag='diagnostic.nc'
   tdebdiag=tdebsor(1)
   tfindiag=tfinsor(1)

   ! meteorology
   ! -----------

   l_meteo_stat=.TRUE.
   l_meteo_hom=.TRUE.
   wind_veloc=0.0_rsh
   wind_dir=0.0_rsh
   l_sflx_solarcst=.TRUE.
   sflx_solarcst=0.0_rsh
   nbatm=0

   ! open boundaries
   ! ---------------

   ! geometry (where are the open boundaries ?)

   l_obc_south=.TRUE.
   l_obc_north=.FALSE.
   l_obc_west=.TRUE.
   l_obc_east=.FALSE.

   ! used scheme to manage obc

   l_obc_mars = .TRUE.  
   l_obc_diri = .FALSE. 
   l_obc_char = .FALSE. 
   l_obc_cycl = .FALSE.
   l_obc_cycl_x=.FALSE.
   l_obc_cycl_y=.FALSE.

   ! physics introduced into the simulated area (tide or ogcm solution)

   l_obc_tide=.TRUE.
   l_tide_harmcompo=.FALSE.
   l_obc_ogcm=.FALSE.

   obc_coefrel=0.0_rsh

   ! bathymetry
   ! ----------

   mslshift=0.0_rsh
   DO i=imin+1,imax
     hx_g(i,:)=10.0_rsh-REAL(i-1,rsh)/REAL(imax-1,rsh)*12.0_rsh
     hy_g(i,:)=10.0_rsh-REAL(i-1,rsh)/REAL(imax-1,rsh)*12.0_rsh
     h0_g(i,:)=10.0_rsh-REAL(i-1,rsh)/REAL(imax-1,rsh)*12.0_rsh
   END DO
   h0fond=0.01_rsh
   hm=1.0_rsh
   PRINT*, 'on passe dans cassedim ... bonne nouvelle !'
   PRINT*,'h0fond=',h0fond
   hy_g(:,jmin)  =-valmanq-h0fond
   hy_g(:,jmin+1)=-valmanq-h0fond
   hy_g(:,jmax)  =-valmanq-h0fond
   hy_g(:,jmax-1)=-valmanq-h0fond   
   hx_g(:,jmin)  =-valmanq-h0fond
   hx_g(:,jmin+1)=-valmanq-h0fond
   hx_g(:,jmax)  =-valmanq-h0fond
   hx_g(:,jmax-1)=-valmanq-h0fond
   hx_g(imax-1,:)=-valmanq-h0fond
   hx_g(imax,:)  =-valmanq-h0fond
   hy_g(imax-1,:)=-valmanq-h0fond
   hy_g(imax,:)  =-valmanq-h0fond

   h0(:,:) = h0_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hx(:,:) = hx_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hy(:,:) = hy_g(liminm2:limaxp2,ljminm2:ljmaxp2)

   ! sponge layer
   !-------------

   fvisc=0.0_rsh
   sponge_nbcell=5
   vismul=1.0_rsh
   l_spongs=.FALSE.
   l_spongn=.FALSE.
   l_sponge=.FALSE.
   l_spongw=.FALSE.

   ! turbulence
   !------------

   l_incbotstress=.FALSE.
   botstressmax=100.0_rsh
   turb_nbeq=0
   turb_0eq_option=1
   turb_2eq_option=2
   z0b(:,:)   =0.005_rsh
   z0s(:,:)   =0.0_rsh
   hminfrot=0.5_rsh
   hmaxfrot=5.0_rsh
   bgdiff=0.000001_rsh
   nzinit=0.001_rsh
   kzinit=0.00001_rsh
   frot_lin=0.0_rsh
   nz(:,:,:) = nzinit
   kz(:,:,:) = kzinit
   ect(:,:,:) = 0.000001_rsh
   vish_xe(:,:,:) = 3.0_rsh*0.01_rsh*dyval**1.15_rsh
   vish_phi(:,:,:)= 3.0_rsh*0.01_rsh*dyval**1.15_rsh
   vis2d(:,:) = 0.0_rsh
   l_smagor=.FALSE.
   vismin=0.001_rsh
   vismax=4.0_rsh
   cosmag=0.1_rsh
   icon=0   ! useless
   kx= 1.0_rsh
   ky= 1.0_rsh
   qmax=8.0_rsh
   qmaxz=10.0_rsh
   rhoref=1027.34_rsh
   saliref=30.0_rsh
   tetaref=10.0_rsh
   l_stateeq_lin=.FALSE.
   rhoair=1.25_rsh
   chp=3986.0_rsh
   coext=0.1_rsh
   file_river='./'

   ! turbulenceOLD
   !------------
   ectmin=1.0e-6
   phieau(:,:,:)=0.0_rlg
   IF (limax==imax) phieau(:,imax-1,:)=200.0_rlg/REAL(kmax,rlg)

   ! Initialization of the sea surface height
   ! ----------------------------------------

   ssh(liminm2:limaxp2,ljminm2:ljmaxp2)  = MAX(0.0_rsh,-h0(liminm2:limaxp2,ljminm2:ljmaxp2))

   ! Initialization of velocity field
   !---------------------------------
   u(:,:)    = 0.0_rsh
   v(:,:)    = 0.0_rsh
   uz(:,:,:) = 0.0_rsh
   vz(:,:,:) = 0.0_rsh
   wz(:,:,:) = 0.0_rsh

! debut commentaire valerie
   ! aymeric cette partie n a pas lieu d etre car tu vas passer dans la partie dynamique
   ! contrairement au cas test casadvspot ou casadv*
   ! peux-tu la virer et voir si cela te va ? Merci
   ! Initialization of other dynamics variables
   !-------------------------------------------

   hex(limin:limax,ljmin:ljmax)=(hx(limin:limax,ljmin:ljmax)+ssh(limin:limax,ljmin:ljmax)+ &
                                dhx(limin:limax,ljmin:ljmax))*goulet(limin:limax,ljmin:ljmax)
   hey(limin:limax,ljmin:ljmax)=(hy(limin:limax,ljmin:ljmax)+ssh(limin:limax,ljmin:ljmax)+ &
                                dhy(limin:limax,ljmin:ljmax))*govlet(limin:limax,ljmin:ljmax)
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljmin,ljmax,hex(liminm1:limaxp1,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,1,1,limin,limax,ljminm1,ljmaxp1,hey(limin:limax,ljminm1:ljmaxp1))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(hex,hey)
#if defined key_siggen || defined key_gencoord
   DO j=ljmin,ljmax
   DO i=limin,limax
   DO k=1,kmax
     hzex(k,i,j)=ssh(i,j)+hcx_sig(i,j)+(hx(i,j)+dhx(i,j)-hcx_sig(i,j))*dcusds(k)
     hzey(k,i,j)=ssh(i,j)+hcy_sig(i,j)+(hy(i,j)+dhy(i,j)-hcy_sig(i,j))*dcusds(k)
   END DO
   END DO
   END DO
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljmin,ljmax,hzex(kmax,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin,limax,ljminm1,ljmax,hzey(kmax,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(hzex,hzey)
#endif

#if defined key_siggen || defined key_gencoord
   fxeau(:,liminm1:limax,ljmin:ljmax)=hzex(:,liminm1:limax,ljmin:ljmax)*uz(:,liminm1:limax,ljmin:ljmax)*halfdt
   fyeau(:,limin:limax,ljminm1:ljmax)=hzey(:,limin:limax,ljminm1:ljmax)*vz(:,limin:limax,ljminm1:ljmax)*halfdt
#else
   DO k=1,kmax
     fxeau(k,liminm1:limax,ljmin:ljmax)=hex(liminm1:limax,ljmin:ljmax)*uz(k,liminm1:limax,ljmin:ljmax)*halfdt
     fyeau(k,limin:limax,ljminm1:ljmax)=hey(limin:limax,ljminm1:ljmax)*vz(k,limin:limax,ljminm1:ljmax)*halfdt
   END DO
#endif
!fin commentaire valerie

   ! initialization of tracers
   !--------------------------
   !DO i=imin+1,imax
   ! x=(i-1)*dx(i,1)
   ! IF(x < 30000.0_rsh ) then
   !   sal(:,i,:)=saliref
   ! ELSEIF(x > 80000.0_rsh ) then
   !   sal(:,i,:)= 0.0_rsh
   ! ELSE
   !   sal(:,i,:)=saliref*(1.0_rsh-(x-30000.0_rsh)/50000.0_rsh)
   ! END if
   !END DO
   sal (:,:,:) = saliref
   temp(:,:,:) = tetaref

! commentaire valeri idem ci dessus
   ! Initialization of other variables used in tracers advection
   !------------------------------------------------------------
   phisal(:,:,:) = 0.0_rlg
   phitemp(:,:,:) = 0.0_rlg

#ifdef key_substance
   nvpc=0
   nvp=0
   nv_cp=0
   nv_dis=0
   nv_fix=0
   nv_dri=0
   nv_int=0
   nv_adv=nb_var
   nv_state=nb_var
   nv_tot=nv_state+nv_dri+nv_int
   ALLOCATE(cv_wat(nv_tot,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   ALLOCATE(f90(nb_var))
   f90(1:nb_var) = 1.0_rsh
   ALLOCATE(t90(nv_state))
   t90(1:nv_state) = 0.0_rsh
   ALLOCATE(irk_fil(nv_tot))
   DO n=1,nv_tot
     irk_fil(nv_tot)=n
   END DO
   ALLOCATE(l_out_subs(nv_state))
   DO n=1,nb_var
     cv_wat(n,:,liminm1:limaxp2,ljminm1:ljmaxp2) = sal(:,liminm1:limaxp2,ljminm1:ljmaxp2)
     name_var(n)='substance'
     unit_var(n)='unknown'
     long_name_var(n)='nitrate'
     standard_name_var(n)='mole_concentration_of_nitrate_in_sea_water'
     valid_min_var(n)=-3.4e+38
     valid_max_var(n)=3.4e+38
   END DO
   l_out_subs(:)=.TRUE.
   !   file_outflow = '../inputs/outflow.dat'
   !   name_filesubs = '../inputs/variable.dat'
   !   perchrono=0.1
   !   l_chrono=.false.
   !   perchrono=perchrono*3600.d0
   !   IF (l_meteo_hom .and. .not.l_meteo_stat) perchrono=3.d0*3600.d0
   !   tdb=tdeb-perchrono
   !   tfi=tfin+perchrono
   !   nvalmax=nint((tfi-tdb)/perchrono+0.5)+1
   !   ALLOCATE(tchrono(nvalmax))
   !   DO i=1,nvalmax
   !     tchrono(i)=tdb+(i-1)*perchrono
   !   END DO
   !   indchrono=1
   !   l_chrono=.true.
   !   phicon(:,:,:,:) = 0.0_rsh
#endif

   ! Percent of wet surface on each cell
   ! -----------------------------------

   hx_t(:,:)=100000.0_rsh
   hy_t(:,:)=100000.0_rsh
   WHERE (hx_g(:,:) /= -valmanq -h0fond) hx_t(:,:)=hx_g(:,:)
   WHERE (hy_g(:,:) /= -valmanq -h0fond) hy_t(:,:)=hy_g(:,:)
   dhxy(:,:)=1.0_rsh
   DO j=ljminp1,ljmax
      DO i=liminp1,limax
         dhxy(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))- &
                   MIN(hx_t(i,j),hx_t(i-1,j),hy_t(i,j),hy_t(i,j-1))+epsilon
      END DO
   END DO

   fwet(:,:) = 1.0_rsh
   fwetp(:,:) = 1.0_rsh
   fwetmin=1.0_rsh
   IF (fwetmin/=1.0_rsh) THEN
     DO j=ljmin,ljmax
       DO i=limin,limax
         IF (dhxy(i,j) == 0.0_rsh) THEN
            fwet(i,j)=1.0_rsh
           fwetp(i,j)=1.0_rsh
         ELSE
            fwet(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
           fwetp(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
         ENDIF
       END DO
     END DO
   END IF

   ! initialization of names of variables in output file
   ! ---------------------------------------------------

   name_out_h0='h0'
   name_out_hx='hx'
   name_out_hy='hy'
   name_out_u='u'
   name_out_v='v'
   name_out_uz='uz'
   name_out_vz='vz'
   name_out_temp='temp'
   name_out_sal='sal'
   name_out_xe='xe'
   name_out_vis2d='VIS2D'
   name_out_vishxe='VISH_XE'
   name_out_vishphi='VISH_PHI'
   name_out_nz='NZ'
   name_out_kz='KZ'
   name_out_bz='BZ'
#ifdef key_sedim
   name_out_eptot='EPTOT'
   name_out_nbniv='NBNIV'
   name_out_dzs='DZS FOND SURF'
   name_out_tenfon='TENFON'
   name_out_tenfonc='TENFONC'
   name_out_tenfonw='TENFONW'
#endif

   ! initialization of attributes (max and min valid values for each variable)
   ! -------------------------------------------------------------------------

   riog_valid_min_h0=-50.0_riosh
   riog_valid_max_h0=10000.0_riosh
   riog_valid_min_u=-100.0_riosh
   riog_valid_max_u=100.0_riosh
   riog_valid_min_temp=0.0_riosh
   riog_valid_max_temp=50.0_riosh
   riog_valid_min_sal=0.0_riosh
   riog_valid_max_sal=50.0_riosh
   riog_valid_min_xe=-50.0_riosh
   riog_valid_max_xe=50.0_riosh
   riog_valid_min_wstress=0.0_riosh
   riog_valid_max_wstress=2000.0_riosh
   riog_valid_min_visc=0.0_riosh
   riog_valid_max_visc=100000.0_riosh
   riog_valid_min_nz=0.0_riosh
   riog_valid_max_nz=1000.0_riosh
   riog_valid_min_kz=0.0_riosh
   riog_valid_max_kz=1000.0_riosh
   riog_valid_min_bz=-50.0_riosh
   riog_valid_max_bz=50.0_riosh
   value=MINVAL(dx(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dx=value-1.0_riosh
   value=MAXVAL(dx(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dx=value+1.0_riosh
   value=MINVAL(dy(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dy=value-1.0_riosh
   value=MAXVAL(dy(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dy=value+1.0_riosh
#ifdef key_sedim
   riog_valid_min_eptot=0.0_riosh
   riog_valid_max_eptot=100.0_riosh
   riog_valid_min_nbniv=0.0_riosh
   riog_valid_max_nbniv=200.0_riosh
   riog_valid_min_dzs=0.0_riosh
   riog_valid_max_dzs=100.0_riosh
   riog_valid_min_tenfon=0.0_riosh
   riog_valid_max_tenfon=50.0_riosh
#endif

   ! Initialization of other attributes
   ! ----------------------------------

   cg_conv_title="name of area"
   cg_conv_comment=" "
   cg_conv_source="MARS CONF"
   cg_conv_area="area"
   cg_conv_institution="IFREMER"
   cg_conv_institution_references="http://www.ifremer.fr"
   cg_conv_data_centre="DYNECO/PHYSED"
   cg_conv_data_centre_references="http://www.ifremer.physed.fr"
   cg_conv_forecast_range=" "
   cg_conv_forecast_type=" "
   cg_conv_field_type="n/a"
   cg_conv_operational_status=" "

   ! initialization of cdate,ijour,imois... for outputs and use of tides
   ! -------------------------------------------------------------------

   cdate = tool_sectodat(t)
   CALL tool_decompdate(cdate,ijour,imois,ian,iheure,iminu,isec)

   ! initialization of vertical scale factor (part relative to dc*sds)
   !------------------------------------------------------------------

#if defined key_siggen || defined key_gencoord
#if ! defined key_siggen_g2
!$OMP DO SCHEDULE(STATIC)
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       DO k=1,kmax-1
         hzdcwsds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcwsds(k)
         hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
         hzdcwsds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcwsds(k)
         hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
         hzdcwsds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcwsds(k)
         hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
       END DO
       k=kmax
       hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
       hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
       hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
     END DO
   END DO
!$OMP END DO
#endif
#endif

#ifdef key_substance
   ! Allocate intermediate variable used in ultimate quickest macho scheme
   ALLOCATE(c2(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   c2(:,:,:,:)=0.0_rsh
   CALL alloc_fxfyfz
#endif

  END SUBROUTINE cas_sedim
