  SUBROUTINE bathy

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE bathy  ***
   !&E
   !&E ** Purpose : read the bathymetry and mean sea level file
   !&E              in the historical DEL/AO format
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : hnivmoy,zlit
   !&E
   !&E ** Modified variables : hx_g, hy_g, h0_g, dhx, dhy
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2005-10-14
   !&E       !  2009-04  (F. Dumas) generalized sigma
   !&E       !  01-2011 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E       !  09-2012 (S. Petton) key_siggen hc = MIN(h0_g(:,:),hc)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE ionc4, ONLY : ionc4_openr,ionc4_read_xy,ionc4_close,&
                     ionc4_createfile,ionc4_createvar,ionc4_write_xy
   USE comvars2d
#ifdef key_tide_saverotatedobc
   USE tidesaveobcrotated, ONLY : lon2d_child,lat2d_child,    &
                                  lon2du_child,lon2dv_child,  &
                                  lat2du_child,lat2dv_child,  &
                                  hx_child,hy_child,h0_child, &
                                  ig_child,id_child,jh_child,jb_child
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc,hc_sig_g,hcx_sig_g,hcy_sig_g
#endif
   USE_MPI toolmpi,   ONLY : MASTER
   USE obccombine,ONLY : l_obc_tide

   IMPLICIT NONE

  !! * Local declarations
   CHARACTER(lchain)                             :: chain
   INTEGER                                       :: igg,idd,i,j
   REAL(kind=riosh),DIMENSION(imin:imax,jmin:jmax) :: zlit
   REAL(kind=riolg),dimension(imin:imax,jmin:jmax) :: tmp2drlg
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_t,hy_t
#ifdef key_tide_saverotatedobc
   REAL(KIND=riolg),DIMENSION(imin_child:imax_child,jmin_child:jmax_child) :: tmp2drlg2
   REAL(KIND=riosh),DIMENSION(imin_child:imax_child,jmin_child:jmax_child) :: zlit2
#endif
  !!----------------------------------------------------------------------
  !! * Executable part


   CALL ionc4_openr(file_bathy)
   CALL ionc4_read_xy(file_bathy,name_in_longitude,tmp2drlg,imin,imax,jmin,jmax)
   lon2d_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_latitude,tmp2drlg,imin,imax,jmin,jmax)
   lat2d_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_longitude_u,tmp2drlg,imin,imax,jmin,jmax)
   lon2du_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_latitude_u,tmp2drlg,imin,imax,jmin,jmax)
   lat2du_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_longitude_v,tmp2drlg,imin,imax,jmin,jmax)
   lon2dv_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_latitude_v,tmp2drlg,imin,imax,jmin,jmax)
   lat2dv_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_longitude_f,tmp2drlg,imin,imax,jmin,jmax)
   lon2df_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_read_xy(file_bathy,name_in_latitude_f,tmp2drlg,imin,imax,jmin,jmax)
   lat2df_g(:,:)=tmp2drlg(:,:)
   CALL ionc4_close(file_bathy)

#ifdef key_tide_saverotatedobc
   ! latitude and longitude in the child grid
   CALL ionc4_openr(file_bathy_child)
   CALL ionc4_read_xy(file_bathy_child,name_in_longitude,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lon2d_child(:,:)=tmp2drlg2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_in_latitude,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lat2d_child(:,:)=tmp2drlg2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_in_longitude_u,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lon2du_child(:,:)=tmp2drlg2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_in_latitude_u,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lat2du_child(:,:)=tmp2drlg2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_in_longitude_v,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lon2dv_child(:,:)=tmp2drlg2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_in_latitude_v,tmp2drlg2,imin_child,imax_child,jmin_child,jmax_child)
   lat2dv_child(:,:)=tmp2drlg2(:,:)

   ! bathymetry of the child grid
   CALL ionc4_read_xy(file_bathy_child,name_bathy_hx,zlit2,imin_child,imax_child,jmin_child,jmax_child, &
                      fillvar=-rg_valmanq_io-REAL(h0fond,riosh))
   hx_child(:,:)=zlit2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_bathy_hy,zlit2,imin_child,imax_child,jmin_child,jmax_child, &
                      fillvar=-rg_valmanq_io-REAL(h0fond,riosh))
   hy_child(:,:)=zlit2(:,:)
   CALL ionc4_read_xy(file_bathy_child,name_bathy_h0,zlit2,imin_child,imax_child,jmin_child,jmax_child,fillvar=-rg_valmanq_io)
   h0_child(:,:)=zlit2(:,:)
   CALL ionc4_close(file_bathy_child)

!cval this cpp key may be not relevant
#if ! defined key_bathy_final
   WHERE (hx_child(:,:) < hminim) hx_child(:,:) = -valmanq -h0fond
   WHERE (hy_child(:,:) < hminim) hy_child(:,:) = -valmanq -h0fond
   hx_child(:,:)=hx_child(:,:)+0.96-mslshift
   hy_child(:,:)=hy_child(:,:)+0.96-mslshift
   WHERE (hx_child(:,:) < -valmanq+20.0_rsh) hx_child(:,:) = -valmanq -h0fond
   WHERE (hy_child(:,:) < -valmanq+20.0_rsh) hy_child(:,:) = -valmanq -h0fond

   DO j=jmin_child+1,jmax_child
      DO i=imin_child+1,imax_child
         h0_child(i,j)=MAX(hx_child(i,j),hx_child(i-1,j),hy_child(i,j),hy_child(i,j-1))+h0fond
      END DO
   END DO

   DO j=jmin_child+1,jmax_child
      h0_child(imin_child,j)=MAX(hx_child(imin_child,j),hy_child(imin_child,j),hy_child(imin_child,j-1))+h0fond
   END DO
   DO i=imin_child+1,imax_child
      h0_child(i,jmin_child)=MAX(hx_child(i,jmin_child),hx_child(i-1,jmin_child),hy_child(i,jmin_child))+h0fond
   END DO
   h0_child(imin_child,jmin_child)=MAX(hx_child(imin_child,jmin_child),hy_child(imin_child,jmin_child))+h0fond
   WHERE (h0_child(:,:) < -valmanq+20.0_rsh) h0_child(:,:) = -valmanq

  !  redefine hx_child and hy_child
   DO i=imin_child,imax_child-1
      DO j=jmin_child,jmax_child-1
        IF (hx_child(i,j) >= hrdef) hx_child(i,j)=(h0_child(i,j)+h0_child(i+1,j))/2.0_rsh
        IF (hy_child(i,j) >= hrdef) hy_child(i,j)=(h0_child(i,j)+h0_child(i,j+1))/2.0_rsh
      END DO
   END DO
#endif

   ! initialize obc indexes (ig,id,jb,jh) in the child grid
   ig_child(jmin_child:jmax_child)=imin_child+1
   id_child(jmin_child:jmax_child)=imax_child
   jb_child(imin_child:imax_child)=jmin_child+1
   jh_child(imin_child:imax_child)=jmax_child
#endif

   ! Bathymetry of the current simulation
   ! ------------------------------------

#ifdef key_offline

   CALL ionc4_openr(file_bathy)
   CALL ionc4_read_xy(file_bathy,name_bathy_hx,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hx_g(:,:)=zlit(:,:)
   CALL ionc4_read_xy(file_bathy,name_bathy_hy,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hy_g(:,:)=zlit(:,:)
   CALL ionc4_read_xy(file_bathy,name_bathy_h0,zlit,imin,imax,jmin,jmax,fillvar=-rg_valmanq_io)
   h0_g(:,:)=zlit(:,:)

#else

   hnivmoy_g(:,:)=0.0_rsh
#ifdef key_bathy_final
   CALL ionc4_openr(file_bathy)
   CALL ionc4_read_xy(file_bathy,name_bathy_hx,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hx_g(:,:)=zlit(:,:)
   CALL ionc4_read_xy(file_bathy,name_bathy_hy,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hy_g(:,:)=zlit(:,:)
   CALL ionc4_read_xy(file_bathy,name_bathy_h0,zlit,imin,imax,jmin,jmax,fillvar=-rg_valmanq_io)
   h0_g(:,:)=zlit(:,:)
   IF (l_bathy_meanlev) THEN
     CALL ionc4_read_xy(file_bathy,name_bathy_meanlev,zlit,imin,imax,jmin,jmax)
     hnivmoy_g(:,:)=zlit(:,:)
   END IF

   ! to manage with rsh=8
   WHERE (hx_g(:,:) < -valmanq+20.0_rsh) hx_g(:,:) = -valmanq -h0fond
   WHERE (hy_g(:,:) < -valmanq+20.0_rsh) hy_g(:,:) = -valmanq -h0fond
   WHERE (h0_g(:,:) < -valmanq+20.0_rsh) h0_g(:,:) = -valmanq

#else
   CALL ionc4_openr(file_bathy)
   CALL ionc4_read_xy(file_bathy,name_bathy_hx,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hx_g(:,:)=zlit(:,:)
   CALL ionc4_read_xy(file_bathy,name_bathy_hy,zlit,imin,imax,jmin,jmax, &
                      -rg_valmanq_io-REAL(h0fond,riosh))
   hy_g(:,:)=zlit(:,:)

   IF (l_bathy_meanlev) THEN                                        ! read mean level
     CALL ionc4_read_xy(file_bathy,name_bathy_meanlev,zlit,imin,imax,jmin,jmax)
     hnivmoy_g(:,:)=zlit(:,:)
   END IF
#endif

#if ! defined key_bathy_final

   WHERE (hx_g(:,:) < hminim) hx_g(:,:) = -valmanq -h0fond
   WHERE (hy_g(:,:) < hminim) hy_g(:,:) = -valmanq -h0fond
   hx_g(:,:)=hx_g(:,:)+hnivmoy_g(:,:)-mslshift
   hy_g(:,:)=hy_g(:,:)+hnivmoy_g(:,:)-mslshift
   WHERE (hx_g(:,:) < -valmanq+20.0_rsh) hx_g(:,:) = -valmanq -h0fond
   WHERE (hy_g(:,:) < -valmanq+20.0_rsh) hy_g(:,:) = -valmanq -h0fond


   DO j=jmin+1,jmax
      DO i=imin+1,imax
         h0_g(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))+h0fond
      END DO
   END DO

!   IF (no==0) THEN
!      PRINT*, 'WARNING : wall at the eastern boundary for rank 0'
!      PRINT*, 'i.e. the Channel is closed'
!      PRINT*, 'this correction is available for the default rank 0 over the North East Atlantic'
!      PRINT*, 'See bathy.F90'
!      PRINT*, ' '
!      h0_g(imax,:)=-valmanq
!      hx_g(imax-1,:)=-valmanq-h0fond
!   END IF

   DO j=jmin+1,jmax
      h0_g(imin,j)=MAX(hx_g(imin,j),hy_g(imin,j),hy_g(imin,j-1))+h0fond
   END DO
   DO i=imin+1,imax
      h0_g(i,jmin)=MAX(hx_g(i,jmin),hx_g(i-1,jmin),hy_g(i,jmin))+h0fond
   END DO
   h0_g(imin,jmin)=MAX(hx_g(imin,jmin),hy_g(imin,jmin))+h0fond
   WHERE (h0_g(:,:) < -valmanq+20.0_rsh) h0_g(:,:) = -valmanq

  !  redefine hx_g and hy_g
   DO i=imin,imax-1
      DO j=jmin,jmax-1
        IF (hx_g(i,j) >= hrdef) hx_g(i,j)=(h0_g(i,j)+h0_g(i+1,j))/2.0_rsh
        IF (hy_g(i,j) >= hrdef) hy_g(i,j)=(h0_g(i,j)+h0_g(i,j+1))/2.0_rsh
      END DO
   END DO

#endif

   hx_t(:,:)=100000.0_rsh
   hy_t(:,:)=100000.0_rsh
   WHERE (hx_g(:,:) /= -valmanq -h0fond) hx_t(:,:)=hx_g(:,:)
   WHERE (hy_g(:,:) /= -valmanq -h0fond) hy_t(:,:)=hy_g(:,:)
   dhxy_g(:,:)=1.0_rsh
   DO j=jmin+1,jmax
      DO i=imin+1,imax
         dhxy_g(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))- &
                     MIN(hx_t(i,j),hx_t(i-1,j),hy_t(i,j),hy_t(i,j-1))+epsilon
      END DO
   END DO

   dhx_g(:,:)=0.0_rsh
   dhy_g(:,:)=0.0_rsh
   WHERE (hx_g(:,:) < hrdef) dhx_g(:,:)=dhj
   WHERE (hy_g(:,:) < hrdef) dhy_g(:,:)=dhj

#if defined key_siggen || defined key_gencoord
   hc_sig_g(:,:)=-valmanq
   hcx_sig_g(:,:)=-valmanq
   hcy_sig_g(:,:)=-valmanq
   IF (l_bathy_meanlev) THEN
      WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:),MAX(hnivmoy_g(:,:),hc))
      WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:),MAX(hnivmoy_g(:,:),hc))
      WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:),MAX(hnivmoy_g(:,:),hc))
   ELSE
      WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:),hc)
      WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:),hc)
      WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:),hc)
   END IF
#endif

   ! Save the bathymetry
   ! -------------------

   IF (l_bathy_save) THEN

     IF_MPI (MASTER) THEN
       CALL ionc4_createfile('bathy_verif.nc',lon2d_g,lat2d_g, &
                             imin,imax,1, &
                             jmin,jmax,1, &
                             1,kmax,1,kmax,0,l_out_nc4par=.false.)
       CALL ionc4_createvar('bathy_verif.nc','longitude_u',                            &
                            units='degrees_east',long_name='longitude at u location',   &
                            standard_name='longitude_at_u_location',location_in_cell='u', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_u',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_u',                             &
                            units='degrees_north',long_name='latitude at u location',   &
                            standard_name='latitude_at_u_location',location_in_cell='u',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_u',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','longitude_v',                            &
                            units='degrees_east',long_name='longitude at v location',   &
                            standard_name='longitude_at_v_location',location_in_cell='v', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_v',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_v',                             &
                            units='degrees_north',long_name='latitude at v location',   &
                            standard_name='latitude_at_v_location',location_in_cell='v',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_v',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','longitude_f',                            &
                            units='degrees_east',long_name='longitude at f location',   &
                            standard_name='longitude_at_f_location',location_in_cell='f', &
                            valid_min=-180.0_riolg,valid_max=180.0_riolg,              &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lon2du_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','longitude_f',tmp2drlg,imin,imax,jmin,jmax)
       CALL ionc4_createvar('bathy_verif.nc','latitude_f',                             &
                            units='degrees_north',long_name='latitude at f location',   &
                            standard_name='latitude_at_f_location',location_in_cell='f',  &
                            valid_min=-90.0_riolg,valid_max=90.0_riolg,                &
                            dims='xy',fill_value=dg_valmanq_io,l_auxcoordinate=.false.)
       tmp2drlg(:,:)=lat2dv_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','latitude_f',tmp2drlg,imin,imax,jmin,jmax)

       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level'
       ELSE
         chain='model_sea_floor_depth_below_geoid'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','H0',units='m',                     &
                            long_name='bathymetry relative to the mean level',   &
                            standard_name = TRIM(chain),                         &
                            fill_value=-rg_valmanq_io,dims='xy')
       zlit(:,:)=h0_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','H0',zlit,imin,imax,jmin,jmax)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_u_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_u_location'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','HX',units='m',location_in_cell='u',              &
                            long_name = 'bathymetry at u-location relative to the mean level', &
                            standard_name = TRIM(chain),                         &
                            fill_value=-rg_valmanq_io-REAL(h0fond,riosh),dims='xy')
       zlit(:,:)=hx_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','HX',zlit,imin,imax,jmin,jmax)
       IF (l_obc_tide) THEN
         chain='model_sea_floor_depth_below_sea_level_at_v_location'
       ELSE
         chain='model_sea_floor_depth_below_geoid_at_v_location'
       END IF
       CALL ionc4_createvar('bathy_verif.nc','HY',units='m',location_in_cell='v',              &
                            long_name = 'bathymetry at v-location relative to the mean level', &
                            standard_name = TRIM(chain),                         &
                            fill_value=-rg_valmanq_io-REAL(h0fond,riosh),dims='xy')
       zlit(:,:)=hy_g(:,:)
       CALL ionc4_write_xy('bathy_verif.nc','HY',zlit,imin,imax,jmin,jmax)

       CALL ionc4_close('bathy_verif.nc')
     ENDIF_MPI
   END IF


  ! key_offline
#endif

   CALL ionc4_close(file_bathy)

   PRINT_DBG*, 'exit BATHY routine'

  END SUBROUTINE bathy
