MODULE interpolation_methods

   !!======================================================================
   !!                   ***  MODULE interpolation_methods  ***
   !!
   !! ** Description :

   !!      !!======================================================================
   !! * Modules used
   USE parameters

   USE comcolig2d, ONLY : mn,ms,whx               !
 
   IMPLICIT NONE
  
   CONTAINS
!---------------------------
!INTERPOLATION EAST TO NORTH
!---------------------------
!
        FUNCTION interp_u_to_north(u, i, j) RESULT(u_v)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u

                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: u_v

                 u_v = 0.25*(u(i,j)+u(i,j+1)+u(i-1,j+1)+u(i-1,j))
                     
        END FUNCTION interp_u_to_north

        FUNCTION interp_uz_to_north(uz, i, j) RESULT(uz_v)

                 IMPLICIT NONE

                 !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax, liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz

                INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh), DIMENSION(kmax)  :: uz_v

                 uz_v(:) = 0.25*(uz(:,i,j)+uz(:,i,j+1)+uz(:,i-1,j+1)+uz(:,i-1,j))

        END FUNCTION interp_uz_to_north

!---------------------------
!INTERPOLATION NORTH TO EAST
!---------------------------

        FUNCTION interp_v_to_east(v, i, j) RESULT(v_u)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v

                 INTEGER,        INTENT(IN) :: i, j
                 !Output parameter
                 REAL(kind=rsh)  :: v_u


                 v_u = 0.25*(v(i,j)+v(i+1,j)+v(i+1,j-1)+v(i,j-1))

        END FUNCTION interp_v_to_east

        FUNCTION interp_vz_to_east(vz, i, j)  RESULT(vz_u)

                 IMPLICIT NONE 

                 !Inpout parameter

                 REAL(KIND=rsh), DIMENSION(kmax, liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz

                 INTEGER, INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh), DIMENSION(kmax)  ::vz_u

                 vz_u(:) = 0.25_rsh*(vz(:,i,j)+vz(:,i+1,j)+vz(:,i,j-1)+vz(:,i+1,j-1))

        END FUNCTION interp_vz_to_east 

!----------------------------
!INTERPOLATION CENTER TO EAST
!----------------------------

        FUNCTION interp_ssh_to_east(xe, i, j) RESULT(xe_u)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: xe

                 INTEGER,        INTENT(IN) :: i, j
                 !Output parameter
                 REAL(kind=rsh)  :: xe_u

                 xe_u = 0.5*(xe(i,j)+xe(i+1,j))

        END FUNCTION interp_ssh_to_east


        FUNCTION interp_visc_to_east(nz, k, i, j)  RESULT(nz_u)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(0:kmax,limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: nz

                 INTEGER, INTENT(IN) :: k, i, j

                 !Output parameter
                 REAL(kind=rsh)  :: nz_u

                 nz_u = 0.5_rsh*(nz(k,i,j)+nz(k,i+1,j))

        END FUNCTION interp_visc_to_east

!-----------------------------
!INTERPOLATION CENTER TO NORTH
!-----------------------------

        FUNCTION interp_ssh_to_north(xe, i, j) RESULT(xe_v)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: xe
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: xe_v

                 xe_v = 0.5*(xe(i,j)+xe(i,j+1))

        END FUNCTION interp_ssh_to_north

        FUNCTION interp_visc_to_north(nz, k, i, j) RESULT(nz_v)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(0:kmax,limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: nz

                 INTEGER,        INTENT(IN) :: k, i, j

                 !Output parameter
                 REAL(kind=rsh)  :: nz_v

                 nz_v =0.5*(nz(k,i,j)+nz(k,i,j+1))

        END FUNCTION interp_visc_to_north

END MODULE interpolation_methods
