!!COMPUTE THE GRADIENT USING THE CENTERED DIFFERENCE (SECOND ORDER)
!------------------------------------------------------------------

!x-direction
        !2d variable
        FUNCTION centered_dudx(u, i, j) RESULT(dudx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dudx

                dudx = (u(i+1,j)-u(i-1,j))/(dx(i+1,j)+dx(i,j))

        END FUNCTION centered_dudx

        !3d variable
        FUNCTION centered_duzdx(uz, k, i, j) RESULT(duzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz

                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdx

                duzdx = (uz(k,i+1,j)-uz(k,i-1,j))/(dx(i+1,j)+dx(i,j))

        END FUNCTION centered_duzdx


        FUNCTION centered_dvdx(v, i, j) RESULT(dvdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dvdx

                dvdx = ((v(i  ,j)-v(i-1,j))*mo(i,j)/dx_f(i-1,j) &
                       +(v(i+1,j)-v(i,j))*me(i,j)/dx_f(i,j))*0.5_rsh

        END FUNCTION centered_dvdx

        !3d variable
        FUNCTION centered_dvzdx(vz, k, i, j) RESULT(dvzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdx

                dvzdx = ((vz(k,i,j)-vz(k,i-1,j))*mo(i,j)/dx_f(i-1,j) &
                        +(vz(k,i+1,j)-vz(k,i,j))*me(i,j)/dx_f(i,j))*0.5_rsh

        END FUNCTION centered_dvzdx

!y-direction

        FUNCTION centered_dudy(u, i, j) RESULT(dudy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dudy

                dudy = ((u(i,j)-u(i,j-1))*ms(i,j)/dy_f(i,j-1) +(u(i,j+1)-u(i,j))*mn(i,j)/dy_f(i,j))*0.5_rsh

        END FUNCTION centered_dudy

        !3d variable
        FUNCTION centered_duzdy(uz, k, i, j) RESULT(duzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdy

                duzdy = ((uz(k,i,j)-uz(k,i,j-1))*ms(i,j)/dy_f(i,j-1) +(uz(k,i,j+1)-uz(k,i,j))*mn(i,j)/dy_f(i,j))*0.5_rsh

        END FUNCTION centered_duzdy

        FUNCTION centered_dvdy(v, i, j) RESULT(dvdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dvdy

                dvdy = (v(i,j+1)-v(i,j-1))/(dy(i,j)+dy(i,j+1))

        END FUNCTION centered_dvdy

        !3d variable
        FUNCTION centered_dvzdy(vz, k, i, j) RESULT(dvzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdy

                dvzdy = (vz(k,i,j+1)-vz(k,i,j-1))/(dy(i,j)+dy(i,j+1))

        END FUNCTION centered_dvzdy

!z-direction
        FUNCTION centered_duzdz(uz, dsigw, k, i, j) RESULT(duzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                REAL(kind=rsh), DIMENSION(kmax), INTENT(IN)  :: dsigw
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdz

                duzdz = (uz(k+1,i,j)-uz(k-1,i,j))/(dsigw(k)+dsigw(k+1))

        END FUNCTION centered_duzdz

        FUNCTION centered_dvzdz(vz, dsigw, k, i, j) RESULT(dvzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(kind=rsh), DIMENSION(kmax), INTENT(IN)  :: dsigw
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdz

                dvzdz = (vz(k+1,i,j)-vz(k-1,i,j))/(dsigw(k)+dsigw(k+1))

        END FUNCTION centered_dvzdz
