MODULE conjugate_gradient

    IMPLICIT NONE

    CONTAINS

    !Solve linear system using the conjugate gradient method
    SUBROUTINE CGsolve(A,b,x,info,delta)

        !! * Modules used
        USE parameters

        IMPLICIT NONE
      
        REAL(KIND=rsh), intent(in)  :: A(:,:)
        REAL(KIND=rsh), intent(in)  :: b(:)        
        REAL(KIND=rsh), intent(inout) :: x(:)
        integer,        intent(out) :: info
        REAL(KIND=rsh), intent(out) :: delta

        INTEGER                     :: n,maxit,i,j,t   
        REAL(KIND=rsh)              :: eps
        REAL(KIND=rsh)              :: alpha,rsold,rsnew,drs

        REAL(KIND=rsh), dimension(:), allocatable :: r(:),p(:),Ap(:)

        n     = size(A,1)     ! assume # columns = # lines
        ALLOCATE(r(n),p(n),Ap(n))
        maxit = n
        eps   = 1e-10         ! accuracy goal
        r = b - MATMUL(A,x)
        p = r
        rsold = DOT_PRODUCT(r,r)
        
        IF (sqrt(rsold) .lt. eps) THEN
           return
        ELSE
                DO t = 1,maxit
                        Ap    = MATMUL(A,p)
                        alpha = rsold/DOT_PRODUCT(p,Ap)
                        x     = x + alpha*p
                        r     = r - alpha*Ap
                        rsnew = DOT_PRODUCT(r,r)
                        IF (sqrt(rsnew) .lt. eps) THEN
                                info  = t
                                delta = sqrt(rsnew)
                                RETURN
                        END IF
                        drs   = rsnew/rsold
                        p     = r + drs*p
                        rsold = rsnew
                END DO        
                info  = -maxit
                delta = sqrt(rsnew)
        END IF

        DEALLOCATE(r,p,Ap)

     END SUBROUTINE CGsolve

END MODULE conjugate_gradient
