 MODULE extendfield
#if defined key_lans_turb || defined key_leray_turb
   !!=======================================================================
   !!                        ***  MODULE obcapplyu  ***
   !! Open boundary treatment : apply open boundary schemes
   !!                           for 2d velocities
   !!                           at eastern and western boundaries.
   !!=======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,ex_i_rlg,ex_j_rlg,ex_peri_x_rsh,ex_peri_y_rsh
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC extend_u, extend_v, extend_w, extend_intz_exp ! dyn2dx, dyn3dx


   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE extend_u(u, uext)

  !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u
   REAL(KIND=rsh),DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(inout) :: uext

  !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter extend_u routine'

   uext(liminm2:limaxp2,ljminm2:ljmaxp2) = u(liminm2:limaxp2,ljminm2:ljmaxp2)
 
OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,uext(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,uext(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(uext)

   PRINT_DBG*, 'exit extend_u routine'

  END SUBROUTINE extend_u

  SUBROUTINE extend_v(v, vext)

  !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in)   :: v
   REAL(KIND=rsh),DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(inout) :: vext

  !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter extend_u routine'

   vext(liminm2:limaxp2,ljminm2:ljmaxp2) = v(liminm2:limaxp2,ljminm2:ljmaxp2)

OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,vext(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,vext(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(vext)

   PRINT_DBG*, 'exit extend_u routine'

  END SUBROUTINE extend_v

  SUBROUTINE extend_w(w, wext)

  !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm1:limaxp1,ljminm1:ljmaxp1),INTENT(in)   :: w
   REAL(KIND=rsh),DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(inout) :: wext

  !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter extend_w routine'

   wext(liminm1:limaxp1,ljminm1:ljmaxp1) = w(liminm1:limaxp1,ljminm1:ljmaxp1)

OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,wext(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,wext(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(wext)

   PRINT_DBG*, 'exit extend_w routine'

  END SUBROUTINE extend_w

  SUBROUTINE extend_intz_exp(intz_exp, intz_exp_ext)

  !! * Arguments
   REAL(KIND=rsh),DIMENSION(limin:limax,ljmin:ljmax),INTENT(in)    :: intz_exp 
   REAL(KIND=rsh),DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(inout) :: intz_exp_ext

  !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter extend_intz_exp: routine'

   intz_exp_ext(limin:limax,ljmin:ljmax) = intz_exp(limin:limax,ljmin:ljmax)

OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,intz_exp_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,intz_exp_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(intz_exp_ext)

   PRINT_DBG*, 'exit extend_intz_exp routine'

  END SUBROUTINE extend_intz_exp

   !!======================================================================
#endif
 END MODULE extendfield
