!---------------------------------------------------------------
!COMPUTE THE GRADIENT USING THE FORWARD DIFFERENCE (FIRST ORDER)
!---------------------------------------------------------------

!x-direction
        !2d variable
        FUNCTION forward_dpadx(pa, i, j) RESULT(dpadx)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: pa
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dpadx

                 dpadx = (pa(i+1,j)-pa(i,j))/dx_u(i,j)

        END FUNCTION forward_dpadx

        FUNCTION forward_dxedx(xe, i, j) RESULT(dxedx)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: xe
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dxedx

                 dxedx = (xe(i+1,j)-xe(i,j))/dx_u(i,j)

        END FUNCTION forward_dxedx

        FUNCTION forward_dvisdx(vis2d, i, j) RESULT(dvisdx)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm1:limaxp1,ljminm1:ljmaxp1),INTENT(IN) :: vis2d
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dvisdx

                 dvisdx = (vis2d(i+1,j)-vis2d(i,j))/dx_u(i,j)

        END FUNCTION forward_dvisdx

        FUNCTION forward_dudx(u, i, j) RESULT(dudx)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dudx

                 dudx = (u(i+1,j)-u(i,j))/dx(i+1,j)

        END FUNCTION forward_dudx


        FUNCTION forward_dpotgenedx(potgene, i, j) RESULT(dpotgenedx)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: potgene
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dpotgenedx

                 dpotgenedx = (potgene(i+1,j)-potgene(i,j))/dx_u(i,j)

        END FUNCTION forward_dpotgenedx

        !3d variable
        FUNCTION forward_duzdx(uz, i, j) RESULT(duzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh), DIMENSION(kmax) :: duzdx

                duzdx(:) = (uz(:,i+1,j)-uz(:,i,j))/dx_u(i,j)

        END FUNCTION forward_duzdx

!y-direction
        !2d variable
        FUNCTION forward_dpady(pa, i, j) RESULT(dpady)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: pa
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dpady

                 dpady = (pa(i,j+1)-pa(i,j))/dy_v(i,j)

        END FUNCTION forward_dpady

        FUNCTION forward_dxedy(xe, i, j) RESULT(dxedy)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: xe
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dxedy

                 dxedy = (xe(i,j+1)-xe(i,j))/dy_v(i,j)

        END FUNCTION forward_dxedy

        FUNCTION forward_dvisdy(vis2d, i, j) RESULT(dvisdy)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm1:limaxp1,ljminm1:ljmaxp1),INTENT(IN) :: vis2d
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dvisdy

                 dvisdy = (vis2d(i,j+1)-vis2d(i,j))/dy_v(i,j)

        END FUNCTION forward_dvisdy

        FUNCTION forward_dudy(u, i, j) RESULT(dudy)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dudy

                 dudy = (u(i,j+1)-u(i,j))/dy_f(i,j)

        END FUNCTION forward_dudy


        FUNCTION forward_dpotgenedy(potgene, i, j) RESULT(dpotgenedy)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(limin:limaxp1,ljmin:ljmaxp1),INTENT(IN) :: potgene
                 INTEGER,        INTENT(IN) :: i, j

                 !Output parameter
                 REAL(kind=rsh) :: dpotgenedy

                 dpotgenedy = (potgene(i,j+1)-potgene(i,j))/dy_v(i,j)

        END FUNCTION forward_dpotgenedy

        !3d variable
        FUNCTION forward_duzdy(uz, i, j) RESULT(duzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh), DIMENSION(kmax) :: duzdy

                duzdy(:) = (uz(:,i,j+1)-uz(:,i,j))/dy_v(i,j)

        END FUNCTION forward_duzdy

!z-direction
        FUNCTION forward_derivative_zdir(phi, dsig, k, i, j) RESULT(phi_deriv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)  :: phi_deriv

                phi_deriv = (phi(k+1,i,j)-phi(k,i,j))/dsig

        END FUNCTION forward_derivative_zdir
