!---------------------
!LANS DYNAMIC PRESSURE
!---------------------

FUNCTION pdyn_lans_lon(uzs, vzs_u, k, i, j) RESULT(pdyn)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: uzs, vzs_u 

                 INTEGER, INTENT(IN) :: k, i, j

                 !Output parameter
                 REAL(KIND=rsh) :: pdyn

                 !Local declaration
                 REAL(KIND=rsh) :: dudx, dudy, dvdx, dvdy, alpha

                 IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN 
		 
			alpha = smooth_coeff*dx(i,j)
 
                 	dudx  = centered_duzdx(uzs, k, i, j)
                 	dudy  = centered_duzdy(uzs, k, i, j)
                        dvdx  = centered_dvzdx(vzs_u, k, i, j)
                        dvdy  = centered_dvzdy(vzs_u, k, i, j)

                       	pdyn = 0.5_rsh*(alpha*alpha*(dudx*dudx+dudy*dudy+dvdx*dvdx+dvdy*dvdy))

		 END IF

END FUNCTION pdyn_lans_lon

FUNCTION pdyn_lans_lat(uzs_v, vzs, k, i, j) RESULT(pdyn)

                 IMPLICIT NONE

                 !Inpout parameter
                 REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: uzs_v, vzs

                 INTEGER, INTENT(IN) :: k, i, j

                 !Output parameter
                 REAL(KIND=rsh) :: pdyn

                 !Local declaration
                 REAL(KIND=rsh) :: dudx, dudy, dvdx, dvdy, alpha

                 IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN

                        alpha = smooth_coeff*dy(i,j)

                        dudx  = centered_duzdx(uzs_v, k, i, j)
                        dudy  = centered_duzdy(uzs_v, k, i, j)
                        dvdx  = centered_dvzdx(vzs, k, i, j)
                        dvdy  = centered_dvzdy(vzs, k, i, j)
                     

                        pdyn = 0.5_rsh*(alpha*alpha*(dudx*dudx+dudy*dudy+dvdx*dvdx+dvdy*dvdy))

                 END IF

END FUNCTION pdyn_lans_lat
