!-------------------------------------------
!COMPUTE GRADIENT USING QUICK SCHEME
!-------------------------------------------

!x-direction

        FUNCTION quick_dudx(u, vel_adv, i, j) RESULT(dudx)

                IMPLICIT NONE

                !Inpout parameter
                 REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dudx

                IF(vel_adv>0) THEN

                        dudx = (3.0_rsh*u(i,j)-7.0_rsh*u(i-1,j)+3.0_rsh*u(i+1,j)+1.0_rsh*u(i-2,j)*whx(i-1,j))/(8.0_rsh*dx_u(i,j))

                ELSE
                        dudx = (-3.0_rsh*u(i,j)+7.0_rsh*u(i+1,j)-3.0_rsh*u(i-1,j)-1.0_rsh*u(i+2,j)*whx(i+1,j))/(8.0_rsh*dx_u(i,j))

                END IF

       END FUNCTION quick_dudx

        FUNCTION quick_duzdx(uz, vel_adv, k, i, j) RESULT(duzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz

                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: duzdx

                IF(vel_adv>0) THEN

                        duzdx = (3.0_rsh*uz(k,i,j)-7.0_rsh*uz(k,i-1,j)+3.0_rsh*uz(k,i+1,j)+1.0_rsh*uz(k,i-2,j)*whx(i-1,j))/(8.0_rsh*dx_u(i,j))

                ELSE
                        duzdx = (-3.0_rsh*uz(k,i,j)+7.0_rsh*uz(k,i+1,j)-3.0_rsh*uz(k,i-1,j)-1.0_rsh*uz(k,i+2,j)*whx(i+1,j))/(8.0_rsh*dx_u(i,j))

                END IF

        END FUNCTION quick_duzdx

        FUNCTION quick_dvdx(v, vel_adv, i, j) RESULT(dvdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dvdx

                IF(vel_adv>0) THEN

                        dvdx = (3.0_rsh*v(i,j)-7.0_rsh*v(i-1,j)+3.0_rsh*v(i+1,j)+1.0_rsh*v(i-2,j)*mo(i-1,j)*mo(i,j))/(8.0_rsh*dx_v(i,j))

                ELSE
                        dvdx = (-3.0_rsh*v(i,j)+7.0_rsh*v(i+1,j)-3.0_rsh*v(i-1,j)-1.0_rsh*v(i+2,j)*me(i+1,j)*me(i,j))/(8.0_rsh*dx_v(i,j))

                END IF

        END FUNCTION quick_dvdx

        FUNCTION quick_dvzdx(vz, vel_adv, k, i, j) RESULT(dvzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: dvzdx, vzkim2j

                IF(vel_adv>0) THEN

                        dvzdx = (3.0_rsh*vz(k,i,j)-7.0_rsh*vz(k,i-1,j)+3.0_rsh*vz(k,i+1,j)+1.0_rsh*vz(k,i-2,j)*mo(i-1,j)*mo(i,j))/(8.0_rsh*dx_v(i,j))

                ELSE
                        dvzdx = (-3.0_rsh*vz(k,i,j)+7.0_rsh*vz(k,i+1,j)-3.0_rsh*vz(k,i-1,j)-1.0_rsh*vz(k,i+2,j)*me(i+1,j)*me(i,j))/(8.0_rsh*dx_v(i,j))

                END IF

        END FUNCTION quick_dvzdx

!y-direction
        !2d variable
        FUNCTION quick_dudy(u, vel_adv, i, j) RESULT(dudy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dudy

                IF(vel_adv>0) THEN

                        dudy = (3.0_rsh*u(i,j)-7.0_rsh*u(i,j-1)+3.0_rsh*u(i,j+1)+1.0_rsh*u(i,j-2)*ms(i,j-1)*ms(i,j))/(8.0_rsh*dy_u(i,j))

                ELSE
                        dudy = (-3.0_rsh*u(i,j)+7.0_rsh*u(i,j+1)-3.0_rsh*u(i,j+1)-1.0_rsh*u(i,j+2)*mn(i,j+1)*mn(i,j))/(8.0_rsh*dy_u(i,j))

                END IF

        END FUNCTION quick_dudy

        !3d variable
        FUNCTION quick_duzdy(uz, vel_adv, k, i, j) RESULT(duzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdy

                IF(vel_adv>0) THEN

                        duzdy =  (3.0_rsh*uz(k,i,j)-7.0_rsh*uz(k,i,j-1)+3.0_rsh*uz(k,i,j+1)+1.0_rsh*uz(k,i,j-2)*ms(i,j-1)*ms(i,j))/(8.0_rsh*dy_u(i,j))

                ELSE
                        duzdy = (-3.0_rsh*uz(k,i,j)+7.0_rsh*uz(k,i,j+1)-3.0_rsh*uz(k,i,j-1)-1.0_rsh*uz(k,i,j+2)*mn(i,j+1)*mn(i,j))/(8.0_rsh*dy_u(i,j))

                END IF

        END FUNCTION quick_duzdy

        FUNCTION quick_dvdy(v, vel_adv, i, j) RESULT(dvdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh) :: dvdy

                IF(vel_adv>0) THEN

                        dvdy = (3.0_rsh*v(i,j)-7.0_rsh*v(i,j-1)+3.0_rsh*v(i,j+1)+1.0_rsh*v(i,j-2)*why(i,j-1))/(8.0_rsh*dy_v(i,j))

                ELSE
                        dvdy = (-3.0_rsh*v(i,j)+7.0_rsh*v(i,j+1)-3.0_rsh*v(i,j+1)-1.0_rsh*v(i,j+2)*why(i,j+1))/(8.0_rsh*dy_v(i,j))

                END IF

        END FUNCTION quick_dvdy

        !3d variable
        FUNCTION quick_dvzdy(vz, vel_adv, k, i, j) RESULT(dvzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz 
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdy

                IF(vel_adv>0) THEN

                        dvzdy = (3.0_rsh*vz(k,i,j)-7.0_rsh*vz(k,i,j-1)+3.0_rsh*vz(k,i,j+1)+1.0_rsh*vz(k,i,j-2)*why(i,j-1))/(8.0_rsh*dy_v(i,j))

                ELSE
                        dvzdy = (-3.0_rsh*vz(k,i,j)+7.0_rsh*vz(k,i,j+1)-3.0_rsh*vz(k,i,j-1)-1.0_rsh*vz(k,i,j+2)*why(i,j+1))/(8.0_rsh*dy_v(i,j))

                END IF

        END FUNCTION quick_dvzdy

!z-direction
        FUNCTION quick_duzdz(uz, vel_adv, dsig, k, i, j) RESULT(duzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdz

                IF(vel_adv>0) THEN

                        duzdz = (3.0_rsh*uz(k,i,j)-7.0_rsh*uz(k-1,i,j)+3.0_rsh*uz(k+1,i,j)+1.0_rsh*uz(k-2,i,j))/(8.0_rsh*dsig)

                ELSE
                        duzdz = (-3.0_rsh*uz(k,i,j)+7.0_rsh*uz(k+1,i,j)-3.0_rsh*uz(k-1,i,j)-1.0_rsh*uz(k+2,i,j))/(8.0_rsh*dsig)

                END IF

        END FUNCTION quick_duzdz

        FUNCTION quick_dvzdz(vz, vel_adv, dsig, k, i, j) RESULT(dvzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdz

                IF(vel_adv>0) THEN

                        dvzdz = (3.0_rsh*vz(k,i,j)-7.0_rsh*vz(k-1,i,j)+3.0_rsh*vz(k+1,i,j)+1.0_rsh*vz(k-2,i,j))/(8.0_rsh*dsig)

                ELSE
                        dvzdz = (-3.0_rsh*vz(k,i,j)+7.0_rsh*vz(k+1,i,j)-3.0_rsh*vz(k-1,i,j)-1.0_rsh*vz(k+2,i,j))/(8.0_rsh*dsig)

                END IF

        END FUNCTION quick_dvzdz
