!-------------------------------------------
!COMPUTE GRADIENT USING QUICKEST SCHEME
!-------------------------------------------

!x-direction
        FUNCTION quickest_duzdx(uz, vel_adv, k, i, j) RESULT(duzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz     
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: duzdx, cfl

                cfl = vel_adv*dt/dx_u(i,j)

                IF(vel_adv>0) THEN

                        duzdx =  0.5_rsh*(1.0_rsh*uz(k,i+1,j)-1.0_rsh*uz(k,i-1,j))/dx_u(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*uz(k,i+1,j)-2.0_rsh*uz(k,i,j)+uz(k,i-1,j))/dx_u(i,j) &
                                +(1.0_rsh-cfl*cfl)*(1.0_rsh*uz(k,i-2,j)*whx(i-1,j)-3.0_rsh*uz(k,i-1,j)+3.0_rsh*uz(k,i,j)-1.0_rsh*uz(k,i+1,j))/(6.0_rsh*dx_u(i,j))
                ELSE

                        duzdx =  0.5_rsh*(1.0_rsh*uz(k,i+1,j)-1.0_rsh*uz(k,i-1,j))/dx_u(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*uz(k,i+1,j)-2.0_rsh*uz(k,i,j)+uz(k,i-1,j))/dx_u(i,j) &
                                +(1.0_rsh-cfl*cfl)*(-1.0_rsh*uz(k,i+2,j)*whx(i+1,j)+3.0_rsh*uz(k,i+1,j)-3.0_rsh*uz(k,i,j)+1.0_rsh*uz(k,i-1,j))/(6.0_rsh*dx_u(i,j))
                END IF

        END FUNCTION quickest_duzdx

        FUNCTION quickest_dvzdx(vz, vel_adv, k, i, j) RESULT(dvzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: dvzdx, cfl

                cfl = vel_adv*dt/dx_v(i,j)

                IF(vel_adv>0) THEN

                        dvzdx = 0.5_rsh*(1.0_rsh*vz(k,i+1,j)-1.0_rsh*vz(k,i-1,j))/dx_v(i,j) &
                               -0.5_rsh*cfl*(1.0_rsh*vz(k,i+1,j)-2.0_rsh*vz(k,i,j)+vz(k,i-1,j))/dx_v(i,j) &
                               +(1.0_rsh-cfl*cfl)*(1.0_rsh*vz(k,i-2,j)*mo(i-1,j)*mo(i,j)-3.0_rsh*vz(k,i-1,j)+3.0_rsh*vz(k,i,j)-1.0_rsh*vz(k,i+1,j))/(6.0_rsh*dx_v(i,j))
                ELSE
                        dvzdx = 0.5_rsh*(1.0_rsh*vz(k,i+1,j)-1.0_rsh*vz(k,i-1,j))/dx_v(i,j) &
                               -0.5_rsh*cfl*(1.0_rsh*vz(k,i+1,j)-2.0_rsh*vz(k,i,j)+vz(k,i-1,j))/dx_v(i,j) &
                               +(1.0_rsh-cfl*cfl)*(-1.0_rsh*vz(k,i+2,j)*me(i+1,j)*me(i,j)+3.0_rsh*vz(k,i+1,j)-3.0_rsh*vz(k,i,j)+1.0_rsh*vz(k,i-1,j))/(6.0_rsh*dx_v(i,j))
                END IF

        END FUNCTION quickest_dvzdx


!y-direction
        FUNCTION quickest_duzdy(uz, vel_adv, k, i, j) RESULT(duzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdy, cfl

                cfl = vel_adv*dt/dy_u(i,j)

                IF(vel_adv>0) THEN

                        duzdy =  0.5_rsh*(1.0_rsh*uz(k,i,j+1)-1.0_rsh*uz(k,i,j-1))/dy_u(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*uz(k,i,j+1)-2.0_rsh*uz(k,i,j)+uz(k,i,j-1))/dy_u(i,j)  &
                                +(1.0_rsh-cfl*cfl)*(1.0_rsh*uz(k,i,j-2)*ms(i,j-1)*ms(i,j)-3.0_rsh*uz(k,i,j-1)+3.0_rsh*uz(k,i,j)-1.0_rsh*uz(k,i,j+1))/(6.0_rsh*dy_u(i,j))
                ELSE

                        duzdy =  0.5_rsh*(1.0_rsh*uz(k,i,j+1)-1.0_rsh*uz(k,i,j-1))/dy_u(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*uz(k,i,j+1)-2.0_rsh*uz(k,i,j)+uz(k,i,j-1))/dy_u(i,j) &
                                +(1.0_rsh-cfl*cfl)*(-1.0_rsh*uz(k,i,j+2)*mn(i,j+1)*mn(i,j)+3.0_rsh*uz(k,i,j+1)-3.0_rsh*uz(k,i,j)+1.0_rsh*uz(k,i,j-1))/(6.0_rsh*dy_u(i,j))
                END IF

        END FUNCTION quickest_duzdy

        FUNCTION quickest_dvzdy(vz, vel_adv, k, i, j) RESULT(dvzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdy, cfl

                cfl = vel_adv*dt/dy_v(i,j)

                IF(vel_adv>0) THEN

                        dvzdy =  0.5_rsh*(1.0_rsh*vz(k,i,j+1)-1.0_rsh*vz(k,i,j-1))/dy_v(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*vz(k,i,j+1)-2.0_rsh*vz(k,i,j)+vz(k,i,j-1))/dy_v(i,j) &
                                +(1.0_rsh-cfl*cfl)*(1.0_rsh*vz(k,i,j-2)*why(i,j-1)-3.0_rsh*vz(k,i,j-1)+3.0_rsh*vz(k,i,j)-1.0_rsh*vz(k,i,j+1))/(6.0_rsh*dy_v(i,j))
                ELSE

                        dvzdy =  0.5_rsh*(1.0_rsh*vz(k,i,j+1)-1.0_rsh*vz(k,i,j-1))/dy_v(i,j) &
                                -0.5_rsh*cfl*(1.0_rsh*vz(k,i,j+1)-2.0_rsh*vz(k,i,j)+vz(k,i,j-1))/dy_v(i,j) &
                                +(1.0_rsh-cfl*cfl)*(-1.0_rsh*vz(k,i,j+2)*why(i,j+1)+3.0_rsh*vz(k,i,j+1)-3.0_rsh*vz(k,i,j)+1.0_rsh*vz(k,i,j-1))/(6.0_rsh*dy_v(i,j))
                END IF

        END FUNCTION quickest_dvzdy

!z-direction
        FUNCTION quickest_duzdz(uz, vel_adv, dsig, k, i, j) RESULT(duzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: duzdz, cfl

                cfl = vel_adv*dt/dsig

                IF(vel_adv>0) THEN

                        duzdz = 0.5_rsh*(1.0_rsh*uz(k+1,i,j)-1.0_rsh*uz(k-1,i,j))/dsig &
                               -0.5_rsh*cfl*(1.0_rsh*uz(k+1,i,j)-2.0_rsh*uz(k,i,j)+uz(k-1,i,j))/dsig &
                               +(1.0_rsh-cfl*cfl)*(1.0_rsh*uz(k-2,i,j)-3.0_rsh*uz(k-1,i,j)+3.0_rsh*uz(k,i,j)-1.0_rsh*uz(k+1,i,j))/(6.0_rsh*dsig)
                ELSE

                        duzdz = 0.5_rsh*(1.0_rsh*uz(k+1,i,j)-1.0_rsh*uz(k-1,i,j))/dsig &
                               -0.5_rsh*cfl*(1.0_rsh*uz(k+1,i,j)-2.0_rsh*uz(k,i,j)+uz(k-1,i,j))/dsig &
                               +(1.0_rsh-cfl*cfl)*(-1.0_rsh*uz(k+2,i,j)+3.0_rsh*uz(k+1,i,j)-3.0_rsh*uz(k,i,j)+1.0_rsh*uz(k-1,i,j))/(6.0_rsh*dsig)
                END IF

        END FUNCTION quickest_duzdz

        FUNCTION quickest_dvzdz(vz, vel_adv, dsig, k, i, j) RESULT(dvzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh) :: dvzdz, cfl

                cfl = vel_adv*dt/dsig

                IF(vel_adv>0) THEN

                        dvzdz = 0.5_rsh*(1.0_rsh*vz(k+1,i,j)-1.0_rsh*vz(k-1,i,j))/dsig &
                               -0.5_rsh*cfl*(1.0_rsh*vz(k+1,i,j)-2.0_rsh*vz(k,i,j)+vz(k-1,i,j))/dsig &
                               +(1.0_rsh-cfl*cfl)*(1.0_rsh*vz(k-2,i,j)-3.0_rsh*vz(k-1,i,j)+3.0_rsh*vz(k,i,j)-1.0_rsh*vz(k+1,i,j))/(6.0_rsh*dsig)
                ELSE

                        dvzdz = 0.5_rsh*(1.0_rsh*vz(k+1,i,j)-1.0_rsh*vz(k-1,i,j))/dsig &
                               -0.5_rsh*cfl*(1.0_rsh*vz(k+1,i,j)-2.0_rsh*vz(k,i,j)+vz(k-1,i,j))/dsig &
                               +(1.0_rsh-cfl*cfl)*(-1.0_rsh*vz(k+2,i,j)+3.0_rsh*vz(k+1,i,j)-3.0_rsh*vz(k,i,j)+1.0_rsh*vz(k-1,i,j))/(6.0_rsh*dsig)
                END IF

        END FUNCTION quickest_dvzdz
