!COMPUTE THE GRADIENT USING UPWIND SCHEME
!----------------------------------------

!x-direction

        !2d variable
        FUNCTION upwind_dudx(u, vel_adv, i, j) RESULT(dudx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dudx

                IF(vel_adv>0) THEN

                        dudx = (u(i,j)-u(i-1,j))/dx(i,j)

                ELSE
                        dudx = (u(i+1,j)-u(i,j))/dx(i+1,j)

                END IF

        END FUNCTION upwind_dudx

        !3d variable
        FUNCTION upwind_duzdx(uz, vel_adv, k, i, j) RESULT(duzdx)

                IMPLICIT NONE

                !Inpout parameter

                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: duzdx

                IF(vel_adv>0) THEN

                        duzdx = (uz(k,i,j)-uz(k,i-1,j))/dx(i,j)

                ELSE
                        duzdx = (uz(k,i+1,j)-uz(k,i,j))/dx(i+1,j)

                END IF

        END FUNCTION upwind_duzdx

        FUNCTION upwind_dvdx(v, vel_adv, i, j) RESULT(dvdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dvdx

                IF(vel_adv>0) THEN

                        dvdx = (v(i,j)-v(i-1,j))/dx_f(i-1,j)

                ELSE
                        dvdx = (v(i+1,j)-v(i,j))/dx_f(i,j)

                END IF

        END FUNCTION upwind_dvdx

        !3d variable
        FUNCTION upwind_dvzdx(vz, vel_adv, k, i, j) RESULT(dvzdx)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: dvzdx

                IF(vel_adv>0) THEN

                        dvzdx = (vz(k,i,j)-vz(k,i-1,j))/dx_f(i-1,j)

                ELSE
                        dvzdx = (vz(k,i+1,j)-vz(k,i,j))/dx_f(i,j)

                END IF

        END FUNCTION upwind_dvzdx

!y-direction
        !2d variable
        FUNCTION upwind_dudy(u, vel_adv, i, j) RESULT(dudy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: u
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dudy

                IF(vel_adv>0) THEN

                        dudy = (u(i,j)-u(i,j-1))/dy_f(i,j-1)

                ELSE
                        dudy = (u(i,j+1)-u(i,j))/dy_f(i,j)

                END IF

        END FUNCTION upwind_dudy

        !3d variable
        FUNCTION upwind_duzdy(uz, vel_adv, k, i, j) RESULT(duzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: uz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: duzdy, uzkijm1

                IF(vel_adv>0) THEN

                        duzdy = (uz(k,i,j) - uz(k,i,j-1))/dy_f(i,j-1)

                ELSE
                        duzdy = (uz(k,i,j+1) - uz(k,i,j))/dy_f(i,j)

                END IF

        END FUNCTION upwind_duzdy

        FUNCTION upwind_dvdy(v, vel_adv, i, j) RESULT(dvdy)

                IMPLICIT NONE

               !Inpout parameter
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: v
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: i, j

                !Output parameter
                REAL(kind=rsh)             :: dvdy

                IF(vel_adv>0) THEN

                        dvdy = (v(i,j)-v(i,j-1))/dy(i,j+1)

                ELSE
                        dvdy = (v(i,j+1)-v(i,j))/dy(i,j)

                END IF

        END FUNCTION upwind_dvdy


        !3d variable
        FUNCTION upwind_dvzdy(vz, vel_adv, k, i, j) RESULT(dvzdy)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: dvzdy

                IF(vel_adv>0) THEN

                        dvzdy = (vz(k,i,j) - vz(k,i,j-1))/dy(i,j+1)

                ELSE
                        dvzdy = (vz(k,i,j+1) - vz(k,i,j))/dy(i,j)

                END IF

        END FUNCTION upwind_dvzdy

!z-direction
        FUNCTION upwind_duzdz(uz, vel_adv, dsigkm1, dsigk, k, i, j) RESULT(duzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: uz

                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsigkm1, dsigk

                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: duzdz

                IF(vel_adv>0) THEN

                        duzdz = (uz(k,i,j)-uz(k-1,i,j))/dsigkm1

                ELSE
                        duzdz = (uz(k+1,i,j)-uz(k,i,j))/dsigk

                END IF

        END FUNCTION upwind_duzdz

        FUNCTION upwind_dvzdz(vz, vel_adv, dsigkm1, dsigk, k, i, j) RESULT(dvzdz)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: vz
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsigkm1, dsigk

                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: dvzdz

                IF(vel_adv>0) THEN

                        dvzdz = (vz(k,i,j)-vz(k-1,i,j))/dsigkm1

                ELSE
                        dvzdz = (vz(k+1,i,j)-vz(k,i,j))/dsigk

                END IF

        END FUNCTION upwind_dvzdz
