MODULE correct_obc_y2dvs 
#if defined key_lans_turb || defined key_leray_turb
#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comfluxdo
   USE comcolig2d, ONLY : y2dvs,av,bv,cv

   IMPLICIT NONE

   CONTAINS

   SUBROUTINE y2du_tls_obc(i1,i2,j,tfg,tfd,y2dv_aux)

        USE obccombine, ONLY : obc_ue, obc_uw, obc_sshe, obc_sshw, &
                          l_obc_west,l_obc_east,              &
                          l_obc_char 

        IMPLICIT NONE

        !! * Arguments
        INTEGER,                                                    INTENT(IN) :: i1,i2,j
        REAL(KIND=rsh),                                             INTENT(IN) :: tfg,tfd
        REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv_aux

        IF (.NOT. l_obc_char) THEN

        ! introduce sea suface height at open boundaries

                IF (l_obc_west) THEN
                        IF_MPI ( i1==imin+1+ishift_obc_west .AND. limin<=i1 .AND. i1<=limax) THEN
                                IF_AGRIF (Agrif_Root()) THEN
                                        y2dvs(i1,j) = y2dv_aux(i1  ,j)
                                ELSE_AGRIF
                                        INST_AGRIF y2dvs(i1  ,j) = y2dv_aux(i1 ,j)
                                ENDIF_AGRIF
                        ENDIF_MPI
                END IF

                IF (l_obc_east) THEN
                        IF_MPI ( i2==imax-ishift_obc_east .AND. limin<=i2 .AND. i2<=limax) THEN
                                IF_AGRIF (Agrif_Root()) THEN
                                        y2dvs(i2-1,j)=y2dv_aux(i2-1,j) 
                                ELSE_AGRIF
                                        INST_AGRIF y2dvs(i2-1,j)=y2dv_aux(i2-1,j)
                                ENDIF_AGRIF
                        ENDIF_MPI
                END IF

        ELSE    ! Characteristics method

                ! western boundary impact

                IF (l_obc_west .AND. i1==imin+1+ishift_obc_west) THEN
                        IF_MPI (limin<=i1 .AND. i1<=limax) THEN
                                IF (tfg.eq.1) THEN
                                        y2dvs(i1,j) = y2dv_aux(i1  ,j)
                                END IF
                        ENDIF_MPI
                END IF

                ! eastern boundary impact

                IF (l_obc_east .AND. i2==imax-ishift_obc_east) THEN
                        IF_MPI (limin<=i2 .AND. i2<=limax) THEN
                                IF (tfd.eq.1) THEN
                                        y2dvs(i2-1,j) = y2dv_aux(i2-1,j)
                                END IF
                        ENDIF_MPI
                END IF
        END IF

   END SUBROUTINE y2du_tls_obc

  SUBROUTINE y2dv_tls_obc(j1,j2,i,tfb,tfh,y2dv_aux)

   USE obccombine, ONLY : obc_vn, obc_vs, obc_sshn, obc_sshs, &
                          l_obc_south,l_obc_north,            &
                          l_obc_char
   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                     INTENT(IN)  :: j1,j2,i
   REAL(KIND=rsh),                                              INTENT(IN)  :: tfh,tfb
   REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),  INTENT(IN ) :: y2dv_aux

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (.NOT. l_obc_char) THEN

        ! introduce sea suface height at open boundaries

        IF (l_obc_south) THEN
                IF_MPI ( j1==jmin+1+jshift_obc_south .AND. ljmin<=j1 .AND. j1<=ljmax) THEN
                        IF_AGRIF (Agrif_Root()) THEN
                                y2dvs(i,j1  )=y2dv_aux(i,j1)
                        ELSE_AGRIF
                                INST_AGRIF y2dvs(i,j1) = y2dv_aux(i,j1)
                        ENDIF_AGRIF
                ENDIF_MPI
        END IF

        IF (l_obc_north) THEN
                IF_MPI ( j2==jmax-jshift_obc_north .AND. ljmin<=j2 .AND. j2<=ljmax) THEN
                        IF_AGRIF (Agrif_Root()) THEN
                                y2dvs(i,j2-1) = y2dv_aux(i,j2-1)*bv(i,j2-1)
                        ELSE_AGRIF
                                INST_AGRIF y2dvs(i,j2-1) = y2dv_aux(i,j2-1)
                        ENDIF_AGRIF
                ENDIF_MPI
        END IF


        ELSE    ! Characteristics method

        ! southern boundary impact
        IF (l_obc_south .AND. j1==jmin+1+jshift_obc_south) THEN
                IF_MPI (ljmin<=j1 .AND. j1<=ljmax) THEN
                        IF (tfb.eq.1) THEN
                                y2dvs(i,j1) = y2dv_aux(i,j1)
                        END IF
                ENDIF_MPI
        END IF

        ! northern boundary impact
        IF (l_obc_north .AND. j2==jmax-jshift_obc_north) THEN
                IF_MPI (ljmin<=j2 .AND. j2<=ljmax) THEN
                        IF (tfh.eq.1) THEN
                                y2dvs(i,j2-1) = y2dv_aux(i,j2-1)
                        END IF
                ENDIF_MPI
        END IF

   END IF

   END SUBROUTINE y2dv_tls_obc 
#endif
END MODULE correct_obc_y2dvs 
