 MODULE deconvolution 
#ifdef key_lans_turb

   !!=======================================================================
   !!                        ***  MODULE obcapplyu  ***
   !! Open boundary treatment : apply open boundary schemes
   !!                           for 2d velocities
   !!                           at eastern and western boundaries.
   !!=======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE smoothing
   USE extendfield
   USE obcapplyu
   USE obcapplyv

   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send

   IMPLICIT NONE

   !! * Accessibility
   PUBLIC 

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

   SUBROUTINE deconv_u(us, u) 

              IMPLICIT NONE

              !INPUT
              REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(INOUT) :: u
              !OUTPUT
              REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: us

              !LOCAL DECLARATION
              REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4)  :: u_ext
              REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)  :: us_tmp

              LOGICAL :: Agrif_Root_OMP,Agrif_GrandMother_OMP

              INTEGER :: iter

              u(limin:limax,ljmin:ljmax) = us(limin:limax,ljmin:ljmax)

!                Agrif_Root_OMP = agrif_root()
!                INST_AGRIF Agrif_GrandMother_OMP = agrif_grandmother()
                      
!                DO iter=1,nb_deconv

!                        CALL extend_u(u, u_ext)

 !                       CALL smoothu(u_ext, us_tmp)

!BC CONDITIONS
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_u(us_tmp)
!     IF (Agrif_Root_OMP) THEN
!        CALL obc_apply_u(us_tmp)
!     END IF
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(us_tmp)

!                        u(limin:limax,ljmin:ljmax) = u(limin:limax,ljmin:ljmax) + (us(limin:limax,ljmin:ljmax)-us_tmp(limin:limax,ljmin:ljmax))
!
!
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_u(u)
!     IF (Agrif_Root_OMP) THEN
!        CALL obc_apply_u(u)
!     END IF
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(u)

!                END DO 

   END SUBROUTINE deconv_u


   SUBROUTINE deconv_v(vs, v)

                IMPLICIT NONE

                !INPUT
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(INOUT) :: v
                !OUTPUT
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: vs

                !LOCAL DECLARATION
                REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4)  :: v_ext
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)  :: vs_tmp

                LOGICAL :: Agrif_Root_OMP,Agrif_GrandMother_OMP

                INTEGER :: iter

                v(limin:limax,ljmin:ljmax) = vs(limin:limax,ljmin:ljmax)

!                Agrif_Root_OMP = agrif_root()
!                INST_AGRIF Agrif_GrandMother_OMP = agrif_grandmother()

!                DO iter=1,nb_deconv
!
!                        CALL extend_v(v, v_ext)
!
!                        CALL smoothv(v_ext, vs_tmp)
!
!BC CONDITIONS
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_v(vs_tmp)
!     IF (Agrif_Root_OMP) THEN
!        CALL obc_apply_v(vs_tmp)
!     END IF
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(vs_tmp)

 !                       v(limin:limax,ljmin:ljmax) = v(limin:limax,ljmin:ljmax) + (vs(limin:limax,ljmin:ljmax)-vs_tmp(limin:limax,ljmin:ljmax))

!
!BC CONDITIONS
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_v(v)
!     IF (Agrif_Root_OMP) THEN
!        CALL obc_apply_v(v)
!     END IF
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(v)

!                END DO

   END SUBROUTINE deconv_v

  !!======================================================================

#endif
 END MODULE deconvolution 
