MODULE smooth_Helmholtz 
#if defined key_lans_turb || defined key_leray_turb

    USE parameters

    USE conjugate_gradient

    IMPLICIT NONE

    CONTAINS

    SUBROUTINE assemblyHelmholtzMat(matHelm, id, jd)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(:,:), INTENT(OUT) :: matHelm ! Helmholtz matrix
        INTEGER, INTENT(IN) :: id, jd

        REAL(KIND=rsh), DIMENSION(1:id,1:id) :: DiagMat, Id_alpha ! Block matrix
        INTEGER  :: i,j
        INTEGER  :: ij1,ij2,ij_low1,ij_low2,ij_up1,ij_up2
        REAL     :: Kparam

        Kparam = smooth_coeff**2

        DiagMat(:,:) = 0; Id_alpha(:,:) =  0; matHelm(:,:) = 0

        !BLOCK MATRIX DEFINITION
        !-----------------------

        !First and lawst line
        !-------------------

        DiagMat(1,1) = 1 + 4*Kparam;  DiagMat(1,2) = -Kparam;
        DiagMat(id,id-1) = -Kparam;  DiagMat(id,id) = 1+4*Kparam;

        DO i=2,id-1

                DiagMat(i,i)   =  1+4*Kparam
                DiagMat(i,i-1) =  -Kparam
                DiagMat(i,i+1) =  -Kparam

        END DO

        DO i=1,id

                Id_alpha(i,i)   =  -Kparam

        END DO

        !BUILD THE HELMHOTZ MATRIX
        !-------------------------

        !First block
        matHelm(1:id, 1:id) = DiagMat(1:idu,1:id)

        matHelm(1:id,id+1:2*id) = Id_alpha(1:id,1:id)

        !Central block
        DO j=1,jd-2

                ij_low1 = (j-1)*id+1; ij_low2 = j*id;
                ij1 = j*id+1; ij2 = (j+1)*id
                ij_up1 = (j+1)*id+1; ij_up2 = (j+2)*id

                matHelm(ij1:ij2,ij_low1:ij_low2) = Id_alpha(1:id,1:id)
                matHelm(ij1:ij2,ij1:ij2)  = DiagMat(1:id,1:id)
                matHelm(ij1:ij2,ij_up1:ij_up2) = Id_alpha(1:id,1:id)

        END DO

        !Last block
        matHelm((jd-1)*id+1:jd*id, (jd-2)*id+1:(jd-1)*id) = Id_alpha(1:id,1:id)
        matHelm((jd-1)*id+1:jd*id, (jd-1)*id+1:jd*id) = DiagMat(1:id,1:id)

    END SUBROUTINE assemblyHelmholtzMat

    SUBROUTINE assemblyHelmholtzSm(smHelm, u, id, jd, idmin, jdmin, idmax, jdmax)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(:), INTENT(OUT) :: smHelm! Helmholtz second member

        REAL(KIND=rsh), DIMENSION(:,:), INTENT(IN) :: u

        INTEGER, INTENT(IN)  :: id, jd, idmin, jdmin, idmax, jdmax

        !Loacl decalration
        INTEGER  :: i,j,l
        REAL     :: alpha_param, Kparam

        Kparam = smooth_coeff**2

!i=1
        i=1
!j=1
        j=1
        smHelm(i) = u(i+idmin,j+jdmin) + Kparam*u(idmin,j) + Kparam*u(i,jdmin)
!j=2:ju-1
        DO j=2,jd-1
                l = (j-1)*id+i
                smHelm(l) = u(1+idmin,j+jdmin) + Kparam*u(idmin,j)
        END DO
!j=ju
        j=jd
        l = (j-1)*id+i
        smHelm(l) = u(i+idmin,i+jdmin) + Kparam*u(idmin,j) + Kparam*u(i,jdmax)

!i=2:iu-1
        DO i=2,id-1
!j=1
                smHelm(i) = u(i+idmin,j+jdmin) + Kparam*u(i,jdmin)
!j=2:ju-1
                DO j=2,jd-1
                        l = (j-1)*id+i
                        smHelm(l) = u(i+idmin,j+jdmin)
                END DO
!j=ju
                j = jd
                l = (j-1)*id+i
                SmHelm(l) = u(i+idmin,j+jdmin) + Kparam*u(i,jdmax)
        END DO

!i=iu
        i=id
!j=1
        j=1
        l = (j-1)*id+i
        smHelm(l) = u(i+idmin,j+jdmin) + Kparam*u(idmax,j) + Kparam*u(i,jdmin)
!j=2:ju-1
        DO j=2,jd-1
                l = (j-1)*id+i
                smHelm(l) = u(i+idmin,i+jdmin) + Kparam*u(idmax,j)
        END DO
!j=ju
        j = jd
        l = (j-1)*id+i
        smHelm(l) = u(i+idmin,j+jdmin) + Kparam*u(idmax,j) + Kparam*u(i,jdmax)

    END SUBROUTINE assemblyHelmholtzSm

    SUBROUTINE smoothu_Helmhotz(u, us)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(imin:imax,jmin:jmax),INTENT(IN) :: u

        REAL(KIND=rsh), DIMENSION(imin:imax,jmin:jmax),INTENT(OUT) :: us

        !local declaration
        REAL(kind=rsh), ALLOCATABLE,DIMENSION(:)    :: smHelm, usvec
        REAL(kind=rsh), ALLOCATABLE,DIMENSION(:,:)  :: matHelm, us_int

        !local declaration
        INTEGER :: i,j

        INTEGER :: iter_number = 0

        REAL(KIND=rsh) :: tol = 0.00000001_rsh

        ALLOCATE(smHelm(1:ijdu))
        ALLOCATE(usvec(1:ijdu))

        ALLOCATE(matHelm(1:idu,1:jdu))
        ALLOCATE(us_int(1:idu,1:jdu))

        CALL assemblyHelmholtzSm(smHelm, u, idu, jdu, iumin, jumin, iumax,  jumax)

        CALL assemblyHelmholtzMat(matHelm, idu, jdu)

        usvec(1:ijdu) = smHelm(1:ijdu)

        CALL CGsolve(MatHelm,smHelm,usvec,iter_number,tol)

#ifdef key_grad_conj_infos
        WRITE(*,*) 'Conjugate Gradient Info:'
        WRITE(*,*) 'number_of_iteration:', iter_number
        WRITE(*,*) 'rewsidual:', tol
#endif

        us_int(1:idu,1:jdu) = reshape(usvec, (/ idu, jdu /))

        us(iumin+1:iumax-1,jumin+1:jumax-1) = us_int(1:idu,1:jdu)

!Gauche
        us(imin:iumin,jmin:jmax) = u(imin:iumin,jmin:jmax)
!Droite
        us(iumax:imax,jmin:jmax) = u(iumax:imax,jmin:jmax)
!Bas
        us(imin:imax,jmin:jumin) = u(imin:imax,jmin:jumin)
!Haute
        us(imin:imax,jumax:jmax) = u(imin:imax,jumax:jmax)

        DEALLOCATE(smHelm, matHelm, usvec, us_int)

    END SUBROUTINE smoothu_Helmhotz

    SUBROUTINE smoothv_Helmhotz(v, vs)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(imin:imax,jmin:jmax),INTENT(IN) :: v

        REAL(KIND=rsh), DIMENSION(imin:imax,jmin:jmax),INTENT(OUT) :: vs

        !local declaration
        REAL(kind=rsh), ALLOCATABLE,DIMENSION(:)    :: smHelm, vsvec
        REAL(kind=rsh), ALLOCATABLE,DIMENSION(:,:)  :: matHelm, vs_int

        !local declaration
        INTEGER :: i, j

        INTEGER :: iter_number = 0

        REAL(KIND=rsh) :: tol = 0.00000001_rsh

        ALLOCATE(smHelm(1:ijdv))
        ALLOCATE(vsvec(1:ijdv))
        ALLOCATE(matHelm(1:ijdv,1:ijdv))
        ALLOCATE(vs_int(1:idv,1:jdv))

        CALL assemblyHelmholtzSm(smHelm, v, idv, jdv, ivmin, jvmin, ivmax,  jvmax)

        CALL assemblyHelmholtzMat(matHelm, idv, jdv)

        vsvec(1:ijdv) = smHelm(1:ijdv)

        CALL CGsolve(matHelm,smHelm,vsvec,iter_number,tol)

#ifdef key_grad_conj_infos
        WRITE(*,*) 'Conjugate Gradient Info:'
        WRITE(*,*) 'number_of_iteration:', iter_number
        WRITE(*,*) 'rewsidual:', tol
#endif

        vs_int(1:idv,1:jdv) = reshape(vsvec, (/ idv, jdv /))

        vs(ivmin+1:ivmax-1,jvmin+1:jvmax-1) = vs_int(1:idv,1:jdv)

!Gauche
        vs(imin:ivmin,jmin:jmax) = v(imin:ivmin,jmin:jmax)
!Droite
        vs(ivmax:imax,jmin:jmax) = v(ivmax:imax,jmin:jmax)
!Baws
        vs(imin:imax,jmin:jvmin) = v(imin:imax,jmin:jvmin)
!Haute
        vs(imin:imax,jvmax:jmax) = v(imin:imax,jvmax:jmax)

        DEALLOCATE(smHelm, matHelm, vsvec, vs_int)

    END SUBROUTINE smoothv_Helmhotz

#endif
END MODULE smooth_Helmholtz
