MODULE smooth_fw3
#if defined key_lans_turb || defined key_leray_turb

    USE parameters
    USE comcolig2d
    USE comvars2d
    USE weighting 

    IMPLICIT NONE

    CONTAINS 

    FUNCTION calc_smooth_vel_fw3(vel, w, sum_w, i, j) RESULT(vel_smooth)

         IMPLICIT NONE

         !Inpout parameter
         REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: vel
         REAL(KIND=rsh), DIMENSION(1:9,1:9), INTENT(IN) :: w
         REAL(KIND=rsh), INTENT(IN) :: sum_w
         INTEGER, INTENT(IN) :: i, j

         !Output parameter
         REAL(kind=rsh) :: vel_smooth

         vel_smooth = (w(4,4)*vel(i-1,j-1) + w(4,5)*vel(i,j-1) + w(4,6)*vel(i+1,j-1) + &
                       w(5,4)*vel(i-1,j)   + w(5,5)*vel(i,j)   + w(5,6)*vel(i+1,j)   + &
                       w(6,4)*vel(i-1,j+1) + w(6,5)*vel(i,j+1) + w(6,6)*vel(i+1,j+1)   &
                      )/sum_w

    END FUNCTION calc_smooth_vel_fw3

    FUNCTION calc_smooth_y2dv_fw3(y2dv, w, sum_w, i, j) RESULT(y2dvs)

         IMPLICIT NONE

         !Inpout parameter
         REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv
         REAL(KIND=rsh), DIMENSION(1:9,1:9), INTENT(IN) :: w
         REAL(KIND=rsh), INTENT(IN) :: sum_w
         INTEGER, INTENT(IN) :: i, j

         !Output parameter
         REAL(kind=rlg) :: y2dvs

         y2dvs   = (w(4,4)*y2dv(i-1,j-1) + w(4,5)*y2dv(i,j-1) + w(4,6)*y2dv(i+1,j-1) + &
                    w(5,4)*y2dv(i-1,j)   + w(5,5)*y2dv(i,j)   + w(5,6)*y2dv(i+1,j)   + &
                    w(6,4)*y2dv(i-1,j+1) + w(6,5)*y2dv(i,j+1) + w(6,6)*y2dv(i+1,j+1)   &
                   )/sum_w

    END FUNCTION calc_smooth_y2dv_fw3

    SUBROUTINE UPDATE_HX_FW3_WATER_CELL(FW3_WATER_CELL, i, j)

           IMPLICIT NONE

           INTEGER, INTENT(IN) :: i, j

           LOGICAL, INTENT(INOUT) :: FW3_WATER_CELL

           REAL(KIND=rsh) :: him1jp1, hijp1, hip1jp1, him1j, hij, hip1j, him1jm1, hijm1, hip1jm1

           him1jp1 = igdu_ext(i-1,j+1)
           hijp1   = igdu_ext(i,j+1)
           hip1jp1 = igdu_ext(i+1,j+1)

           him1j   = igdu_ext(i-1,j)
           hij     = igdu_ext(i,j)
           hip1j   = igdu_ext(i+1,j)

           him1jm1 = igdu_ext(i-1,j-1)
           hijm1   = igdu_ext(i,j-1)
           hip1jm1 = igdu_ext(i+1,j-1)

           FW3_WATER_CELL = (him1jp1 /= 4).AND.(hijp1  /= 4).AND.(hip1jp1  /= 4).AND. &
                            (him1j   /= 4).AND.(hij    /= 4).AND.(hip1j    /= 4).AND. &
                            (him1jm1 /= 4).AND.(hijm1  /= 4).AND.(hip1jm1  /= 4)

    END SUBROUTINE UPDATE_HX_FW3_WATER_CELL

    SUBROUTINE UPDATE_HY_FW3_WATER_CELL(FW3_WATER_CELL, i, j)

           IMPLICIT NONE

           INTEGER, INTENT(IN) :: i, j

           LOGICAL, INTENT(INOUT) :: FW3_WATER_CELL

           REAL(KIND=rsh) :: him1jp1, hijp1, hip1jp1, him1j, hij, hip1j, him1jm1, hijm1, hip1jm1

           him1jp1 = igdv_ext(i-1,j+1)
           hijp1   = igdv_ext(i,j+1)
           hip1jp1 = igdv_ext(i+1,j+1)

           him1j   = igdv_ext(i-1,j)
           hij     = igdv_ext(i,j)
           hip1j   = igdv_ext(i+1,j)

           him1jm1 = igdv_ext(i-1,j-1)
           hijm1   = igdv_ext(i,j-1)
           hip1jm1 = igdv_ext(i+1,j-1)

           FW3_WATER_CELL = (him1jp1 /= 4).AND.(hijp1  /= 4).AND.(hip1jp1  /= 4).AND. &
                            (him1j   /= 4).AND.(hij    /= 4).AND.(hip1j    /= 4).AND. &
                            (him1jm1 /= 4).AND.(hijm1  /= 4).AND.(hip1jm1  /= 4)

    END SUBROUTINE UPDATE_HY_FW3_WATER_CELL

    SUBROUTINE smoothuInd_fw3

           IMPLICIT NONE

           REAL(KIND=rsh) :: him1jp1, hijp1, hip1jp1, him1j, hij, hip1j, him1jm1, hijm1, hip1jm1

           LOGICAL :: WATER_CELL

           INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                        IF (i >= imin+2 .AND. i <= imax-2 .AND. j >= jmin+3 .AND. j <= jmax-2) THEN

                                CALL UPDATE_HX_FW3_WATER_CELL(WATER_CELL, i, j)

                                IF (WATER_CELL == .TRUE.) THEN

                                        Indu(i,j) = 3.0_rsh
                                ELSE
                                        Indu(i,j) = 0.0_rsh
                                END IF
                        ELSE
                                Indu(i,j) = 0.0_rsh
                        END IF
                END DO
           END DO
!$OMP END DO

    END SUBROUTINE smoothuInd_fw3

    SUBROUTINE smoothvInd_fw3

           IMPLICIT NONE

           REAL(KIND=rsh) :: him1jp1, hijp1, hip1jp1, him1j, hij, hip1j, him1jm1, hijm1, hip1jm1

           LOGICAL :: WATER_CELL

           INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                        IF (i >= imin+3 .AND. i <= imax-2 .AND. j >= jmin+2 .AND. j <= jmax-2) THEN

                                CALL UPDATE_HY_FW3_WATER_CELL(WATER_CELL, i, j)

                                IF (WATER_CELL == .TRUE.) THEN

                                        Indv(i,j) = 3.0_rsh
                                ELSE
                                        Indv(i,j) = 0.0_rsh
                                END IF
                        ELSE
                                Indv(i,j) = 0.0_rsh
                        END IF
                END DO
           END DO
!$OMP END DO
    END SUBROUTINE smoothvInd_fw3

    SUBROUTINE smoothu_fw3(u, us) 

           IMPLICIT NONE

           REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN) :: u

           REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(INOUT) :: us

           INTEGER :: i, j

           LOGICAL :: FW3_WATER_CELL

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                        IF (Indu(i,j) == 3.0_rsh) THEN
                                us(i,j) = calc_smooth_vel_fw3(u, weight_fw3, sum_fw3, i, j)
                        ELSE
                                us(i,j) = u(i,j)
                        END IF
                END DO
           END DO
!$OMP END DO

    END SUBROUTINE smoothu_fw3

    SUBROUTINE smoothy2du_fw3(y2dv, y2dvs)

           IMPLICIT NONE

           REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv

           REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax), INTENT(INOUT) :: y2dvs

           INTEGER :: i, j

           LOGICAL :: FW3_WATER_CELL

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                        IF (Indu(i,j) == 3.0_rsh) THEN
                                y2dvs(i,j) = calc_smooth_y2dv_fw3(y2dv, weight_fw3, sum_fw3, i, j)
                        ELSE
                                y2dvs(i,j) = y2dv(i,j)
                        END IF
                END DO
           END DO
!$OMP END DO

    END SUBROUTINE smoothy2du_fw3

    SUBROUTINE smoothv_fw3(v, vs)

           IMPLICIT NONE

           REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN) :: v

           REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(INOUT) :: vs

           INTEGER :: i, j

           LOGICAL :: FW3_WATER_CELL

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax
                           
                        IF (Indv(i,j) == 3.0_rsh) THEN
                                vs(i,j) = calc_smooth_vel_fw3(v, weight_fw3, sum_fw3, i, j)
                        ELSE
                                vs(i,j) = v(i,j)
                        END IF
             END DO
           END DO
!$OMP END DO

    END SUBROUTINE smoothv_fw3

    SUBROUTINE smoothy2dv_fw3(y2dv, y2dvs)

           IMPLICIT NONE

           REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv

           REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax), INTENT(INOUT) :: y2dvs

           INTEGER :: i, j

           LOGICAL :: FW3_WATER_CELL

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax
                        IF (Indv(i,j) == 3.0_rsh) THEN
                                y2dvs(i,j) = calc_smooth_y2dv_fw3(y2dv, weight_fw3, sum_fw3, i, j)
                        ELSE
                                y2dvs(i,j) = y2dv(i,j)
                        END IF
              END DO
           END DO
!$OMP END DO
    END SUBROUTINE smoothy2dv_fw3
#endif
END MODULE smooth_fw3
