MODULE smooth_fw5
#if defined key_lans_turb || defined key_leray_turb

    USE parameters
    USE comvars2d
    USE smooth_fw3

    IMPLICIT NONE

    CONTAINS

    FUNCTION calc_smooth_vel_fw5(vel, w, sum_w, i, j) RESULT(vel_smooth)

         IMPLICIT NONE

         !Inpout parameter
         REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: vel

         REAL(KIND=rsh), DIMENSION(1:9,1:9), INTENT(IN) :: w

         REAL(KIND=rsh), INTENT(IN) :: sum_w

         INTEGER, INTENT(IN) :: i, j

         !Output parameter
         REAL(kind=rsh) :: vel_smooth

         vel_smooth =  (w(3,3)*vel(i-2,j-2) + w(3,4)*vel(i-1,j-2) + w(3,5)*vel(i,j-2) + w(3,6)*vel(i+1,j-2) + w(3,7)*vel(i+2,j-2) + &
                        w(4,3)*vel(i-2,j-1) + w(4,4)*vel(i-1,j-1) + w(4,5)*vel(i,j-1) + w(4,6)*vel(i+1,j-1) + w(4,7)*vel(i+2,j-1) + &
                        w(5,3)*vel(i-2,j)   + w(5,4)*vel(i-1,j)   + w(5,5)*vel(i,j)   + w(5,6)*vel(i+1,j)   + w(5,7)*vel(i+2,j)   + &
                        w(6,3)*vel(i-2,j+1) + w(6,4)*vel(i-1,j+1) + w(6,5)*vel(i,j+1) + w(6,6)*vel(i+1,j+1) + w(6,7)*vel(i+2,j+1) + &
                        w(7,3)*vel(i-2,j+2) + w(7,4)*vel(i-1,j+2) + w(7,5)*vel(i,j+2) + w(7,6)*vel(i+1,j+2) + w(7,7)*vel(i+2,j+2)   &
                       )/sum_w

    END FUNCTION calc_smooth_vel_fw5

    FUNCTION calc_smooth_y2dv_fw5(y2dv, w, sum_w, i, j) RESULT(y2dvs)

         IMPLICIT NONE

         !Inpout parameter
         REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv

         REAL(KIND=rsh), DIMENSION(1:9,1:9), INTENT(IN) :: w

         REAL(KIND=rsh), INTENT(IN) :: sum_w

         INTEGER, INTENT(IN) :: i, j

         !Output parameter
         REAL(kind=rsh) :: y2dvs

         y2dvs =  (w(3,3)*y2dv(i-2,j-2) + w(3,4)*y2dv(i-1,j-2) + w(3,5)*y2dv(i,j-2) + w(3,6)*y2dv(i+1,j-2) + w(3,7)*y2dv(i+2,j-2) + &
                   w(4,3)*y2dv(i-2,j-1) + w(4,4)*y2dv(i-1,j-1) + w(4,5)*y2dv(i,j-1) + w(4,6)*y2dv(i+1,j-1) + w(4,7)*y2dv(i+2,j-1) + &
                   w(5,3)*y2dv(i-2,j)   + w(5,4)*y2dv(i-1,j)   + w(5,5)*y2dv(i,j)   + w(5,6)*y2dv(i+1,j)   + w(5,7)*y2dv(i+2,j)   + &
                   w(6,3)*y2dv(i-2,j+1) + w(6,4)*y2dv(i-1,j+1) + w(6,5)*y2dv(i,j+1) + w(6,6)*y2dv(i+1,j+1) + w(6,7)*y2dv(i+2,j+1) + &
                   w(7,3)*y2dv(i-2,j+2) + w(7,4)*y2dv(i-1,j+2) + w(7,5)*y2dv(i,j+2) + w(7,6)*y2dv(i+1,j+2) + w(7,7)*y2dv(i+2,j+2)   &
                       )/sum_w

    END FUNCTION calc_smooth_y2dv_fw5


    SUBROUTINE UPDATE_HX_FW5_WATER_CELL(FW3_WATER_CELL, FW5_WATER_CELL, i, j)

           IMPLICIT NONE

           INTEGER, INTENT(IN) :: i, j

           LOGICAL, INTENT(INOUT) :: FW3_WATER_CELL, FW5_WATER_CELL

           REAL(KIND=rsh) :: him2jp2,  him1jp2,  hijp2,  hip1jp2, hip2jp2, &
                             him2jp1,  him1jp1,  hijp1,  hip1jp1, hip2jp1, &
                             him2j,    him1j,    hij,    hip1j,   hip2j, &
                             him2jm1,  him1jm1,  hijm1,  hip1jm1, hip2jm1, &
                             him2jm2,  him1jm2,  hijm2,  hip1jm2, hip2jm2

           him2jp2 = igdu_ext(i-2,j+2)
           him1jp2 = igdu_ext(i-1,j+2)
           hijp2   = igdu_ext(i,  j+2)
           hip1jp2 = igdu_ext(i+1,j+2)
           hip2jp2 = igdu_ext(i+2,j+2)

           him2jp1 = igdu_ext(i-2,j+1)
           him1jp1 = igdu_ext(i-1,j+1)
           hijp1   = igdu_ext(i,  j+1)
           hip1jp1 = igdu_ext(i+1,j+1)
           hip2jp1 = igdu_ext(i+2,j+1)

           him2j   = igdu_ext(i-2,j)
           him1j   = igdu_ext(i-1,j)
           hij     = igdu_ext(i,  j)
           hip1j   = igdu_ext(i+1,j)
           hip2j   = igdu_ext(i+2,j)

           him2jm1 = igdu_ext(i-2,j-1)
           him1jm1 = igdu_ext(i-1,j-1)
           hijm1   = igdu_ext(i,  j-1)
           hip1jm1 = igdu_ext(i+1,j-1)
           hip2jm1 = igdu_ext(i+2,j-1)

           him2jm2 = igdu_ext(i-2,j-2)
           him1jm2 = igdu_ext(i-1,j-2)
           hijm2   = igdu_ext(i,  j-2)
           hip1jm2 = igdu_ext(i+1,j-2)
           hip2jm2 = igdu_ext(i+2,j-2)

           FW3_WATER_CELL = (him1jp1 /= 4).AND.(hijp1  /= 4).AND.(hip1jp1  /= 4).AND. &
                            (him1j   /= 4).AND.(hij    /= 4).AND.(hip1j    /= 4).AND. &
                            (him1jm1 /= 4).AND.(hijm1  /= 4).AND.(hip1jm1  /= 4)

           FW5_WATER_CELL = FW3_WATER_CELL.AND. &
                            (him2jp2  /= 4).AND.(him1jp2 /= 4).AND.(hijp2 /= 4).AND.  &
                            (hip1jp2  /= 4).AND.(hip2jp2 /= 4).AND. &
                            (him2jp1  /= 4).AND.(hip2jp1 /= 4).AND. &
                            (him2j    /= 4).AND.(hip2j   /= 4).AND. &
                            (him2jm1  /= 4).AND.(hip2jm1 /= 4).AND. &
                            (him2jm2  /= 4).AND.(him1jm2 /= 4).AND.(hijm2  /= 4).AND. &
                            (hip1jm2  /= 4).AND.(hip2jm2 /= 4)
 
    END SUBROUTINE UPDATE_HX_FW5_WATER_CELL

    SUBROUTINE UPDATE_HY_FW5_WATER_CELL(FW3_WATER_CELL, FW5_WATER_CELL, i, j)

           IMPLICIT NONE

           INTEGER, INTENT(IN) :: i, j

           LOGICAL, INTENT(INOUT) :: FW3_WATER_CELL, FW5_WATER_CELL

           REAL(KIND=rsh) :: him2jp2,  him1jp2,  hijp2,  hip1jp2, hip2jp2, &
                             him2jp1,  him1jp1,  hijp1,  hip1jp1, hip2jp1, &
                             him2j,    him1j,    hij,    hip1j,   hip2j, &
                             him2jm1,  him1jm1,  hijm1,  hip1jm1, hip2jm1, &
                             him2jm2,  him1jm2,  hijm2,  hip1jm2, hip2jm2

           him2jp2 = igdv_ext(i-2,j+2)
           him1jp2 = igdv_ext(i-1,j+2)
           hijp2   = igdv_ext(i,  j+2)
           hip1jp2 = igdv_ext(i+1,j+2)
           hip2jp2 = igdv_ext(i+2,j+2)

           him2jp1 = igdv_ext(i-2,j+1)
           him1jp1 = igdv_ext(i-1,j+1)
           hijp1   = igdv_ext(i,  j+1)
           hip1jp1 = igdv_ext(i+1,j+1)
           hip2jp1 = igdv_ext(i+2,j+1)

           him2j   = igdv_ext(i-2,j)
           him1j   = igdv_ext(i-1,j)
           hij     = igdv_ext(i,  j)
           hip1j   = igdv_ext(i+1,j)
           hip2j   = igdv_ext(i+2,j)

           him2jm1 = igdv_ext(i-2,j-1)
           him1jm1 = igdv_ext(i-1,j-1)
           hijm1   = igdv_ext(i,  j-1)
           hip1jm1 = igdv_ext(i+1,j-1)
           hip2jm1 = igdv_ext(i+2,j-1)

           him2jm2 = igdv_ext(i-2,j-2)
           him1jm2 = igdv_ext(i-1,j-2)
           hijm2   = igdv_ext(i,  j-2)
           hip1jm2 = igdv_ext(i+1,j-2)
           hip2jm2 = igdv_ext(i+2,j-2)

           FW3_WATER_CELL = (him1jp1 /= 4).AND.(hijp1  /= 4).AND.(hip1jp1  /= 4).AND. &
                            (him1j   /= 4).AND.(hij    /= 4).AND.(hip1j    /= 4).AND. &
                            (him1jm1 /= 4).AND.(hijm1  /= 4).AND.(hip1jm1  /= 4)

           FW5_WATER_CELL = FW3_WATER_CELL.AND. &
                            (him2jp2  /= 4).AND.(him1jp2 /= 4).AND.(hijp2 /= 4).AND.  &
                            (hip1jp2  /= 4).AND.(hip2jp2 /= 4).AND. &
                            (him2jp1  /= 4).AND.(hip2jp1 /= 4).AND. &
                            (him2j    /= 4).AND.(hip2j   /= 4).AND. &
                            (him2jm1  /= 4).AND.(hip2jm1 /= 4).AND. &
                            (him2jm2  /= 4).AND.(him1jm2 /= 4).AND.(hijm2  /= 4).AND. &
                            (hip1jm2  /= 4).AND.(hip2jm2 /= 4)

    END SUBROUTINE UPDATE_HY_FW5_WATER_CELL

    SUBROUTINE smoothuInd_fw5

           IMPLICIT NONE

           LOGICAL :: FW3_WATER_CELL, FW5_WATER_CELL

           INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                       IF (i >= imin+3 .AND. i <= imax-3 .AND. j >= jmin+4 .AND. j <= jmax-3) THEN

                               CALL UPDATE_HX_FW5_WATER_CELL(FW3_WATER_CELL,FW5_WATER_CELL, i, j)

                               IF (FW5_WATER_CELL == .TRUE.) THEN

                                        Indu(i,j) = 5.0_rsh

                               ELSE IF (FW3_WATER_CELL == .TRUE.) THEN

                                        Indu(i,j) = 3.0_rsh
                               ELSE
                                        Indu(i,j) = 0.0_rsh
                               END IF
!                       ELSE 
                       ELSE IF (i >= imin+2 .AND. i <= imax-2 .AND. j >= jmin+3 .AND. j <= jmax-2) THEN

                               CALL UPDATE_HX_FW3_WATER_CELL(FW3_WATER_CELL, i,j)

                               IF (FW3_WATER_CELL == .TRUE.) THEN

                                       Indu(i,j) = 3.0_rsh
                               ELSE
                                       Indu(i,j) = 0.0_rsh

                               END IF

                       END IF
              END DO
           END DO
!$OMP END DO
    END SUBROUTINE smoothuInd_fw5

    SUBROUTINE smoothvInd_fw5

           IMPLICIT NONE

           LOGICAL :: FW3_WATER_CELL, FW5_WATER_CELL

           INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax

                       IF (i >= imin+4 .AND. i <= imax-3 .AND. j >= jmin+3 .AND. j <= jmax-3) THEN

                               CALL UPDATE_HY_FW5_WATER_CELL(FW3_WATER_CELL,FW5_WATER_CELL, i, j)

                               IF (FW5_WATER_CELL == .TRUE.) THEN 

                                        Indv(i,j) = 5.0_rsh

                               ELSE IF (FW3_WATER_CELL == .TRUE.) THEN

                                        Indv(i,j) = 3.0_rsh
                               ELSE
                                        Indv(i,j) = 0.0_rsh
                               END IF
!                       ELSE
                       ELSE IF (i >= imin+3 .AND. i <= imax-2 .AND. j >= jmin+2 .AND. j <= jmax-2) THEN

                               CALL UPDATE_HY_FW3_WATER_CELL(FW3_WATER_CELL, i, j)

                               IF (FW3_WATER_CELL == .TRUE.) THEN

                                       Indv(i,j) = 3.0_rsh
                               ELSE
                                       Indv(i,j) = 0.0_rsh
                               END IF                
                       END IF
               END DO
           END DO
!$OMP END DO
    END SUBROUTINE smoothvInd_fw5

    SUBROUTINE smoothu_fw5(u, us)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN) :: u

        REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(INOUT) :: us

        INTEGER :: i, j 

!$OMP DO SCHEDULE(runtime)
           DO j=ljmin,ljmax
                DO i=limin,limax
                       IF (Indu(i,j) == 5.0_rsh) THEN
                                us(i,j) = calc_smooth_vel_fw5(u, weight_fw5, sum_fw5, i, j)
                       ELSE IF (Indu(i,j) == 3.0_rsh) THEN
                                us(i,j) = calc_smooth_vel_fw3(u, weight_fw5, sum_fw3, i, j)
                       ELSE
                                us(i,j) = u(i,j)        
                       END IF
                END DO
           END DO
!$OMP END DO

    END SUBROUTINE smoothu_fw5

    SUBROUTINE smoothy2du_fw5(y2dv, y2dvs)

        IMPLICIT NONE

        REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN) :: y2dv

        REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax),INTENT(INOUT) :: y2dvs

        INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
        DO j=ljmin,ljmax
                DO i=limin,limax
                       IF (Indu(i,j) == 5.0_rsh) THEN
                               y2dvs(i,j) = calc_smooth_y2dv_fw5(y2dv, weight_fw5, sum_fw5, i, j)
                       ELSE IF (Indu(i,j) == 3.0_rsh) THEN
                               y2dvs(i,j) = calc_smooth_y2dv_fw3(y2dv, weight_fw5, sum_fw3, i, j)
                       ELSE
                               y2dvs(i,j) = y2dv(i,j) 
                       END IF
                END DO
        END DO
!$OMP END DO

    END SUBROUTINE smoothy2du_fw5

    SUBROUTINE smoothv_fw5(v, vs)

        IMPLICIT NONE

        REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN) :: v

        REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(INOUT) :: vs

        INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
        DO j=ljmin,ljmax
                DO i=limin,limax                            
                        IF (Indv(i,j) == 5.0_rsh) THEN
                                vs(i,j) = calc_smooth_vel_fw5(v, weight_fw5, sum_fw5, i, j)
                        ELSE IF (Indv(i,j) == 3.0_rsh) THEN
                                vs(i,j) = calc_smooth_vel_fw3(v, weight_fw5, sum_fw3, i, j)
                        ELSE
                                vs(i,j) = v(i,j) 
                        END IF
                END DO
        END DO
!$OMP END DO

    END SUBROUTINE smoothv_fw5

    SUBROUTINE smoothy2dv_fw5(y2dv, y2dvs)

        IMPLICIT NONE

        REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4),INTENT(IN)  :: y2dv

        REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax),INTENT(INOUT)   :: y2dvs

        INTEGER :: i, j

!$OMP DO SCHEDULE(runtime)
        DO j=ljmin,ljmax
                DO i=limin,limax                                                       
                        IF (Indv(i,j) == 5.0_rsh) THEN
                                y2dvs(i,j) = calc_smooth_y2dv_fw5(y2dv, weight_fw5, sum_fw5, i, j)
                        ELSE IF (Indv(i,j) == 3.0_rsh) THEN
                                y2dvs(i,j) = calc_smooth_y2dv_fw3(y2dv, weight_fw5, sum_fw3, i, j)
                        ELSE
                                y2dvs(i,j) = y2dv(i,j)
                        END IF
                END DO
        END DO
!$OMP END DO

    END SUBROUTINE smoothy2dv_fw5
#endif
END MODULE smooth_fw5
