MODULE smoothing
#if defined key_lans_turb || defined key_leray_turb
 USE parameters
           
 USE  comvars2d

 USE  smooth_Helmholtz
 USE  smooth_fw3
 USE  smooth_fw5
 USE  smooth_fw7
 USE  smooth_fw9

 IMPLICIT NONE

 CONTAINS

       SUBROUTINE smoothuInd(Indu)

                IMPLICIT NONE

                !OUTPUT
                REAL(KIND=rsh), DIMENSION(limin:limax,ljmin:ljmax), INTENT(INOUT) :: Indu

                IF (smooth_method == 1) THEN

                        IF (filter_width==3) CALL smoothuInd_fw3

                        IF (filter_width==5) CALL smoothuInd_fw5

                        IF (filter_width==7) CALL smoothuInd_fw7

                        IF (filter_width==9) CALL smoothuInd_fw9
                END IF

        END SUBROUTINE smoothuInd

        SUBROUTINE smoothvInd(Indv)

                IMPLICIT NONE

                !OUTPUT
                REAL(KIND=rsh), DIMENSION(limin:limax,ljmin:ljmax), INTENT(INOUT) :: Indv

                IF (smooth_method == 1) THEN

                        IF (filter_width==3) CALL smoothvInd_fw3

                        IF (filter_width==5) CALL smoothvInd_fw5

                        IF (filter_width==7) CALL smoothvInd_fw7

                        IF (filter_width==9) CALL smoothvInd_fw9 
                END IF

        END SUBROUTINE smoothvInd
        
        SUBROUTINE smoothu(u, us)

                IMPLICIT NONE

                !INPUT
                REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: u
                !OUTPUT
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(INOUT) :: us

                IF (smooth_method == 2) THEN 

                        IF (ABS(smooth_coeff)<1E-8_rsh) THEN           

                                us(limin:limax,ljmin:ljmax) = u(limin:limax,ljmin:ljmax)

                        ELSE 
                                CALL smoothu_Helmhotz(u, us)

                        END IF               
 
                ELSE IF (smooth_method == 1) THEN
                        
                        IF (filter_width==3) CALL smoothu_fw3(u, us)

                        IF (filter_width==5) CALL smoothu_fw5(u, us)

                        IF (filter_width==7) CALL smoothu_fw7(u, us)

                        IF (filter_width==9) CALL smoothu_fw9(u, us)           

               END IF

        END SUBROUTINE smoothu

        SUBROUTINE smoothu_y2dv(y2dv, y2dvs)

                IMPLICIT NONE

                !INPUT
                REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv
                !OUTPUT
                REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax),  INTENT(INOUT) :: y2dvs

                IF (smooth_method == 2) THEN

                ELSE IF (smooth_method == 1) THEN

                        IF (filter_width==3) CALL smoothy2du_fw3(y2dv, y2dvs)

                        IF (filter_width==5) CALL smoothy2du_fw5(y2dv, y2dvs)

                        IF (filter_width==7) CALL smoothy2du_fw7(y2dv, y2dvs)

                        IF (filter_width==9) CALL smoothy2du_fw9(y2dv, y2dvs)                        
                END IF

        END SUBROUTINE smoothu_y2dv

        SUBROUTINE smoothv(v, vs)

                IMPLICIT NONE

                !INPUT
                REAL(KIND=rsh), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: v 

                !OUTPUT
                REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(INOUT) :: vs

                IF (smooth_method == 2) THEN

                        IF (ABS(smooth_coeff)<1E-8_rsh) THEN

                                vs(limin:limax,ljmin:ljmax) = v(limin:limax,ljmin:ljmax)

                        ELSE

                                CALL smoothv_Helmhotz(v, vs)

                        END IF

                ELSE IF (smooth_method == 1) THEN

                        IF (filter_width==3) CALL smoothv_fw3(v, vs)

                        IF (filter_width==5) CALL smoothv_fw5(v, vs) 

                        IF (filter_width==7) CALL smoothv_fw7(v, vs)

                        IF (filter_width==9) CALL smoothv_fw9(v, vs)                    

               END IF
        END SUBROUTINE smoothv

        SUBROUTINE smoothv_y2dv(y2dv, y2dvs)

                IMPLICIT NONE

                !INPUT
                REAL(KIND=rlg), DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4), INTENT(IN) :: y2dv
                !OUTPUT
                REAL(KIND=rlg), DIMENSION(liminm1:limax,ljminm1:ljmax),  INTENT(INOUT) :: y2dvs

                IF (smooth_method == 2) THEN

                ELSE IF (smooth_method == 1) THEN

                       IF (filter_width==3) CALL smoothy2dv_fw3(y2dv, y2dvs)

                       IF (filter_width==5) CALL smoothy2dv_fw5(y2dv, y2dvs)

                       IF (filter_width==7) CALL smoothy2dv_fw7(y2dv, y2dvs)

                       IF (filter_width==9) CALL smoothy2dv_fw9(y2dv, y2dvs) 

                END IF
        END SUBROUTINE smoothv_y2dv
#endif
END MODULE smoothing
