!***************************************************
!
       subroutine tide_harmcompo
!
!***************************************************
!
!     objet    : calcul des niveaux de la surface libre a la
!                limite par composition harmonique des donnees
!                issues du modele de maree de Schwiderski (1980).
!                La composition harmonique est
!                basee sur 8 ondes (M2, N2, S2, K2, O1, K1, P1, Q1)
!                ou bien sur M2 seule et une modulation par
!                coefficients de maree renvoyes par tide_calcoef.
!                Cette option est l option par defaut. Elle ne
!                fonctionne que pour le modele a 5km qui va du Portugal a
!                l Islande.  
!
!     appelant         : init_xe, stepu, stpev, step2dxy, step2dyx
!                      :
!     appeles          : tide_calcoef
!                      :
!     canal d ecriture :
!                      :
!     canal de lecture : 7
!                      :
!     cle cpp          :
!                      :
!     variables in     : t, h0, l_tide_M2harm, omegat, file_tide_harmcp
!     variables in     : paref, nbouc, tide_M2coef, pa, mslshift, tide_nbharmcp
!     variables in     : valmanq
!                      :
!     variables out    :
!                      :
!     variables inout  :
!                      :
!     dernieres modifs :             P. Garreau (Original code)
!     dernieres modifs : 14/07/2004
!     dernieres modifs : 2008-03-07 C. Herry (debug : appel tide_astroarg)
   !&E
   !&E ** History :
   !&E       !          Original code
   !&E       !  2009-03 (V. Garnier)  removing of xeuv argument
   !&E                                initialization of obc_tide_ssh*
   !&E       !  2010-02 (V. Garnier)  parametrisation nbfes=8 mais je n en sais rien ATTENTION
   !&E
!                      :
!***********************************************************

#include "toolcpp.h"
!
!
USE parameters
use comvars2d
use cometdyn
   USE comstatic, ONLY : omegat
use comaree
   USE obccombine, ONLY : obc_tide_sshn,obc_tide_sshs,obc_tide_sshe,obc_tide_sshw
implicit none
!
character(len=20)                                :: nom_onde
integer,parameter                                :: nbfes=8     ! cval je n en sais rien ATTENTION
integer                                          :: i,j,n,ibid
real(kind=rsh)                                   :: degrad,tide_coef
real(kind=rsh),dimension(nmarmax)                :: aNw,aNe,aSw,aSe,aWn,aWs,aEn,aEs
real(kind=rsh),dimension(nmarmax)                :: pNw,pNe,pSw,pSe,pWn,pWs,pEn,pEs
real(kind=rsh),dimension(nmarmax,imin:imax),save :: a0sup,a0inf,ph0sup,ph0inf
real(kind=rsh),dimension(nmarmax,jmin:jmax),save :: a0g,ph0g
real(kind=rsh),dimension(limin:limaxp1)          :: xes,xen
real(kind=rsh),dimension(ljmin:ljmaxp1)          :: xew
!
!************************************************************
!     Executable statements
!************************************************************
!
    degrad=180.0_rsh/pi
    tide_coef=1.0_rsh
    tide_nbharmcp=nbfes
!
!   initialisation
!---------------------
!
    if(nbouc == 0) then
!
! lecture des conditions limite en phase et en amplitude
!-------------------------------------------------------
!
     open(7,file=file_tide_harmcp,status='old')
      do n=1,tide_nbharmcp
       read(7,'(a20)') nom_onde
       read(7,*)
       read(7,*)
       read(7,*)
       read(7,*) aNw(n),aNe(n)
       read(7,*) aWn(n),aEn(n)
       read(7,*) aWs(n),aEs(n)
       read(7,*) aSw(n),aSe(n)
       read(7,*)
       read(7,*) pNw(n),pNe(n)
       read(7,*) pWn(n),pEn(n)
       read(7,*) pWs(n),pEs(n)
       read(7,*) pSw(n),pSe(n)
       do i=imin+1,imax
        read(7,*) ibid,a0inf(n,i),ph0inf(n,i),a0sup(n,i),ph0sup(n,i)
       enddo
       do j=jmax-1,jmin,-1
        read(7,*) ibid,a0g(n,j),ph0g(n,j)
       enddo
      enddo
     close(7)
!
! transformation des phases en radians
!-------------------------------------
!
     ph0inf(:,:)=ph0inf(:,:)/degrad
     ph0sup(:,:)=ph0sup(:,:)/degrad
     ph0g(:,:)=ph0g(:,:)/degrad
     pNw(:)=pNw(:)/degrad
     pNe(:)=pNe(:)/degrad
     pWn(:)=pWn(:)/degrad
     pEn(:)=pEn(:)/degrad
     pWs(:)=pWs(:)/degrad
     pEs(:)=pEs(:)/degrad
     pSw(:)=pSw(:)/degrad
     pSe(:)=pSe(:)/degrad
!
! petite correction sur les hauteurs et les phases
!-------------------------------------------------
!
     do n=1,tide_nbharmcp
      do i=imin+1,imax
        a0inf(n,i)=a0inf(n,i)+(aSw(n)*(imax-i)+aSe(n)*i)/real(imax)
        a0sup(n,i)=a0sup(n,i)+(aNw(n)*(imax-i)+aNe(n)*i)/real(imax)
        ph0inf(n,i)=ph0inf(n,i)+(pSw(n)*(imax-i)+pSe(n)*i)/real(imax)
        ph0sup(n,i)=ph0sup(n,i)+(pNw(n)*(imax-i)+pNe(n)*i)/real(imax)
      enddo
      do j=jmin,jmax-1
        a0g(n,j)=a0g(n,j)+(aWs(n)*(jmax-j)+aWn(n)*j)/real(jmax)
        ph0g(n,j)=ph0g(n,j)+(pWs(n)*(jmax-j)+pWn(n)*j)/real(jmax)
      enddo
     enddo
!
    endif
!
!  deroulement normal
!---------------------
!
!
     call tide_astroarg
!
! calcul du coefficient de maree dans le cas
! forcage par modulation de M2 seule
!-------------------------------------------
!
      if (l_tide_M2harm) then
       tide_coef=tide_M2coef
       if(.not.l_tide_M2statcoef) call tide_calcoef(tide_coef)
       tide_coef=tide_coef/70.0_rsh
      endif
!
!     composition sur la frontiere sud
!-------------------------------------------
#ifdef key_MPI_2D
write(*,*)'pa is local; so MPI is not ready yet'
stop
#endif

   IF_MPI (ljmin==jmin) THEN

      do i=liminp1,limaxp1
       xes(i)=mslshift-(pa(i,jmin+1)-paref)/10000.0_rsh
       do n=1,tide_nbharmcp
        xes(i)=xes(i)+a0inf(n,i)*cos(omegat(n)-ph0inf(n,i))*tide_coef
       enddo
      enddo
!
      do i=liminp2,limaxm1p1
       if(h0(i,jmin+1) > -valmanq) obc_tide_sshs(i)=0.5_rsh*xes(i)+0.25_rsh*(xes(i-1)+xes(i+1))
      enddo

   ENDIF_MPI
!
!     composition sur la frontiere nord
!-------------------------------------------
!
   IF_MPI (ljmax==jmax) THEN

      do i=liminp1,limaxp1
       xen(i)=mslshift-(pa(i,jmax)-paref)/10000.0_rsh
       do n=1,tide_nbharmcp
        xen(i)=xen(i)+a0sup(n,i)*cos(omegat(n)-ph0sup(n,i))*tide_coef
       enddo
      enddo

      do i=liminp2,limaxm1p1
       if(h0(i,jmax) > -valmanq) obc_tide_sshn(i)=0.5_rsh*xen(i)+0.25_rsh*(xen(i-1)+xen(i+1))
      enddo

   ENDIF_MPI
!
!     composition sur la frontiere ouest
!-------------------------------------------
!
   IF_MPI (limin==imin) THEN

      do j=ljminp1,ljmaxm1p1
       xew(j)=mslshift-(pa(imin+1,j)-paref)/10000.0_rsh
       do n=1,tide_nbharmcp
        xew(j)=xew(j)+a0g(n,j)*cos(omegat(n)-ph0g(n,j))*tide_coef
       enddo
      enddo
!
      do j=ljminp2,ljmaxm1p1
       if(h0(imin+1,j) > -valmanq) obc_tide_sshw(j)=0.5_rsh*xew(j)+0.25_rsh*(xew(j-1)+xew(j+1))
      enddo

   ENDIF_MPI
!
end subroutine tide_harmcompo
