MODULE toolmarsww3cpl
#ifdef key_oasis_mars_ww3
   !===========================================================================
   !                   ***  MODULE  toolmarsww3cpl  ***
   !
   ! Module used for coupling applications between MARS and WW3 with OASIS3-MCT 
   !
   !===========================================================================

   IMPLICIT NONE
   PRIVATE

  !! * Accessibility
  PUBLIC snd_fields_to_ww3 
  PUBLIC rcv_fields_from_ww3

  CONTAINS


   !======================================================================

  
  SUBROUTINE snd_fields_to_ww3(id_oasis_time)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE snd_fields_to_ww3  ***
   !&E
   !&E ** Purpose : Send coupling fields to WW3 
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-10 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------

   USE parameters,   ONLY : limax, ljmax, kmax, &
                            limin, ljmin,       &
                            liminp1, ljminp1,   &
                            valmanq, rlg
   USE tooloasismct, ONLY : snd_fld, cpl_oasis_snd, il_nb_snd                           
   USE comvarp2d,    ONLY : ssh
   USE comvarp3d,    ONLY : uz, vz   
   USE comvars2d,    ONLY : h0, hx, hy, h0fond, ig, id                      
                          
   !! * Argument
   INTEGER, INTENT(IN) :: id_oasis_time
   
   !! * Local declaration
   REAL(kind=8), DIMENSION(limax-liminp1+1, ljmax-ljminp1+1) :: rla_oasis_snd
   INTEGER                                                   :: ib_do, i, j
   LOGICAL                                                   :: ll_action   
   REAL(kind=8), DIMENSION(limin:limax,ljmin:ljmax)          :: mask_h0,mask_hx,mask_hy

   !!----------------------------------------------------------------------
   !! * Executable part
  
   mask_h0(:,:)=0.0_rlg
   mask_hx(:,:)=0.0_rlg
   mask_hy(:,:)=0.0_rlg
   
   DO j=ljminp1,ljmax
      DO i=MAX(limin,ig(j)),MIN(limax,id(j))
         ! For heights smaller than h0fond, the cell is considered as wet
         IF ( h0(i,j)+ssh(i,j) > h0fond ) mask_h0(i,j)=1.0_rlg
! en fait les masques pour les vitesses u (hx) et v (hy) sont inchanges dans le temps, CAR les vitesses u et v sont nulles a terre ()Sinon le test est plus penible (et couteux) mais possible)
         IF ( hx(i,j) /= -valmanq -h0fond ) mask_hx(i,j)=1.0_rlg
         IF ( hy(i,j) /= -valmanq -h0fond ) mask_hy(i,j)=1.0_rlg
      ENDDO
   ENDDO
  
   DO ib_do = 1, il_nb_snd
      ! Banc découvrant pour la SSH
      IF (snd_fld(ib_do)%cl_field_name == 'MARS_BDH') THEN
         rla_oasis_snd(:,:) = mask_h0(liminp1:limax,ljminp1:ljmax)      
      ENDIF 
      
      ! Banc découvrant pour uz
      IF (snd_fld(ib_do)%cl_field_name == 'MARS_BDU') THEN
         rla_oasis_snd(:,:) = mask_hx(liminp1:limax,ljminp1:ljmax)      
      ENDIF 
      
      ! Banc découvrant pour vz
      IF (snd_fld(ib_do)%cl_field_name == 'MARS_BDV') THEN
         rla_oasis_snd(:,:) = mask_hy(liminp1:limax,ljminp1:ljmax)      
      ENDIF 

      ! sea surface height (m)
      IF (snd_fld(ib_do)%cl_field_name == 'MARS_SSH') THEN
         rla_oasis_snd(:,:) = DBLE(ssh(liminp1:limax,ljminp1:ljmax))      
      ENDIF 

      ! zonal sea surface velocity (m s-1)
      IF (snd_fld(ib_do)%cl_field_name == 'MARS__UZ') THEN
         rla_oasis_snd(:,:) = DBLE(uz(kmax,liminp1:limax,ljminp1:ljmax))
      ENDIF
 
      ! meridional sea surface velocity (m s-1)
      IF (snd_fld(ib_do)%cl_field_name == 'MARS__VZ') THEN
         rla_oasis_snd(:,:) = DBLE(vz(kmax,liminp1:limax,ljminp1:ljmax))
      ENDIF

      ! ici on envoie a chaque pas de temps, du coup oasis peut cumuler faire
      ! des moyennes... puisque oasis recoit l info mais ne la redonne pas a
      ! WW3 quand ce n est pas la date de couplage. 
      ! si on cherche a limiter les copies ci-dessus, on pert cette
      ! possibilite. La limitation peut se faire en precisant en dur la pas
      ! de couplage (MODt,120) par exemple.
      CALL cpl_oasis_snd(ib_do, id_oasis_time, rla_oasis_snd, ll_action)
      
   ENDDO

  END SUBROUTINE snd_fields_to_ww3


   !======================================================================

  
  SUBROUTINE rcv_fields_from_ww3(id_oasis_time)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE rcv_fields_from_ww3  ***
   !&E
   !&E ** Purpose : Receive coupling fields from WW3 
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-10 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------
   
#include "toolcpp.h"

   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh
   USE parameters,   ONLY : limin, ljmin, limax, ljmax, rsh, &
                     &      limaxp1,ljmaxp1, liminp1, ljminp1
   USE tooloasismct, ONLY : rcv_fld, cpl_oasis_rcv, il_nb_rcv 
   USE comwave,      ONLY : wave_hs                ! significant wave height
   USE wave,         ONLY : wave_t0m1,           & ! mean period of the waves
                            wave_thm,            & ! mean direction of the waves
                            wave_bhd,            & ! pression induite par les vagues
                            wave_tauox,          & ! tension liee au deferlement, composante en x
                            wave_tauoy,          & !tension liee au deferlement, composante en y
                            wave_tauwx,          & ! tension de vent supportee par les vagues, composante en x
                            wave_tauwy,          & ! tension de vent supportee par les vagues, composante en y
                            wave_uba,            & ! amplitude of orbital velocity of the waves
                            wave_phioc,          & ! flux d energie associee au deferlement
                            wave_wlm               ! mean length wave
                         
   !! * Argument
   INTEGER, INTENT(IN) :: id_oasis_time
   
   !! * Local declaration
   REAL(kind=8), DIMENSION(limax-liminp1+1, ljmax-ljminp1+1) :: rla_oasis_rcv, mars_dry
   INTEGER                                                   :: ib_do 
   LOGICAL                                                   :: ll_action   

   !!----------------------------------------------------------------------
   !! * Executable part

   DO ib_do = 1, il_nb_rcv
  
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_DRY') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            mars_dry(:,:) = rla_oasis_rcv(:,:)
         ENDIF
      ENDIF

      ! Mean wave period (tmn in s) (m0,-1)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARST0M1') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_t0m1(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF

      ! Significant wave height (hs in m)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS__HS') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_hs(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF

      ! Mean wave direction (degrees)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_DIR') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_thm(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF

      ! Wave-induced Bernoulli head pressure (bhd in N.m-1) (J term, Smith JPO 2006)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_BHD') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_bhd(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
            
            CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,wave_bhd(limin:limaxp1,ljmin:ljmaxp1))
            CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,wave_bhd(limin:limaxp1,ljmin:ljmaxp1))
         ENDIF
      ENDIF

      ! Wave-ocean momentum flux, x-component (tauox in m2.s-2)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTWOX') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_tauox(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
            CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_tauox(limin:limaxp1,ljmin:ljmax))
         ENDIF
      ENDIF

      ! Wave-ocean momentum flux, y-component (tauoy in m2.s-2)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTWOY') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_tauoy(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
            CALL_MPI ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_tauoy(limin:limax,ljmin:ljmaxp1))
         ENDIF
      ENDIF

      ! rms amplitude of orbital bottom velocity of the waves (uba in m/s)
      ! Bottom rms velocity (m/s)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_UBR') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_uba(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF

      ! Wave-to-ocean TKE flux (phioc in W.m-2)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_FOC') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_phioc(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF

      ! Net wave-supported stress, x-component (tauwix in m2.s-2)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTAWX') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_tauwx(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
            CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_tauwx(limin:limaxp1,ljmin:ljmax))
         ENDIF
      ENDIF

      ! Net wave-supported stress, y-component (tauwiy in m2.s-2)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTAWY') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_tauwy(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
            CALL_MPI ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_tauwy(limin:limax,ljmin:ljmaxp1))
         ENDIF
      ENDIF

      ! Mean wave length (wlm in m)
      IF (rcv_fld(ib_do)%cl_field_name == 'MARS__LM') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            wave_wlm(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv(:,:)*mars_dry(:,:), rsh)
         ENDIF
      ENDIF
      
   ENDDO

  END SUBROUTINE rcv_fields_from_ww3

#else
   ! Dummy module 
#endif
END MODULE toolmarsww3cpl
