#ifdef key_substance
  SUBROUTINE parsub_advz_macho(xe2,xe3,c,c2,setvel,ll,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advz_macho  ***
   !&E
   !&E ** Purpose : Estimate of intermediate tracers
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,wz,sal,temp,c(cv_wat),dsigu
   !&E
   !&E ** Modified variables : sal2,temp2,c2
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E ** History :
   !&E           !  2010-05 (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : h0,hm,h0fond,ig,id,halfdt
   USE comvars3d,    ONLY : wz,dsigu
   USE comsubstance, ONLY : nv_adv,nvpi,nv_ws_max,iv_ws_max,nvp
#if defined key_sedim || defined key_biolo
   USE comsubstance,ONLY : ws3
#endif
#if defined key_siggen || defined key_gencoord
   USE toolgeom,      ONLY : f_hzu
#endif
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in)    :: niter,num_ws,ll
   REAL(KIND=rsh),                                                       INTENT(in)    :: setvel
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: c
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: c2


   !! * Local declarations
   INTEGER                   :: i,j,k,iv,l
   REAL(KIND=rsh)            :: h3,xee
   REAL(KIND=rlg), DIMENSION(kmax)      :: wadv
   REAL(KIND=rsh),DIMENSION(1:kmax)     :: ws
   REAL(KIND=rsh),DIMENSION(1:kmax,nvp) :: wsz


   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVZ_MACHO.F90 routine'

   ! estimate of intermediate tracers (from upwind scheme)
   ! -----------------------------------------------------

!cval toseelater HUM que se passe t til pour les partie non initialiesee lors des echanges ?
!cval que fait on au fond

   ! at open boundaries (east and west)

!$OMP SINGLE
   DO l=1,nv_ws_max(num_ws)
     iv=iv_ws_max(num_ws,l)

   IF_MPI (limin==imin) THEN
     c2(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax) = &
                        c(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (limax==imax) THEN
     c2(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax) = &
                        c(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (ljmin==jmin) THEN
     c2(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south) = &
                        c(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south)
   ENDIF_MPI
   IF_MPI (ljmax==jmax) THEN
     c2(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2)) = &
                        c(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2))
   ENDIF_MPI
   ENDDO

!$OMP END SINGLE

   ! inside the domain

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,wadv,h3)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       xee=REAL(ll,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)
       h3=h0(i,j)+xee
       IF(h3 > hm) THEN

         iv=1
#if defined key_sedim || defined key_biolo
          ws(:)=-ws3(:,iv,i,j)
#else
          ! constant settling velocity
          ws(:)=-setvel
#endif
          
!         k=1
!         DO l=1,nv_ws_max(num_ws)
!             iv=iv_ws_max(num_ws,l)
!             IF (wsz(k,iv) >= 0.0_rsh) THEN
!              wadv(k)=0.0_rsh
!             ELSE
!              wadv(k)=wsz(k,iv)
!             ENDIF
!             c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/dsigu(k))*wadv(k) &
!                              *(c(iv,k+1,i,j)-c(iv,k,i,j))/REAL(niter,rsh)
!         ENDDO
!         k=kmax
!         DO l=1,nv_ws_max(num_ws)
!             iv=iv_ws_max(num_ws,l)
!             IF (wsz(k,iv) <= 0.0_rsh) THEN
!              wadv(k)=0.0_rsh
!             ELSE
!              wadv(k)=wsz(k-1,iv)
!             ENDIF
!             c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/dsigu(k))*wadv(k) &
!                              *(c(iv,k,i,j)-c(iv,k-1,i,j))/REAL(niter,rsh)
!         ENDDO


         DO k=2,kmax-1

           ! estimate of intermediate tracers
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)

          ! estimate of the advective velocity
! a revoir car pas sur que mpyenne soit correct en siggen
           !wadv(k)=0.5_rsh*(wz(k-1,i,j)+wz(k,i,j))
            IF (wz(k,i,j)*wz(k-1,i,j) > 0.0_rsh) THEN
             IF (wz(k,i,j) > 0.0_rsh) THEN
               wadv(k)=wz(k-1,i,j)
             ELSE
               wadv(k)=wz(k,i,j)
             END IF
           ELSE
             wadv(k)=0.0_rsh
           END IF
#if defined key_siggen || defined key_gencoord
           c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/dsigu(k))*(wadv(k)+ws(k)/f_hzu(h0(i,j),xee,k,i,j)) &
                                        *0.5_rsh*(c(iv,k+1,i,j)-c(iv,k-1,i,j))/REAL(niter,rsh)
#else
           c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/dsigu(k))*(wadv(k)+ws(k)/h3) &
                                        *0.5_rsh*(c(iv,k+1,i,j)-c(iv,k-1,i,j))/REAL(niter,rsh)
#endif

           ENDDO
         END DO   ! ends loop on k


       ELSE  ! to initialize correctly (for nbouc_macho/=0 and coherent values of tracers above small depths)
 
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             c2(iv,:,i,j)  = c(iv,:,i,j)
           ENDDO
       END IF   ! ends test for selecting right j loop

     END DO   ! ends loop on i
   END DO   ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c2)

   PRINT_DBG*, 'exit PARSUB_ADVZ_MACHO.F90 routine'

  END SUBROUTINE parsub_advz_macho
#endif
