 MODULE parsubobc3dapply

#ifdef key_substance

   !!======================================================================
   !!                   ***  MODULE  parsubobc3dapply  ***
   !! Ocean dynamics:  apply open boundary conditions of variables
   !!                  (tracers other than temperature or salinity)
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC parsub_obc3d_apply             ! routine called by stepu.F90 stepv.F90
#ifdef key_agrif
   PUBLIC parsub_obc3d_apply_agrif       ! routine called by stepu.F90 stepv.F90
#endif

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE parsub_obc3d_apply(u,v,cv_wat,dtm,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_obc3d_apply  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : cv_wat,u,obc_cv_wat_w,hx,dx_u,obc_cv_wat_e,dy_v,
   !&E                     hy,obc_cv_wat_s,v,obc_cv_wat_n
   !&E 
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-06
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04 (A. Mano) MPI2D
   !&E       !  2010-04 (T. Odaka) MPI2D cyclic
   !&E       !  2011-02 (B. Thouvenin) correction for key_subs_part_eqsubdt key
   !&E                  WARNING : cyclic obc may not work anymore with MPI
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : dt,ig,hx,h0fond,dx_u,id,jb,hy,dy_v,jh,          &
                            l_obc_cycl_x,l_obc_cycl_y
   USE comsubstance, ONLY : obc_cv_wat_w,nvp,obc_cv_wat_e,                  &
                            obc_cv_wat_s,obc_cv_wat_n,obc_cv_name,          &
                            iv_ws_max,nv_ws_max
   USE obcogcm,      ONLY : obc_coefrel
#ifdef key_biolo
   USE obccombine,    ONLY : l_obc_ogcm_rc
#if defined key_N_tracer
   USE bioloinit,     ONLY : nb_source_tracerN,p_marque_tracerN,iv_tracer_N,iv_signed_N,nb_source_river_tracerN
#endif
#if defined key_P_tracer
   USE bioloinit,     ONLY : nb_source_tracerP,p_marque_tracerP,iv_tracer_P,iv_signed_P,nb_source_river_tracerP
#endif

#endif
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh

   !! * Arguments
   INTEGER,                                                       INTENT(in) :: num_ws
   REAL(KIND=rsh),                                                INTENT(in) :: dtm
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u,v
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                                                   :: i,j,k,l,iv,ijgnul
   REAL(KIND=rsh)                                            :: uu,vv,cou
   !REAL(KIND=rlg)                                            :: tf
   REAL(KIND=rsh)                                            :: dtrelax
#if defined key_N_tracer || defined key_P_tracer
   INTEGER                                                   :: is,ivtra,iv_tra,iv_sign
#endif
 

   !!----------------------------------------------------------------------
   !! * Executable part

   !dtm=0.5_rlg*dt
   !tf=1.0_rlg/obc_coefrel
#if ! defined key_biolo
   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh
   ijgnul=0
#endif 


OMPNOAGRIF SINGLE
   ! periodic conditions along x-axis
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
#ifdef key_shiftobc
       CALL_MPI ex_peri_x_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
     END DO
#else
     DO j=ljmin,ljmax
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
   ! western boundary
          cv_wat(iv,:,ig(j),j)=cv_wat(iv,:,id(j)-1,j)
          cv_wat(iv,:,ig(j)-1,j)=cv_wat(iv,:,id(j)-2,j)
!!!!!!#if defined key_tssub_adv_fourthorder  || defined key_tssub_adv_fifthorder
#ifdef key_shiftobc
          cv_wat(iv,:,ig(j)-2,j)=cv_wat(iv,:,id(j)-3,j)
#endif
   ! eastern boundary
          cv_wat(iv,:,id(j),j)=cv_wat(iv,:,ig(j)+1,j)
#ifdef key_shiftobc
          cv_wat(iv,:,id(j)+1,j)=cv_wat(iv,:,ig(j)+2,j)
          cv_wat(iv,:,id(j)+2,j)=cv_wat(iv,:,ig(j)+3,j)
#endif

       ENDDO
     END DO
#endif
   END IF

   ! periodic conditions along y-axis
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
!!!!!#if defined key_tssub_adv_fourthorder  || defined key_tssub_adv_fifthorder
#ifdef key_shiftobc
       CALL_MPI ex_peri_y_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
     END DO
#else
     DO i=limin,limax
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
        ! southern boundary
          cv_wat(iv,:,i,jb(i))=cv_wat(iv,:,i,jh(i)-1)
          cv_wat(iv,:,i,jb(i)-1)=cv_wat(iv,:,i,jh(i)-2)
!!!#if defined key_tssub_adv_fourthorder  || defined key_tssub_adv_fifthorder
#ifdef key_shiftobc
          cv_wat(iv,:,i,jb(i)-2)=cv_wat(iv,:,i,jh(i)-3)
#endif
        ! northern boundary
          cv_wat(iv,:,i,jh(i))=cv_wat(iv,:,i,jb(i)+1)
!!!!#if defined key_tssub_adv_fourthorder  || defined key_tssub_adv_fifthorder
#ifdef key_shiftobc
          cv_wat(iv,:,i,jh(i)+1)=cv_wat(iv,:,i,jb(i)+2)
          cv_wat(iv,:,i,jh(i)+2)=cv_wat(iv,:,i,jb(i)+3)
#endif
       ENDDO
     ENDDO
#endif
   END IF

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites gauche et droite
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_x) THEN
     DO j=ljminp2,ljmaxm1

   ! western boundary
       i=ig(j)
       IF_MPI (limin==i-1) THEN
       IF (hx(i,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i,j)
           IF (uu > 0.0_rsh) THEN
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
#ifdef key_biolo
               IF (obc_cv_name(iv)=='zerograd') THEN
                   ! zero gradient
                   ijgnul=1
                   dtrelax=0.0_rsh
               ELSE
                   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
                   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh
                   ijgnul=0
               ENDIF
#endif
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i+ijgnul,j)*(1.0-dtrelax)+obc_cv_wat_w(iv,k,1,j)*dtrelax
             ENDDO
#if defined key_N_tracer
             DO is=nb_source_river_tracerN+1,nb_source_tracerN
                 DO ivtra=1,nb_var_tracerN
                    iv_tra=iv_tracer_N(ivtra,is)
                    iv_sign=iv_signed_N(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerN(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
             ENDDO
#endif
#if defined key_P_tracer
             DO is=nb_source_river_tracerP+1,nb_source_tracerP
                 DO ivtra=1,nb_var_tracerP
                    iv_tra=iv_tracer_P(ivtra,is)
                    iv_sign=iv_signed_P(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerP(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
             ENDDO
#endif

           ELSE
             cou=uu*dtm/dx_u(i,j)
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(iv,k,i+1,j)
             ENDDO
           END IF
         END DO
       END IF
       ENDIF_MPI

   ! eastern boundary
       i=id(j)
       IF_MPI (limax==i) THEN
       IF (hx(i-1,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i-1,j)
           IF (uu < 0.0_rsh) THEN
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
#ifdef key_biolo
               IF (obc_cv_name(iv)=='zerograd') THEN
                   ! zero gradient
                   ijgnul=1
                   dtrelax=0.0_rsh
               ELSE
                   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
                   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh
                   ijgnul=0
               ENDIF
#endif
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i-ijgnul,j)*(1.0-dtrelax)+obc_cv_wat_e(iv,k,imax,j)*dtrelax
             ENDDO
#if defined key_N_tracer
             DO is=nb_source_river_tracerN+1,nb_source_tracerN
                 DO ivtra=1,nb_var_tracerN
                    iv_tra=iv_tracer_N(ivtra,is)
                    iv_sign=iv_signed_N(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerN(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
              ENDDO
#endif
#if defined key_P_tracer
             DO is=nb_source_river_tracerP+1,nb_source_tracerP
                 DO ivtra=1,nb_var_tracerP
                    iv_tra=iv_tracer_P(ivtra,is)
                    iv_sign=iv_signed_P(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerP(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
              ENDDO
#endif
           ELSE
             cou=uu*dtm/dx_u(i-1,j)
            DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(iv,k,i-1,j)
             ENDDO
           END IF
         END DO
       END IF
       ENDIF_MPI

     END DO
   END IF

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites basse et haute
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_y) THEN
     DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

   ! southern boundary
       IF (j == jb(i)) THEN
         IF (hy(i,j) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j)
             IF (vv > 0.0_rsh) THEN
              DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
#ifdef key_biolo
                 IF (obc_cv_name(iv)=='zerograd') THEN
                   ! zero gradient
                   ijgnul=1
                   dtrelax=0.0_rsh
                 ELSE
                   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
                   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh
                   ijgnul=0
                 ENDIF
#endif
                 cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j+ijgnul)*(1.0-dtrelax)+obc_cv_wat_s(iv,k,i,1)*dtrelax
              ENDDO
#if defined key_N_tracer
              DO is=nb_source_river_tracerN+1,nb_source_tracerN
                 DO ivtra=1,nb_var_tracerN
                    iv_tra=iv_tracer_N(ivtra,is)
                    iv_sign=iv_signed_N(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerN(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
              ENDDO
#endif
#if defined key_P_tracer
              DO is=nb_source_river_tracerP+1,nb_source_tracerP
                 DO ivtra=1,nb_var_tracerP
                    iv_tra=iv_tracer_P(ivtra,is)
                    iv_sign=iv_signed_P(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerP(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
              ENDDO
#endif
             ELSE
               cou=vv*dtm/dy_v(i,j)
               DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
                 cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(iv,k,i,j+1)
               ENDDO
             END IF
           END DO
         END IF
       END IF

   ! northern boundary
       IF (j == jh(i)) THEN
         IF (hy(i,j-1) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j-1)
             IF (vv < 0.0_rsh) THEN
               DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
#ifdef key_biolo
                 IF (obc_cv_name(iv)=='zerograd') THEN
                   ! zero gradient
                   ijgnul=1
                   dtrelax=0.0_rsh
                 ELSE
                   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
                   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh
                   ijgnul=0
                 ENDIF
#endif
                 cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j-ijgnul)*(1.0-dtrelax)+obc_cv_wat_n(iv,k,i,jmax)*dtrelax
               ENDDO
#if defined key_N_tracer
               DO is=nb_source_river_tracerN+1,nb_source_tracerN
                 DO ivtra=1,nb_var_tracerN
                    iv_tra=iv_tracer_N(ivtra,is)
                    iv_sign=iv_signed_N(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerN(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
               ENDDO
#endif
#if defined key_P_tracer
               DO is=nb_source_river_tracerP+1,nb_source_tracerP
                 DO ivtra=1,nb_var_tracerP
                    iv_tra=iv_tracer_P(ivtra,is)
                    iv_sign=iv_signed_P(ivtra,is)
                    IF (iv_tra <= nvp) cv_wat(iv_tra,k,i,j)=p_marque_tracerP(ivtra,is)*cv_wat(iv_sign,k,i,j)
                 ENDDO
               ENDDO
#endif
             ELSE
               cou=vv*dtm/dy_v(i,j-1)
               DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
                 cv_wat(iv,k,i,j)=cv_wat(iv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(iv,k,i,j-1)
               ENDDO
             END IF
           END DO
         END IF
       END IF

     END DO
     END DO
   END IF
OMPNOAGRIF END SINGLE

!cval trop d echange si nombre d iteration varie avec les variables
OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(cv_wat)

  END SUBROUTINE parsub_obc3d_apply

   !!======================================================================

#ifdef key_agrif

  SUBROUTINE parsub_obc3d_apply_agrif(num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_obc3d_apply_agrif  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E    
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2005-10
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvarp2d
   USE toolmpi
   USE agrifcommonid
   USE comsubstance, ONLY : nvp,iv_ws_max,nv_ws_max
   USE comvars2d, ONLY : c_agrif_interp

   !! * Arguments
   INTEGER,                                                       INTENT(in) :: num_ws

   !! * Local declarations
   INTEGER  :: l,iv
   EXTERNAL :: interp3dsub

   !!----------------------------------------------------------------------
   !! * Executable part

   Agrif_UseSpecialValue = .TRUE.
   Agrif_SpecialValue = valmanq
   
   c_agrif_interp='cv_wat_parsub_bc'
   agrif_indic = num_ws
   CALL Agrif_Bc_Variable(cv_wat_id,procname=interp3dsub)

   Agrif_UseSpecialValue = .FALSE.
   
OMPMPINOAGRIF barrier
OMPMPINOAGRIF master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPINOAGRIF end master
OMPMPINOAGRIF barrier
OMPMPINOAGRIF flush(cv_wat)


  END SUBROUTINE parsub_obc3d_apply_agrif
#endif
#endif

 END MODULE parsubobc3dapply
