  MODULE wavediag

   !!======================================================================
   !!                   ***  MODULE wavediag  ***
   !!
   !! ** Description :
   !!
   !! ** Reference :
   !!
   !! ** History :  !  2011-04  (P. Lazure)  Original code
   !!
   !!======================================================================

   !! * Modules used
   USE parameters, ONLY : lchain,rsh,rlg,riolg,riosh,kmax,imin,imax,jmin,jmax, &
                          limin,limax,ljmin,ljmax

   IMPLICIT NONE

#ifdef key_MPI_2D
   INCLUDE 'mpif.h'
#endif

   PRIVATE

   !! * Accessibility

   PUBLIC wave_diag_qeuler_alloc,   &       ! called by alloc.F90
          wave_diag_qeuler_dealloc, &       ! called by dealloc.F90
          wave_diag_qeuler_init,   &       ! called by main.F90
          wave_diag_qeuler_output          ! called by dyn3dxztlssolve.F90, dyn3dyztlssolve.F90

   !! * Shared module variables

   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:),PUBLIC :: diagx_wf_1,diagx_wf_2,diagx_wf_3,diagx_wf_4,diagx_wf_5,  &
                                                         diagx_wf_6,diagx_wf_7,diagx_wf_8,diagx_wf_9,diagx_wf_10, &
                                                         diag_prod_ect,diagx_wf_11,diagx_wf_12,diagx_wf_13,       &
                                                         diag_diff_ect,diag_disp_ect

   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:),PUBLIC :: diagy_wf_1,diagy_wf_2,diagy_wf_3,diagy_wf_4,diagy_wf_5,  &
                                                         diagy_wf_6,diagy_wf_7,diagy_wf_8,diagy_wf_9,diagy_wf_10, &
                                                         diag_prod_psi,diagy_wf_11,diagy_wf_12,diagy_wf_13,       & 
                                                         diag_diff_psi,diag_disp_psi
   
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:),PUBLIC     :: ectm
   LOGICAL,ALLOCATABLE,DIMENSION(:,:),PUBLIC          :: mask_diagx,mask_diagy

   !! * Private variables

   REAL(KIND=rlg)  :: tdebdiagdyn,tfindiagdyn,dtsordiagdyn,tnextout,tcumul,tprecmoy,dtmoy
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: diagx_wf_1_save,diagx_wf_2_save,diagx_wf_3_save,diagx_wf_5_save,diagx_wf_6_save,&
                                                  diagx_wf_7_save,diagx_wf_10_save,diagx_wf_12_save,diagx_wf_13_save,diag_boz_save
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: diagy_wf_1_save,diagy_wf_2_save,diagy_wf_6_save,diagy_wf_4_save,diagy_wf_7_save,&
                                                  diagy_wf_10_save,diagy_wf_12_save,diagy_wf_13_save,diagy_dudt_save,diagy_vel_save
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:) :: diag_xe_save

   LOGICAL,ALLOCATABLE,DIMENSION(:,:)  :: mask_h0_diag,mask_hx_diag,mask_hy_diag

   LOGICAL     :: l_moy_save
   INTEGER     :: nbmoy
   CHARACTER(LEN=lchain) :: filediag
   INTEGER      :: iextmindiag,iextmaxdiag,stepidiag,jextmindiag,jextmaxdiag,stepjdiag,kextmindiag,kextmaxdiag,stepkdiag
   INTEGER      :: igextmindiag,igextmaxdiag,jgextmindiag,jgextmaxdiag
   LOGICAL      :: l_turb,l_botstress,l_xe,l_late

   CHARACTER(len=lchain) :: name_outdiag_h0,name_outdiag_xe
   CHARACTER(len=lchain) :: name_outdiag_hx,name_outdiag_hy

   CHARACTER(len=lchain) :: name_outdiagx_wf_1,name_outdiagx_wf_2,name_outdiagx_wf_3,name_outdiagx_wf_4, &
                            name_outdiagx_wf_5,name_outdiagx_wf_6,name_outdiagx_wf_7, &
                            name_outdiagx_wf_8,name_outdiagx_wf_9,name_outdiagx_wf_10, &
                            name_outdiagx_wf_11,name_outdiagx_wf_12,name_outdiagx_wf_13, &
                            name_outdiagy_wf_1,name_outdiagy_wf_2,name_outdiagy_wf_3,name_outdiagy_wf_4, &
                            name_outdiagy_wf_5,name_outdiagy_wf_6,name_outdiagy_wf_7, &
                            name_outdiagy_wf_8,name_outdiagy_wf_9,name_outdiagy_wf_10, &
                            name_outdiagy_wf_11,name_outdiagy_wf_12,name_outdiagy_wf_13


  CONTAINS

   !!======================================================================

  SUBROUTINE wave_diag_qeuler_alloc

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE wave_diag_qeuler_alloc  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-09 (S. Michel, HOCER) correct i dimension of the arrays
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ALLOCATE(diagx_wf_1(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_2(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_3(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_4(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_5(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_6(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_7(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_8(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_9(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_10(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_11(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_12(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_13(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diag_prod_ect(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_prod_psi(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_diff_ect(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_diff_psi(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_disp_ect(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diag_disp_psi(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagy_wf_1(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_2(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_3(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_4(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_5(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_6(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_7(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_8(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_9(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_10(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_11(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_12(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_13(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagx_wf_1_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_2_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_3_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_5_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_6_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_7_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_10_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_12_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagx_wf_13_save(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diagy_wf_1_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_2_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_4_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_6_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_7_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_10_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_12_save(1:kmax,limin:limax,ljmin:ljmax) )
   ALLOCATE(diagy_wf_13_save(1:kmax,limin:limax,ljmin:ljmax) )
!   ALLOCATE(diagy_dudt_save(1:kmax,limin:limax,ljmin:ljmax) )
!   ALLOCATE(diagy_vel_save(1:kmax,limin:limax,ljmin:ljmax) )

!   ALLOCATE(diag_boz(1:kmax,limin:limax,ljmin:ljmax) )
!   ALLOCATE(diag_boz_save(1:kmax,limin:limax,ljmin:ljmax) )

   ALLOCATE(diag_xe_save(limin:limax,ljmin:ljmax) )

   ALLOCATE(mask_h0_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_hx_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_hy_diag(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_diagx(limin:limax,ljmin:ljmax) )
   ALLOCATE(mask_diagy(limin:limax,ljmin:ljmax) )

   ALLOCATE(ectm(1:kmax) )

   diagx_wf_1(:,:,:)=0.0_rlg
   diagx_wf_2(:,:,:)=0.0_rlg
   diagx_wf_3(:,:,:)=0.0_rlg
   diagx_wf_6(:,:,:)=0.0_rlg
   diagx_wf_7(:,:,:)=0.0_rlg
   diagx_wf_10(:,:,:)=0.0_rlg
   diagx_wf_5(:,:,:)=0.0_rlg
   diagx_wf_12(:,:,:)=0.0_rlg
   diagx_wf_13(:,:,:)=0.0_rlg

   diagy_wf_1(:,:,:)=0.0_rlg
   diagy_wf_2(:,:,:)=0.0_rlg
   diagy_wf_6(:,:,:)=0.0_rlg
   diagy_wf_4(:,:,:)=0.0_rlg
   diagy_wf_10(:,:,:)=0.0_rlg
   diagy_wf_12(:,:,:)=0.0_rlg
   diagy_wf_13(:,:,:)=0.0_rlg
   diagy_wf_7(:,:,:)=0.0_rlg
!   diagy_dudt(:,:,:)=0.0_rlg
!   diagy_vel(:,:,:)=0.0_rlg

  END SUBROUTINE wave_diag_qeuler_alloc

   !!======================================================================

  SUBROUTINE wave_diag_qeuler_dealloc

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE wave_diag_qeuler_dealloc  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   !! * Arguments

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   DEALLOCATE(diagx_wf_1,diagx_wf_2,diagx_wf_3,diagx_wf_4,diagx_wf_5, &
              diagx_wf_6,diagx_wf_7,diagx_wf_8,diagx_wf_9,diagx_wf_10, &
              diagx_wf_11,diagx_wf_12,diagx_wf_13)

   DEALLOCATE(diagy_wf_1,diagy_wf_2,diagy_wf_3,diagy_wf_4,diagy_wf_5, &
              diagy_wf_6,diagy_wf_7,diagy_wf_8,diagy_wf_9,diagy_wf_10, &
              diagy_wf_11,diagy_wf_12,diagy_wf_13)

   DEALLOCATE(diag_prod_ect,diag_prod_psi,diag_diff_ect,diag_diff_psi, &
              diag_disp_ect,diag_disp_psi)                                  

   DEALLOCATE(diagx_wf_1_save,diagx_wf_2_save,diagx_wf_3_save,diagx_wf_5_save,diagx_wf_6_save, &
              diagx_wf_7_save,diagx_wf_10_save,diagx_wf_12_save,diagx_wf_13_save)

   DEALLOCATE(diagy_wf_1_save,diagy_wf_2_save,diagy_wf_6_save,diagy_wf_4_save,diagy_wf_7_save,diagy_wf_10_save, &
              diagy_wf_12_save,diagy_wf_13_save)
!,diagy_dudt_save,diagy_vel_save)

!   DEALLOCATE(diag_xe,diag_boz,diag_xe_save,diag_boz_save)
   DEALLOCATE(diag_xe_save)

   DEALLOCATE(mask_h0_diag,mask_hx_diag,mask_hy_diag,mask_diagx,mask_diagy)

   DEALLOCATE(ectm)

  END SUBROUTINE wave_diag_qeuler_dealloc

   !!======================================================================

  SUBROUTINE wave_diag_qeuler_init

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE wave_diag_qeuler_init  ***
   !&E
   !&E ** Purpose : Initialize diagnostics of dynamical terms
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :  !  2011-04  (P. Lazure)  Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE comvars2d,   ONLY : lon2d_g,lat2d_g,lon2d,lat2d,ierrorlog
   USE parameters,  ONLY : rsh,rlg,riosh,kmax,imin,imax,limin,limax,ljmin,ljmax,  &
                           rg_valmanq_io,ljminp1,riolg,valmanq,lchain
   USE ionc4,       ONLY : ionc4_createfile,ionc4_gatt,ionc4_createvar,           &
                           ionc4_write,ionc4_write_sig,ionc4_write_sigw,          &
                           ionc4_write_xy,ionc4_write_time,ionc4_vatt_chain
   USE output,      ONLY : l_out_nc4par,l_out_pack,l_out_zaxis_w
   USE comvars2d,   ONLY : hx,hy,h0,h0fond,riog_valid_min_h0,riog_valid_max_h0,   &
                           name_out_xe,name_out_h0
   USE comvars2d,   ONLY : ig,id,dx_u,dy_v,jb,jh,htx,hty,tdeb3d
   USE comvars3d,   ONLY : sig
   USE_MPI toolmpi, ONLY : ADD_TAG,IPROC,NPROCS,ADD_ALL_MPI_INT,MPI_COMM_MARS

#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : b_sig,theta_sig,hc_sig,Csu_sig,Csw_sig
#endif

   !! * Local declarations
   CHARACTER(LEN=lchain) :: chain
   REAL(kind=riosh),DIMENSION(limin:limax,ljmin:ljmax) :: tecrit
   REAL(KIND=riosh),DIMENSION(0:kmax+1)            :: sigecrit
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)      :: h0_local,hx_local,hy_local
   REAL(KIND=riosh),DIMENSION(kmax,limin:limax,ljmin:ljmax) :: tzecrit
   REAL(KIND=riolg),DIMENSION(imin:imax)                    :: lon
   REAL(KIND=riosh)                            :: riog_valid_min_diag,riog_valid_max_diag
   REAL(kind=rlg)        :: tool_datosec
   INTEGER :: i,j,IERR_MPI
   INTEGER :: grp_world,grp_zoom,comm_zoom,ncpu,ip
   INTEGER, DIMENSION(:), ALLOCATABLE  :: numrank,listrank
   REAL(KIND=riosh)                                              :: vecrit

   !!----------------------------------------------------------------------
   !! * Executable part

   filediag=TRIM('diag_wave_qeuler.nc')

   tdebdiagdyn=tool_datosec('01/01/1900 00:00:00')
   tdebdiagdyn=MAX(tdebdiagdyn, tdeb3d)
   tfindiagdyn=tool_datosec('01/01/2870 00:00:00')
   dtsordiagdyn=3.*3600.0_rlg
   dtsordiagdyn=30.0_rlg
!   dtsordiagdyn=0.00001_rlg
   l_moy_save=.TRUE.
   l_moy_save=.FALSE.

   l_late=.TRUE.
   l_xe=.TRUE.
   l_turb=.TRUE.
   l_botstress=.TRUE.

   igextmindiag=imin
   igextmaxdiag=imax
   iextmindiag=imin
   iextmaxdiag=imax
   stepidiag=1
   jextmindiag=jmin
   jextmaxdiag=jmax
   jgextmindiag=jmin
   jgextmaxdiag=jmax
   stepjdiag=1
#if defined key_MPI_2D
   ! If we do not use the parallel writing, the spatial extension
   ! must be bounded by limin and limax
   IF (.NOT.l_out_nc4par) THEN
     DO WHILE ( iextmindiag < limin )
       iextmindiag=iextmindiag+stepidiag
     END DO
     iextmaxdiag=MIN(limax,iextmaxdiag)
     DO WHILE ( jextmindiag < ljmin )
       jextmindiag=jextmindiag+stepjdiag
     END DO
     jextmaxdiag=MIN(ljmax,jextmaxdiag)
   END IF
#endif
   kextmindiag=1  ! should not be changed
   kextmaxdiag=kmax ! id
   stepkdiag=1 ! id

   riog_valid_min_diag=-10000.0_riosh
   riog_valid_max_diag=10000.0_riosh

   name_outdiag_h0='h0'
   name_outdiag_hx='hx'
   name_outdiag_hy='hy'
   name_outdiag_xe='xe'

   name_outdiagx_wf_1='F1X'
   name_outdiagx_wf_2='F2X'
   name_outdiagx_wf_3='F3X'
!   name_outdiagx_wf_4='F4X'
   name_outdiagx_wf_5='F5X'
   name_outdiagx_wf_6='F6X'
   name_outdiagx_wf_7='F7X'
!   name_outdiagx_wf_8='F8X'
!   name_outdiagx_wf_9='F9X'
   name_outdiagx_wf_10='F10X'
!   name_outdiagx_w_f11='F11X'
   name_outdiagx_wf_12='F12X'
   name_outdiagx_wf_13='F13X'
   name_outdiagy_wf_1='F1Y'
   name_outdiagy_wf_2='F2Y'
!   name_outdiagy_wf_3='F3Y'
   name_outdiagy_wf_4='F4Y'
!   name_outdiagy_wf_5='F5Y'
   name_outdiagy_wf_6='F6Y'
   name_outdiagy_wf_7='F7Y'
!   name_outdiagy_wf_8='F8Y'
!   name_outdiagy_wf_9='F9Y'
   name_outdiagy_wf_10='F10Y'
!   name_outdiagy_wf_11='F11Y'
   name_outdiagy_wf_12='F12Y'
   name_outdiagy_wf_13='F13Y'


   tnextout=tdebdiagdyn+dtsordiagdyn


! Verifications
! -------------

   IF ( jextmindiag > jextmaxdiag .OR. iextmindiag > iextmaxdiag ) THEN
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) ' Routine WAVE_DIAG_INIT in WAVEDIAG.F90'
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) 'Check your domain : because'
     WRITE(ierrorlog,*) ' jextmindiag > jextmaxdiag '
     WRITE(ierrorlog,*) '           or'
     WRITE(ierrorlog,*) ' iextmindiag > iextmaxdiag '
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) ' The simulation is stopped '
     WRITE(ierrorlog,*) ' '
     CALL_MPI MPI_FINALIZE(IERR_MPI)
     STOP
   END IF


! Masks
! -----

   mask_diagx(:,:)=.FALSE.
   DO j=jextmindiag,jextmaxdiag,stepjdiag
     DO i=iextmindiag,iextmaxdiag,stepidiag
       IF ( i >= limin .AND. i <= limax .AND. j >= ljmin .AND. j <= ljmax) THEN
         mask_diagx(i,j)=.TRUE.
       END IF
     END DO
   END DO

   mask_diagy(:,:)=.FALSE.
   DO j=jextmindiag,jextmaxdiag,stepjdiag
     DO i=iextmindiag,iextmaxdiag,stepidiag
       IF ( i >= limin .AND. i <= limax .AND. j >= ljmin .AND. j <= ljmax) THEN
         mask_diagy(i,j)=.TRUE.
       END IF
     END DO
   END DO



   h0_local(limin:limax,ljmin:ljmax)=h0(limin:limax,ljmin:ljmax)
   hx_local(limin:limax,ljmin:ljmax)=hx(limin:limax,ljmin:ljmax)
   hy_local(limin:limax,ljmin:ljmax)=hy(limin:limax,ljmin:ljmax)

   mask_h0_diag(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( h0_local(i,j)> h0fond ) mask_h0_diag(i,j)=.FALSE.
     END DO
   END DO

! pascal
! mouais pas clair c t affaire
   mask_hx_diag(:,:)=.TRUE.
   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)-1),MIN(limax,id(j))
       IF ( hx_local(i,j) /= -valmanq -h0fond ) mask_hx_diag(i,j)=.FALSE.
     END DO
   END DO
   mask_hx_diag(:,:)=.FALSE.

   mask_hy_diag(:,:)=.TRUE.
   DO j=ljmin,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF ( (j >= jb(i)-1 .AND. j <= jh(i)) .AND. (hy_local(i,j) /= -valmanq -h0fond) ) mask_hy_diag(i,j)=.FALSE.
     END DO
   END DO


   comm_zoom=0
#ifdef key_MPI_2D
     comm_zoom=MPI_COMM_MARS
     ! what a code ! Not a clever code probably but not clear in my head at the moment
     ! Count the number of processors active over the area to be saved
     ncpu=0
     IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) ncpu=ncpu+1
     CALL ADD_ALL_MPI_INT(ncpu)
     ! Save the number of active processors
     ALLOCATE (numrank(ncpu))
     ALLOCATE (listrank(nprocs))
     numrank(:)=0   ! must be initialized if the proc 0 is active on the saved zoom
     listrank(:)=0
     ! which cpu is active ?
     IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) listrank(1)=iproc
     ! all cpus get the number of active cpus plus sleeping cpus (set to 0)
     CALL MPI_ALLGATHER(listrank(1),1,MPI_INTEGER,listrank,1,MPI_INTEGER,MPI_COMM_MARS,ierr_mpi)
     ! all cpus keep the active cpus only
     IF (ncpu/=nprocs) THEN
       ip=0
       DO i=1,nprocs
         IF_MPI ( listrank(i) /= 0 ) THEN
         ip=ip+1
         numrank(ip)=listrank(i)
         ENDIF
       END DO
       ! Group relative to the MPI_COMM_MARS
       CALL MPI_COMM_GROUP(MPI_COMM_MARS,grp_world,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_GROUP_INCL(grp_world,ncpu,numrank,grp_zoom,ierr_mpi)
       ! Create the group of active cpus
       CALL MPI_COMM_CREATE(MPI_COMM_MARS,grp_zoom,comm_zoom,ierr_mpi)
     ENDIF
#endif


   ! to work with acive cpus over a small area
   IF ( jextmindiag <= jextmaxdiag .AND. iextmindiag <= iextmaxdiag ) THEN


   sigecrit(:)=sig(:)
   IF ( l_out_nc4par ) THEN
     ! Create a global file because of the parallel writing
     CALL ionc4_createfile(filediag,lon2d_g,lat2d_g,igextmindiag,igextmaxdiag,stepidiag, &
                           jgextmindiag,jgextmaxdiag,stepjdiag, &
                           kextmindiag,kextmaxdiag,stepkdiag,   &
                           kmax,0,l_out_nc4par = .TRUE.,comm_active=comm_zoom, &
                           zaxis_var=sigecrit(1:kmax))
   ELSE
     ! Create a local file bounded by limin,limax,ljmin,ljmax
     CALL_MPI ADD_TAG(filediag,LEN(filediag))
     CALL ionc4_createfile(filediag,lon2d,lat2d,iextmindiag,iextmaxdiag,stepidiag, &
                           jextmindiag,jextmaxdiag,stepjdiag, &
                           kextmindiag,kextmaxdiag,stepkdiag, &
                           kmax,0,zaxis_var=sigecrit(1:kmax))
   END IF

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
   CALL ionc4_gatt(filediag,'global_imin',iextmindiag)
   CALL ionc4_gatt(filediag,'global_imax',iextmaxdiag)
   CALL ionc4_gatt(filediag,'global_jmin',jextmindiag)
   CALL ionc4_gatt(filediag,'global_jmax',jextmaxdiag)

   ! Add the vertical axis
   CALL out_zaxis_w(filediag,kextmindiag,kextmaxdiag,stepkdiag,comm_zoom)

   CALL ionc4_createvar(filediag,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                        standard_name= 'ocean_s_variable',&
#else
                        standard_name= 'ocean_sigma_variable',&
#endif
                        dims='z',valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io)
   CALL ionc4_write_sig(filediag,'SIG',sigecrit,kmax)

#if defined key_siggen || defined key_gencoord

   sigecrit(:)=Csu_sig(:)
   CALL ionc4_createvar(filediag,'Csu_sig',' ','C(s) law at mid layer',dims='z', &
                        standard_name='ocean_s_coordinate_function_at_midlayer', &
                        valid_min=-1.0_riosh,valid_max=10000.0_riosh,fill_value= rg_valmanq_io)
   CALL ionc4_write_sig(filediag,'Csu_sig',sigecrit,kmax)
   CALL ionc4_vatt_chain(filediag,'Csu_sig','formula_definition', &
          'C(k) = (1-b)*sinh(a*s(k))/sinh(a) + b*[tanh(a*(s(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

   sigecrit(:)=Csw_sig(:)
   CALL ionc4_createvar(filediag,'Csw_sig',' ','C(s) law at interface',dims='z', &
                        standard_name='ocean_s_coordinate_function_at_interface', &
                        valid_min= -1.0_riosh,valid_max= 10000.0_riosh,fill_value= rg_valmanq_io)
   !                        l_auxcoordinate=.false.,location_in_cell='w')
   CALL ionc4_write_sigw(filediag,'Csw_sig',sigecrit,kmax)
   CALL ionc4_vatt_chain(filediag,'Csw_sig','formula_definition', &
          'C_w(k) = (1-b)*sinh(a*s_w(k))/sinh(a) + b*[tanh(a*(s_w(k)+0.5))-tanh(0.5*a)]/(2*tanh(0.5*a))')

   CALL ionc4_createvar(filediag,'hc','m', 'depth over which the resolution is increased', &
                        standard_name='ocean_s_coordinate_thick_of_surface_resolution',dims='xy', &
                        valid_min= 0.0_riosh,valid_max= 1000.0_riosh,fill_value= rg_valmanq_io,&
                        l_out_nc4par=l_out_nc4par,l_pack=l_out_pack )
   tecrit(limin:limax,ljmin:ljmax)=hc_sig(limin:limax,ljmin:ljmax)
   CALL ionc4_write_xy(filediag,'hc',tecrit,limin,limax,ljmin,ljmax,fill_value=rg_valmanq_io,l_out_nc4par=l_out_nc4par)

   CALL ionc4_createvar(filediag,'b',' ','bottom control parameter',  &
                        standard_name='ocean_s_coordinate_bottom_control', &
                        valid_min=0.0_riosh,valid_max=1.0_riosh,fill_value=rg_valmanq_io)
   vecrit=b_sig
   CALL ionc4_write(filediag,'b',vecrit)

   CALL ionc4_createvar(filediag,'theta',' ','surface control parameter', &
                        standard_name ='ocean_s_coordinate_surface_control', &
                        valid_min=0.0_riosh,valid_max=20.0_riosh,fill_value=rg_valmanq_io)
   vecrit=theta_sig
   CALL ionc4_write(filediag,'theta',vecrit)

#endif

   ! Add attributes to follow OCO netCDF convention
   CALL out_netcdf_conv(filediag,1,comm_zoom,igextmindiag,igextmaxdiag,jgextmindiag,jgextmaxdiag)

   IF_MPI (l_out_nc4par .AND. comm_zoom/=MPI_COMM_MARS) CALL MPI_COMM_FREE(comm_zoom,ierr_mpi)


! Save the bathymetry
! -------------------

! h0
! --

#if ! defined key_agrif
   ! avec agrif on ne peut compiler car The type of the actual argument differs from the type of the dummy argument.   [ARRAY0]
   !  &,  Agrif_tabvars(606) % var% array0 ,  Agrif_tabvars(605) &
   ! Pas de souci si on utilise riog_valid_min_diag a la place de riog_valid_min_h0 !!!!!!!!!!
   ! Cela m exaspere, et c est un pb recurant, voir mail du 3/3/2010 a Romain Ramel (PREVI_OMPMPI)
   ! Laurent peux-tu faire qq chose STP ou donner une explication rationnelle !!!!

   chain='model_sea_floor_depth_below_geoid'
   CALL ionc4_createvar(filediag,name_outdiag_h0,units='m',&
                        long_name='bathymetry relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -rg_valmanq_io,&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(limin:limax,ljmin:ljmax)=h0_local(limin:limax,ljmin:ljmax)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_h0),tecrit,limin,limax,ljmin,ljmax,&
                       fill_value=-rg_valmanq_io,l_out_nc4par = l_out_nc4par )


! hx
! --
   chain='model_sea_floor_depth_below_geoid_at_u_location'
   CALL ionc4_createvar(filediag,name_outdiag_hx,units='m',&
                        long_name = 'bathymetry at u-location relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(:,:)=hx_local(:,:)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_hx),tecrit,limin,limax,ljmin,ljmax ,&
                       fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)

! hy
! --

   chain='model_sea_floor_depth_below_geoid_at_v_location'
   CALL ionc4_createvar(filediag,name_outdiag_hy,units='m',&
                        long_name = 'bathymetry at v-location relative to the mean level',&
                        standard_name = TRIM(chain),&
                        fill_value = -REAL(rg_valmanq_io+h0fond,riosh),&
                        valid_min = riog_valid_min_h0,&
                        valid_max = riog_valid_max_h0,&
                        dims = 'xy',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
   tecrit(:,:)=hy_local(:,:)
   CALL ionc4_write_xy(filediag,TRIM(name_outdiag_hy),tecrit,limin,limax,ljmin,ljmax,&
                       fill_value = -REAL(rg_valmanq_io+h0fond,riosh),l_out_nc4par = l_out_nc4par)

#endif

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_wf_1,units= 'm s-2',&
                          long_name = 'x_vortex_force',&
                          standard_name = 'x_vortex_force',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_wf_1,units= 'm s-2',&
                          long_name = 'y_vortex_force',&
                          standard_name = 'y_vortex_force',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagx_wf_2,units= 'm s-2',&
                          long_name = 'diff_ect',&
                          standard_name = 'diff_ect',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
   END IF

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_wf_3,units= 'm s-2',&
                          long_name = 'x_advection',&
                          standard_name = 'x_advection',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagx_wf_5,units= 'm s-2',&
                          long_name = 'x_vertical_mixing',&
                          standard_name = 'x_vertical_mixing',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_wf_7,units= 'm s-2',&
                          long_name = 'ect',&
                          standard_name = 'ect',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
#ifdef key_plustard
     CALL ionc4_createvar(filediag,name_outdiagy_wf_6,units= 'm s-2',&
                          long_name = 'non linear terms at v location',&
                          standard_name = 'non_linear_terms_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
#endif

   END IF

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagy_wf_10,units= 'm s-2',&
                          long_name = 'surface_source_term_for_ect',&
                          standard_name = 'surface_source_term_for_ect',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_wf_4,units= 'm s-2',&
                          long_name = 'y_bernouilli_head_and_breaking',&
                          standard_name = 'y_bernouilli_head_and_breaking',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagy_wf_2,units= 'm s-2',&
                          long_name = 'diff_eps',&
                          standard_name = 'diff_eps',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagx_wf_10,units= 'm s-2',&
                          long_name = 'bottom_source_term_for_ect',&
                          standard_name = 'bottom_source_term_for_ect',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
   END IF

   IF (l_turb) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_wf_12,units= 'm s-2',&
                          long_name = 'name_outdiag_f12_x',&
                          standard_name = 'name_outdiag_f12_x',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagx_wf_13,units= 'm s-2',&
                          long_name = 'disp_ect',&
                          standard_name = 'disp_ect',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

!   END IF

!   IF (l_vdif) THEN
     CALL ionc4_createvar(filediag,name_outdiagy_wf_7,units= 'm s-2',&
                          long_name = 'turbulent_dissipation',&
                          standard_name = 'turbulent_dissipation',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_wf_12,units= 'm s-2',&
                          long_name = 'prod_eps',&
                          standard_name = 'prod_eps',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_late) THEN
     CALL ionc4_createvar(filediag,name_outdiagy_wf_13,units= 'm s-2',&
                          long_name = 'disp_eps',&
                          standard_name = 'disp_eps',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

#ifdef key_plustard
     CALL ionc4_createvar(filediag,name_outdiagy_dudt,units= 'm s-2',&
                          long_name = 'time trend at v location',&
                          standard_name = 'time_trend_v_location',&
                          fill_value = rg_valmanq_io,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)
#endif

   END IF

   IF (l_botstress) THEN
     CALL ionc4_createvar(filediag,name_outdiagx_wf_6,units= 'm s-1',&
                          long_name = 'x_bottom_friction',&
                          standard_name = 'x_bottom_friction',&
                          fill_value = 0.0_riosh,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

     CALL ionc4_createvar(filediag,name_outdiagy_wf_6,units= 'm s-1',&
                          long_name = 'y_bottom_friction',&
                          standard_name = 'y_bottom_friction',&
                          fill_value = 0.0_riosh,&
                          valid_min = riog_valid_min_diag,&
                          valid_max = riog_valid_max_diag,&
                          dims= 'zxyt',&
                          l_pack = l_out_pack,&
                          l_out_nc4par = l_out_nc4par)

   END IF

   IF (l_xe) &
   CALL ionc4_createvar(filediag,name_outdiag_xe,units= 'm',&
                        long_name = 'sea surface height',&
                        standard_name = 'sea surface height',&
                        fill_value = rg_valmanq_io,&
                        valid_min = riog_valid_min_diag,&
                        valid_max = riog_valid_max_diag,&
                        dims= 'xyt',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)

#ifdef key_plustard
!   IF (l_boz) &
   CALL ionc4_createvar(filediag,name_outdiag_boz,units= 'm s-2',&
                        long_name = 'Buoyancy of sea water at atmospheric pressure',&
                        standard_name = 'sea_water_potential_buoyancy',&
                        fill_value = rg_valmanq_io,&
                        valid_min = riog_valid_min_diag,&
                        valid_max = riog_valid_max_diag,&
                        dims= 'zxyt',&
                        l_pack = l_out_pack,&
                        l_out_nc4par = l_out_nc4par)
#endif

   END IF  ! ends test on iextmin...

  END SUBROUTINE wave_diag_qeuler_init

   !!======================================================================

  SUBROUTINE wave_diag_qeuler_output

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE outputwave_diag_qeuler  ***
   !&E
   !&E ** Purpose : Save diagnostics of dynamical terms
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :  !  2011-04  (P. Lazure)  Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters,  ONLY : rsh,rlg,riosh,kmax,imin,imax,limin,limax,ljmin,ljmax,  &
                           rg_valmanq_io,ljminp1,riolg,valmanq,lchain
   USE outtools,    ONLY : out_write_xytvar,out_write_zxytvar
   USE ionc4,       ONLY : ionc4_write_time,ionc4_sync
   USE output,      ONLY : l_out_nc4par
   USE comvars2d,   ONLY : hx,hy,h0,h0fond,riog_valid_min_h0,riog_valid_max_h0
   USE comvars2d,   ONLY : ig,id,dx_u,dy_v,jb,jh,htx,hty
   USE comvars2d,   ONLY : t,h0fond
   USE comvars3d,   ONLY : sig
   USE comvarp2d,   ONLY : u,v,ssh
   USE_MPI toolmpi, ONLY : MASTER

   !! * Local declarations
   REAL(kind=riosh),DIMENSION(imin:imax,jmin:jmax) :: tecrit
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)      :: h0_local,hx_local,hy_local
   REAL(KIND=riosh),DIMENSION(kmax,limin:limax,ljmin:ljmax) :: tzecrit
   REAL(KIND=riolg),DIMENSION(imin:imax)                    :: lon
   REAL(KIND=riolg)                            :: riog_valid_min_diag,riog_valid_max_diag
   INTEGER :: i,j
   CHARACTER (len=19) :: tool_sectodat

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter WAVE_DIAG_QEULER_OUTPUT routine'

   IF ( t < tdebdiagdyn .OR. t > tfindiagdyn )  RETURN

!print*, ' prochaine ecriture:',tool_sectodat(tnextout)

   IF (l_moy_save) THEN
     nbmoy=nbmoy+1

     IF (nbmoy ==1) THEN
       tprecmoy=t
       tcumul=0
     ELSE
       dtmoy=t-tprecmoy
       tcumul=tcumul+dtmoy

       diagx_wf_1_save(:,:,:)=diagx_wf_1_save(:,:,:)+diagx_wf_1(:,:,:)*dtmoy
       diagx_wf_2_save(:,:,:)=diagx_wf_2_save(:,:,:)+diagx_wf_2(:,:,:)*dtmoy
       diagx_wf_3_save(:,:,:)=diagx_wf_3_save(:,:,:)+diagx_wf_3(:,:,:)*dtmoy
       diagx_wf_5_save(:,:,:)=diagx_wf_5_save(:,:,:)+diagx_wf_5(:,:,:)*dtmoy
       diagx_wf_6_save(:,:,:)=diagx_wf_6_save(:,:,:)+diagx_wf_6(:,:,:)*dtmoy
       diagx_wf_7_save(:,:,:)=diagx_wf_7_save(:,:,:)+diagx_wf_7(:,:,:)*dtmoy
       diagx_wf_10_save(:,:,:)=diagx_wf_10_save(:,:,:)+diagx_wf_10(:,:,:)*dtmoy
       diagx_wf_12_save(:,:,:)=diagx_wf_12_save(:,:,:)+diagx_wf_12(:,:,:)*dtmoy
       diagx_wf_13_save(:,:,:)=diagx_wf_13_save(:,:,:)+diagx_wf_13(:,:,:)*dtmoy


       diagy_wf_1_save(:,:,:)=diagy_wf_1_save(:,:,:)+diagy_wf_1(:,:,:)*dtmoy
       diagy_wf_2_save(:,:,:)=diagy_wf_2_save(:,:,:)+diagy_wf_2(:,:,:)*dtmoy
       diagy_wf_6_save(:,:,:)=diagy_wf_6_save(:,:,:)+diagy_wf_6(:,:,:)*dtmoy
       diagy_wf_4_save(:,:,:)=diagy_wf_4_save(:,:,:)+diagy_wf_4(:,:,:)*dtmoy
       diagy_wf_7_save(:,:,:)=diagy_wf_7_save(:,:,:)+diagy_wf_7(:,:,:)*dtmoy
       diagy_wf_10_save(:,:,:)=diagy_wf_10_save(:,:,:)+diagy_wf_10(:,:,:)*dtmoy
       diagy_wf_12_save(:,:,:)=diagy_wf_12_save(:,:,:)+diagy_wf_12(:,:,:)*dtmoy
       diagy_wf_13_save(:,:,:)=diagy_wf_13_save(:,:,:)+diagy_wf_13(:,:,:)*dtmoy
!       diagy_dudt_save(:,:,:)=diagy_dudt_save(:,:,:)+diagy_dudt*dtmoy
!       diagy_vel_save(:,:,:)=diagy_vel_save(:,:,:)+diagy_vel*dtmoy

!       diag_boz_save(:,:,:)=diag_boz_save(:,:,:)+diag_boz*dtmoy
       diag_xe_save(:,:)=diag_xe_save(:,:)+ssh(limin:limax,ljmin:ljmax)*dtmoy
     END IF

   ELSE

     nbmoy=1
     tcumul=1._rlg
     diagx_wf_1_save=diagx_wf_1
     diagx_wf_2_save=diagx_wf_2
     diagx_wf_3_save=diagx_wf_3
     diagx_wf_5_save=diagx_wf_5
     diagx_wf_6_save=diagx_wf_6
     diagx_wf_7_save=diagx_wf_7
     diagx_wf_10_save=diagx_wf_10
     diagx_wf_12_save=diagx_wf_12
     diagx_wf_13_save=diagx_wf_13

     diagy_wf_1_save=diagy_wf_1
     diagy_wf_2_save=diagy_wf_2
     diagy_wf_6_save=diagy_wf_6
     diagy_wf_4_save=diagy_wf_4
     diagy_wf_7_save=diagy_wf_7
     diagy_wf_10_save=diagy_wf_10
     diagy_wf_12_save=diagy_wf_12
     diagy_wf_13_save=diagy_wf_13
!     diagy_dudt_save=diagy_dudt
!     diagy_vel_save=diagy_vel

!     diag_boz_save=diag_boz
     diag_xe_save(:,:)=ssh(limin:limax,ljmin:ljmax)
   END IF


   IF ( t >= tnextout) THEN

     IF_MPI (MASTER) THEN
     print*,' WAVE_DIAG_QEULER_OUTPUT : on va ecrire dans fichier :',TRIM(filediag)
     print*,' date:',tool_sectodat(t)
     print*, 'date sortie prevue',tool_sectodat(tnextout)
     print*, 'longueur moyenne (en h)',tcumul/3600.
     ENDIF_MPI

     tnextout=tnextout+dtsordiagdyn

     diagx_wf_1_save=diagx_wf_1_save/tcumul
     diagx_wf_2_save=diagx_wf_2_save/tcumul
     diagx_wf_3_save=diagx_wf_3_save/tcumul
     diagx_wf_5_save=diagx_wf_5_save/tcumul
     diagx_wf_6_save=diagx_wf_6_save/tcumul
     diagx_wf_7_save=diagx_wf_7_save/tcumul
     diagx_wf_10_save=diagx_wf_10_save/tcumul
     diagx_wf_12_save=diagx_wf_12_save/tcumul
     diagx_wf_13_save=diagx_wf_13_save/tcumul

     diagy_wf_1_save=diagy_wf_1_save/tcumul
     diagy_wf_2_save=diagy_wf_2_save/tcumul
     diagy_wf_6_save=diagy_wf_6_save/tcumul
     diagy_wf_4_save=diagy_wf_4_save/tcumul
     diagy_wf_7_save=diagy_wf_7_save/tcumul
     diagy_wf_10_save=diagy_wf_10_save/tcumul
     diagy_wf_12_save=diagy_wf_12_save/tcumul
     diagy_wf_13_save=diagy_wf_13_save/tcumul
!     diagy_dudt_save=diagy_dudt_save/tcumul
!     diagy_vel_save=diagy_vel_save/tcumul

!     diag_boz_save=diag_boz_save/tcumul
     diag_xe_save=diag_xe_save/tcumul

     CALL ionc4_write_time(filediag,0,t)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_1,rg_valmanq_io,mask_hx_diag, &
                                       diagx_wf_1_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagy_wf_1,rg_valmanq_io,mask_hx_diag, &
                                       diagy_wf_1_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagy_wf_2,rg_valmanq_io,mask_hx_diag, &
                                       diagy_wf_2_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_3,rg_valmanq_io,mask_hx_diag, &
                                       diagx_wf_3_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagy_wf_4,rg_valmanq_io,mask_hx_diag, &
                                       diagy_wf_4_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_2,rg_valmanq_io,mask_hx_diag, &
                                       diagx_wf_2_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_botstress) CALL out_write_zxytvar(filediag,name_outdiagy_wf_6,rg_valmanq_io,mask_hx_diag, &
                                       diagy_wf_6_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_5,rg_valmanq_io,mask_hx_diag, &
                                        diagx_wf_5_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_turb) CALL out_write_zxytvar(filediag,name_outdiagy_wf_7,rg_valmanq_io,mask_hx_diag, &
                                        diagy_wf_7_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_botstress) CALL out_write_zxytvar(filediag,name_outdiagx_wf_6,rg_valmanq_io,mask_hx_diag, &
                                       diagx_wf_6_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagy_wf_10,rg_valmanq_io,mask_hx_diag, &
                                       diagy_wf_10_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_7,rg_valmanq_io,mask_hx_diag, &
                                          diagx_wf_7_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_turb) CALL out_write_zxytvar(filediag,name_outdiagy_wf_12,rg_valmanq_io,mask_hx_diag, &
                                          diagy_wf_12_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagx_wf_10,rg_valmanq_io,mask_hx_diag, &
                                        diagx_wf_10_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
     IF (l_late) CALL out_write_zxytvar(filediag,name_outdiagy_wf_13,rg_valmanq_io,mask_hx_diag, &
                                        diagy_wf_13_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_turb) CALL out_write_zxytvar(filediag,name_outdiagx_wf_12,rg_valmanq_io,mask_hx_diag, &
                                        diagx_wf_12_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
!     IF (l_dudt) CALL out_write_zxytvar(filediag,name_outdiagy_dudt,rg_valmanq_io,mask_hx_diag, &
!                                        diagy_dudt_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_turb) CALL out_write_zxytvar(filediag,name_outdiagx_wf_13,0.0_riosh,mask_hx_diag, &
                                       diagx_wf_13_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)
!     IF (l_vel) CALL out_write_zxytvar(filediag,name_outdiagy_vel,0.0_riosh,mask_hx_diag, &
!                                       diagy_vel_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     IF (l_xe) CALL out_write_xytvar(filediag,name_outdiag_xe,rg_valmanq_io,mask_hx_diag, &
                                     diag_xe_save(limin:limax,ljmin:ljmax),0,l_out_nc4par = l_out_nc4par)

!     IF (l_boz) CALL out_write_zxytvar(filediag,name_outdiag_boz,rg_valmanq_io,mask_hx_diag, &
!                                       diag_boz_save(:,limin:limax,ljmin:ljmax),0,1,l_out_nc4par = l_out_nc4par)

     ! To write the data on the disk and not loose data in case of run crash
     CALL ionc4_sync(filediag)

     !print*, ' avant reinitialisation'

     nbmoy=0
     diagx_wf_1_save=0.0_rsh
     diagx_wf_2_save=0.0_rsh
     diagx_wf_3_save=0.0_rsh
     diagx_wf_5_save=0.0_rsh
     diagx_wf_6_save=0.0_rsh
     diagx_wf_7_save=0.0_rsh
     diagx_wf_10_save=0.0_rsh
     diagx_wf_12_save=0.0_rsh
     diagx_wf_13_save=0.0_rsh

     diagy_wf_1_save=0.0_rsh
     diagy_wf_2_save=0.0_rsh
     diagy_wf_4_save=0.0_rsh
     diagy_wf_6_save=0.0_rsh
     diagy_wf_7_save=0.0_rsh
     diagy_wf_10_save=0.0_rsh
     diagy_wf_12_save=0.0_rsh
     diagy_wf_13_save=0.0_rsh
!     diagy_dudt_save=0.0_rsh
!     diagy_vel_save=0.0_rsh

!     diag_boz_save=0.0_rsh
     diag_xe_save=0.0_rsh

   END IF

   PRINT_DBG*, 'exit WAVE_DIAG_QEULER_OUTPUT routine'

  END SUBROUTINE wave_diag_qeuler_output

   !!======================================================================

  END MODULE wavediag
