 MODULE toolmpi

   !!======================================================================
   !!                   ***  MODULE toolmpi  ***
   !! 
   !! NPROCS= total no of procs using  =>MYPROCS
   !! NPROCJ= number of division on J direction =>MYPROCJ
   !! NPROCI= number of division on I direction =>MYPROCI
   !! IGROUNDPOINT= BLANK point, i.e. no mpi process is used for that local area
   !! NBBLANK=COUNT(PROC==IGROUDPOINT). i.e. total number of non-wet points
   !! IWETPOINT= wet point i.e. COUNT(PROC==IWETPOINT)==NPROCS
   !! NPROCI * NPROCJ=  NPROCS+NBBLANK
   !!
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE

#if defined key_MPI || defined key_MPI_2D
   PRIVATE

   !! * Accessibility
   PUBLIC :: MPI_START1 ,MPI_END1,MPI_START2,MPI_START3
   PUBLIC :: TAG_PROC,ADD_TAG
   PUBLIC :: MPI_COMM_MARS
   PUBLIC :: MASTER,IPROC,NPROCS
   PUBLIC :: give_ccount,get_ccount,MINcBUDY,MAXcBUDY,cBUDY,MINCPROC,MAXCPROC
   PUBLIC :: give_rcount,get_rcount,MINrBUDY,MAXrBUDY,rBUDY,MINRPROC,MAXRPROC
   PUBLIC :: myiproc ,MYRPROC
   PUBLIC :: myjproc ,MYCPROC
   ! public:: myi_uproc,myi_dproc,myj_uproc,myj_dproc
   PUBLIC :: LI,LJ,CI,RJ,proc
   PUBLIC :: ciminp2,cimaxm1
   PUBLIC :: rjminp2,rjmaxm1
   PUBLIC :: exchange_i_rlg   !by akiko 
#if defined key_MPI_2D
   PUBLIC :: exchange_i_int   !by akiko 
#endif

   PUBLIC :: exchange_rlg,exchange_rsh ,exchange_i_rsh
   PUBLIC :: exchange_int

   PUBLIC :: ex_j_rsh,ex_j_rlg,ex_j_int
   PUBLIC :: ex_i_rsh,ex_i_rlg,ex_i_int
   PUBLIC :: ex_peri_x_rsh,ex_peri_y_rsh
   PUBLIC :: ex_peri_x_avcv,ex_peri_y_avcv
   PUBLIC :: exchange_vectcpu_int
#if defined key_trajec2d || defined key_trajec3d || defined key_ibm
   PUBLIC :: ex_traj
#endif


   PUBLIC :: correct_rsh
   PUBLIC :: sendrecv_rlg,sendrecv_rsh
#if defined key_MPI_2D
   PUBLIC :: sendrecv_int !akiko
#endif
   PUBLIC :: MPI_EXJ_SEND,MPI_EXJ_RECV,MPI_EXI_SEND,MPI_EXI_RECV
   PUBLIC :: BCAST_MPI,FINDMAX_MPI,FINDMAX_MPI_1,FINDMAX_MPI_3
   PUBLIC :: FINDMIN_MPI_1,ADD_ALL_MPI_RSH,ADD_ALL_MPI_RLG,   &
             MAX_ALL_MPI_RSH,MIN_ALL_MPI_RSH,ADD_ALL_MPI_INT, &
             MAX_ALL_MPI_INT,MIN_ALL_MPI_INT
   PUBLIC :: ADD_MPI_RSH,ADD_MPI_RLG
   PUBLIC :: CREATE_COMM_ZONAL,ADD_ZONAL_RSH
   !PUBLIC :: ADD_MPI_INT

   !! * Interface
   INTERFACE BCAST_MPI
     MODULE PROCEDURE BCAST_MPI_LOGIC
     MODULE PROCEDURE BCAST_MPI_INT
     MODULE PROCEDURE BCAST_MPI_RSH
     MODULE PROCEDURE BCAST_MPI_RLG
     MODULE PROCEDURE BCAST_MPI_CHAR
   END INTERFACE

   INCLUDE 'mpif.h'
#ifdef key_palm_ww3
   INCLUDE 'palmlib.h'
#endif   

   !! * Private variables
   INTEGER                            :: MPI_COMM_MARS
   INTEGER                            :: NPROCS,IERR_MPI,IPROC,MAXP,MINP
   INTEGER                            :: MINIPROC,MAXIPROC,MINJPROC,maxjproc,NBBLANK
   INTEGER                            :: MYIPROC,MYJPROC,MYCPROC,MYRPROC
   INTEGER                            :: MINCPROC,MAXCPROC,MINRPROC,MAXRPROC
   INTEGER                            :: MINcBUDY,MAXcBUDY,MINRBUDY,MAXRBUDY
   INTEGER                            :: MINiBUDY,MAXiBUDY,MINjBUDY,MAXjBUDY
   INTEGER                            :: NUMRBUDY,NUMCBUDY
   INTEGER                            :: NUMiBUDY,NUMjBUDY
   INTEGER                            :: ciminp2,cimaxm1 !,cimin,cimax
   INTEGER                            :: rjminp2,rjmaxm1
   INTEGER,PARAMETER                  :: IGROUNDPOINT=-100
   INTEGER,PARAMETER                  :: IWETPOINT=-2
   INTEGER                            :: myi_uproc,myi_dproc,myj_uproc,myj_dproc
   INTEGER                            :: impierror
   INTEGER                            :: njbrecv, njhrecv,njbsend,njhsend
   LOGICAL                            :: MASTER=.FALSE.,LPARAMREAD=.FALSE.
   INTEGER,ALLOCATABLE,DIMENSION(:,:) :: LJ,CI,LI,RJ
   INTEGER,ALLOCATABLE,DIMENSION(:)   :: cbudy,give_ccount,get_ccount
   INTEGER,ALLOCATABLE,DIMENSION(:)   :: rbudy,give_rcount,get_rcount
   INTEGER,ALLOCATABLE,DIMENSION(:)   :: ibudy,jbudy
   INTEGER,ALLOCATABLE,DIMENSION(:)   :: proc_i,proc_j,proc_size,proc_displs
   INTEGER,ALLOCATABLE,DIMENSION(:)   :: proc_limin,proc_limax,proc_ljmin,proc_ljmax
   INTEGER,ALLOCATABLE,DIMENSION(:,:) :: PROC

   CONTAINS

   !!======================================================================

  SUBROUTINE MPI_START1

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE  MPI_START1  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E       !  2013-07 (A. Thevenin - CERFACS) coupling using OASIS3-MCT
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : imax,jmax,l_testcase,num_testcase

#ifdef key_oasis
   USE tooloasismct, ONLY : cpl_oasis_init
#endif

   IMPLICIT NONE
   logical ios,ioshyb
   character(len=100) :: filempitxt

   !!----------------------------------------------------------------------
   !! * Executable part

   !Here are the list of trapp
#endif
   !#if defined key_substance && defined key_MPI
   !print*, "key_substance can not be used with key_MPI on this version of MARS"
   ! print*, "The computation stops here"
   ! stop 
   !#endif
#if defined key_champroche && ( defined key_MPI || defined key_MPI_2D )
   WRITE(impierror,*) "key_champroche can not be used with key_MPI on this version of MARS"
   WRITE(impierror,*) "The computation stops here"
   CALL MPI_FINALIZE(IERR_MPI) ; STOP 
#endif
#if defined (key_MPI_strict) && ( ! defined key_MPI && ! defined key_MPI_2D )
   WRITE(impierror,*) "key_MPI_strict can not be used without key_MPI"
   WRITE(impierror,*) "The computation stops here"
   CALL MPI_FINALIZE(IERR_MPI) ; STOP 
#endif
#if defined key_MPI || defined key_MPI_2D

#if defined key_agrif
   IF (agrif_root()) THEN
#endif  
     CALL MPI_INIT(IERR_MPI)
#ifdef key_oasis
     CALL cpl_oasis_init(MPI_COMM_MARS)
#elif key_palm_ww3
!cvalacben utilise nulle part     MPI_COMM=PL_COMM_EXEC
     MPI_COMM_MARS=PL_COMM_EXEC
#else
     MPI_COMM_MARS = MPI_COMM_WORLD
#endif
#if defined key_agrif
     CALL Agrif_MPI_Init(MPI_COMM_MARS)
   ELSE
     MPI_COMM_MARS = Agrif_MPI_get_grid_comm()
   END IF
#endif  

   CALL MPI_COMM_SIZE ( MPI_COMM_MARS, NPROCS, IERR_MPI )
   CALL MPI_COMM_RANK ( MPI_COMM_MARS, IPROC, IERR_MPI )
   MINP=0
   MAXP=NPROCS-1
   IF (IPROC.EQ.0) MASTER=.TRUE.
   IF (MASTER) WRITE(*,*) IPROC,"I AM THE MASTER"

   MINIPROC=0
   MINJPROC=0
   NBBLANK=0
#if defined key_MPI_2D  && ! defined key_MPI_1D
   filempitxt='mpi.txt'
   IF_AGRIF (.NOT. agrif_root()) filempitxt='mpi'//TRIM(Agrif_Cfixed())//'.txt'
   INQUIRE(file     = filempitxt, EXIST = ios)
   filempitxt='hybrid'
   INQUIRE(file     = filempitxt, EXIST = ioshyb)

   IF (l_testcase .AND. .NOT.ioshyb) THEN
     IF (num_testcase==3 .OR. num_testcase==11) THEN
       MAXIPROC=1
       MAXJPROC=1
     ELSE IF (num_testcase==2) THEN
       MAXIPROC=7
       MAXJPROC=0
     ELSE IF (num_testcase==17) THEN
       MAXIPROC=2
       MAXJPROC=2
     ELSE IF (num_testcase==13) THEN   ! casacc
       MAXIPROC=1
       MAXJPROC=0
#if key_cas_5km
       MAXIPROC=1
       MAXJPROC=1
#elif key_cas_2km
       MAXIPROC=4
       MAXJPROC=3
#elif key_cas_1km
       MAXIPROC=7  !256 OMPMPI
       MAXJPROC=3
       MAXIPROC=5  !144 OMPMPI
       MAXJPROC=2
#endif
     ELSE IF (num_testcase==26) THEN
       MAXIPROC=3
       MAXJPROC=0
     ELSE IF (num_testcase==35) THEN
       MAXIPROC=1   ! pour 4 procs
       MAXJPROC=1
!       MAXIPROC=3  ! pour 16 procs
!       MAXJPROC=3
     ELSE IF (num_testcase==37) THEN
#if defined key_MPIOMP
       MAXIPROC=1
       MAXJPROC=0
#else
       MAXIPROC=2
       MAXJPROC=4
#endif
     ELSE
       MAXIPROC=3
       MAXJPROC=3
     END IF

     ALLOCATE(LI(4,miniproc:maxiproc))
     ALLOCATE(LJ(4,minjproc:maxjproc))
     ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
     proc=-2

   ELSE IF (.NOT.ios .AND. .NOT.ioshyb) then

     WRITE(*,*)'mpi.txt does not exit'
     WRITE(*,*)'mpi job will go to 1D mode '

   !MAXIPROC=3
   !MAXJPROC=3
   ! MAXIPROC=15
   ! MAXJPROC=0
   !IF(nprocs.NE.16)STOP !if(nprocs.ne.25)stop
     IF (jmax.gt.imax) THEN
       MAXIPROC=0
       MAXJPROC=NPROCS-1
     ELSE
       MAXIPROC=NPROCS-1
       MAXJPROC=0
     END IF

     ALLOCATE(LI(4,miniproc:maxiproc))
     ALLOCATE(LJ(4,minjproc:maxjproc))
     ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
     proc=IWETPOINT

   ELSE IF (.NOT.ios .AND. ioshyb) then

     WRITE(*,*)'openMP and MPI job'
     WRITE(*,*)'mpi job will go to 1D mode (along x)'

     MAXIPROC=NPROCS-1
     MAXJPROC=0

     ALLOCATE(LI(4,miniproc:maxiproc))
     ALLOCATE(LJ(4,minjproc:maxjproc))
     ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
     proc=IWETPOINT

   ELSE
     IF (MASTER) THEN
       WRITE(*,*)'mpi.txt  exist'
       WRITE(*,*)'read the mpi.txt'
     END IF
     LPARAMREAD=.TRUE.
     call MPI_PARAMREAD
   END IF

#else
   ! 1-D simulation specification
   MAXIPROC=0
   MAXJPROC=NPROCS-1
   ALLOCATE(LI(4,miniproc:maxiproc))
   ALLOCATE(LJ(4,minjproc:maxjproc))
   ! end 1-D simulation specification
   ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
   proc=IWETPOINT
#endif

   CALL DEFINE_PROC
   CALL DEFINE_UDPROC

#ifdef key_MPI_printparam
   CALL MPI_DEBUG_PRINTPARAMS1
#endif

   RETURN

  END SUBROUTINE MPI_START1

   !!======================================================================

  SUBROUTINE MPI_END1

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE  MPI_END1  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E       !  2013-07 (A. Thevenin - CERFACS) coupling using OASIS3-MCT
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

#ifdef key_oasis
   USE tooloasismct, ONLY : cpl_oasis_finalize
#endif

   IMPLICIT NONE

   !!----------------------------------------------------------------------
   !! * Executable part

   DEALLOCATE(cbudy,LI,LJ,CI,give_ccount,get_ccount)
   DEALLOCATE(rbudy,give_rcount,get_rcount)
   DEALLOCATE(PROC,ibudy,jbudy)
   DEALLOCATE(proc_i,proc_j,proc_size,proc_displs)
   DEALLOCATE(proc_limin,proc_limax,proc_ljmin,proc_ljmax)

#ifdef key_oasis
   CALL cpl_oasis_finalize
#endif

#if ! defined key_palm_ww3
   CALL MPI_Finalize(IERR_MPI)
#endif   
#ifdef key_MPI_printparam
   CLOSE(impierror)
#endif   
   RETURN

  END SUBROUTINE MPI_end1

   !!======================================================================

  !subroutine MPI_START2(LIP,LJP,CIP,RJP)
  SUBROUTINE MPI_START2(CIP,LJP)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_START2  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : imin,imax
   USE parameters, ONLY : limin,limax,liminp1,liminp2,limaxm1,liminm1,limaxp1,liminm2,limaxp2
   USE parameters, ONLY : limaxm1p1,liminp1m1,liminp1m2,ljminp1m2,limaxp3
#if defined key_lans_turb || defined key_leray_turb
   USE parameters, ONLY : ljminm3, liminm3, ljminm4, ljmaxp4, liminm4, limaxp4
#endif
   ! USE parameters, only : liminm2,limaxp2,liminm1,limaxp1
   USE parameters, ONLY : jmin,jmax
   USE parameters, ONLY : ljmin,ljmax,ljminp1,ljminp2,ljmaxm1,ljmaxm1p1,ljminp1m1
   USE parameters, ONLY : ljminm2,ljmaxp2,ljminm1,ljmaxp1,ljmaxp3
   USE parameters, ONLY : ishift_obc_west,ishift_obc_east,jshift_obc_south,jshift_obc_north

   IMPLICIT NONE

   !! * Arguments
   INTEGER, INTENT(in) :: CIP,LJP

   !! * Local declarations
   INTEGER :: i,j,n
   INTEGER :: LIP,RJP,maxfakeproc,countblank
   INTEGER,DIMENSION(imin:imax) :: ISUM
   INTEGER,DIMENSION(jmin:jmax) :: JSUM
   CHARACTER(len=7)  :: tag
   CHARACTER(len=30) :: filename

   !!----------------------------------------------------------------------
   !! * Executable part

   LIP=LJP
   RJP=CIP


   IF (.not.LPARAMREAD) THEN
     ! DEFINE LI 
     IF (LIP.EQ.0) THEN
       CALL divide_simple1d(imin,imax,miniproc,maxiproc,maxiproc,LI)
     ELSE
       CALL get_iSUM(isum)
       CALL divide_withsum1d(imin,imax,miniproc,maxiproc,maxiproc,isum,LI)
     END IF
     ! DEFINE LJ 
     IF (LJP.EQ.0) THEN
       CALL divide_simple1d(jmin,jmax,minjproc,maxjproc,maxjproc,LJ)
     ELSE
       CALL get_JSUM(jsum)
       CALL divide_withsum1d(jmin,jmax,minjproc,maxjproc,maxjproc,jsum,LJ)
     END IF
   END IF

   limin=LI(1,myiproc)
   liminp1=limin
   liminp2=limin
   liminm2=limin -2
#if defined key_lans_turb || defined key_leray_turb
   liminm3=limin - 3
   liminm4=limin - 4
#endif
   liminm1=limin -1
   liminp1m1=limin -1
   liminp1m2=limin -2
   limax=LI(2,myiproc)
   limaxm1=limax
   limaxp2=limax +2
   limaxp3=limax +3
#if defined key_lans_turb || defined key_leray_turb
   limaxp4=limax +4
#endif
   limaxp1=limax +1
   limaxm1p1=limax +1
   IF (myiproc.EQ.miniproc) THEN
     liminp1=imin+1+ishift_obc_west
     liminp2=imin+2+ishift_obc_west
     liminm2=imin
#if defined key_lans_turb || defined key_leray_turb
     liminm3=imin
     liminm4=imin
#endif
     liminm1=imin
     liminp1m1=imin+1+ishift_obc_west
     liminp1m2=imin+1+ishift_obc_west
   END IF
   IF (myiproc.EQ.maxiproc) THEN
     limaxm1=imax-1-ishift_obc_east
     limaxp2=imax
     limaxp3=imax
#if defined key_lans_turb || defined key_leray_turb
     limaxp4=imax
#endif
     limaxp1=imax
     limaxm1p1=limax-1-ishift_obc_east
   END IF

   ljmin=LJ(1,myjproc)
   ljminp1=ljmin
   ljminp2=ljmin
   ljminm2=ljmin -2
#if defined key_lans_turb || defined key_leray_turb
   ljminm3=ljmin -3
   ljminm4=ljmin -4
#endif
   ljminm1=ljmin -1
   ljminp1m1=ljmin -1
   ljminp1m2=ljmin -2
   ljmax=LJ(2,myjproc)
   ljmaxm1=ljmax
   ljmaxp2=ljmax + 2
   ljmaxp3=ljmax + 3
#if defined key_lans_turb || defined key_leray_turb
   ljmaxp4=ljmax + 4
#endif
   ljmaxp1=ljmax +1
   ljmaxm1p1=ljmax +1
   IF (myjproc.EQ.minjproc) THEN
     ljminp1=jmin+1+jshift_obc_south
     ljminp2=jmin+2+jshift_obc_south
     ljminm2=jmin
#if defined key_lans_turb || defined key_leray_turb
     ljminm3=jmin
     ljminm4=jmin
#endif
     ljminm1=jmin
     ljminp1m1=jmin+1+jshift_obc_south
     ljminp1m2=jmin+1+jshift_obc_south
   END IF
   IF (myjproc.EQ.maxjproc) THEN
     ljmaxm1=jmax-1-jshift_obc_north
     ljmaxp2=jmax
     ljmaxp3=jmax
     ljmaxp1=jmax
#if defined key_lans_turb || defined key_leray_turb
     ljmaxp4=jmax
#endif
     ljmaxm1p1=ljmax-1-jshift_obc_north
   END IF

   ! Make the shift of the OBC is not too large
   ! To simplify the exchanges, the OBCs must be inside the cpus
   ! delimitating the domain
!cval FAUT-IL PRENDRE PLUS DE MARGE ?
   ! il faut ig+3 dans miniproc (> suffisant)
   IF (myiproc.EQ.miniproc) THEN
     IF ((ishift_obc_west+3)>=limax) THEN
       WRITE(impierror,*) 'ishift_obc_west ,',ishift_obc_west
       WRITE(impierror,*) ', is larger than'
       WRITE(impierror,*) 'limax, ', limax
       WRITE(impierror,*) 'you MUST HAVE ishift_obc_west+3 < limax'
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
   END IF
   ! il faut 3 a gauche de id dans maxiproc (> suffisant)
   IF (myiproc.EQ.maxiproc) THEN
     IF (ishift_obc_east+3>=(limax-limin)) THEN
       WRITE(impierror,*) 'ishift_obc_east ,',ishift_obc_east
       WRITE(impierror,*) ', is larger than the size of the cpu'
       WRITE(impierror,*) 'bounding the eastern boundary , ', limax-limin
       WRITE(impierror,*) 'you MUST HAVE ishift_obc_east < (limax-limin)'
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
   END IF
   ! il faut jb+3 dans minjproc (> suffisant)
   IF (myjproc.EQ.minjproc) THEN
     IF ((jshift_obc_south+3)>=ljmax) THEN
       WRITE(impierror,*) 'jshift_obc_south ,',jshift_obc_south
       WRITE(impierror,*) ', is larger than'
       WRITE(impierror,*) 'ljmax, ', ljmax
       WRITE(impierror,*) 'you MUST HAVE jshift_obc_south+3 < ljmax'
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
   END IF
   ! il faut 3 en bas de jh dans maxjproc (> suffisant)
   IF (myjproc.EQ.maxjproc) THEN
     IF (jshift_obc_north+3>=(ljmax-ljmin)) THEN
       WRITE(impierror,*) 'jshift_obc_north ,',jshift_obc_north
       WRITE(impierror,*) ', is larger than the size of the cpu'
       WRITE(impierror,*) 'bounding the northern boundary , ', ljmax-ljmin
       WRITE(impierror,*) 'you MUST HAVE jshift_obc_north < (ljmax-ljmin)'
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
   END IF

   ! define CI here

   ALLOCATE(CI(4,mincproc:maxcproc))
   CI=0
   COUNTBLANK=COUNT(PROC(MYIPROC,mincproc:maxcproc)==IGROUNDPOINT)
   maxfakeproc=maxcproc-countblank

   IF (CIP.EQ.0) THEN
     CALL divide_simple1d(liminp2,limaxm1,mincproc,maxcproc,maxfakeproc,CI)
   ELSE
     CALL get_iSUM(isum)
     CALL divide_withsum1d(liminp2,limaxm1,mincproc,maxcproc,maxfakeproc,isum(liminp2:limaxm1),CI)
   END IF
   if(countblank.ne.0)CALL shiftforblank(CI,mincproc,maxcproc,PROC(myiproc,mincproc:maxcproc))


   ! define RJ here
   

   ALLOCATE(RJ(4,minrproc:maxrproc))
   RJ=0
   COUNTBLANK=COUNT(PROC(minrproc:maxrproc,myjproc)==IGROUNDPOINT)
   maxfakeproc=maxrproc-countblank
   IF (RJP.EQ.0) THEN
     CALL divide_simple1d(ljminp2,ljmaxm1,minrproc,maxrproc,maxfakeproc,RJ)
   ELSE
     CALL get_jSUM(jsum)
     CALL divide_withsum1d(ljminp2,ljmaxm1,minrproc,maxrproc,maxfakeproc,jsum(ljminp2:ljmaxm1),RJ)
   END IF

   if(countblank.ne.0)CALL shiftforblank(RJ,minrproc,maxrproc,PROC(minrproc:maxrproc,myjproc))

   ciminp2=CI(1,mycproc)
   cimaxm1=CI(2,mycproc)
   rjminp2=RJ(1,myrproc)
   rjmaxm1=RJ(2,myrproc)

   IF (LI(2,maxiproc).NE.imax) WRITE(impierror,*) imax,LI(2,maxiproc),"MPI_BUG1 report to tina ASAP" !;stop
   IF (LJ(2,maxjproc).NE.jmax) WRITE(impierror,*) jmax,LJ(2,maxjproc),"MPI_BUG2 report to tina ASAP" !;stop
   IF (CI(2,maxcproc).NE.limaxm1) WRITE(impierror,*) limaxm1,CI(2,maxcproc),"MPI_BUG3 report to tina ASAP "!;stop
   IF (RJ(2,maxrproc).NE.ljmaxm1) WRITE(impierror,*) ljmaxm1,RJ(2,maxrproc),"MPI_BUG4 report to tina ASAP",iproc !;stop

! cval val2tina : any MPI job is stopped with the following lines why ?
!   IF ( LI(2,maxiproc).NE.imax .OR. LJ(2,maxjproc).NE.jmax .OR. CI(2,maxcproc).NE.limaxm1 .OR. RJ(2,maxrproc).NE.ljmaxm1) &
!     CALL MPI_FINALIZE(IERR_MPI) ; STOP

   ALLOCATE(proc_i(minp:maxp),proc_j(minp:maxp))
   ALLOCATE(proc_limin(minp:maxp),proc_ljmin(minp:maxp))
   ALLOCATE(proc_limax(minp:maxp),proc_ljmax(minp:maxp))
   ALLOCATE(proc_size(minp:maxp),proc_displs(minp:maxp+1))

   DO j=minjproc,maxjproc
     DO i=miniproc,maxiproc
       n=proc(i,j)
       IF (n.NE.IGROUNDPOINT) THEN
         proc_i(n)=i
         proc_j(n)=j
         proc_size(n)=LI(3,i)*LJ(3,j)
         proc_limin(n)=LI(1,i)
         proc_limax(n)=LI(2,i)
         proc_ljmin(n)=LJ(1,j)
         proc_ljmax(n)=LJ(2,j)
       END IF
     END DO
   END DO
   proc_displs=0
   DO n=minp+1,maxp+1
     proc_displs(n)=proc_displs(n-1)+proc_size(n-1)
   END DO
   NBBLANK=COUNT(PROC==IGROUNDPOINT)

   filename='mpierror'
   CALL TAG_PROC_IJ(tag)
   filename=TRIM(filename)//'.'//tag
   impierror=5123
#ifdef key_MPI_printparam
   OPEN(impierror,file=filename)
#endif

   CALL MPI_DEBUG_PRINTPARAMS2

   RETURN

  END SUBROUTINE MPI_START2

   !!======================================================================

  SUBROUTINE MPI_START3

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_START3  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY: ljmin,ljmax
   USE parameters, ONLY: limin,limax
   USE comvars2d , ONLY: jb,jh,ig,id
   IMPLICIT NONE

   !! * Local declarations
   INTEGER :: x,y,i,j,k

   !!----------------------------------------------------------------------
   !! * Executable part

   ALLOCATE(give_ccount(MINcBUDY:MAXcBUDY))
   ALLOCATE(get_ccount(MINcBUDY:MAXcBUDY))

   give_ccount=0
   get_ccount=0

   A: DO x=MINcBUDY,MAXcBUDY
     y=cbudy(x)
     IF (y.EQ.mycproc) CYCLE A
     IF (y.LT.mincproc) CYCLE A
     IF (y.GT.maxcproc) CYCLE A
     k=0
     DO j=ljmin,ljmax
       DO i=CI(1,y),CI(2,y)
         IF (j.LT.jb(i))CYCLE
         IF (j.GT.jh(i))CYCLE
         k=k+1
       END DO
     END DO
     give_ccount(x)=k

     k=0
     DO j=LJ(1,y),LJ(2,y)
       DO i=ciminp2,cimaxm1
         IF (j.LT.jb(i))CYCLE
         IF (j.GT.jh(i))CYCLE
         k=k+1
       END DO
     END DO
     get_ccount(x)=k
   END DO A

   ALLOCATE(give_rcount(MINrBUDY:MAXrBUDY))
   ALLOCATE(get_rcount(MINrBUDY:MAXrBUDY))
   give_rcount=0
   get_rcount=0

   B: DO x=MINrBUDY,MAXrBUDY
     y=rbudy(x)
     IF (y.EQ.myrproc) CYCLE B
     IF (y.LT.minrproc) CYCLE B
     IF (y.GT.maxrproc) CYCLE B
     k=0
     DO j=RJ(1,y),RJ(2,y)
       DO i=limin,limax
         IF (i.LT.ig(j))CYCLE
         IF (i.GT.id(j))CYCLE
         k=k+1
       END DO
     END DO
     give_rcount(x)=k

     k=0
     DO j=rjminp2,rjmaxm1
       DO i=LI(1,y),LI(2,y)
         IF (i.LT.ig(j))CYCLE
         IF (i.GT.id(j))CYCLE
         k=k+1
       END DO
     END DO
     get_rcount(x)=k
   END DO B

   if(master) CALL makeprocmap
   if(master) CALL countwetpoint
#ifdef key_MPI_printparam
   CALL MPI_DEBUG_PRINTPARAMS3
#endif

   RETURN

  END SUBROUTINE MPI_START3

   !!======================================================================

  SUBROUTINE MPI_PARAMREAD

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_PARAMREAD  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Local declarations
   LOGICAL :: ios
   INTEGER :: ios2,itoto,itotoo,i,j
   INTEGER :: numiproc,numjproc,numactifs,tmpiproc, tmpjproc, tmpimin,tmpimax, tmpjmin, tmpjmax
   INTEGER :: readnproc
   LOGICAL :: LBLANK
   character(len=100) :: filempitxt

   !!----------------------------------------------------------------------
   !! * Executable part
   
#ifdef key_MPI_BLANK236
   ! here is what each program should read from input
   !open(1111,file='mpiparam.dat'
   readnproc=6
   IF (readnproc.NE.nprocs) THEN !trap to make the code stops if -np xx is wrong
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   NBBLANK=6   !read number of BLANK
   MAXIPROC=2  !read max number of I grid
   MAXjPROC=3  !read max number of J grid
   ALLOCATE(LI(4,miniproc:maxiproc))
   ALLOCATE(LJ(4,minjproc:maxjproc))
   ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
   proc=IWETPOINT
   ! do n=1,nbblank
   !  read(1111,*)i,j
   ! proc(i,j)=IGROUNDPOINT !read coordinate i, j for blank 
   ! end do
   !
   !close(1111)
   proc(0,2)=IGROUNDPOINT
   proc(0,3)=IGROUNDPOINT

   proc(1,0)=IGROUNDPOINT
   proc(1,3)=IGROUNDPOINT

   proc(2,0)=IGROUNDPOINT
   proc(2,1)=IGROUNDPOINT
#else
   filempitxt='mpi.txt'
   IF_AGRIF (.NOT. agrif_root()) filempitxt='mpi'//TRIM(Agrif_Cfixed())//'.txt'
   INQUIRE(file     = filempitxt, EXIST = ios)
   IF (.NOT.ios) stop "probleme a l ouverture mpi.txt en ecriture"
   OPEN(unit     = 1000, file     = filempitxt, &
        form     = "formatted",       &
        access   = "sequential",      &
        action   = "read",           &
        position = "rewind" )

   READ(unit = 1000, fmt = '(I12, A12)', iostat = ios2) itoto  
   READ(unit = 1000, fmt = '(I12, A12)', iostat = ios2) numiproc
   READ(unit = 1000, fmt = '(I12, A12)', iostat = ios2) numjproc
   READ(unit = 1000, fmt = '(I12, A12)', iostat = ios2) itoto
   READ(unit = 1000, fmt = '(I12, A12)', iostat = ios2) numactifs
   miniproc=0
   minjproc=0
   maxiproc=numiproc-1
   maxjproc=numjproc-1
   ALLOCATE(LI(4,miniproc:maxiproc))
   ALLOCATE(LJ(4,minjproc:maxjproc))
   ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
   READ(unit = 1000, fmt = '(8A12)', iostat = ios2) 
   DO j = 1, numjproc
     DO i = 1, numiproc
       READ(unit = 1000, fmt = '(6I12, L12, I12)', iostat = ios2) &
            tmpiproc, tmpjproc, tmpimin,tmpimax, tmpjmin, tmpjmax, LBLANK
            !copy tmpimin imax to mars here??
       IF (.NOT.LBLANK) THEN
         proc(tmpiproc,tmpjproc)=IGROUNDPOINT
       ELSE
         proc(tmpiproc,tmpjproc)=IWETPOINT
         LI(1,tmpiproc)=tmpimin
         LI(2,tmpiproc)=tmpimax
         LJ(1,tmpjproc)=tmpjmin
         LJ(2,tmpjproc)=tmpjmax
       END IF
     END DO
   END DO
   LI(3,:)=LI(2,:)-LI(1,:)+1
   LI(4,:)=LI(1,:)-LI(1,miniproc)
   LJ(3,:)=LJ(2,:)-LJ(1,:)+1
   LJ(4,:)=LJ(1,:)-LJ(1,minjproc)
      
   CLOSE(unit = 1000)

   IF (numactifs.NE.nprocs) THEN
     WRITE(impierror,*)'numiproc,numjproc'
     WRITE(impierror,*)numiproc,numjproc
     WRITE(impierror,*)'stop : numactifs on mpi.txt does not match nprocs : ',nprocs
     CALL MPI_FINALIZE(IERR_MPI) ; STOP  
   END IF
   !NBBLANK=6   !read number of BLANK
   !MAXIPROC=3  !read max number of I grid
   !MAXjPROC=2  !read max number of J grid
   !ALLOCATE(proc(miniproc:maxiproc,minjproc:maxjproc))
   !proc=-2
   !proc(0,0)=IGROUNDPOINT
   !proc(0,1)=IGROUNDPOINT
   !proc(1,2)=IGROUNDPOINT
   !proc(2,2)=IGROUNDPOINT
   !proc(3,0)=IGROUNDPOINT
   !proc(3,2)=IGROUNDPOINT
#endif

   RETURN

  END SUBROUTINE MPI_PARAMREAD

   !!======================================================================

  SUBROUTINE DEFINE_PROC

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEFINE_PROC  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Local declarations
   INTEGER  :: n,i,j,countblank
   INTEGER,ALLOCATABLE,DIMENSION(:) :: proctmp !this does not work yet
   !to avoid using this i need to get rid of exchange_rsh

   !!----------------------------------------------------------------------
   !! * Executable part
   
   n=0
   DO j=minjproc,maxjproc
     DO i=miniproc,maxiproc
       IF (proc(i,j).EQ.IWETPOINT) THEN
         proc(i,j)=n
         n=n+1
       END IF
     END DO
   END DO
   IF (n.NE.nprocs) THEN
     WRITE(impierror,*) 'minjproc,maxjproc,miniproc,maxiproc,n,nprocs'
     WRITE(impierror,*) minjproc,maxjproc,miniproc,maxiproc,n,nprocs
     WRITE(impierror,*)'maxjproc and maxiproc do not correspond to np stop'
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

   ! from here, it defines myiproc,myjproc,mincproc,maxcproc,minrproc,maxcproc
   ! depending on what is your 'iproc' is 
   ! i.e. each process have differfent value!!!
   DO j=minjproc,maxjproc
     DO i=miniproc,maxiproc
       IF (proc(i,j).EQ.iproc) THEN
         myiproc=i
         myjproc=j
       END IF
     END DO
   END DO

   MYCPROC=MYJPROC
   MYRPROC=MYIPROC
   i=myiproc
   mincproc=minjproc
   A:DO j=minjproc,maxjproc
     IF (PROC(i,j).EQ.IGROUNDPOINT) THEN
       mincproc=j+1
     ELSE
       EXIT A
     END IF
   END DO A
   maxcproc=maxjproc
   B:DO j=maxjproc,minjproc,-1
     IF (PROC(i,j).EQ.IGROUNDPOINT) THEN
       maxcproc=j-1
     ELSE
       EXIT B
     END IF
   END DO B

   j=myjproc
   minrproc=miniproc
   C:DO i=miniproc,maxiproc
     IF (PROC(i,j).EQ.IGROUNDPOINT) THEN
       minrproc=i+1
     ELSE
       EXIT C
     END IF
   END DO C

   maxrproc=maxiproc
   D: DO i=maxiproc,miniproc,-1
     IF (PROC(i,j).EQ.IGROUNDPOINT) THEN
       maxrproc=i-1
     ELSE
       EXIT D
     END IF
   END DO D

   MINCBUDY=0
   COUNTBLANK=COUNT(PROC(MYIPROC,mincproc:maxcproc)==IGROUNDPOINT)
   MAXCBUDY=MAXCPROC-MINCPROC-COUNTBLANK
   NUMCBUDY=MAXCBUDY+1
   IF ((numcbudy.NE.1).AND.(MOD(numcbudy,2).EQ.1)) THEN
     MAXCBUDY=MAXCBUDY+1
   END IF
   ALLOCATE(Cbudy(MINcBUDY:MAXcBUDY))
   CALL makebudy(Cbudy,maxcbudy,mincproc,maxcproc,numcbudy,mycproc,PROC(myiproc,mincproc:maxcproc))

   MINRBUDY=0
   COUNTBLANK=COUNT(PROC(minrproc:maxrproc,myjproc)==IGROUNDPOINT)
   MAXRBUDY=MAXRPROC-MINRPROC-COUNTBLANK
   NUMRBUDY=MAXRBUDY+1
   IF ((numRbudy.NE.1).AND.(MOD(numRbudy,2).EQ.1)) THEN
     MAXRBUDY=MAXRBUDY+1
   END IF
   ALLOCATE(rbudy(MINrBUDY:MAXrBUDY))
   CALL makebudy(rbudy,maxrbudy,minrproc,maxrproc,numrbudy,myrproc,PROC(minrproc:maxrproc,myjproc))

   MINIBUDY=MINIPROC
   MAXIBUDY=MAXIPROC
   NUMiBUDY=MAXIBUDY+1
   IF ((numibudy.NE.1).AND.(MOD(numibudy,2).EQ.1)) THEN
     MAXiBUDY=MAXiBUDY+1
   END IF
   ALLOCATE(ibudy(MINiBUDY:MAXiBUDY))
   ALLOCATE(proctmp(MINiproc:MAXiproc))
   proctmp=1
   CALL makebudy(ibudy,maxibudy,miniproc,maxiproc,numibudy,myiproc,PROCtmp)
   deallocate(proctmp)

   MINjBUDY=MINjPROC
   MAXjBUDY=MAXjPROC
   NUMjBUDY=MAXjBUDY+1
   IF ((numjbudy.NE.1).AND.(MOD(numjbudy,2).EQ.1)) THEN
     MAXjBUDY=MAXjBUDY+1
   END IF
   ALLOCATE(Jbudy(MINjBUDY:MAXjBUDY))
   ALLOCATE(proctmp(MINjproc:MAXjproc))
   proctmp=1
   CALL makebudy(jbudy,maxjbudy,minjproc,maxjproc,numjbudy,myjproc,PROCtmp)
   deallocate(proctmp)

  END SUBROUTINE DEFINE_PROC

   !!======================================================================

  SUBROUTINE DEFINE_UDPROC

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE DEFINE_UDPROC  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !!----------------------------------------------------------------------
   !! * Executable part
   
   IF (myjproc==maxjproc) THEN
     myj_uproc=MPI_PROC_NULL
   ELSE IF (PROC(myiproc,myjproc+1).EQ.IGROUNDPOINT) THEN
     myj_uproc=MPI_PROC_NULL
   ELSE
     myj_uproc=PROC(myiproc,myjproc+1)
   END IF

   IF (myjproc==minjproc) THEN
     myj_dproc=MPI_PROC_NULL
   ELSE IF (PROC(myiproc,myjproc-1).EQ.IGROUNDPOINT) THEN
     myj_dproc=MPI_PROC_NULL
   ELSE
     myj_dproc=PROC(myiproc,myjproc-1)
   END IF

   IF (myiproc==maxiproc) THEN
     myi_uproc=MPI_PROC_NULL
   ELSE IF (PROC(myiproc+1,myjproc).EQ.IGROUNDPOINT) THEN
     myi_uproc=MPI_PROC_NULL
   ELSE
     myi_uproc=PROC(myiproc+1,myjproc)
   END IF

   IF (myiproc==miniproc) THEN
     myi_dproc=MPI_PROC_NULL
   ELSE IF (PROC(myiproc-1,myjproc).EQ.IGROUNDPOINT) THEN
     myi_dproc=MPI_PROC_NULL
   ELSE
     myi_dproc=PROC(myiproc-1,myjproc)
   END IF

   RETURN

  END SUBROUTINE DEFINE_UDPROC

   !!======================================================================

  SUBROUTINE MPI_DEBUG_PRINTPARAMS1

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_DEBUG_PRINTPARAMS1  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Local declarations
   INTEGER           :: i,ii
   CHARACTER(len=7)  :: tag
   CHARACTER(len=30) :: filename

   !!----------------------------------------------------------------------
   !! * Executable part
   
   filename='mpidebugoutput'
   CALL TAG_PROC_IJ(tag)
   filename=TRIM(filename)//'.'//tag
   i=1000
   OPEN(i,file=filename)
   WRITE(i,*)'mpi_debug_printparams1 start'
   WRITE(i,*)'iproc,nprocs',iproc,nprocs
   WRITE(i,*)'maxiproc,maxjproc',maxiproc,maxjproc
   WRITE(i,*)'myiproc,myjproc',myiproc,myjproc
   WRITE(i,*)'print proc(:,j) here'
   DO ii=minjproc,maxjproc
     WRITE(i,*)'j=',ii
     WRITE(i,*)proc(:,ii)
   END DO

   WRITE(i,*)'mincproc,maxcproc',mincproc,maxcproc
   WRITE(i,*)'mincbudy,maxcbudy',mincbudy,maxcbudy
   WRITE(i,*)'cbudy',cbudy
   WRITE(i,*)'minrproc,maxrprro',minrproc,maxrproc
   WRITE(i,*)'minrbudy,maxrbudy',minrbudy,maxrbudy
   WRITE(i,*)'rbudy',rbudy
   WRITE(i,*)'mpi_debug_printparams1 end'
   WRITE(i,*)

   RETURN

  END SUBROUTINE MPI_DEBUG_PRINTPARAMS1

   !!======================================================================

  SUBROUTINE MPI_DEBUG_PRINTPARAMS2

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_DEBUG_PRINTPARAMS2  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   IMPLICIT NONE

   !! * Local declarations
   INTEGER :: i,ii,iii,j
   CHARACTER(LEN=lchain) :: file_mpidebugprepare

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (MASTER) THEN
     iii=1100
     file_mpidebugprepare = 'mpidebugprepare'
     IF_AGRIF (.NOT. agrif_root()) file_mpidebugprepare='mpidebugprepare_'//TRIM(Agrif_Cfixed())

     OPEN(iii,file=file_mpidebugprepare)
           WRITE(iii,800)nprocs
     DO j=minjproc,maxjproc
       DO i=miniproc,maxiproc
         IF (proc(i,j).GE.0) THEN
           WRITE(iii,800)proc(i,j),i,j,LI(1,i),LI(2,i),LJ(1,j),LJ(2,j)
         END IF
       END DO
     END DO

800  FORMAT(7(i10.10,' '))

     CLOSE(iii)
   END IF

#ifdef key_MPI_printparam
   ii=1000

   WRITE(ii,*)'mpi_debug_printparams2 start'
   WRITE(ii,*)'print LI'
   DO i=miniproc,maxiproc
     WRITE(ii,*)'for i=',i,'LI =',LI(1:3,i)
   END DO

   WRITE(ii,*)'print LJ'
   DO i=minjproc,maxjproc
     WRITE(ii,*)'for j=',i,'LJ =',LJ(1:3,i)
   END DO

   WRITE(ii,*)'print CI'
   DO i=mincproc,maxcproc
     WRITE(ii,*)'for cproc=',i,'CI =',CI(1:3,i)
   END DO

   WRITE(ii,*)'print RJ'
   DO i=minrproc,maxrproc
     WRITE(ii,*)'for rproc=',i,'RJ =',RJ(1:3,i)
   END DO

   WRITE(ii,*)'print Proc_i,j,size,displs'
   DO i=minp,maxp
     WRITE(ii,*)'for iproc=',i,'proc_i=',proc_i(i),'proc_j=',proc_j(i)
     WRITE(ii,*)'for iproc=',i,'proc_limin=',proc_limin(i),'proc_ljmin=',proc_ljmin(i)
     WRITE(ii,*)'for iproc=',i,'proc_limax=',proc_limax(i),'proc_ljmin=',proc_ljmax(i)
     WRITE(ii,*)'for iproc=',i,'proc_size=',proc_size(i)
     WRITE(ii,*)'for iproc=',i,'proc_displs=',proc_displs(i)
   END DO
     WRITE(ii,*)'all active proc size is',sum(proc_size)
     WRITE(ii,*)'sum of ground mpi process is',nbblank
   WRITE(ii,*)'mpi_debug_printparams2 end'
   WRITE(ii,*)
#endif

  END SUBROUTINE MPI_DEBUG_PRINTPARAMS2

   !!======================================================================

  SUBROUTINE MPI_DEBUG_PRINTPARAMS3

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_DEBUG_PRINTPARAMS3  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Local declarations
   INTEGER :: i,ii

   !!----------------------------------------------------------------------
   !! * Executable part

   ii=1000

   WRITE(ii,*)'mpi_debug_printparams3 start'
   WRITE(ii,*)'print give_ccount'
   WRITE(ii,*)give_ccount
   WRITE(ii,*)'print get_ccount'
   WRITE(ii,*)get_ccount
   WRITE(ii,*)'print give_rcount'
   WRITE(ii,*)give_rcount
   WRITE(ii,*)'print get_rcount'
   WRITE(ii,*)get_rcount
   WRITE(ii,*)'mpi_debug_printparams3 end'

   CLOSE(ii)
   RETURN

  END SUBROUTINE MPI_DEBUG_PRINTPARAMS3

   !!======================================================================

  SUBROUTINE makebudy(bud,maxbudy,minproc,maxproc,numbudy,myproc,aproc)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE makebudy  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : ATT: BUDY counts the number of times each procs transpose data;
   !&E                  budy will let you know who you should exchange the data each N,i,j
   !&E                  N starts always from 0, and ends at maxbudy
   !&E                  minproc indicates the first process  which starts the transpose data
   !&E                  maxproc indicates the last process which ends the transpose data
   !&E
   !&E ** Modified variables : bud
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER                     ,INTENT(in)  :: maxbudy,minproc,maxproc,myproc
   INTEGER,DIMENSION(0:maxbudy),INTENT(out) :: bud
   INTEGER,DIMENSION(minproc:maxproc),INTENT(in) :: aproc

   !! * Local declarations
   INTEGER                            :: myproc_tmp,numbudy,i,j
   INTEGER,ALLOCATABLE,DIMENSION(:,:) :: budy_all

   !!----------------------------------------------------------------------
   !! * Executable part
   !numbudy=maxproc-minproc+1
   IF (numbudy.EQ.1) THEN
     bud=myproc
     RETURN
   END IF

   ALLOCATE(budy_all(0:maxbudy,0:maxbudy))
   CALL calcbudy(budy_all,maxbudy+1)  !calcbudy calculate maxbudy pairs
   !of exchanging process
   !myproc_tmp defines the relative numbering of procs
   myproc_tmp=myproc-minproc-count(aproc(minproc:myproc-1)<0)
!   write(*,*)'makebudy',myiproc,myjproc,myproc_tmp
   bud(:)=budy_all(myproc_tmp,0:maxbudy)
   DEALLOCATE(budy_all)
   IF (MOD(numbudy,2).EQ.1) CALL calcbudy_odd(bud,maxbudy)  !if maxbudy is odd
   IF (minproc.NE.0) bud=bud+minproc  !adjust back so that zero becomes minproc
   if(count(aproc(:)<0).gt.0)then
           doi:do i=minproc+1,maxproc
           if(aproc(i).ne.IGROUNDPOINT)cycle doi
           doj:do j=0,maxbudy
           if(bud(j).lt.i)cycle doj
!   write(*,*)'makebudy budmodified',myiproc,i,j
           bud(j)=bud(j)+1
           enddo doj
           enddo doi
   endif
  END SUBROUTINE makebudy

   !!======================================================================

  SUBROUTINE calcbudy_odd(bud,n)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE calcbudy_odd  ***
   !&E
   !&E ** Purpose : exchange the entry with maxbudy to -1
   !&E
   !&E ** Modified variables : bud
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER               ,INTENT(in)  :: n
   INTEGER,DIMENSION(0:n),INTENT(out) :: bud

   !! * Local declarations
   INTEGER :: i,j

   !!----------------------------------------------------------------------
   !! * Executable part

   DO i=0,n
     j= bud(i)
     IF (j==n) THEN
       bud(i)=-1
     ELSE
       bud(i)=j
     END IF
   END DO

  END SUBROUTINE calcbudy_odd

   !!======================================================================

  SUBROUTINE calcbudy(budy,n)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE calcbudy  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : based on the idea published in
   !&E                J. A. M. Schreuder, "Constructing Timetables for Sport Competitions," 
   !&E                Mathematical Programming Study 13, pp. 58-67 (1980).
   !&E ** History :
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER                       ,INTENT(in)  :: n
   INTEGER,DIMENSION(0:n-1,0:n-1),INTENT(out) :: budy

   !! * Local declarations
   INTEGER                  :: i,j,t,a,b,k,n2
   LOGICAL                  :: test1,test2
   INTEGER,DIMENSION(n,n,n) :: x

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (MOD(n,2).EQ.1) THEN
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

   IF (n==4) THEN
     budy(0,:)=(/ 3,2,1,0 /)
     budy(1,:)=(/ 2,3,0,1 /)
     budy(2,:)=(/ 1,0,3,2 /)
     budy(3,:)=(/ 0,1,2,3 /)
     RETURN
   ELSE IF (n==2) THEN
     budy(0,:)=(/ 1,0 /)
     budy(1,:)=(/ 0,1 /)
     RETURN
   END IF

   test1=.TRUE.
   t=0
   i=0
   x=0
   n2=n/2

10 t=t+1
   i=i+1
   IF (i.EQ.n2) test1=.NOT.test1
   IF (test1) THEN
     x(n,i,t)=1
     test1=.FALSE.
   ELSE
     x(i,n,t)=1
     test1=.TRUE.
   END IF
   test2=.TRUE.
   a=i
   b=i
   DO k=1,n2-1
     a= a-1
     a=MOD(a,n-1)
     b=MOD(b+1,n-1)
     !this part is different from original paper
     IF (a.EQ.0)a=n-1
     IF (b.EQ.0)b=n-1
     !untill here.

     IF (test2) THEN
       x(a,b,t)=1
       test2=.FALSE.
     ELSE
       x(b,a,t)=1
       test2=.TRUE.
     END IF
   END DO
   IF (t.LT.n-1) GOTO 10

   !from now it is adjusted for mars budy.
   budy=0
   DO t=0,n-1
     DO j=1,n
       IF (t.EQ.0) THEN
         budy(j-1,0)=j-1
       ELSE
         DO i=1,n
           IF (x(i,j,t).EQ.1) THEN
             budy(i-1,t)=j-1
             budy(j-1,t)=i-1
           END IF
         END DO
       END IF
     END DO
   END DO

   ! do i=0,n-1
   ! write(*,*)"budyi=",i,budy(i,:)
   ! enddo

  END SUBROUTINE calcbudy

   !!======================================================================

  SUBROUTINE TAG_PROC(name)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE TAG_PROC  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : name
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=3),INTENT(out) :: name 

   !! * Local declarations
   INTEGER :: i1,i2,i3,k

   !!----------------------------------------------------------------------
   !! * Executable part

   k=iproc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   name=CHAR(i3)//CHAR(i2)//CHAR(i1)
   RETURN

  END SUBROUTINE TAG_PROC

   !!======================================================================

  SUBROUTINE TAG_PROC_IJ(nameall)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE TAG_PROC_IJ  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : nameall
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(len=7),INTENT(out) :: nameall

   !! * Local declarations
   CHARACTER(len=3) :: namei,namej
   INTEGER          :: i1,i2,i3,k

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_MPI_2D
   k=myiproc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   namei=CHAR(i3)//CHAR(i2)//CHAR(i1)
#endif

   k=myjproc
   i3=k/100+48
   i2=(k-(i3-48)*100)/10+48
   i1=k-(i3-48)*100-(i2-48)*10+48
   namej=CHAR(i3)//CHAR(i2)//CHAR(i1)
#ifdef key_MPI_2D
   nameall=TRIM(namei)//'.'//TRIM(namej)
#else
   nameall=TRIM(namej)
#endif
   RETURN

  END SUBROUTINE TAG_PROC_IJ

   !!======================================================================

  SUBROUTINE ADD_TAG(filename,ndim)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ADD_TAG  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : filename
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER,          INTENT(in)  :: ndim
   CHARACTER(len=*), INTENT(out) :: filename

   !! * Local declarations
   CHARACTER(len=7)    :: tag
   CHARACTER(len=ndim) :: filenametmp
   INTEGER             :: nn,ndim_nn

   !!----------------------------------------------------------------------
   !! * Executable part

   IF ((LEN_TRIM(filename)+8).GT.ndim)WRITE(impierror,*)"filename: ",filename," trancated"
   CALL TAG_PROC_IJ(tag)
   ! filename=tag//"."//filename
   ! print *, "iproc=",iproc,"tag=",tag,filename
   ndim_nn=LEN(filename)
   nn=LEN_TRIM(filename)
   IF ((nn+8).GT.ndim_nn) THEN
     filenametmp=TRIM(filename)
     filename=filenametmp(1:nn-8)//'.'//tag
   ELSE
     filename=TRIM(filename)//'.'//tag
   END IF
   !PRINT *, filename
   RETURN

  END SUBROUTINE ADD_TAG

   !!======================================================================

  SUBROUTINE makeprocmap

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE makeprocmap  ***
   !&E
   !&E ** Purpose : 
   !&E           to verify if mpi.txt includes all thepoints between 
   ! ig:id, and jb;jh (skips blank in the middle)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : jmin,jmax,imin,imax,valmanq
   use comvars2d, only : ig,id,jb,jh,h0_g
   IMPLICIT NONE

   !! * Arguments
   INTEGER ,DIMENSION(imin:imax,jmin:jmax) :: procmap

   !! * Local declarations
   INTEGER :: ip,jp,i,j,ierrr

   !!----------------------------------------------------------------------
   !! * Executable part

   DO jp=minjproc,maxjproc
   DO ip=miniproc,maxiproc
   procmap(LI(1,ip):LI(2,ip),LJ(1,jp):LJ(2,jp))=proc(ip,jp)
    
   enddo
   enddo
   ierrr=0
!   write(*,*) 'check if mpi.txt include all the wet point'
   do j=jmin,jmax
   do i=ig(j),id(j)
      if(procmap(i,j).eq.IGROUNDPOINT)then 
        if((i.ge.imin+2.and.i.le.imax-2).and.(j.ge.jmin+2.and.j.le.jmax-2))then
! modif pgarreau  pour etre moins mere poule (tina dixit)...
!         if(count(h0(i-2:i+2,j-2:j+2)>-valmanq).ne.0)then
          if(h0_g(i,j) .gt. -valmanq)then
       write(*,*) 'stop: bad mpi.txt from igid i=',i,'j=',j
       ierrr=ierrr+1
       endif
       endif
      endif
   enddo
   enddo
   do i=imin,imax
   do j=jb(i),jh(i)
      if(procmap(i,j).eq.IGROUNDPOINT)then 
        if((i.ge.imin+2.and.i.le.imax-2).and.(j.ge.jmin+2.and.j.le.jmax-2))then
! modif pgarreau  pour etre moins mere poule (tina dixit)...
!         if(count(h0(i-2:i+2,j-2:j+2)>-valmanq).ne.0)then
          if(h0_g(i,j) .gt. -valmanq)then
          write(*,*) 'stop: bad mpi.txt from jbjh i=',i,'j=',j
          ierrr=ierrr+1
         endif
        endif
      endif
   enddo
   enddo

   if(ierrr.eq.0)then
!   write(*,*) 'check mpi.txt OK'
   else
   write(*,*) 'check mpi.txt not OK,kill your code and edit your mpi.txt'
   stop
   endif
   return
  END SUBROUTINE makeprocmap

  SUBROUTINE countwetpoint

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE countwetpoint  ***
   !&E
   !&E ** Purpose : 
   !&E           to verify if mpi.txt  put almost same no of wet point for each
   !nodes
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : jmin,jmax,imin,imax,valmanq
   use comvars2d, only : h0_g
   IMPLICIT NONE

   !! * Arguments
   INTEGER ,DIMENSION(imin:imax,jmin:jmax) :: procmap
!   LOGICAL ,DIMENSION(imin-2:imax+2,jmin-2:jmax+2) :: logich0

   !! * Local declarations
   INTEGER :: ip,jp,i,j,iwet,ierror

   !!----------------------------------------------------------------------
   !! * Executable part
   
   ! Set to .true. over wet points
!   logich0(:,:)=.FALSE.
!   WHERE(h0 > -valmanq) logich0(imin:imax,jmin:jmax)=.TRUE.
!   write(*,*) 'count wet point for each mpi grid'
   ierror=0

   ! Loop on domain division
   DO jp=minjproc,maxjproc
     DO ip=miniproc,maxiproc

       iwet=0
   ! Loop in ljmin,ljmax and limin,limax of each subdomain
       DO j=LJ(1,jp),LJ(2,jp)
         DO i=LI(1,ip),LI(2,ip)
           IF (h0_g(i,j) > -valmanq)iwet=iwet+1
           !cval test too large because of +2 -2 if(count( logich0(i-2:i+2,j-2:j+2)).ne.0)iwet=iwet+1
         END DO
       END DO
!       write(*,*) 'ip=',ip,'jp=',jp,'wetpoint=',iwet

   ! Make sure there is none wet cell (iwet/=0) for sleeping CPU (IGROUNDPOINT=.true.)
       IF (iwet /= 0 .AND. proc(ip,jp) == IGROUNDPOINT) THEN
         WRITE(*,*) 'stop:ip=',ip,'jp=',jp,'You should not have any wet cell in this CPU'
         ierror=ierror+1
       END IF

     END DO
   END DO
   IF (ierror /= 0) THEN
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

  END SUBROUTINE countwetpoint

  SUBROUTINE shiftforblank(LA,minaproc,maxaproc,APROC)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE shiftforblank  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : LA
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER           ,INTENT(in)  :: maxaproc,minaproc
   INTEGER ,DIMENSION(4,minaproc:maxaproc) :: LA
   INTEGER ,DIMENSION(minaproc:maxaproc),INTENT(in) ::aproc

   !! * Local declarations
   INTEGER :: i

   !!----------------------------------------------------------------------
   !! * Executable part

   DO i=minaproc,maxaproc
     if(aproc(i).eq.IGROUNDPOINT) then
!          write(*,*),'shift',i+1,maxaproc
!          write(*,*),'shift',i,maxaproc-1 
!          write(*,*),'shiftmae',LA(1,i:maxaproc)
          LA(:,i+1:maxaproc)=LA(:,i:maxaproc-1)
          LA(:,i)=IGROUNDPOINT
!          write(*,*),'shiftato',LA(1,i:maxaproc)
     endif
   enddo


  END SUBROUTINE shiftforblank

   !!======================================================================
  SUBROUTINE divide_simple1d(amin,amax,minaproc,maxdimaproc,maxaproc,LA)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE divide_simple1d  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : LA
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER           ,INTENT(in)  :: amin,amax,maxaproc,minaproc,maxdimaproc
   INTEGER ,DIMENSION(4,minaproc:maxdimaproc),INTENT(out) :: LA

   !! * Local declarations
   INTEGER :: n,m,i,numaproc,ndf

   !!----------------------------------------------------------------------
   !! * Executable part

   numaproc=maxaproc-minaproc+1
   LA(1,minaproc)=amin
   n=(amax-amin  +1)/ numaproc 
   m=MOD((amax-amin+1) ,numaproc )
   ndf=0
   DO i=minaproc,maxaproc
     LA(2,i)=LA(1,i)+n-1
     if(i.eq.maxaproc)then
       if(LA(2,maxaproc).ne.amax) then
           write(*,*)'stop: error maxaproc la2 ne amax'
           write(*,*)maxaproc,amax,iproc
           stop
       else
               cycle
       endif
     endif
   if(m.gt.ndf) LA(2,i)=LA(2,i)+1
     LA(1,i+1)=LA(2,i)+1
     ndf=ndf+1
   enddo

   LA(3,:)=LA(2,:)-LA(1,:)+1
   LA(4,:)=LA(1,:)-LA(1,minaproc)

  END SUBROUTINE divide_simple1d

   !!======================================================================

  SUBROUTINE divide_withsum1d(amin,amax,minaproc,maxdimaproc,maxaproc,asum,LA)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE divide_withsum1d  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : LA
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER        ,INTENT(in)  :: amin,amax,maxaproc,minaproc,maxdimaproc
   INTEGER ,DIMENSION(amin:amax)          ,INTENT(in)  :: aSUM
   INTEGER ,DIMENSION(4,minaproc:maxdimaproc),INTENT(out) :: LA

   !! * Local declarations
   INTEGER ::n,y,x,j ,numaproc,iiout

   !!----------------------------------------------------------------------
   !! * Executable part

   LA(1,minaproc)=amin
   numaproc=maxaproc-minaproc+1
   !iiout=10000+myiproc*100+myjproc
   !        write(iiout,*)amin,amax
   !        write(iiout,*)minaproc,maxaproc,maxdimaproc
   !DO j=amin,amax
   !        write(iiout,*)asum(j)
   !enddo
   if(numaproc.eq.1)goto 101
          
   n=SUM(aSUM(amin:amax))/numaproc
   y=minaproc
   x=0
   DO j=amin,amax
     x=x+aSUM(j)
     IF (x.Ge.n) THEN
       La(2,Y)=j
       y=y+1
       La(1,Y)=j+1
       IF (y.EQ.maxaproc) GOTO 101
       x=0
       n=SUM(aSUM(j+1:amax))/(numaproc-(y-minaproc))
     END IF
   END DO
101 LA(2,maxaproc)=amax
   LA(3,:)=LA(2,:)-LA(1,:)+1
   LA(4,:)=LA(1,:)-LA(1,minaproc)

  END SUBROUTINE divide_withsum1d

   !!======================================================================

  SUBROUTINE get_JSUM(JSUM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE get_JSUM  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : JSUM
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : jmin,jmax,imin,imax,valmanq
   !USE comvars2d, ONLY : h0
   use comvars2d, only : ig,id
   IMPLICIT NONE

   !! * Arguments
   INTEGER ,DIMENSION(jmin:jmax),INTENT(out) :: JSUM

   !! * Local declarations
   INTEGER i,j,x,y

   !!----------------------------------------------------------------------
   !! * Executable part

   DO j=jmin,jmax
     !jsum(j)=0
     !DO i=imin,imax
     !  IF (h0(i,j) > -valmanq) jsum(j)=jsum(j)+1
     !ENDDO
     if(ig(j).le.id(j)) then
     jsum(j)=id(j)-ig(j)+1
     else
     jsum(j)=0
     endif
   END DO
   RETURN

  END SUBROUTINE get_JSUM

   !!======================================================================

  SUBROUTINE get_iSUM(iSUM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE get_iSUM  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : iSUM
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : jmin,jmax,imin,imax,valmanq
   !USE comvars2d, ONLY : h0
   use comvars2d,only : jb,jh
   IMPLICIT NONE

   !! * Arguments
   INTEGER ,DIMENSION(imin:imax),INTENT(out) :: iSUM

   !! * Local declarations
   INTEGER i,j,x,y

   !!----------------------------------------------------------------------
   !! * Executable part

   DO i=imin,imax
!     DO j=jmin,jmax
!       IF (h0(i,j) > -valmanq) isum(i)=isum(i)+1
!     END DO
     if(jb(i).le.jh(i)) then
     isum(i)=jh(i)-jb(i)+1
     else
     isum(i)=0
     endif
   END DO
   RETURN

  END SUBROUTINE get_iSUM

   !!======================================================================
  
  SUBROUTINE sendrecv_rlg(dest,size_give,give,size_get,get) 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sendrecv_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg
   IMPLICIT NONE

   !! * Arguments
   INTEGER                            ,INTENT(inout) :: dest,size_give,size_get
   REAL(kind=rlg),DIMENSION(size_give),INTENT(inout) :: GIVE
   REAL(kind=rlg),DIMENSION(size_get) ,INTENT(inout) :: GET

   !! * Local declarations
   INTEGER :: send_tag,recv_tag,NDIM_SEND,NDIM_RECV
   INTEGER :: stat(MPI_STATUS_SIZE)
   INTEGER :: TYPE_NDIM_SEND,TYPE_NDIM_RECV

   !!----------------------------------------------------------------------
   !! * Executable part

   NDIM_SEND=1
   NDIM_RECV=1
   IF (rlg.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_REAL,TYPE_NDIM_SEND, IERR_MPI )
     CALL MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_REAL,TYPE_NDIM_RECV, IERR_MPI )
   ELSE IF (rlg.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_DOUBLE_PRECISION,TYPE_NDIM_SEND, IERR_MPI )
     CALL MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_DOUBLE_PRECISION,TYPE_NDIM_RECV, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rlg=4 and 8'
     WRITE(impierror,*)'you gave rlg=',rlg
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_RECV, IERR_MPI )

   send_tag=300*iproc+dest
   recv_tag=300*dest+iproc
   CALL MPI_SENDRECV(GIVE,SIZE_GIVE,TYPE_NDIM_SEND,dest,send_tag &
     &,GET,SIZE_GET,TYPE_NDIM_RECV,dest,recv_tag &
     &,MPI_COMM_MARS,stat,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_FREE(TYPE_NDIM_RECV, IERR_MPI )
   RETURN

  END SUBROUTINE sendrecv_rlg

   !!======================================================================

#ifdef key_MPI_2D
  SUBROUTINE sendrecv_int(dest,size_give,give,size_get,get) !!!akiko copy sendrecv_rlg

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sendrecv_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER                            ,INTENT(inout) :: dest,size_give,size_get
   INTEGER,DIMENSION(size_give),INTENT(inout) :: GIVE
   INTEGER,DIMENSION(size_get) ,INTENT(inout) :: GET

   !! * Local declarations
   INTEGER :: send_tag,recv_tag,NDIM_SEND,NDIM_RECV
   INTEGER :: stat(MPI_STATUS_SIZE)
   INTEGER :: TYPE_NDIM_SEND,TYPE_NDIM_RECV

   !!----------------------------------------------------------------------
   !! * Executable part

   NDIM_SEND=1
   NDIM_RECV=1
   !if (int.eq.4)then
   !  call MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_REAL,TYPE_NDIM_SEND, IERR_MPI )
   !  call MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_REAL,TYPE_NDIM_RECV, IERR_MPI )
   !elseif(int.eq.8)then
   CALL MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_INTEGER,TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_INTEGER,TYPE_NDIM_RECV, IERR_MPI )
   !else
   !        print*,' this version of exchange works only for int=4 and 8'
   !        print*,'you gave int=',int
   !        stop
   !endif
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_RECV, IERR_MPI )

   send_tag=300*iproc+dest
   recv_tag=300*dest+iproc
   CALL MPI_SENDRECV(GIVE,SIZE_GIVE,TYPE_NDIM_SEND,dest,send_tag &
     &,GET,SIZE_GET,TYPE_NDIM_RECV,dest,recv_tag &
     &,MPI_COMM_MARS,stat,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_FREE(TYPE_NDIM_RECV, IERR_MPI )
   RETURN

  END SUBROUTINE sendrecv_int
#endif

   !!======================================================================

  SUBROUTINE sendrecv_rsh(dest,size_give,give,size_get,get)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sendrecv_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   INTEGER                            ,INTENT(inout) :: dest,size_give,size_get
   REAL(kind=rsh),DIMENSION(size_give),INTENT(inout) :: GIVE
   REAL(kind=rsh),DIMENSION(size_get) ,INTENT(inout) :: GET

   !! * Local declarations
   INTEGER :: send_tag,recv_tag,NDIM_SEND,NDIM_RECV
   INTEGER :: stat(MPI_STATUS_SIZE)
   INTEGER :: TYPE_NDIM_SEND,TYPE_NDIM_RECV

   !!----------------------------------------------------------------------
   !! * Executable part

   NDIM_SEND=1
   NDIM_RECV=1
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_REAL,TYPE_NDIM_SEND, IERR_MPI )
     CALL MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_REAL,TYPE_NDIM_RECV, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM_SEND,MPI_DOUBLE_PRECISION,TYPE_NDIM_SEND, IERR_MPI )
     CALL MPI_TYPE_CONTIGUOUS(NDIM_RECV,MPI_DOUBLE_PRECISION,TYPE_NDIM_RECV, IERR_MPI )
   ELSE
      WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
      WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM_RECV, IERR_MPI )

   send_tag=300*iproc+dest
   recv_tag=300*dest+iproc
   CALL MPI_SENDRECV(GIVE,SIZE_GIVE,TYPE_NDIM_SEND,dest,send_tag &
     &,GET,SIZE_GET,TYPE_NDIM_RECV,dest,recv_tag &
     &,MPI_COMM_MARS,stat,IERR_MPI)
    IF (IERR_MPI.ne.0)  WRITE(impierror,*)'you have error in sendrecv_rsh',dest,ierr_mpi,stat
   CALL MPI_TYPE_FREE(TYPE_NDIM_SEND, IERR_MPI )
   CALL MPI_TYPE_FREE(TYPE_NDIM_RECV, IERR_MPI )
   RETURN

  END SUBROUTINE sendrecv_rsh

  !!======================================================================

#if defined key_trajec2d || defined key_trajec3d || defined key_ibm

  SUBROUTINE ex_traj(D_give,U_give,R_give,L_give)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_j_traj  ***
   !&E
   !&E ** Purpose :
   !&E
   !&E ** Description : Exchanges for the trajectories
   !&E
   !&E 1. when part is allocated and initialised,
   !&E    part(n,m)%inoutflag must be initialised as 'in' for limin-limax
   !&E    ljmin-ljmax region
   !&E    part(n,m)%limitbye must be initliased as 0
   !&E 2. in the calculation for part(n,m), all the calculation must be done with
   !&E    if(part(n,m)%inoutflag) THEN
   !&E      toto
   !&E    endif
   !&E 3. when part got calculated from step (once all the calculation is finished)
   !&E    all the value which is part(n,m)%inoutflag=.true. and part(n,m)%lpos ipos
   !&E    went out of limin-limax,ljmin-ljmax region,
   !&E    keep inoutflag as true, but set the value 'limitbye' to 1-8 depending
   !&E    on where it is.  1 is sw, 2=s, 3=se, 4=e, 5=ne,6=n,7=nw,8=west
   !&E    during this calculation, count the values D_give U_give R_give L_give which corresponds to
   !&E    limitbye 1,2,3 -> D_give
   !&E    limitbye 5,6,7 -> U_give
   !&E    limitbye 4 -> R_give
   !&E    limitbye 8 -> L_give
   !&E    then just do
   !&E    call ex_traj(d_give,u_give,r_give,L,give
   !&E
   !&E    subroutin ex_traj will
   !&E 1. put back the part(n,m)%inoutflag true ->false
   !&E 2. put back the part(n,m)%limitbye to 0
   !&E 3. move the particle to the right mpi process
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : ljmax,ljmin,limin,limax
   USE comtraj, ONLY : base_3d_part,part,nb_part,nb_patch, &
                       nbvartraj_log,nbvartraj_int,nbvartraj_rlg,nbvartraj_rsh
   use mpi
   IMPLICIT NONE

   !! * Arguments
   INTEGER                ,INTENT(inout)    :: D_give,U_give
   INTEGER                ,INTENT(inout)    :: R_give,L_give
   !!
   !! * Local declarations
   INTEGER,DIMENSION(1):: INT_GIVE
   INTEGER,DIMENSION(1)  :: INT_GET
   INTEGER                                     :: D_get,U_get
   INTEGER                                     :: R_get,L_get
   INTEGER                                     :: I_GIVE,I_GET,INDEX_GIVE,INDEX_GET
   INTEGER                                     :: GIVE_PROC,GET_PROC
   INTEGER                                     :: ierr,Apart_size,i,n,m
   INTEGER                                     :: size_give,size_get
   INTEGER                                     :: TYPE_NDIM,mpisize,ndim_part
   INTEGER                                     :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   TYPE(base_3d_part),ALLOCATABLE,DIMENSION(:) :: GET,GIVE
   INTEGER,allocatable,DIMENSION(:)  :: GET_N,GIVE_N,GET_M,GIVE_M

   !!----------------------------------------------------------------------
   !! * Executable part

   ! set up TYPE_NDIM
   !CALL MPI_SIZEOF(part(1,1),part_size,ierr
   ! MPI_BYTE
   !CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   ! automatic setting not yet done. this is manual setting, thus eachtime
   ! part definition changes this have to be changed
   ! we will see later to optimise this
   ndim_part=0
   ! logical
   call MPI_sizeof (part(1,1)%inoutflag, mpisize, ierr)
   ndim_part=ndim_part+mpisize*nbvartraj_log
   ! integer
   call MPI_sizeof (part(1,1)%limitbye, mpisize, ierr)
   ndim_part=ndim_part+mpisize*nbvartraj_int !limitbye, itypevert
   !rlg
   call MPI_sizeof (part(1,1)%date_orig, mpisize, ierr)
   ndim_part=ndim_part+mpisize*nbvartraj_rlg !date_orig
   ! rsh
   call MPI_sizeof (part(1,1)%xpos, mpisize, ierr)
   ndim_part=ndim_part+mpisize*nbvartraj_rsh !xpos etc rsh

   ndim_part=ndim_part-mod(ndim_part,8)+8   ! good job Martin !
   CALL MPI_TYPE_CONTIGUOUS(ndim_part,MPI_BYTE,TYPE_NDIM, IERR )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR )

   !up and down exchange part!!!!!
   ! exchange the info for how many data will move
   size_give=1
   size_get=1
   IF (myj_dproc.NE.MPI_PROC_NULL) THEN
     INT_GIVE(1)=D_give
     call sendrecv_int(myj_dproc,size_give,int_give,size_get,int_get)
     D_get=int_get(1)
   ELSE
     D_get=0
   END IF
   IF (myj_uproc.NE.MPI_PROC_NULL) THEN
     INT_GIVE(1)=u_give
     call sendrecv_int(myj_uproc,size_give,int_give,size_get,int_get)
     u_get=int_get(1)
   ELSE
     U_get=0
   END IF

   !Here we send values limitbye=1,2,3 to  8 0 4
   IF ((d_give.eq.0).and.(u_get.eq.0)) goto 100
   I_GIVE=D_GIVE; I_GET=U_GET; INDEX_GIVE=MAX(0,ljmin-1); INDEX_GET=ljmax
   ALLOCATE(GET(I_get),GIVE(I_give),GET_N(I_get),GIVE_N(I_give),GET_M(I_get),GIVE_M(I_give))
   IF (I_get .eq.0) THEN
     GET_PROC=MPI_PROC_NULL
   ELSE
     GET_PROC=myj_UPROC
   END IF
   IF (I_give.eq.0) THEN
     GIVE_PROC=MPI_PROC_NULL
   ELSE
     GIVE_PROC=myj_DPROC
   END IF
   !down
   IF (I_give.ne.0) THEN
     !make packet  limitbye=1,2,3
     i=0
     outer0:do n=1,nb_patch
     DO m=1,nb_part(n)
       IF ((part(n,m)%limitbye.ge.1).and.(part(n,m)%limitbye.le.3)) THEN
         i=i+1
         GIVE_n(i)=n
         GIVE_m(i)=m
         GIVE(i)=part(n,m)
         IF (.not.part(n,m)%inoutflag) stop !error, this value must be true here
         part(n,m)%inoutflag=.FALSE.
         part(n,m)%limitbye=0
         IF (i.eq.I_give) exit outer0
       END IF
     END DO
   END DO outer0
   END IF
   ! here 3 send-recv should be packed one day when i have time
   !sending indecees N
   !print*,'bUDgive100',iproc,I_GIVE,GIVE_PROC,INDEX_GIVE
   !print*,'bUDget100',iproc,I_GET,GET_PROC,INDEX_GET
   CALL MPI_SENDRECV(GIVE_N, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_N, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'aUD100',iproc
   !sending indecees M
   CALL MPI_SENDRECV(GIVE_M, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_M, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abUD100',iproc
   !now send packet
   CALL MPI_SENDRECV(GIVE, I_give, TYPE_NDIM, GIVE_PROC,  INDEX_GIVE,&
                     GET, I_get, TYPE_NDIM, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abcUD100',iproc
   IF (I_get.ne.0) THEN
     DO i=1,I_get
       !unpack packet
       n=GET_n(i); m=GET_m(i); part(n,m)=GET(i)
       !update R_give and L_give
       IF (.not.part(n,m)%inoutflag) stop !error
       IF (part(n,m)%limitbye.eq.1) THEN
         part(n,m)%limitbye=8
         L_give=L_give+1
       ELSE IF(part(n,m)%limitbye.eq.2) THEN
         part(n,m)%limitbye=0
       ELSE IF (part(n,m)%limitbye.eq.3) THEN
         part(n,m)%limitbye=4
         R_give=R_give+1
       ELSE
         stop  !this value should not exist here!!
       END IF
     END DO
   END IF
   DEALLOCATE(GIVE,GIVE_N,GIVE_M,GET,GET_N,GET_M)
   100 continue

   !up   5 6 7  will be 4 0 8

   IF ((u_give.eq.0).and.(d_get.eq.0)) goto 200
   I_GIVE=U_GIVE; I_GET=D_GET; INDEX_GIVE=ljmax+1; INDEX_GET=ljmin
   ALLOCATE(GET(I_get),GIVE(I_give),GET_N(I_get),GIVE_N(I_give),GET_M(I_get),GIVE_M(I_give))
   IF (I_get .eq.0) THEN
     GET_PROC=MPI_PROC_NULL
   ELSE
     GET_PROC=myj_DPROC
   END IF
   IF (I_give.eq.0) THEN
     GIVE_PROC=MPI_PROC_NULL
   ELSE
     GIVE_PROC=myj_UPROC
   END IF
   !down
   IF (I_give.ne.0) THEN
   !make packet  limitbye=1,2,3
     i=0
!        outer2:do m=1,nbmax_part
!        DO n=1,nb_patch
     outer2:do n=1,nb_patch
       DO m=1,nb_part(n)
         IF ((part(n,m)%limitbye.ge.5).and.(part(n,m)%limitbye.le.7)) THEN
           i=i+1
           GIVE_n(i)=n
           GIVE_m(i)=m
           GIVE(i)=part(n,m)
           IF (.not.part(n,m)%inoutflag) stop !error, this value must be true here
           part(n,m)%inoutflag=.FALSE.
           part(n,m)%limitbye=0
           IF (i.eq.I_give) exit outer2
         END IF
       END DO
     END DO outer2
   END IF
   ! here 3 send-recv should be packed one day when i have time
   !sending indices N
   !print*,'bUDgive200',iproc,I_GIVE,GIVE_PROC,INDEX_GIVE
   !print*,'bUDget200',iproc,I_GET,GET_PROC,INDEX_GET
   CALL MPI_SENDRECV(GIVE_N, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_N, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'aUD200',iproc
        !sending indecees M
   CALL MPI_SENDRECV(GIVE_M, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_M, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abUD200',iproc
        !now send packet
   CALL MPI_SENDRECV(GIVE, I_GIVE, TYPE_NDIM, GIVE_PROC,  INDEX_GIVE,&
                     GET, I_GET, TYPE_NDIM, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)

   !print*,'abcUD200',iproc
   IF (I_get.ne.0) THEN
     DO i=1,I_get
       !unpack packet
       n=GET_n(i); m=GET_m(i); part(n,m)=GET(i)
       !update R_give and L_give
       IF (.not.part(n,m)%inoutflag) stop !error
       IF (part(n,m)%limitbye.eq.7) THEN
         part(n,m)%limitbye=8
         L_give=L_give+1
       ELSE IF(part(n,m)%limitbye.eq.6) THEN
         part(n,m)%limitbye=0
       ELSE IF (part(n,m)%limitbye.eq.5) THEN
         part(n,m)%limitbye=4
         R_give=R_give+1
       ELSE
         stop  !this value should not exist here!!
       END IF
     END DO
   END IF
   DEALLOCATE(GIVE,GIVE_N,GIVE_M,GET,GET_N,GET_M)
   200 continue
   !!!!right and left exchange part!!!!!
   ! down corresponds to Left (=west) up corresponds to right(=east)
   ! exchange the info for how many data will move
   IF (myi_dproc.NE.MPI_PROC_NULL) THEN
     INT_GIVE(1)=L_give
     call sendrecv_int(myi_dproc,size_give,int_give,size_get,int_get)
     L_get=int_get(1)
   ELSE
     L_get=0
   END IF
   IF (myi_uproc.NE.MPI_PROC_NULL) THEN
     INT_GIVE(1)=R_give
     call sendrecv_int(myi_uproc,size_give,int_give,size_get,int_get)
     R_get=int_get(1)
   ELSE
     R_get=0
   END IF

   !Here we send values limitbye=8 to   0
   IF ((l_give.eq.0).and.(r_get.eq.0)) goto 300
   I_GIVE=l_GIVE; I_GET=r_GET; INDEX_GIVE=MAX(0,limin-1); INDEX_GET=limax
   ALLOCATE(GET(I_get),GIVE(I_give),GET_N(I_get),GIVE_N(I_give),GET_M(I_get),GIVE_M(I_give))
   IF (I_get .eq.0) THEN
     GET_PROC=MPI_PROC_NULL
   ELSE
     GET_PROC=myi_UPROC
   END IF
   IF (I_give.eq.0) THEN
     GIVE_PROC=MPI_PROC_NULL
   ELSE
     GIVE_PROC=myi_DPROC
   END IF
   !down
   IF (I_give.ne.0) THEN
     !make packet  limitbye=8
     i=0
!        outer3:do m=1,nbmax_part
!        DO n=1,nb_patch
     outer3:do n=1,nb_patch
       DO m=1,nb_part(n)
         IF (part(n,m)%limitbye.eq.8) THEN
           i=i+1
           GIVE_n(i)=n
           GIVE_m(i)=m
           GIVE(i)=part(n,m)
           IF (.not.part(n,m)%inoutflag) stop !error, this value must be true here
           part(n,m)%inoutflag=.FALSE.
           part(n,m)%limitbye=0
           IF (i.eq.I_give) exit outer3
         END IF
       END DO
     END DO outer3
   END IF
   ! here 3 send-recv should be packed one day when i have time
   !sending indecees N
   !print*,'bUDgive300',iproc,I_GIVE,GIVE_PROC,INDEX_GIVE
   !print*,'bUDget300',iproc,I_GET,GET_PROC,INDEX_GET
   CALL MPI_SENDRECV(GIVE_N, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_N, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'aUD300',iproc
   !sending indecees M
   CALL MPI_SENDRECV(GIVE_M, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_M, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abUD300',iproc
   !now send packet
   CALL MPI_SENDRECV(GIVE, I_give, TYPE_NDIM, GIVE_PROC,  INDEX_GIVE,&
                     GET, I_get, TYPE_NDIM, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abcUD300',iproc
   IF (I_get.ne.0) THEN
     DO i=1,I_get
       !unpack packet
       n=GET_n(i); m=GET_m(i); part(n,m)=GET(i)
       !update R_give and L_give
       IF (.not.part(n,m)%inoutflag) stop !error
       IF (part(n,m)%limitbye.ne.8)stop !error
       part(n,m)%limitbye=0
     END DO
   END IF
   DEALLOCATE(GIVE,GIVE_N,GIVE_M,GET,GET_N,GET_M)
   300 continue

   !Here we send values limitbye=4 to   0
   IF ((r_give.eq.0).and.(l_get.eq.0)) goto 400
   I_GIVE=r_GIVE; I_GET=l_GET; INDEX_GIVE=limax+1; INDEX_GET=limin
   ALLOCATE(GET(I_get),GIVE(I_give),GET_N(I_get),GIVE_N(I_give),GET_M(I_get),GIVE_M(I_give))
   IF (I_get .eq.0) THEN
     GET_PROC=MPI_PROC_NULL
   ELSE
     GET_PROC=myi_DPROC
   END IF
   IF (I_give.eq.0) THEN
     GIVE_PROC=MPI_PROC_NULL
   ELSE
     GIVE_PROC=myi_UPROC
   END IF
   !down
   IF (I_give.ne.0) THEN
     !make packet  limitbye=4
     i=0
!        outer4:do m=1,nbmax_part
!        DO n=1,nb_patch
     outer4:do n=1,nb_patch
       DO m=1,nb_part(n)
         IF (part(n,m)%limitbye.eq.4) THEN
           i=i+1
           GIVE_n(i)=n
           GIVE_m(i)=m
           GIVE(i)=part(n,m)
           IF (.not.part(n,m)%inoutflag) stop !error, this value must be true here
           part(n,m)%inoutflag=.FALSE.
           part(n,m)%limitbye=0
           IF (i.eq.I_give) exit outer4
         END IF
       END DO
     END DO outer4
   END IF
   ! here 3 send-recv should be packed one day when i have time
   !sending indecees N
   !print*,'bUDgive400',iproc,I_GIVE,GIVE_PROC,INDEX_GIVE
   !print*,'bUDget400',iproc,I_GET,GET_PROC,INDEX_GET
   CALL MPI_SENDRECV(GIVE_N, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_N, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'aUD400',iproc
   !sending indecees M
   CALL MPI_SENDRECV(GIVE_M, I_give, MPI_INTEGER, GIVE_PROC,  INDEX_GIVE,&
                     GET_M, I_get, MPI_INTEGER, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abUD400',iproc
   !now send packet
   CALL MPI_SENDRECV(GIVE, I_give, TYPE_NDIM, GIVE_PROC,  INDEX_GIVE,&
                     GET, I_get, TYPE_NDIM, GET_PROC, INDEX_GET ,&
                     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   !print*,'abcUD400',iproc
   IF (I_get.ne.0) THEN
     DO i=1,I_get
       !unpack packet
       n=GET_n(i); m=GET_m(i); part(n,m)=GET(i)
       !update R_give and L_give
       IF (.not.part(n,m)%inoutflag) stop !error
       IF (part(n,m)%limitbye.ne.4)stop !error
       part(n,m)%limitbye=0
     END DO
   END IF
   DEALLOCATE(GIVE,GIVE_N,GIVE_M,GET,GET_N,GET_M)
   400 continue

   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )

   RETURN

  END SUBROUTINE ex_traj

#endif


   !!======================================================================


  SUBROUTINE ex_j_rsh(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_j_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for j-1 j-2 ,,,
   !&E                   y is to exchange up    (for j+1 j+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rsh),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: x,aisize
   INTEGER                                     :: TYPE_NDIM
   INTEGER                                     :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   aisize=aimax-aimin+1
   IF (xx.GT.0) THEN  !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

   x=-(xx)   !make xx plus

   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*) ' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*) 'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,aimin:aimax,x))
   ALLOCATE(DS(NDIM,aimin:aimax,x))
   ALLOCATE(UR(NDIM,aimin:aimax,y))
   ALLOCATE(US(NDIM,aimin:aimax,y))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myj_uproc.NE.MPI_PROC_NULL) DS=A(:,:,ljmax-x+1:ljmax)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myj_UPROC,  ljmax,&
       DR, x, TYPE_NDIM, myj_DPROC, (ljmin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myj_dproc.NE.MPI_PROC_NULL)A(:,:,ljmin-x:ljmin-1)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myj_dproc.NE.MPI_PROC_NULL)US=A(:,:,ljmin:ljmin+y-1)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myj_DPROC,  ljmin,&
       UR, y, TYPE_NDIM, myj_UPROC, (ljmax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myj_uproc.NE.MPI_PROC_NULL)A(:,:,ljmax+1:ljmax+y)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_j_rsh

   !!======================================================================

  SUBROUTINE ex_i_rsh(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_i_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for i-1 i-2 ,,,
   !&E                   y is to exchange up    (for i+1 i+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,limin,limax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rsh),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: x,ajsize
   INTEGER                                     :: TYPE_NDIM
   INTEGER                                     :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   ajsize=ajmax-ajmin+1
   IF (xx.GT.0) THEN !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

#ifdef key_later_test_on_bounds
   IF ((ajmin.NE.ljmin).OR.(ajmax.NE.ljmax)) THEN
     WRITE(*,*) 'in ex_i_rsh, aimin.ne.ljmin.or.aimax.ne.ljmax'
     WRITE(*,*) 'which is strange...not as tina planned to exchange'
     WRITE(*,*) ' '
     CALL MPI_FINALIZE(IERR_MPI) ; STOP  !xx must be - or 0
   END IF
#endif

   x=-(xx)   !make xx plus

   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,x,ajmin:ajmax))
   ALLOCATE(DS(NDIM,x,ajmin:ajmax))
   ALLOCATE(UR(NDIM,y,ajmin:ajmax))
   ALLOCATE(US(NDIM,y,ajmin:ajmax))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myi_uproc.NE.MPI_PROC_NULL) DS=A(:,limax-x+1:limax,:)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myi_UPROC,  limax,&
       DR, x, TYPE_NDIM, myi_DPROC, (limin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myi_dproc.NE.MPI_PROC_NULL)A(:,limin-x:limin-1,:)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myi_dproc.NE.MPI_PROC_NULL)US=A(:,limin:limin+y-1,:)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myi_DPROC,  limin,&
       UR, y, TYPE_NDIM, myi_UPROC, (limax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myi_uproc.NE.MPI_PROC_NULL)A(:,limax+1:limax+y,:)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_i_rsh

   !!======================================================================

  SUBROUTINE ex_i_rlg(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_i_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for i-1 i-2 ,,,
   !&E                   y is to exchange up    (for i+1 i+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,limin,limax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rlg),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: x,ajsize
   INTEGER                                     :: TYPE_NDIM
   INTEGER                                     :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   ajsize=ajmax-ajmin+1
   IF (xx.GT.0) THEN !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

#ifdef key_later_test_on_bounds
   IF ((ajmin.NE.ljmin).OR.(ajmax.NE.ljmax)) THEN
     WRITE(*,*) 'in ex_i_rlg, aimin.ne.ljmin.or.aimax.ne.ljmax'
     WRITE(*,*) 'which is strange...not as tina planned to exchange'
     WRITE(*,*) ' '
     CALL MPI_FINALIZE(IERR_MPI) ; STOP  !xx must be - or 0
   END IF
#endif

   x=-(xx)   !make xx plus

   IF (rlg.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rlg.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rlg=4 and 8'
     WRITE(impierror,*)'you gave rlg=',rlg
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,x,ajmin:ajmax))
   ALLOCATE(DS(NDIM,x,ajmin:ajmax))
   ALLOCATE(UR(NDIM,y,ajmin:ajmax))
   ALLOCATE(US(NDIM,y,ajmin:ajmax))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myi_uproc.NE.MPI_PROC_NULL) DS=A(:,limax-x+1:limax,:)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myi_UPROC,  limax,&
       DR, x, TYPE_NDIM, myi_DPROC, (limin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myi_dproc.NE.MPI_PROC_NULL)A(:,limin-x:limin-1,:)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myi_dproc.NE.MPI_PROC_NULL)US=A(:,limin:limin+y-1,:)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myi_DPROC,  limin,&
       UR, y, TYPE_NDIM, myi_UPROC, (limax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myi_uproc.NE.MPI_PROC_NULL)A(:,limax+1:limax+y,:)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_i_rlg

   !!======================================================================

  SUBROUTINE ex_i_int(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_i_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for i-1 i-2 ,,,
   !&E                   y is to exchange up    (for i+1 i+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : limin,limax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                        ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   INTEGER,DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                              :: x,ajsize
   INTEGER                              :: TYPE_NDIM
   INTEGER                              :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   INTEGER,ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   ajsize=ajmax-ajmin+1
   IF (xx.GT.0) THEN !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

#ifdef key_later_test_on_bounds
   IF ((ajmin.NE.ljmin).OR.(ajmax.NE.ljmax)) THEN
     WRITE(*,*) 'in ex_i_int, aimin.ne.ljmin.or.aimax.ne.ljmax'
     WRITE(*,*) 'which is strange...not as tina planned to exchange'
     WRITE(*,*) ' '
     CALL MPI_FINALIZE(IERR_MPI) ; STOP  !xx must be - or 0
   END IF
#endif

   x=-(xx)   !make xx plus

   CALL MPI_TYPE_CONTIGUOUS(ajsize*NDIM,MPI_INTEGER,TYPE_NDIM, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,x,ajmin:ajmax))
   ALLOCATE(DS(NDIM,x,ajmin:ajmax))
   ALLOCATE(UR(NDIM,y,ajmin:ajmax))
   ALLOCATE(US(NDIM,y,ajmin:ajmax))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myi_uproc.NE.MPI_PROC_NULL) DS=A(:,limax-x+1:limax,:)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myi_UPROC,  limax,&
       DR, x, TYPE_NDIM, myi_DPROC, (limin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myi_dproc.NE.MPI_PROC_NULL)A(:,limin-x:limin-1,:)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myi_dproc.NE.MPI_PROC_NULL)US=A(:,limin:limin+y-1,:)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myi_DPROC,  limin,&
       UR, y, TYPE_NDIM, myi_UPROC, (limax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myi_uproc.NE.MPI_PROC_NULL)A(:,limax+1:limax+y,:)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_i_int

   !!======================================================================

  SUBROUTINE ex_j_rlg(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_j_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for j-1 j-2 ,,,
   !&E                   y is to exchange up    (for j+1 j+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,ljmin,ljmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rlg),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: x,aisize
   INTEGER                                     :: TYPE_NDIM
   INTEGER                                     :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   aisize=aimax-aimin+1
   IF (xx.GT.0) THEN  !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

   x=-(xx)   !make xx plus

   IF (rlg.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rlg.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rlg=4 and 8'
     WRITE(impierror,*)'you gave rlg=',rlg
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,aimin:aimax,x))
   ALLOCATE(DS(NDIM,aimin:aimax,x))
   ALLOCATE(UR(NDIM,aimin:aimax,y))
   ALLOCATE(US(NDIM,aimin:aimax,y))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myj_uproc.NE.MPI_PROC_NULL) DS=A(:,:,ljmax-x+1:ljmax)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myj_UPROC,  ljmax,&
       DR, x, TYPE_NDIM, myj_DPROC, (ljmin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myj_dproc.NE.MPI_PROC_NULL)A(:,:,ljmin-x:ljmin-1)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myj_dproc.NE.MPI_PROC_NULL)US=A(:,:,ljmin:ljmin+y-1)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myj_DPROC,  ljmin,&
       UR, y, TYPE_NDIM, myj_UPROC, (ljmax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myj_uproc.NE.MPI_PROC_NULL)A(:,:,ljmax+1:ljmax+y)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_j_rlg

   !!======================================================================

  SUBROUTINE ex_j_int(xx,y,NDIM,aimin,aimax,ajmin,ajmax,A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_j_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange down  (for j-1 j-2 ,,,
   !&E                   y is to exchange up    (for j+1 j+2 ..
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : ljmin,ljmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                        ,INTENT(in)    :: xx,y,ndim,aimin,aimax,ajmin,ajmax
   INTEGER,DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                              :: x,aisize
   INTEGER                              :: TYPE_NDIM
   INTEGER                              :: STATUS1(MPI_STATUS_SIZE),STATUS2(MPI_STATUS_SIZE)
   INTEGER,ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS,UR,US

   !!----------------------------------------------------------------------
   !! * Executable part

   aisize=aimax-aimin+1
   IF (xx.GT.0) THEN !xx must be - or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF (y.LT.0) THEN  !y must be + or 0
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF

   x=-(xx)   !make xx plus

   CALL MPI_TYPE_CONTIGUOUS(aisize*NDIM,MPI_INTEGER,TYPE_NDIM, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

   ALLOCATE(DR(NDIM,aimin:aimax,x))
   ALLOCATE(DS(NDIM,aimin:aimax,x))
   ALLOCATE(UR(NDIM,aimin:aimax,y))
   ALLOCATE(US(NDIM,aimin:aimax,y))
   !down x(m)  
   IF (x.NE.0) THEN
     IF (myj_uproc.NE.MPI_PROC_NULL) DS=A(:,:,ljmax-x+1:ljmax)
     CALL MPI_SENDRECV(DS, x, TYPE_NDIM, myj_UPROC,  ljmax,&
       DR, x, TYPE_NDIM, myj_DPROC, (ljmin-1),&
       & MPI_COMM_MARS , STATUS1, IERR_MPI)
     IF (myj_dproc.NE.MPI_PROC_NULL)A(:,:,ljmin-x:ljmin-1)=DR
   END IF

   !up   y(p) 
   IF (y.NE.0) THEN
     IF (myj_dproc.NE.MPI_PROC_NULL)US=A(:,:,ljmin:ljmin+y-1)
     CALL MPI_SENDRECV(US, y, TYPE_NDIM, myj_DPROC,  ljmin,&
       UR, y, TYPE_NDIM, myj_UPROC, (ljmax+1),&
       & MPI_COMM_MARS , STATUS2, IERR_MPI)
     IF (myj_uproc.NE.MPI_PROC_NULL)A(:,:,ljmax+1:ljmax+y)=UR
   END IF

   DEALLOCATE(DR,DS,UR,US)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_j_int

   !!======================================================================

  SUBROUTINE ex_peri_y_rsh(xx,NDIM,aimin,aimax,ajmin,ajmax,A) 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_peri_y_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange 1 or 2
   !&E **             : ajmin==ljmin, ajmax==ljmax
   !&E **             : aimin and aimax is equal to the limit where each node holds data.
   !&E **             : (take it a bit bigger just in case)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,jmin,jmax,limin,limax
   USE comvars2d, ONLY : jb,jh
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in) :: xx,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rsh),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                              :: x,y,aisize,stag,rtag,uproc,jn,js
   INTEGER                              :: TYPE_NDIM,STATUS1(MPI_STATUS_SIZE)
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS

   !!----------------------------------------------------------------------
   !! * Executable part

   jn=jh((limax-limin)/2)
   js=jb((limax-limin)/2)
   IF ((xx < 1).AND.(xx > 3)) THEN
     WRITE(*,*)'you can use only 1,2 or 3 as xx!!'
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   IF ((ljmin.NE.jmin).AND.(ljmax.NE.jmax)) RETURN
   IF ((ljmin.EQ.jmin).AND.(ljmax.EQ.jmax)) THEN
     IF (xx >= 2) A(:,:,js-1)=A(:,:,jn-2)
     A(:,:,js)=A(:,:,jn-1)
     A(:,:,jn)=A(:,:,js+1)
     IF (xx == 3) THEN
       A(:,:,js-2)=A(:,:,jn-3)
       A(:,:,jn+1)=A(:,:,js+2)
       A(:,:,jn+2)=A(:,:,js+3)
     END IF
     RETURN
   END IF

   aisize=aimax-aimin+1

   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*aisize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   IF (myjproc==minjproc) THEN
     uproc=PROC(myiproc,maxjproc)
     stag=0+1000*myiproc
     rtag=1+1000*myiproc
     x=xx
     y=1
     IF (xx == 3) y=3
     ALLOCATE(DR(NDIM,aimin:aimax,x))
     ALLOCATE(DS(NDIM,aimin:aimax,y))
     DS(:,:,1)=A(:,:,js+1)
     IF (xx == 3) DS(:,:,1:3)=A(:,:,js+1:js+3)
   ELSE IF (myjproc==maxjproc) THEN
     uproc=PROC(myiproc,minjproc)
     stag=1+1000*myiproc
     rtag=0+1000*myiproc
     x=1
     IF (xx == 3) x=3
     y=xx
     ALLOCATE(DR(NDIM,aimin:aimax,x))
     ALLOCATE(DS(NDIM,aimin:aimax,y))
     IF (xx == 3) THEN
       DS(:,:,1:3)=A(:,:,jn-3:jn-1)
     ELSE IF (xx == 2) THEN
       DS(:,:,1:2)=A(:,:,jn-2:jn-1)
     ELSE IF (xx == 1) THEN
       DS(:,:,1)=A(:,:,jn-1)
     END IF
   END IF

   CALL MPI_SENDRECV(DS, y, TYPE_NDIM, UPROC,  stag,&
     DR, x, TYPE_NDIM, UPROC,  rtag,&
     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   DEALLOCATE(DS)

   IF (myjproc==minjproc) THEN
     IF (xx == 1) A(:,:,js)=DR(:,:,1)
     IF (xx == 2) A(:,:,js-1:js)=DR(:,:,1:2)
     IF (xx == 3) A(:,:,js-2:js)=DR(:,:,1:3)
   ELSE IF (myjproc==maxjproc) THEN
     A(:,:,jn)=DR(:,:,1)
     IF (xx == 3) A(:,:,jn:jn+2)=DR(:,:,1:3)
   END IF
   DEALLOCATE(DR)

   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_peri_y_rsh

   !!======================================================================

  SUBROUTINE ex_peri_y_avcv(aimin,aimax,ajmin,ajmax,A,C) 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_peri_y_avcv  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : copy av(jmax-1) into av(jmin+1)
   !&E **             : and cv(jmin+1) into cv(jmax-1)
   !&E **             : ajmin==ljmin, ajmax==ljmax
   !&E **             : aimin and aimax is equal to the limit where each node holds data.
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,ljmin,ljmax,jmin,jmax,limin,limax
   USE comvars2d, ONLY : jb,jh
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                          ,INTENT(in)    :: aimin,aimax,ajmin,ajmax
   REAL(kind=rlg),DIMENSION(aimin:aimax,ajmin:ajmax),INTENT(inout) :: A,C

   !! * Local declarations
   INTEGER                              :: x,y,aisize,stag,rtag,uproc,jn,js
   INTEGER                              :: TYPE_NDIM,STATUS1(MPI_STATUS_SIZE)
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:) :: DR,DS

   !!----------------------------------------------------------------------
   !! * Executable part

   jn=jh((limax-limin)/2)
   js=jb((limax-limin)/2)
   IF ((ljmin.NE.jmin).AND.(ljmax.NE.jmax)) RETURN
   IF ((ljmin.EQ.jmin).AND.(ljmax.EQ.jmax)) THEN
     A(:,js)=A(:,jn-1)
     C(:,jn-1)=C(:,js)
     RETURN
   END IF

   aisize=aimax-aimin+1

   CALL MPI_TYPE_CONTIGUOUS(aisize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   IF (myjproc==minjproc) THEN
     uproc=PROC(myiproc,maxjproc)
     stag=0+1000*myiproc
     rtag=1+1000*myiproc
     x=1
     y=1
     ALLOCATE(DR(aimin:aimax,x))
     ALLOCATE(DS(aimin:aimax,y))
     DS(:,y)=C(:,js)
   ELSE IF (myjproc==maxjproc) THEN
     uproc=PROC(myiproc,minjproc)
     stag=1+1000*myiproc
     rtag=0+1000*myiproc
     x=1
     y=1
     ALLOCATE(DR(aimin:aimax,x))
     ALLOCATE(DS(aimin:aimax,y))
     DS(:,1)=A(:,jn-1)         
   END IF

   CALL MPI_SENDRECV(DS, y, TYPE_NDIM, UPROC,  stag,&
     DR, x, TYPE_NDIM, UPROC,  rtag,&
     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   DEALLOCATE(DS)

   IF (myjproc==minjproc) THEN
     A(:,js)=DR(:,1)
   ELSE IF (myjproc==maxjproc) THEN
     C(:,jn-1)=DR(:,x)
   END IF
   DEALLOCATE(DR)

   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_peri_y_avcv

   !!======================================================================

  SUBROUTINE ex_peri_x_rsh(xx,NDIM,aimin,aimax,ajmin,ajmax,A) 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_peri_x_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : xx is to exchange 1 or 2
   !&E **             : aimin==limin, aimax==limax
   !&E **             : ajmin and ajmax is equal to the limit where each node holds data.
   !&E **             : (take it a bit bigger just in case)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,imin,imax,limin,limax,ljmin,ljmax
   USE comvars2d, ONLY : ig,id
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in) :: xx,ndim,aimin,aimax,ajmin,ajmax
   REAL(kind=rsh),DIMENSION(NDIM,aimin:aimax,ajmin:ajmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                              :: x,y,ajsize,stag,rtag,uproc,ie,iw
   INTEGER                              :: TYPE_NDIM,STATUS1(MPI_STATUS_SIZE)
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: DR,DS

   !!----------------------------------------------------------------------
   !! * Executable part

   ie=id((ljmax-ljmin)/2)
   iw=ig((ljmax-ljmin)/2)

   IF ((xx < 1).AND.(xx > 3)) THEN
     WRITE(*,*)'you can use only 1,2 or 3 as xx!!'
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   ! interior of the domain, none OBC
   IF ((limin.NE.imin).AND.(limax.NE.imax)) RETURN
   ! MPI division along horizontal stripes
   IF ((limin.EQ.imin).AND.(limax.EQ.imax)) THEN
     IF (xx >= 2)   A(:,iw-1,:)=A(:,ie-2,:)
     A(:,iw,:)=A(:,ie-1,:)
     A(:,ie,:)=A(:,iw+1,:)
     IF (xx==3) THEN
       A(:,iw-2,:)=A(:,ie-3,:)
       A(:,ie+1,:)=A(:,iw+2,:)
       A(:,ie+2,:)=A(:,iw+3,:)
     END IF
     RETURN
   END IF

   ajsize=ajmax-ajmin+1

   ! create contiguous datatype
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM*ajsize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   ! A datatype object has to be committed before it can be used in a communication
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   IF (myiproc==miniproc) THEN
     uproc=PROC(maxiproc,myjproc)
     stag=0+1000*myjproc
     rtag=1+1000*myjproc
     x=xx
     y=1
     IF (xx==3) y=3
     ALLOCATE(DR(NDIM,x,ajmin:ajmax))
     ALLOCATE(DS(NDIM,y,ajmin:ajmax))
     ! copy into send buffer
     DS(:,1,:)=A(:,iw+1,:)
     IF (xx==3) DS(:,1:3,:)=A(:,iw+1:iw+3,:)
   ELSE IF (myiproc==maxiproc) THEN
     uproc=PROC(miniproc,myjproc)
     stag=1+1000*myjproc
     rtag=0+1000*myjproc
     x=1
     IF (xx==3) x=3
     y=xx
     ALLOCATE(DR(NDIM,x,ajmin:ajmax))
     ALLOCATE(DS(NDIM,y,ajmin:ajmax))
     IF (xx == 3) THEN
       DS(:,1:3,:)=A(:,ie-3:ie-1,:)
     ELSE IF (xx == 2) THEN
       DS(:,1:2,:)=A(:,ie-2:ie-1,:)
     ELSE IF (xx == 1) THEN
       DS(:,1,:)=A(:,ie-1,:)
     END IF
     IF (xx.EQ.1)DS(:,1,:)=A(:,ie-1,:)
   END IF

   ! exchange and deallocation of the initial address of send buffer (DS)
   CALL MPI_SENDRECV(DS, y, TYPE_NDIM, UPROC,  stag,&
     DR, x, TYPE_NDIM, UPROC,  rtag,&
     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   DEALLOCATE(DS)

   ! copy of the initial address of the receive buffer (DR) into the common variable (A)
   IF (myiproc==miniproc) THEN
     IF (xx == 1) A(:,iw,:)=DR(:,1,:)
     IF (xx == 2) A(:,iw-1:iw,:)=DR(:,1:2,:)
     IF (xx == 3) A(:,iw-2:iw,:)=DR(:,1:3,:)
   ELSE IF (myiproc==maxiproc) THEN
     A(:,ie,:)=DR(:,1,:)
     IF (xx == 3) A(:,ie:ie+2,:)=DR(:,1:3,:)
   END IF
   DEALLOCATE(DR)

   ! free the data type
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_peri_x_rsh

   !!======================================================================

  SUBROUTINE ex_peri_x_avcv(aimin,aimax,ajmin,ajmax,A,C) 

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ex_peri_x_avcv  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description : copy av(imax-1) into av(imin+1)
   !&E **             : and cv(imin+1) into cv(imax-1)
   !&E **             : aimin==limin, aimax==limax
   !&E **             : ajmin and ajmax is equal to the limit where each node holds data.
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,imin,imax,limin,limax,ljmin,ljmax
   USE comvars2d, ONLY : ig,id
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                          ,INTENT(in)    :: aimin,aimax,ajmin,ajmax
   REAL(kind=rlg),DIMENSION(aimin:aimax,ajmin:ajmax),INTENT(inout) :: A,C

   !! * Local declarations
   INTEGER                              :: x,y,ajsize,stag,rtag,uproc,iw,ie
   INTEGER                              :: TYPE_NDIM,STATUS1(MPI_STATUS_SIZE)
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:) :: DR,DS

   !!----------------------------------------------------------------------
   !! * Executable part

   ie=id((ljmax-ljmin)/2)
   iw=ig((ljmax-ljmin)/2)

   ! interior of the domain, none OBC
   IF ((limin.NE.imin).AND.(limax.NE.imax)) RETURN
   ! MPI division along horizontal stripes
   IF ((limin.EQ.imin).AND.(limax.EQ.imax)) THEN
     A(iw,:)=A(ie-1,:)
     C(ie-1,:)=C(iw,:)
     RETURN
   END IF

   ajsize=ajmax-ajmin+1

   ! create contiguous datatype
   CALL MPI_TYPE_CONTIGUOUS(ajsize,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ! A datatype object has to be committed before it can be used in a communication
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   IF (myiproc==miniproc) THEN
     uproc=PROC(maxiproc,myjproc)
     stag=0+1000*myjproc
     rtag=1+1000*myjproc
     x=1
     y=1
     ALLOCATE(DR(x,ajmin:ajmax))
     ALLOCATE(DS(y,ajmin:ajmax))
     ! copy into send buffer
     DS(y,:)=C(iw,:)
   ELSE IF (myiproc==maxiproc) THEN
     uproc=PROC(miniproc,myjproc)
     stag=1+1000*myjproc
     rtag=0+1000*myjproc
     x=1
     y=1
     ALLOCATE(DR(x,ajmin:ajmax))
     ALLOCATE(DS(y,ajmin:ajmax))
     DS(1,:)=A(ie-1,:)
   END IF

   ! exchange and deallocation of the initial address of send buffer (DS)
   CALL MPI_SENDRECV(DS, y, TYPE_NDIM, UPROC,  stag,&
     DR, x, TYPE_NDIM, UPROC,  rtag,&
     & MPI_COMM_MARS , STATUS1, IERR_MPI)
   DEALLOCATE(DS)

   ! copy of the initial address of the receive buffer (DR) into the common variables (A and C)
   IF (myiproc==miniproc) THEN
     A(iw,:)=DR(1,:)
   ELSE IF (myiproc==maxiproc) THEN
     C(ie-1,:)=DR(x,:)
   END IF
   DEALLOCATE(DR)

   ! free the data type
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
   RETURN

  END SUBROUTINE ex_peri_x_avcv

   !!======================================================================

  SUBROUTINE correct_rsh(A,B,NDIM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE correct_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A (, B ?)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax,jmin,jmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                               ,INTENT(in)    :: ndim
   REAL(kind=rsh),DIMENSION(NDIM,imin:imax,jmin:jmax)    ,INTENT(inout) :: A
   REAL(kind=rsh),DIMENSION(NDIM,limin:limax,ljmin:ljmax),INTENT(in)    :: B
   !REAL(kind=rsh),DIMENSION(NDIM,limin:limax,ljmin:ljmax)               :: F

   !! * Local declarations
   INTEGER                                     :: x
   INTEGER                                     :: TYPE_NDIM
   INTEGER                                     :: count_send,count_get,dest,etigo,n
   INTEGER                                     :: stat(MPI_STATUS_SIZE)
   INTEGER,DIMENSION(nprocs)                   :: recvcounts,displs
   INTEGER                                     :: nsize,i,j,nij,sendcount,iroot
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: C,D
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:) :: E
   REAL(kind=rsh)                              :: toto

   !!----------------------------------------------------------------------
   !! * Executable part


   !WRITE(impierror,*)   "i am in correct_rsh"
   IF (nbblank.NE.0) THEN 
   !  WRITE(impierror,*)   "i am in nbblanc.ne0 correct_rsh"

     IF (rsh.EQ.4) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
     ELSE IF (rsh.EQ.8) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
     ELSE
       WRITE(impierror,*)' this version of correct works only for rsh=4 and 8'
       WRITE(impierror,*)'you gave rsh=',rsh
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
     nsize=SUM(proc_size(minp:maxp))
     !tinatestWRITE(impierror,*)   "nsize",nsize
     ALLOCATE(C(NDIM,LI(3,myiproc),LJ(3,myjproc)))
     
     !  DO j=ljmin,ljmax
     !     DO i=limin,limax
     !      F(1,i,j)=dble(1000*i+j)
     !    END DO
     !  ENDDO
     !C(:,:,:)=F(:,limin:limax,ljmin:ljmax)  !data that i hold, which will be send to everyone
     C(:,:,:)=B(:,limin:limax,ljmin:ljmax)  !data that i hold, which will be send to everyone
     ALLOCATE(E(NDIM,nsize))                ! data box that i keep all the data from everyone
     sendcount=proc_size(iproc)
     recvcounts(1:nprocs)=proc_size(minp:maxp)
     displs(1:nprocs)=proc_displs(minp:maxp)
      !WRITE(impierror,*)   "just before gatherv"
      !WRITE(impierror,*)   "jusmaster",master,iproc
     iroot=0 !here 0 means master
     CALL MPI_GATHERV(C,sendcount , TYPE_NDIM, &
       & E, recvcounts, displs,     TYPE_NDIM, iroot, & 
       &MPI_COMM_MARS, IERR_MPI)
     !WRITE(impierror,*)   "just after gatherv",ierr_mpi
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     !WRITE(impierror,*)   "just before mpi_barrier"
     !call MPI_BARRIER(MPI_COMM_MARS,IERR_MPI)
     !WRITE(impierror,*)   "just after mpi_barrier"
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     DEALLOCATE(c)
     nij=0
     DO n=minp,maxp
       !IF (n==iproc)CYCLE
       !nij=proc_displs(N)
       ! here i can also use function reshape
       DO j=proc_ljmin(n),proc_ljmax(n)
         DO i=proc_limin(n),proc_limax(n)
           nij=nij+1
           A(:,i,j)=E(:,nij)
           !if(A(1,i,j).ne.dble(1000*i+j))then
           !  WRITE(impierror,*)   "a.ne.dble",a(1,i,j),i,j
           !  if(master)  then
           !      WRITE(impierror,*)   "master is a.ne.dble stop"
           !   CALL MPI_FINALIZE(IERR_MPI) ; STOP
           !  endif
           !else
           !   WRITE(impierror,*)   "a.eq.dble",a(1,i,j),i,j
           !END IF
         END DO
       END DO
       IF (nij.NE.proc_displs(n+1)) THEN
       WRITE(impierror,*)   "nij.ne.proc_displs(n+1) in correct_rsh",nij,proc_displs(n+1)
         CALL MPI_FINALIZE(IERR_MPI) ; STOP
       END IF
     END DO
     DEALLOCATE(e)
     !WRITE(impierror,*)   "no error with exchange_rsh"
     RETURN
   ELSE  !this is a subroutin for the case there is only wet points
   !correct on row direction(make imin:imax complete on minrproc)
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(1,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(1,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE 
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   IF (myrproc==minrproc) THEN
     ALLOCATE(C(ndim,imin:imax,ljmin:ljmax))
     C=0.0_rsh
     C(:,limin:limax,ljmin:ljmax)=B(:,limin:limax,ljmin:ljmax)
   END IF

   DO x=minrproc+1,maxrproc

     IF (myrproc.EQ.minrproc) THEN
       count_get=NDIM*LI(3,x)*LJ(3,myjproc)
       ALLOCATE(D(NDIM, LI(3,x),LJ(3,myjproc)))
       dest=PROC(x,myjproc)
       etigo=x*100
       CALL MPI_RECV(D,count_get,TYPE_NDIM,dest,etigo,MPI_COMM_MARS,stat,IERR_MPI)
       C(:,LI(1,x):LI(2,x),ljmin:ljmax)=D
       DEALLOCATE(D)
     ELSE IF (myrproc.EQ.x) THEN
       count_send=NDIM*LI(3,myrproc)*LJ(3,myjproc)
       dest=PROC(minrproc,myjproc)
       etigo=x*100
       CALL MPI_SEND(B,count_send,TYPE_NDIM,dest,etigo,MPI_COMM_MARS,IERR_MPI)
     END IF
   END DO

   IF (iproc==0) A(:,imin:imax,ljmin:ljmax)=C

   IF (myrproc==minrproc) THEN
     DO  x=minjproc+1,maxjproc
       IF (iproc.EQ.0) THEN
         count_get=NDIM*(imax-imin+1)*LJ(3,x)
         !must found most left PROC(:,x) which is not -1)
         DO n=miniproc,maxiproc
           dest=PROC(n,x)
           IF (dest.NE.-1)EXIT
         END DO
         etigo=x*1000
         ALLOCATE(D(NDIM, imin:imax,LJ(3,x)))
         CALL MPI_RECV(D(:,:,:),count_get,TYPE_NDIM,dest,etigo,MPI_COMM_MARS,stat,IERR_MPI)
         A(:,:,LJ(1,x):LJ(2,x))=D
         DEALLOCATE(D)
       ELSE IF (myjproc==x) THEN
         count_send=NDIM*(imax-imin+1)*LJ(3,myjproc)
         dest=0
         etigo=x*1000
         CALL MPI_SEND(c(:,:,:),count_send,TYPE_NDIM,dest,etigo,MPI_COMM_MARS,IERR_MPI)
       END IF
     END DO
   END IF

   IF (myrproc==minrproc) THEN
     DEALLOCATE(C)
   END IF

   ! sendcount=LJ(3,myjproc) 
   ! recvcounts= LJ(3,:)
   ! displs= LJ(4,:)

   ! call MPI_GATHERV(B,sendcount , TYPE_NDIM, &
   !      & A, recvcounts, displs,     TYPE_NDIM, 0,MPI_COMM_MARS, IERR_MPI)
   !      if(ierr_mpi.ne.0) print*,              IERR_MPI,"ierror at exchange"

   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
  RETURN
  ENDIF

  END SUBROUTINE correct_rsh

   !!======================================================================

  SUBROUTINE exchange_j_int(A,NDIM) !not yet tested!!

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_j_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : jmin,jmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                             ,INTENT(in)    :: ndim
   INTEGER,       DIMENSION(NDIM,limin:limax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   INTEGER,       DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   INTEGER,       ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part

   B=A(:,:,ljmin:ljmax)
   size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
   ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
   doA:DO x=mincbudy,maxcbudy
     y=cbudy(x)
     IF (y.LT.mincproc) CYCLE doA
     IF (y.GT.maxcproc) CYCLE doA
     IF (y.EQ.myjproc) THEN
       A(:,:,ljmin:ljmax)=B
       CYCLE doA
     END IF
     give=B
     size_get=ndim*LI(3,myiproc)*LJ(3,y)
     ALLOCATE(get(ndim,LI(3,myiproc),LJ(3,y)))
     dest=proc(myiproc,y)
     CALL sendrecv_int(dest,size_give,give,size_get,get)
     A(:,:,LJ(1,y):LJ(2,y))=get
     DEALLOCATE(get)
   END DO doA
   RETURN

  END SUBROUTINE exchange_j_int

   !!======================================================================

  SUBROUTINE exchange_j_rsh(A,NDIM) !not yet tested!!

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_j_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : jmin,jmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                             ,INTENT(in)    :: ndim
   REAL(kind=rsh),DIMENSION(NDIM,limin:limax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   REAL(kind=rsh),DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part

   B=A(:,:,ljmin:ljmax)
   size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
   ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
   doA:DO x=mincbudy,maxcbudy
     y=cbudy(x)
     IF (y.LT.mincproc) CYCLE doA
     IF (y.GT.maxcproc) CYCLE doA
     IF (y.EQ.myjproc) THEN
       A(:,:,ljmin:ljmax)=B
       CYCLE doA
     END IF
     give=B
     size_get=ndim*LI(3,myiproc)*LJ(3,y)
     ALLOCATE(get(ndim,LI(3,myiproc),LJ(3,y)))
     dest=proc(myiproc,y)
     !WRITE(impierror,*)   "i am in  exchange_j_rsh"
     !WRITE(impierror,*)   "in  exchange_j_rsh just before sendrecv",myiproc,y,dest
     CALL sendrecv_rsh(dest,size_give,give,size_get,get)
     !WRITE(impierror,*)   "in  exchange_j_rsh just after sendrecv"
     A(:,:,LJ(1,y):LJ(2,y))=get
     DEALLOCATE(get)
   END DO doA
   RETURN

  END SUBROUTINE exchange_j_rsh

   !!======================================================================

  SUBROUTINE exchange_i_rsh(A,NDIM) !not yet tested!!

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_i_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                             ,INTENT(in)    :: ndim
   REAL(kind=rsh),DIMENSION(NDIM,imin:imax,ljmin:ljmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   REAL(kind=rsh),DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part

   B=A(:,limin:limax,:)
   size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
   ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
   doA:DO x=minrbudy,maxrbudy
     y=rbudy(x)
     IF (y.LT.minrproc) CYCLE doA
     IF (y.GT.maxrproc) CYCLE doA
     IF (y.EQ.myiproc) CYCLE doA
     give=B
     size_get=ndim*LI(3,y)*LJ(3,myjproc)
     ALLOCATE(get(ndim,LI(3,y),LJ(3,myjproc)))
     dest=proc(y,myjproc)
     CALL sendrecv_rsh(dest,size_give,give,size_get,get)
     A(:,LI(1,y):LI(2,y),:)=get
     DEALLOCATE(get)
   END DO doA
   RETURN

  END SUBROUTINE exchange_i_rsh

   !!======================================================================

  SUBROUTINE exchange_j_rlg(A,NDIM) !not yet tested!!

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_j_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,ljmin,ljmax,limin,limax
   USE parameters, ONLY : jmin,jmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                             ,INTENT(in)    :: ndim
   REAL(kind=rlg),DIMENSION(NDIM,limin:limax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   REAL(kind=rlg),DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part

   B=A(:,:,ljmin:ljmax)
   size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
   ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
   doA:DO x=mincbudy,maxcbudy
     y=cbudy(x)
     IF (y.LT.mincproc) CYCLE doA
     IF (y.GT.maxcproc) CYCLE doA
     IF (y.EQ.myjproc) THEN
       A(:,:,ljmin:ljmax)=B
       CYCLE doA
     END IF
     give=B
     size_get=ndim*LI(3,myiproc)*LJ(3,y)
     ALLOCATE(get(ndim,LI(3,myiproc),LJ(3,y)))
     dest=proc(myiproc,y)
     !WRITE(impierror,*)   "i am in  exchange_j_rlg"
     !WRITE(impierror,*)   "in  exchange_j_rlg just before sendrecv",myiproc,y,dest
     CALL sendrecv_rlg(dest,size_give,give,size_get,get)
     !WRITE(impierror,*)   "in  exchange_j_rlg just after sendrecv"
     A(:,:,LJ(1,y):LJ(2,y))=get
     DEALLOCATE(get)
   END DO doA
   RETURN

  END SUBROUTINE exchange_j_rlg

   !!======================================================================

  SUBROUTINE exchange_i_rlg(A,NDIM) !!! by akiko, copyed exchange_i_rsh.

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_i_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                             ,INTENT(in)    :: ndim
   REAL(kind=rlg),DIMENSION(NDIM,imin:imax,ljmin:ljmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   REAL(kind=rlg),DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part
   
   B=A(:,limin:limax,:)
   size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
   ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
   doA:DO x=minrbudy,maxrbudy
     y=rbudy(x)
     IF (y.LT.minrproc) CYCLE doA
     IF (y.GT.maxrproc) CYCLE doA
     IF (y.EQ.myiproc) CYCLE doA
     give=B
     size_get=ndim*LI(3,y)*LJ(3,myjproc)
     ALLOCATE(get(ndim,LI(3,y),LJ(3,myjproc)))
     dest=proc(y,myjproc)
     CALL sendrecv_rlg(dest,size_give,give,size_get,get)
     A(:,LI(1,y):LI(2,y),:)=get
     DEALLOCATE(get)
   END DO doA
   RETURN

   END SUBROUTINE exchange_i_rlg

   !!======================================================================

#ifdef key_MPI_2D
   SUBROUTINE exchange_i_int(A,NDIM) !!!akiko, copyed exchange_i_rlg.

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_i_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                      ,INTENT(in)    :: ndim
   INTEGER,DIMENSION(NDIM,imin:imax,ljmin:ljmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                                :: x,y
   INTEGER                                                :: dest,size_give,size_get
   INTEGER,DIMENSION(NDIM,limin:limax,ljmin:ljmax) :: B
   INTEGER,ALLOCATABLE,DIMENSION(:,:,:)            :: GIVE,GET

   !!----------------------------------------------------------------------
   !! * Executable part

    B=A(:,limin:limax,:)
    size_give=ndim*LI(3,myiproc)*LJ(3,myjproc)
    ALLOCATE(GIVE(ndim,LI(3,myiproc),LJ(3,myjproc)))
    doA:DO x=minrbudy,maxrbudy
      y=rbudy(x)
      IF (y.LT.minrproc) CYCLE doA
      IF (y.GT.maxrproc) CYCLE doA
      IF (y.EQ.myiproc) CYCLE doA
      give=B
      size_get=ndim*LI(3,y)*LJ(3,myjproc)
      ALLOCATE(get(ndim,LI(3,y),LJ(3,myjproc)))
      dest=proc(y,myjproc)
      CALL sendrecv_int(dest,size_give,give,size_get,get)
      A(:,LI(1,y):LI(2,y),:)=get
      DEALLOCATE(get)
    END DO doA
    RETURN

   END SUBROUTINE exchange_i_int
#endif

   !!======================================================================
  SUBROUTINE exchange_rlg(A,NDIM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_rlg  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax,jmin,jmax
   IMPLICIT NONE

   !! * Arguments
   INTEGER                                           ,INTENT(in)    :: ndim
   REAL(kind=rlg),DIMENSION(NDIM,imin:imax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: i,j,nsize,n,nij,ni,nj
   INTEGER                                     :: TYPE_NDIM,sendcount
   INTEGER                                     :: size_give,x,y,size_get,dest
   INTEGER,DIMENSION(nprocs)                   :: recvcounts,displs
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:,:) :: B,C,give,get
   REAL(kind=rlg),ALLOCATABLE,DIMENSION(:,:)   :: D

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (maxiproc==0.AND.maxjproc==0) THEN
     RETURN
   ELSE IF ((maxiproc==0).AND.(maxjproc.NE.0)) THEN

     ! a way of avoiding using allgatherv
     !        allocate(C(NDIM,imin:imax,ljmin:ljmax))  
     !         c(:,:,ljmin:ljmax)=A(:,:,ljmin:ljmax)
     !          call exchange_j_rlg(C,ndim)
     !        A=C
     !        deallocate(c)
     !        return

     IF (rlg.EQ.8) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM*(imax-imin+1),MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
     ELSE
       WRITE(impierror,*)' this version of exchange works only for rlg=8'
       WRITE(impierror,*)'you gave rlg=',rlg
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

     sendcount=LJ(3,myjproc)
     recvcounts(1:nprocs)= LJ(3,minjproc:maxjproc)
     displs(1:nprocs)= LJ(4,minjproc:maxjproc)
     ALLOCATE(C(NDIM,1:imax-imin+1,LJ(3,myjproc)))
     C(:,:,:)=A(:,:,ljmin:ljmax)
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & A, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     DEALLOCATE(c)
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     RETURN

   ELSE IF (maxjproc==0.AND.maxiproc.NE.0) THEN  !not yet tested
     CALL exchange_i_rlg(A,ndim)
     RETURN

   ELSE IF (nbblank.NE.0) THEN     !tested with nblank.eq.0,only with ljmin:ljmax
     !tinatestWRITE(impierror,*)   "i am in nbblanc.ne0 exchange"

     IF (rlg.EQ.8) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
     ELSE
       WRITE(impierror,*)' this version of exchange works only for rlg=8'
       WRITE(impierror,*)'you gave rlg=',rlg
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
     nsize=SUM(proc_size(minp:maxp))
     !tinatestWRITE(impierror,*)   "nsize",nsize
     ALLOCATE(C(NDIM,LI(3,myiproc),LJ(3,myjproc)))
     !test tina
     !tinatest DO j=ljmin,ljmax
     !tinatest     DO i=limin,limax
     !tinatest      A(1,i,j)=dble(1000*i+j)
     !tinatest    END DO
     !tinatest  ENDDO
     C(:,:,:)=A(:,limin:limax,ljmin:ljmax)  !data that i hold, which will be send to everyone
     ALLOCATE(D(NDIM,nsize))                ! data box that i keep all the data from everyone
     sendcount=proc_size(iproc)
     !tinatest WRITE(impierror,*)   "sendcount",sendcount
     recvcounts(1:nprocs)=proc_size(minp:maxp)
     displs(1:nprocs)=proc_displs(minp:maxp)
     !tinatest WRITE(impierror,*)   "just before allgatherv"
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & D, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     DEALLOCATE(c)
     !tinatest WRITE(impierror,*)   "just after allgatherv"
     nij=0
     DO n=minp,maxp
       !IF (n==iproc)CYCLE
       !nij=proc_displs(N)
       ! here i can also use function reshape
       DO j=proc_ljmin(n),proc_ljmax(n)
         DO i=proc_limin(n),proc_limax(n)
           nij=nij+1
           A(:,i,j)=D(:,nij)
          !tinatest if(A(1,i,j).ne.dble(1000*i+j))then
          !tinatest WRITE(impierror,*)   "a.ne.dble",a(1,i,j),i,j
          !tinatest CALL MPI_FINALIZE(IERR_MPI) ; STOP
          !tinatest else
          !tinatest WRITE(impierror,*)   "a.eq.dble",a(1,i,j),i,j
          !tinatest END IF
         END DO
       END DO
       IF (nij.NE.proc_displs(n+1)) THEN
       WRITE(impierror,*)   "nij.ne.proc_displs(n+1)",nij,proc_displs(n+1)
         CALL MPI_FINALIZE(IERR_MPI) ; STOP
       END IF
     END DO
     DEALLOCATE(d)
     !tinatest WRITE(impierror,*)   "no error with exchange_rlg"
     RETURN
   ELSE  !this is a subroutin for the case there is only wet points

     !WRITE(impierror,*)   "i am  iafter else exchange,nbblank",nbblank
     ALLOCATE(C(NDIM,LI(3,myiproc),jmin:jmax))   !not yet tested
     IF (mincproc==maxcproc) THEN
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
     ELSE
     !WRITE(impierror,*)   "just before exchangej-rlg"
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
       CALL exchange_j_rlg(C,ndim)
     !WRITE(impierror,*)   "just after exchangej-rlg"
     END IF
     !WRITE(impierror,*)   "just before mpi_barrier"
     call MPI_BARRIER(MPI_COMM_MARS,IERR_MPI)
     !WRITE(impierror,*)   "just after mpi_barrier"
     size_give=ndim*(jmax-jmin+1)*LI(3,myiproc)
     ALLOCATE(GIVE(ndim,LI(3,myiproc),jmin:jmax))
     doA:    DO x=minibudy,maxibudy
       y=ibudy(x)
       IF (y.LT.miniproc) CYCLE doA
       IF (y.GT.maxiproc) CYCLE doA
       IF (y.EQ.myiproc) THEN
         A(:,limin:limax,:)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(jmax-jmin+1)*LI(3,y)
       ALLOCATE(get(ndim,LI(3,y),jmin:jmax))
       dest=proc(y,myjproc)
     !WRITE(impierror,*)   "just before sendrecv"
       CALL sendrecv_rlg(dest,size_give,give,size_get,get)
     !WRITE(impierror,*)   "just after sendrecv"
       A(:,LI(1,y):LI(2,y),jmin:jmax)=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)

#ifdef key_tested
     ALLOCATE(C(NDIM,imin:imax,LJ(3,myjproc)))    !?? LJ(1:LJ(2) ??
     !need to test with i for 16 procs j for 0 procs also
     !memo.tina  tested  need to test with exchange_j_rlg too, to test exchange_j
     IF (minrproc==maxrproc) THEN
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)
     ELSE
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)! this line was missing.
       !need to check if there is anymore bug or not 09.10.2008
       CALL exchange_i_rlg(C,ndim)
     END IF
     size_give=ndim*(imax-imin+1)*LJ(3,myjproc)
     ALLOCATE(GIVE(ndim,imin:imax,LJ(3,myjproc)))
     doA:    DO x=minjbudy,maxjbudy
       y=jbudy(x)                   !here for blank.ne.0, need to find a way
       !which works in a good way
       IF (y.LT.minjproc) CYCLE doA
       IF (y.GT.maxjproc) CYCLE doA
       IF (y.EQ.myjproc) THEN
         A(:,:,ljmin:ljmax)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(imax-imin+1)*LJ(3,y)
       ALLOCATE(get(ndim,imin:imax,LJ(3,y)))
       dest=proc(myiproc,y)
       CALL sendrecv_rlg(dest,size_give,give,size_get,get)
       A(:,imin:imax,LJ(1,y):LJ(2,y))=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)
#endif
     RETURN
   END IF

  END SUBROUTINE exchange_rlg

   !!======================================================================

  SUBROUTINE exchange_rsh(A,NDIM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_rsh  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax,jmin,jmax
   IMPLICIT NONE
   
   !! * Arguments
   INTEGER                                           ,INTENT(in)    :: ndim
   REAL(kind=rsh),DIMENSION(NDIM,imin:imax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: i,j,nsize,n,nij,ni,nj
   INTEGER                                     :: TYPE_NDIM,sendcount
   INTEGER                                     :: size_give,x,y,size_get,dest
   INTEGER,DIMENSION(nprocs)                   :: recvcounts,displs
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:,:) :: B,C,give,get
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:)   :: D

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (maxiproc==0.AND.maxjproc==0) THEN
     RETURN
   ELSE IF ((maxiproc==0).AND.(maxjproc.NE.0)) THEN  

     ! a way of avoiding using allgatherv
     !        allocate(C(NDIM,imin:imax,ljmin:ljmax))  
     !         c(:,:,ljmin:ljmax)=A(:,:,ljmin:ljmax)
     !          call exchange_j_rsh(C,ndim)
     !        A=C
     !        deallocate(c)
     !        return

     IF (rsh.EQ.4) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM*(imax-imin+1),MPI_REAL,TYPE_NDIM, IERR_MPI )
     ELSE IF (rsh.EQ.8) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM*(imax-imin+1),MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
     ELSE 
       WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
       WRITE(impierror,*)'you gave rsh=',rsh
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

     sendcount=LJ(3,myjproc) 
     recvcounts(1:nprocs)= LJ(3,minjproc:maxjproc)
     displs(1:nprocs)= LJ(4,minjproc:maxjproc)
     ALLOCATE(C(NDIM,1:imax-imin+1,LJ(3,myjproc)))
     C(:,:,:)=A(:,:,ljmin:ljmax)
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & A, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     DEALLOCATE(c)
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     RETURN

   ELSE IF (maxjproc==0.AND.maxiproc.NE.0) THEN  !not yet tested
     CALL exchange_i_rsh(A,ndim)
     RETURN

   ELSE IF (nbblank.NE.0) THEN     !tested with nblank.eq.0,only with ljmin:ljmax
     !tinatestWRITE(impierror,*)   "i am in nbblanc.ne0 exchange"

     IF (rsh.EQ.4) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
     ELSE IF (rsh.EQ.8) THEN
       CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
     ELSE 
       WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
       WRITE(impierror,*)'you gave rsh=',rsh
       CALL MPI_FINALIZE(IERR_MPI) ; STOP
     END IF
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
     nsize=SUM(proc_size(minp:maxp))
     !tinatestWRITE(impierror,*)   "nsize",nsize
     ALLOCATE(C(NDIM,LI(3,myiproc),LJ(3,myjproc)))
     !test tina
     !tinatest DO j=ljmin,ljmax
     !tinatest     DO i=limin,limax
     !tinatest      A(1,i,j)=dble(1000*i+j)
     !tinatest    END DO
     !tinatest  ENDDO
     C(:,:,:)=A(:,limin:limax,ljmin:ljmax)  !data that i hold, which will be send to everyone
     ALLOCATE(D(NDIM,nsize))                ! data box that i keep all the data from everyone
     sendcount=proc_size(iproc)
     !tinatest WRITE(impierror,*)   "sendcount",sendcount
     recvcounts(1:nprocs)=proc_size(minp:maxp)
     displs(1:nprocs)=proc_displs(minp:maxp)
     !tinatest WRITE(impierror,*)   "just before allgatherv"
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & D, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     DEALLOCATE(c)
     !tinatest WRITE(impierror,*)   "just after allgatherv"
     nij=0
     DO n=minp,maxp
       !IF (n==iproc)CYCLE
       !nij=proc_displs(N)
       ! here i can also use function reshape
       DO j=proc_ljmin(n),proc_ljmax(n)
         DO i=proc_limin(n),proc_limax(n)
           nij=nij+1
           A(:,i,j)=D(:,nij)
          !tinatest if(A(1,i,j).ne.dble(1000*i+j))then
          !tinatest WRITE(impierror,*)   "a.ne.dble",a(1,i,j),i,j
          !tinatest CALL MPI_FINALIZE(IERR_MPI) ; STOP
          !tinatest else
          !tinatest WRITE(impierror,*)   "a.eq.dble",a(1,i,j),i,j
          !tinatest END IF
         END DO
       END DO
       IF (nij.NE.proc_displs(n+1)) THEN
       WRITE(impierror,*)   "nij.ne.proc_displs(n+1)",nij,proc_displs(n+1)
         CALL MPI_FINALIZE(IERR_MPI) ; STOP
       END IF
     END DO
     DEALLOCATE(d)
     !tinatest WRITE(impierror,*)   "no error with exchange_rsh"
     RETURN
   ELSE  !this is a subroutin for the case there is only wet points

     !WRITE(impierror,*)   "i am  iafter else exchange,nbblank",nbblank
     ALLOCATE(C(NDIM,LI(3,myiproc),jmin:jmax))   !not yet tested
     IF (mincproc==maxcproc) THEN
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
     ELSE
     !WRITE(impierror,*)   "just before exchangej-rsh"
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
       CALL exchange_j_rsh(C,ndim)
     !WRITE(impierror,*)   "just after exchangej-rsh"
     END IF
     !WRITE(impierror,*)   "just before mpi_barrier"
     call MPI_BARRIER(MPI_COMM_MARS,IERR_MPI)
     !WRITE(impierror,*)   "just after mpi_barrier"
     size_give=ndim*(jmax-jmin+1)*LI(3,myiproc)
     ALLOCATE(GIVE(ndim,LI(3,myiproc),jmin:jmax))
     doA:    DO x=minibudy,maxibudy
       y=ibudy(x)                   
       IF (y.LT.miniproc) CYCLE doA
       IF (y.GT.maxiproc) CYCLE doA
       IF (y.EQ.myiproc) THEN
         A(:,limin:limax,:)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(jmax-jmin+1)*LI(3,y)
       ALLOCATE(get(ndim,LI(3,y),jmin:jmax))
       dest=proc(y,myjproc)
     !WRITE(impierror,*)   "just before sendrecv"
       CALL sendrecv_rsh(dest,size_give,give,size_get,get)
     !WRITE(impierror,*)   "just after sendrecv"
       A(:,LI(1,y):LI(2,y),jmin:jmax)=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)

#ifdef key_tested
     ALLOCATE(C(NDIM,imin:imax,LJ(3,myjproc)))    !?? LJ(1:LJ(2) ??
     !need to test with i for 16 procs j for 0 procs also
     !memo.tina  tested  need to test with exchange_j_rsh too, to test exchange_j
     IF (minrproc==maxrproc) THEN
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)
     ELSE
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)! this line was missing.
       !need to check if there is anymore bug or not 09.10.2008
       CALL exchange_i_rsh(C,ndim)
     END IF
     size_give=ndim*(imax-imin+1)*LJ(3,myjproc)
     ALLOCATE(GIVE(ndim,imin:imax,LJ(3,myjproc)))
     doA:    DO x=minjbudy,maxjbudy
       y=jbudy(x)                   !here for blank.ne.0, need to find a way
       !which works in a good way
       IF (y.LT.minjproc) CYCLE doA
       IF (y.GT.maxjproc) CYCLE doA
       IF (y.EQ.myjproc) THEN
         A(:,:,ljmin:ljmax)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(imax-imin+1)*LJ(3,y)
       ALLOCATE(get(ndim,imin:imax,LJ(3,y)))
       dest=proc(myiproc,y)
       CALL sendrecv_rsh(dest,size_give,give,size_get,get)
       A(:,imin:imax,LJ(1,y):LJ(2,y))=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)
#endif
     RETURN
   END IF

  END SUBROUTINE exchange_rsh

   !!======================================================================

  SUBROUTINE exchange_int(A,NDIM)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE exchange_int  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Modified variables : A
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh,ljmin,ljmax,limin,limax
   USE parameters, ONLY : imin,imax,jmin,jmax
   IMPLICIT NONE
   
   !! * Arguments
   INTEGER                                    ,INTENT(in)    :: ndim
   INTEGER,DIMENSION(NDIM,imin:imax,jmin:jmax),INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                     :: i,j,nsize,n,nij,ni,nj
   INTEGER                                     :: TYPE_NDIM,sendcount
   INTEGER                                     :: size_give,x,y,size_get,dest
   INTEGER,DIMENSION(nprocs)                   :: recvcounts,displs
   INTEGER,ALLOCATABLE,DIMENSION(:,:,:) :: B,C,give,get
   INTEGER,ALLOCATABLE,DIMENSION(:,:)   :: D

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (maxiproc==0.AND.maxjproc==0) THEN
     RETURN
   ELSE IF ((maxiproc==0).AND.(maxjproc.NE.0)) THEN  

     ! a way of avoiding using allgatherv
     !        allocate(C(NDIM,imin:imax,ljmin:ljmax))  
     !         c(:,:,ljmin:ljmax)=A(:,:,ljmin:ljmax)
     !          call exchange_j_int(C,ndim)
     !        A=C
     !        deallocate(c)
     !        return

     CALL MPI_TYPE_CONTIGUOUS(NDIM*(imax-imin+1),MPI_INTEGER,TYPE_NDIM, IERR_MPI )
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )

     sendcount=LJ(3,myjproc) 
     recvcounts(1:nprocs)= LJ(3,minjproc:maxjproc)
     displs(1:nprocs)= LJ(4,minjproc:maxjproc)
     ALLOCATE(C(NDIM,1:imax-imin+1,LJ(3,myjproc)))
     C(:,:,:)=A(:,:,ljmin:ljmax)
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & A, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     DEALLOCATE(c)
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     RETURN

   ELSE IF (maxjproc==0.AND.maxiproc.NE.0) THEN  !not yet tested
     CALL exchange_i_int(A,ndim)
     RETURN

   ELSE IF (nbblank.NE.0) THEN     !tested with nblank.eq.0,only with ljmin:ljmax

     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_INTEGER,TYPE_NDIM, IERR_MPI )
     CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
     nsize=SUM(proc_size(minp:maxp))
     ALLOCATE(C(NDIM,LI(3,myiproc),LJ(3,myjproc)))
     C(:,:,:)=A(:,limin:limax,ljmin:ljmax)
     ALLOCATE(D(NDIM,nsize))
     sendcount=proc_size(iproc)
     recvcounts(1:nprocs)=proc_size(minp:maxp)
     displs(1:nprocs)=proc_displs(minp:maxp)
     CALL MPI_ALLGATHERV(C,sendcount , TYPE_NDIM, &
       & D, recvcounts, displs,     TYPE_NDIM, MPI_COMM_MARS, IERR_MPI)
     IF (ierr_mpi.NE.0) WRITE(impierror,*)              IERR_MPI,"ierror at exchange"
     CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
     DEALLOCATE(c)
     DO n=minp,maxp
       IF (n==iproc)CYCLE
       ni=proc_i(N)
       nj=proc_j(N)
       nij=proc_displs(N)
       ! here i can also use function reshape
       DO j=LJ(1,nj),LJ(2,nj)
         DO i=LI(1,ni),LI(2,ni)
           nij=nij+1
           A(:,i,j)=D(:,nij)
         END DO
       END DO
       IF (nij.NE.proc_displs(n+1))STOP
     END DO
     DEALLOCATE(d)
     RETURN



   ELSE 

     ALLOCATE(C(NDIM,LI(3,myiproc),jmin:jmax))   !not yet tested
     IF (mincproc==maxcproc) THEN
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
     ELSE
       C(:,:,ljmin:ljmax)=A(:,limin:limax,ljmin:ljmax)
       CALL exchange_j_int(C,ndim)
     END IF
     size_give=ndim*(jmax-jmin+1)*LI(3,myiproc)
     ALLOCATE(GIVE(ndim,LI(3,myiproc),jmin:jmax))
     doA:    DO x=minibudy,maxibudy
       y=ibudy(x)                   !here for blank.ne.0, need to find a way
       !which works in a good way
       IF (y.LT.miniproc) CYCLE doA
       IF (y.GT.maxiproc) CYCLE doA
       IF (y.EQ.myiproc) THEN
         A(:,limin:limax,:)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(jmax-jmin+1)*LI(3,y)
       ALLOCATE(get(ndim,LI(3,y),jmin:jmax))
       dest=proc(y,myjproc)
       CALL sendrecv_int(dest,size_give,give,size_get,get)
       A(:,LI(1,y):LI(2,y),jmin:jmax)=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)

#ifdef key_tested
     ALLOCATE(C(NDIM,imin:imax,LJ(3,myjproc)))    !?? LJ(1:LJ(2) ??
     !need to test with i for 16 procs j for 0 procs also
     !memo.tina  tested  need to test with exchange_j_int too, to test exchange_j
     IF (minrproc==maxrproc) THEN
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)
     ELSE
       C(:,limin:limax,:)=A(:,limin:limax,ljmin:ljmax)! this line was missing.
       !need to check if there is anymore bug or not 09.10.2008
       CALL exchange_i_int(C,ndim)
     END IF
     size_give=ndim*(imax-imin+1)*LJ(3,myjproc)
     ALLOCATE(GIVE(ndim,imin:imax,LJ(3,myjproc)))
     doA:    DO x=minjbudy,maxjbudy
       y=jbudy(x)                   !here for blank.ne.0, need to find a way
       !which works in a good way
       IF (y.LT.minjproc) CYCLE doA
       IF (y.GT.maxjproc) CYCLE doA
       IF (y.EQ.myjproc) THEN
         A(:,:,ljmin:ljmax)=C
         CYCLE doA
       END IF
       give=C
       size_get=ndim*(imax-imin+1)*LJ(3,y)
       ALLOCATE(get(ndim,imin:imax,LJ(3,y)))
       dest=proc(myiproc,y)
       CALL sendrecv_int(dest,size_give,give,size_get,get)
       A(:,imin:imax,LJ(1,y):LJ(2,y))=get
       DEALLOCATE(get)
     END DO doA
     DEALLOCATE(give)
     DEALLOCATE(c)
#endif
     RETURN
   END IF

  END SUBROUTINE exchange_int

   !!======================================================================

  SUBROUTINE MPI_EXJ_RECV(A,NDIM,eti,i,j,aiteadd)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_EXJ_RECV  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY:rsh,jmin,jmax

   !! * Arguments
   INTEGER                       ,INTENT(inout) :: NDIM,eti,i,j,aiteadd
   REAL(kind=rsh),DIMENSION(NDIM),INTENT(inout) :: A

   !! * Local declarations
   INTEGER :: TYPE_NDIM,etigo,aite,stat

   !!----------------------------------------------------------------------
   !! * Executable part
   
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE 
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   etigo=eti*(jmax-jmin)*j+i
   aite=proc(myiproc,myjproc+aiteadd)
   CALL MPI_RECV(A(:),1,TYPE_NDIM,aite,etigo,MPI_COMM_MARS,stat,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )

  END SUBROUTINE MPI_EXJ_RECV

   !!======================================================================

   SUBROUTINE MPI_EXJ_SEND(A,NDIM,eti,i,j,aiteadd)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_EXJ_SEND  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
    USE parameters,ONLY:rsh,jmin,jmax

   !! * Arguments
   INTEGER                       ,INTENT(inout) :: NDIM,eti,i,j,aiteadd
   REAL(kind=rsh),DIMENSION(NDIM),INTENT(inout) :: A

   !! * Local declarations
   INTEGER :: TYPE_NDIM,etigo,aite

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE 
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   etigo=eti*(jmax-jmin)*j+i
   aite=proc(myiproc,myjproc+aiteadd)
   CALL MPI_SEND(A(:),1,TYPE_NDIM,aite,etigo,MPI_COMM_MARS,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )

  END SUBROUTINE MPI_EXJ_SEND

   !!======================================================================

  SUBROUTINE MPI_EXI_RECV(A,NDIM,eti,i,j,aiteadd)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_EXI_RECV  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY:rsh,imin,imax

   !! * Arguments
   INTEGER                       ,INTENT(inout) :: NDIM,eti,i,j,aiteadd
   REAL(kind=rsh),DIMENSION(NDIM),INTENT(inout) :: A

   !! * Local declarations
   INTEGER :: TYPE_NDIM,etigo,aite,stat

   !!----------------------------------------------------------------------
   !! * Executable part
   
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   etigo=eti*(imax-imin)*j+i
   aite=proc(myiproc+aiteadd,myjproc)
   CALL MPI_RECV(A(:),1,TYPE_NDIM,aite,etigo,MPI_COMM_MARS,stat,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )

  END SUBROUTINE MPI_EXI_RECV

   !!======================================================================

  SUBROUTINE MPI_EXI_SEND(A,NDIM,eti,i,j,aiteadd)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE MPI_EXI_SEND  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
    USE parameters,ONLY:rsh,imin,imax

   !! * Arguments
   INTEGER                       ,INTENT(inout) :: NDIM,eti,i,j,aiteadd
   REAL(kind=rsh),DIMENSION(NDIM),INTENT(inout) :: A

   !! * Local declarations
   INTEGER :: TYPE_NDIM,etigo,aite

   !!----------------------------------------------------------------------
   !! * Executable part
   
   IF (rsh.EQ.4) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ELSE
     WRITE(impierror,*)' this version of exchange works only for rsh=4 and 8'
     WRITE(impierror,*)'you gave rsh=',rsh
     CALL MPI_FINALIZE(IERR_MPI) ; STOP
   END IF
   CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   etigo=eti*(imax-imin)*j+i
   aite=proc(myiproc+aiteadd,myjproc)
   CALL MPI_SEND(A(:),1,TYPE_NDIM,aite,etigo,MPI_COMM_MARS,IERR_MPI)
   CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )

  END SUBROUTINE MPI_EXI_SEND

   !!======================================================================

  SUBROUTINE ADD_MPI_RSH(mine,total)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ADD_MPI_RSH  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),INTENT(in ) ::  mine
   REAL(kind=rsh),INTENT(out) ::  total

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

    ! print*,'before iam=',iproc,'sbruit',mine
    IF (rsh.EQ.4) THEN
      CALL MPI_REDUCE(mine,total,1,MPI_REAL,MPI_SUM,0,MPI_COMM_MARS,IERR_MPI)
    ELSE IF (rsh.EQ.8) THEN
      CALL MPI_REDUCE(mine,total,1,MPI_Double_Precision,MPI_SUM,0,MPI_COMM_MARS,IERR_MPI)
    END IF
    ! print*,'after iam=',iproc,'sbruit',mine

   END SUBROUTINE ADD_MPI_RSH

   !!======================================================================

  SUBROUTINE ADD_MPI_RLG(mine,total)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ADD_MPI_RLG  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rlg),INTENT(in ) ::  mine
   REAL(kind=rlg),INTENT(out) ::  total

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

    ! print*,'before iam=',iproc,'sbruit',mine
    CALL MPI_REDUCE(mine,total,1,MPI_Double_Precision,MPI_SUM,0,MPI_COMM_MARS,IERR_MPI)
    ! print*,'after iam=',iproc,'sbruit',mine

   END SUBROUTINE ADD_MPI_RLG

   !!======================================================================

   SUBROUTINE ADD_MPI_INT(mine,total)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ADD_MPI_INT  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER,INTENT(in ) ::  mine
   INTEGER,INTENT(out) ::  total

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'nb',mine
   CALL MPI_REDUCE(mine,total,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_MARS,IERR_MPI)
   !mine=total
   ! print*,'after iam=',iproc,'nb',mine

  END SUBROUTINE ADD_MPI_INT

   !!======================================================================

  SUBROUTINE BCAST_MPI_LOGIC(val)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE BCAST_MPI_LOGIC  ***
   !&E
   !&E ** Purpose : The MASTER sends a locical value to every cpus
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   LOGICAL,INTENT(inout) ::  val

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'lipa',val
   CALL MPI_BCAST(val,1,MPI_LOGICAL,0,MPI_COMM_MARS,IERR_MPI)
   ! print*,'after iam=',iproc,'lipa',val

  END SUBROUTINE  BCAST_MPI_LOGIC

   !!======================================================================

  SUBROUTINE BCAST_MPI_INT(val)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE BCAST_MPI_INT  ***
   !&E
   !&E ** Purpose : The MASTER sends an integer value to every cpus
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   INTEGER,INTENT(inout) ::  val

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'lipa',val
   CALL MPI_BCAST(val,1,MPI_INTEGER,0,MPI_COMM_MARS,IERR_MPI)
   ! print*,'after iam=',iproc,'lipa',val

  END SUBROUTINE  BCAST_MPI_INT

   !!======================================================================

  SUBROUTINE BCAST_MPI_RSH(val)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE BCAST_MPI_RSH  ***
   !&E
   !&E ** Purpose : The MASTER sends a RSH (single precision) value to every cpus
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=4),INTENT(inout) ::  val

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'lipa',val
   CALL MPI_BCAST(val,1,MPI_REAL,0,MPI_COMM_MARS,IERR_MPI)
   ! print*,'after iam=',iproc,'lipa',val

  END SUBROUTINE  BCAST_MPI_RSH

   !!======================================================================

  SUBROUTINE BCAST_MPI_RLG(val)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE BCAST_MPI_RLG  ***
   !&E
   !&E ** Purpose : The MASTER sends a RLG (double precision) value to every cpus
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rlg
   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=8),INTENT(inout) ::  val

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'lipa',val
   CALL MPI_BCAST(val,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_MARS,IERR_MPI)
   ! print*,'after iam=',iproc,'lipa',val

  END SUBROUTINE  BCAST_MPI_RLG

   !!======================================================================

  SUBROUTINE BCAST_MPI_CHAR(val)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE BCAST_MPI_CHAR  ***
   !&E
   !&E ** Purpose : The MASTER sends an integer value to every cpus
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : lchain
   IMPLICIT NONE

   !! * Arguments
   CHARACTER(LEN=*),INTENT(inout) ::  val

   !!----------------------------------------------------------------------
   !! * Executable part

   ! print*,'before iam=',iproc,'lipa',val
   CALL MPI_BCAST(val,1,MPI_CHAR,0,MPI_COMM_MARS,IERR_MPI)
   ! print*,'after iam=',iproc,'lipa',val

  END SUBROUTINE  BCAST_MPI_CHAR

   !!======================================================================

   ! subroutine GATHER_MPI_RSH(g,l,NDIM)
   ! USE parameters, only : rsh
   ! USE parameters, only : jmin,jmax,ljmin,ljmax
   ! implicit none
   ! integer TYPE_NDIM,sendcount,ndim
   ! integer,dimension(nprocs)::recvcounts,displs
   ! real(kind=rsh) g(ndim,jmin:jmax),l(ndim,ljmin:ljmax)
   ! if(rsh.eq.4)then
   ! call MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
   ! elseif(rsh.eq.8)then
   ! call MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
   ! endif
   ! call MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
   !
   ! sendcount=LJ(3,iproc) 
   ! recvcounts= LJ(3,:)
   ! displs= LJ(4,:)
   ! call MPI_GATHERV(l, SENDCOUNT, TYPE_NDIM, g, RECVCOUNTS,DISPLS, TYPE_NDIM, &
     !    &        0, MPI_COMM_MARS, IERR_MPI)
   ! end subroutine GATHER_MPI_RSH

   !!======================================================================

  SUBROUTINE FINDMIN_MPI_1(LV)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE FINDMIN_MPI_1 ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),INTENT(inout) :: LV

   !! * Local declarations
   REAL(kind=rsh)                            :: LA,LB,LC,SMALL,WHYSV
   INTEGER                                   :: k,kSMALL
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:)   :: V

   !!----------------------------------------------------------------------
   !! * Executable part

   WHYSV=LV
   ALLOCATE(V(nprocs))
   IF (rsh.EQ.4) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_REAL,V, 1,MPI_REAL,MPI_COMM_MARS, IERR_MPI)
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_DOUBLE_PRECISION, V(1),1,MPI_DOUBLE_PRECISION,MPI_COMM_MARS,IERR_MPI)
   END IF

   SMALL=V(1)
   kSMALL=1
   DO k=1,NPROCS
     IF (V(k).LT.SMALL) THEN
       kSMALL=k
       SMALL=V(k)
     END IF
   END DO
   LV=V(kSMALL)
   DEALLOCATE(V)
   RETURN

  END SUBROUTINE FINDMIN_MPI_1

   !!======================================================================

  SUBROUTINE FINDMAX_MPI_1(LV)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE FINDMAX_MPI_1 ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),INTENT(inout) :: LV

   !! * Local declarations
   REAL(kind=rsh)                            :: LA,LB,LC,BIG,WHYSV
   INTEGER                                   :: k,kbig
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:)   :: V

   !!----------------------------------------------------------------------
   !! * Executable part

   WHYSV=LV
   ALLOCATE(V(nprocs))
   IF (rsh.EQ.4) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_REAL,V, 1,MPI_REAL,MPI_COMM_MARS, IERR_MPI)
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_DOUBLE_PRECISION, V(1),1,MPI_DOUBLE_PRECISION,MPI_COMM_MARS,IERR_MPI)
   END IF

   BIG=V(1)
   kBIG=1
   DO k=1,NPROCS
     IF (V(k).GT.BIG) THEN
       kBIG=k
       BIG=V(k)
     END IF
   END DO
   LV=V(kBIG)
   DEALLOCATE(V)
   RETURN

  END SUBROUTINE FINDMAX_MPI_1

   !!======================================================================

  SUBROUTINE FINDMAX_MPI_3(LV,LLI,LLJ)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE FINDMAX_MPI_3  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),INTENT(inout) :: LV
   INTEGER       ,INTENT(inout) :: LLI,LLJ

   !! * Local declarations
   REAL(kind=rsh)                            :: LA,LB,LC,BIG,WHYSV
   INTEGER                                   :: k,kbig,WHYSI(2)
   !real(kind=rsh),dimension(4):: SV
   !integer       ,dimension(2):: SI
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:)   :: V
   INTEGER       ,ALLOCATABLE,DIMENSION(:,:) :: I

   !!----------------------------------------------------------------------
   !! * Executable part

   WHYSV=LV
   ALLOCATE(V(nprocs))
   IF (rsh.EQ.4) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_REAL,V, 1,MPI_REAL,MPI_COMM_MARS, IERR_MPI)
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_ALLGATHER(WHYSV,1,MPI_DOUBLE_PRECISION, V(1),1,MPI_DOUBLE_PRECISION,MPI_COMM_MARS,IERR_MPI)
   END IF
   WHYSI(1)=LLI
   WHYSI(2)=LLJ
   ALLOCATE(I(2,nprocs))
   CALL MPI_ALLGATHER(WHYSI,2,MPI_INTEGER,I,2,MPI_INTEGER,MPI_COMM_MARS, IERR_MPI)

   BIG=V(1)
   kBIG=1
   DO k=1,NPROCS
     IF (V(k).GT.BIG) THEN
       kBIG=k
       BIG=V(k)
     END IF
   END DO
   LV=V(kBIG)
   LLI=I(1,kBIG)
   LLJ=I(2,kBIG)
   DEALLOCATE(V,I)
   RETURN

  END SUBROUTINE FINDMAX_MPI_3

   !!======================================================================

  SUBROUTINE FINDMAX_MPI(LV,LLI,LLJ,LA,LB,LC)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE FINDMAX_MPI  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : rsh
   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),INTENT(inout) :: LV,LA,LB,LC
   INTEGER       ,INTENT(inout) :: LLI,LLJ

   !! * Local declarations
   REAL(kind=rsh)                            :: BIG,WHYSV(4)
   INTEGER                                   :: k,kbig,WHYSI(2)
   !real(kind=rsh),dimension(4):: SV
   !integer       ,dimension(2):: SI
   REAL(kind=rsh),ALLOCATABLE,DIMENSION(:,:) :: V
   INTEGER       ,ALLOCATABLE,DIMENSION(:,:) :: I

   !!----------------------------------------------------------------------
   !! * Executable part
   
   WHYSV(1)=LV
   WHYSV(2)=LA
   WHYSV(3)=LB
   WHYSV(4)=LC
   ALLOCATE(V(4,nprocs))
   IF (rsh.EQ.4) THEN
     CALL MPI_ALLGATHER(WHYSV,4,MPI_REAL,V, 4,MPI_REAL,MPI_COMM_MARS, IERR_MPI)
   ELSE IF (rsh.EQ.8) THEN
     CALL MPI_ALLGATHER(WHYSV,4,MPI_DOUBLE_PRECISION, V(1,1),4,MPI_DOUBLE_PRECISION,MPI_COMM_MARS,IERR_MPI)
   END IF
   WHYSI(1)=LLI
   WHYSI(2)=LLJ
   ALLOCATE(I(2,nprocs))
   CALL MPI_ALLGATHER(WHYSI,2,MPI_INTEGER,I,2,MPI_INTEGER,MPI_COMM_MARS, IERR_MPI)

   !use MPI_PACK so that the program only need to send one value? combine u and v
   !max search??

   BIG=V(1,1)
   kBIG=1
   DO k=1,NPROCS
     IF (V(1,k).GT.BIG) THEN
       kBIG=k
       BIG=V(1,k)
     END IF
   END DO
   LV=V(1,kBIG)
   LA=V(2,kBIG)
   LB=V(3,kBIG)
   LC=V(4,kBIG)
   LLI=I(1,kBIG)
   LLJ=I(2,kBIG)
   DEALLOCATE(V,I)
   RETURN

  END SUBROUTINE FINDMAX_MPI

   !!======================================================================

  SUBROUTINE ADD_ALL_MPI_RSH(mine)
   USE parameters, ONLY : rsh
   IMPLICIT NONE
   REAL(KIND=rsh)  mine, total
   ! print*,'before iam=',iproc,'nb',mine
   IF (rsh == 4) THEN
     CALL MPI_ALLREDUCE(mine,total,1,MPI_REAL,MPI_SUM,MPI_COMM_MARS,IERR_MPI)
   ELSE IF (rsh == 8) THEN
     CALL MPI_ALLREDUCE(mine,total,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_MARS,IERR_MPI)
   END IF
   mine=total
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE  ADD_ALL_MPI_RSH

   !!======================================================================

  SUBROUTINE ADD_ALL_MPI_RLG(mine)
   USE parameters, ONLY : rlg
   IMPLICIT NONE
   REAL(KIND=rlg)  mine, total
   ! print*,'before iam=',iproc,'nb',mine
   CALL MPI_ALLREDUCE(mine,total,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_MARS,IERR_MPI)
   mine=total
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE  ADD_ALL_MPI_RLG

   !!======================================================================

  SUBROUTINE CREATE_COMM_ZONAL(COMM_ZONAL,comm2dverif)

   INTEGER, INTENT(INOUT)     :: comm_zonal
   INTEGER, INTENT(INOUT), OPTIONAL     :: comm2dverif
   INTEGER                    :: comm1d,comm2d
   INTEGER,PARAMETER          :: ndim2d=2
   INTEGER, DIMENSION(ndim2d) :: dim2d
   LOGICAL, DIMENSION(ndim2d) :: period,subdivision
   LOGICAL                    :: reorder

   ! create a topology (2D)
   dim2d(2)=MAXIPROC+1
   dim2d(1)=MAXJPROC+1
   period(:)=.FALSE.
   reorder=.FALSE.
   CALL MPI_CART_CREATE(MPI_COMM_MARS,ndim2d,dim2d,period,reorder,comm2d,IERR_MPI)

   ! moyenne zonale : chaque ligne de la topologie 2D (ou ligne de processeurs) est une topologie cartesienne 1D
   ! Subdivisions inversees car Tina a range les cpus dans ordre i croisant et non j croissant
   subdivision(1)=.FALSE.   ! Tina a range les cpus dans ordre i croisant et non j croissant
   subdivision(2)=.TRUE.
   CALL MPI_CART_SUB(comm2d,subdivision,comm1d,IERR_MPI)
   !print*, __LINE__,"RANK",iproc," DANS COMM",comm1d ! negatif et identique pour tout proc !!!

   comm_zonal=comm1d

   PRINT_DBG*,'exit CREATE_COMM_ZONAL'

  END SUBROUTINE CREATE_COMM_ZONAL

   !!======================================================================

  SUBROUTINE ADD_ZONAL_RSH(mine,mind1,maxd1,mind2,maxd2,comm1d)
   USE parameters, ONLY : rsh

   IMPLICIT NONE

   REAL(KIND=rsh), DIMENSION(mind1:maxd1,mind2:maxd2), INTENT(inout) ::  mine
   INTEGER, INTENT(in) :: mind1,maxd1,mind2,maxd2,comm1d

   INTEGER, DIMENSION(1)            :: dim12
   REAL(KIND=rsh), DIMENSION((maxd1-mind1+1)*(maxd2-mind2+1)) ::  vecmine, vectotal

   PRINT_DBG*,'enter ADD_ZONAL_RSH',mind1,maxd1,mind2,maxd2,comm1d

   dim12=(maxd1-mind1+1)*(maxd2-mind2+1)
   vecmine(:)=RESHAPE(mine,dim12)
   IF (rsh == 4) THEN
     CALL MPI_ALLREDUCE(vecmine,vectotal,dim12,MPI_REAL,MPI_SUM,comm1d,IERR_MPI)
   ELSE IF (rsh == 8) THEN
     CALL MPI_ALLREDUCE(vecmine,vectotal,dim12,MPI_DOUBLE_PRECISION,MPI_SUM,comm1d,IERR_MPI)
   END IF
   mine(:,:)=RESHAPE(vectotal,SHAPE(mine))

   PRINT_DBG*,'exit ADD_ZONAL_RSH'

  END SUBROUTINE ADD_ZONAL_RSH

   !!======================================================================
!! rajoute par benedicte pour faire le max (idem que FINDMAX_MPI_1 ?) 
  SUBROUTINE MAX_ALL_MPI_RSH(mine,comm_active)
   USE parameters, ONLY : rsh
   IMPLICIT NONE
   REAL(KIND=rsh) :: mine, maxi
   INTEGER,INTENT(IN),OPTIONAL  :: comm_active
   INTEGER        :: communicator
   ! print*,'before iam=',iproc,'nb',mine
   IF (PRESENT(comm_active)) THEN
     communicator=comm_active
   ELSE
     communicator=MPI_COMM_MARS
   END IF
   IF (rsh == 4)THEN
     CALL MPI_ALLREDUCE(mine,maxi,1,MPI_REAL,MPI_MAX,communicator,IERR_MPI)
   ELSE IF (rsh == 8)THEN
     CALL MPI_ALLREDUCE(mine,maxi,1,MPI_DOUBLE_PRECISION,MPI_MAX,communicator,IERR_MPI)
   ENDIF
   mine=maxi
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE  MAX_ALL_MPI_RSH
   !!======================================================================
!! cval copie et correction depuis MAX_ALL_MPI_RSH 
  SUBROUTINE MIN_ALL_MPI_RSH(mine,comm_active)
   USE parameters, ONLY : rsh
   IMPLICIT NONE
   REAL(KIND=rsh) :: mine, maxi
   INTEGER,INTENT(IN),OPTIONAL  :: comm_active
   INTEGER        :: communicator
   ! print*,'before iam=',iproc,'nb',mine
   IF (PRESENT(comm_active)) THEN
     communicator=comm_active
   ELSE
     communicator=MPI_COMM_MARS
   END IF
   IF (rsh == 4)THEN
     CALL MPI_ALLREDUCE(mine,maxi,1,MPI_REAL,MPI_MIN,communicator,IERR_MPI)
   ELSE IF (rsh == 8)THEN
     CALL MPI_ALLREDUCE(mine,maxi,1,MPI_DOUBLE_PRECISION,MPI_MIN,communicator,IERR_MPI)
   ENDIF
   mine=maxi
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE  MIN_ALL_MPI_RSH
   !!======================================================================
!! from MAX_ALL_MPI_RSH
  SUBROUTINE MAX_ALL_MPI_INT(mine,comm_active)
   IMPLICIT NONE
   INTEGER,INTENT(INOUT) :: mine
   INTEGER,INTENT(IN),OPTIONAL  :: comm_active
   INTEGER        :: maxi
   INTEGER        :: communicator
   ! print*,'before iam=',iproc,'nb',mine
   IF (PRESENT(comm_active)) THEN
     communicator=comm_active
   ELSE
     communicator=MPI_COMM_MARS
   END IF
   CALL MPI_ALLREDUCE(mine,maxi,1,MPI_INTEGER,MPI_MAX,communicator,IERR_MPI)
   mine=maxi
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE MAX_ALL_MPI_INT
   !!======================================================================
!! from MIN_ALL_MPI_RSH
  SUBROUTINE MIN_ALL_MPI_INT(mine,comm_active)
   IMPLICIT NONE
   INTEGER,INTENT(INOUT) :: mine
   INTEGER,INTENT(IN),OPTIONAL  :: comm_active
   INTEGER        :: mini
   INTEGER        :: communicator
   ! print*,'before iam=',iproc,'nb',mine
   IF (PRESENT(comm_active)) THEN
     communicator=comm_active
   ELSE
     communicator=MPI_COMM_MARS
   END IF
   CALL MPI_ALLREDUCE(mine,mini,1,MPI_INTEGER,MPI_MIN,communicator,IERR_MPI)
   mine=mini
   ! print*,'after iam=',iproc,'nb',mine
  END SUBROUTINE MIN_ALL_MPI_INT
   !!======================================================================
!! below 3 are obsolute, will be deleted !!!

   SUBROUTINE ADD_ALL_MPI_INT(mine)
    IMPLICIT NONE
    INTEGER  mine, total
    ! print*,'before iam=',iproc,'nb',mine
    CALL MPI_ALLREDUCE(mine,total,1,MPI_INTEGER,MPI_SUM,MPI_COMM_MARS,IERR_MPI)
    mine=total
    ! print*,'after iam=',iproc,'nb',mine
   END SUBROUTINE  ADD_ALL_MPI_INT

   SUBROUTINE MPI_t_bilmat(tmp,NDIM)  !test done?
    USE parameters, ONLY : rlg
    IMPLICIT NONE
    INTEGER                            ::NDIM,TYPE_NDIM
    REAL(kind=rlg)  ,DIMENSION(ndim)   ::tmp,added
    IF (rlg.EQ.4) THEN
      CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_REAL,TYPE_NDIM, IERR_MPI )
    ELSE IF (rlg.EQ.8) THEN
      CALL MPI_TYPE_CONTIGUOUS(NDIM,MPI_DOUBLE_PRECISION,TYPE_NDIM, IERR_MPI )
    END IF
    CALL MPI_TYPE_COMMIT(TYPE_NDIM, IERR_MPI )
    CALL MPI_REDUCE(tmp,added,1,TYPE_NDIM,MPI_SUM,0,MPI_COMM_MARS,IERR_MPI)
    CALL MPI_TYPE_FREE(TYPE_NDIM, IERR_MPI )
    IF (MASTER)tmp=added
   END SUBROUTINE MPI_t_bilmat

   !!======================================================================

  SUBROUTINE exchange_vectcpu_int(A)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE EXCHANGE_VECTCPU_INT ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   IMPLICIT NONE

   !! * Arguments
   INTEGER, DIMENSION(0:nprocs-1), INTENT(inout) :: A

   !! * Local declarations
   INTEGER                                   :: V

   !!----------------------------------------------------------------------
   !! * Executable part

   V=A(iproc)
   CALL MPI_ALLGATHER(V,1,MPI_INTEGER,A,1,MPI_INTEGER,MPI_COMM_MARS,IERR_MPI)

  END SUBROUTINE exchange_vectcpu_int
#endif

 END MODULE toolmpi
